@@ -24,32 +24,25 @@
 #include <isc/hash.h>
 #include <isc/log.h>
 #include <isc/mem.h>
-#include <isc/print.h>
 #include <isc/result.h>
 #include <isc/string.h>
 #include <isc/util.h>
 
 #include <dns/db.h>
 #include <dns/fixedname.h>
-#include <dns/log.h>
 #include <dns/name.h>
 #include <dns/rdataclass.h>
 #include <dns/rootns.h>
 #include <dns/zone.h>
 
+#include <isccfg/check.h>
 #include <isccfg/grammar.h>
 #include <isccfg/namedconf.h>
 
-#include <bind9/check.h>
-
 #include "check-tool.h"
 
 static const char *program = "named-checkconf";
 
-static bool loadplugins = true;
-
-isc_log_t *logc = NULL;
-
 #define CHECK(r)                             \
 	do {                                 \
 		result = (r);                \
@@ -58,13 +51,13 @@ isc_log_t *logc = NULL;
 	} while (0)
 
 /*% usage */
-noreturn static void
+ISC_NORETURN static void
 usage(void);
 
 static void
 usage(void) {
 	fprintf(stderr,
-		"usage: %s [-chijlvz] [-p [-x]] [-t directory] "
+		"usage: %s [-achijlvz] [-p [-x]] [-t directory] "
 		"[named.conf]\n",
 		program);
 	exit(EXIT_SUCCESS);
@@ -87,13 +80,13 @@ directory_callback(const char *clausename, const cfg_obj_t *obj, void *arg) {
 	directory = cfg_obj_asstring(obj);
 	result = isc_dir_chdir(directory);
 	if (result != ISC_R_SUCCESS) {
-		cfg_obj_log(obj, logc, ISC_LOG_ERROR,
+		cfg_obj_log(obj, ISC_LOG_ERROR,
 			    "change directory to '%s' failed: %s\n", directory,
 			    isc_result_totext(result));
-		return (result);
+		return result;
 	}
 
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 }
 
 static bool
@@ -101,10 +94,10 @@ get_maps(const cfg_obj_t **maps, const char *name, const cfg_obj_t **obj) {
 	int i;
 	for (i = 0;; i++) {
 		if (maps[i] == NULL) {
-			return (false);
+			return false;
 		}
 		if (cfg_map_get(maps[i], name, obj) == ISC_R_SUCCESS) {
-			return (true);
+			return true;
 		}
 	}
 }
@@ -120,7 +113,7 @@ get_checknames(const cfg_obj_t **maps, const cfg_obj_t **obj) {
 
 	for (i = 0;; i++) {
 		if (maps[i] == NULL) {
-			return (false);
+			return false;
 		}
 		checknames = NULL;
 		result = cfg_map_get(maps[i], "check-names", &checknames);
@@ -129,7 +122,7 @@ get_checknames(const cfg_obj_t **maps, const cfg_obj_t **obj) {
 		}
 		if (checknames != NULL && !cfg_obj_islist(checknames)) {
 			*obj = checknames;
-			return (true);
+			return true;
 		}
 		for (element = cfg_list_first(checknames); element != NULL;
 		     element = cfg_list_next(element))
@@ -143,7 +136,7 @@ get_checknames(const cfg_obj_t **maps, const cfg_obj_t **obj) {
 				continue;
 			}
 			*obj = cfg_tuple_get(value, "mode");
-			return (true);
+			return true;
 		}
 	}
 }
@@ -156,23 +149,23 @@ configure_hint(const char *zfile, const char *zclass, isc_mem_t *mctx) {
 	isc_textregion_t r;
 
 	if (zfile == NULL) {
-		return (ISC_R_FAILURE);
+		return ISC_R_FAILURE;
 	}
 
-	DE_CONST(zclass, r.base);
+	r.base = UNCONST(zclass);
 	r.length = strlen(zclass);
 	result = dns_rdataclass_fromtext(&rdclass, &r);
 	if (result != ISC_R_SUCCESS) {
-		return (result);
+		return result;
 	}
 
 	result = dns_rootns_create(mctx, rdclass, zfile, &db);
 	if (result != ISC_R_SUCCESS) {
-		return (result);
+		return result;
 	}
 
 	dns_db_detach(&db);
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 }
 
 /*% configure the zone */
@@ -228,33 +221,33 @@ configure_zone(const char *vclass, const char *view, const cfg_obj_t *zconfig,
 		printf("%s %s %s in-view %s\n", zname, zclass, view, inview);
 	}
 	if (inviewobj != NULL) {
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 
 	cfg_map_get(zoptions, "type", &typeobj);
 	if (typeobj == NULL) {
-		return (ISC_R_FAILURE);
+		return ISC_R_FAILURE;
 	}
 
 	if (list) {
 		const char *ztype = cfg_obj_asstring(typeobj);
 		printf("%s %s %s %s\n", zname, zclass, view, ztype);
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 
 	/*
 	 * Skip checks when using an alternate data source.
 	 */
 	cfg_map_get(zoptions, "database", &dbobj);
-	if (dbobj != NULL && strcmp("rbt", cfg_obj_asstring(dbobj)) != 0 &&
-	    strcmp("rbt64", cfg_obj_asstring(dbobj)) != 0)
+	if (dbobj != NULL &&
+	    strcmp(ZONEDB_DEFAULT, cfg_obj_asstring(dbobj)) != 0)
 	{
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 
 	cfg_map_get(zoptions, "dlz", &dlzobj);
 	if (dlzobj != NULL) {
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 
 	cfg_map_get(zoptions, "file", &fileobj);
@@ -268,12 +261,12 @@ configure_zone(const char *vclass, const char *view, const cfg_obj_t *zconfig,
 	 * master and redirect
 	 */
 	if (strcasecmp(cfg_obj_asstring(typeobj), "hint") == 0) {
-		return (configure_hint(zfile, zclass, mctx));
+		return configure_hint(zfile, zclass, mctx);
 	} else if ((strcasecmp(cfg_obj_asstring(typeobj), "primary") != 0) &&
 		   (strcasecmp(cfg_obj_asstring(typeobj), "master") != 0) &&
 		   (strcasecmp(cfg_obj_asstring(typeobj), "redirect") != 0))
 	{
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 
 	/*
@@ -286,12 +279,12 @@ configure_zone(const char *vclass, const char *view, const cfg_obj_t *zconfig,
 		}
 
 		if (primariesobj != NULL) {
-			return (ISC_R_SUCCESS);
+			return ISC_R_SUCCESS;
 		}
 	}
 
 	if (zfile == NULL) {
-		return (ISC_R_FAILURE);
+		return ISC_R_FAILURE;
 	}
 
 	obj = NULL;
@@ -403,6 +396,17 @@ configure_zone(const char *vclass, const char *view, const cfg_obj_t *zconfig,
 		zone_options |= DNS_ZONEOPT_CHECKSPF;
 	}
 
+	obj = NULL;
+	if (get_maps(maps, "check-svcb", &obj)) {
+		if (cfg_obj_asboolean(obj)) {
+			zone_options |= DNS_ZONEOPT_CHECKSVCB;
+		} else {
+			zone_options &= ~DNS_ZONEOPT_CHECKSVCB;
+		}
+	} else {
+		zone_options |= DNS_ZONEOPT_CHECKSVCB;
+	}
+
 	obj = NULL;
 	if (get_maps(maps, "check-wildcard", &obj)) {
 		if (cfg_obj_asboolean(obj)) {
@@ -458,7 +462,7 @@ configure_zone(const char *vclass, const char *view, const cfg_obj_t *zconfig,
 		fprintf(stderr, "%s/%s/%s: %s\n", view, zname, zclass,
 			isc_result_totext(result));
 	}
-	return (result);
+	return result;
 }
 
 /*% configure a view */
@@ -493,7 +497,7 @@ configure_view(const char *vclass, const char *view, const cfg_obj_t *config,
 			result = tresult;
 		}
 	}
-	return (result);
+	return result;
 }
 
 static isc_result_t
@@ -503,11 +507,11 @@ config_getclass(const cfg_obj_t *classobj, dns_rdataclass_t defclass,
 
 	if (!cfg_obj_isstring(classobj)) {
 		*classp = defclass;
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
-	DE_CONST(cfg_obj_asstring(classobj), r.base);
+	r.base = UNCONST(cfg_obj_asstring(classobj));
 	r.length = strlen(r.base);
-	return (dns_rdataclass_fromtext(classp, &r));
+	return dns_rdataclass_fromtext(classp, &r);
 }
 
 /*% load zones from the configuration */
@@ -565,15 +569,15 @@ load_zones_fromconfig(const cfg_obj_t *config, isc_mem_t *mctx,
 	}
 
 cleanup:
-	return (result);
+	return result;
 }
 
 static void
 output(void *closure, const char *text, int textlen) {
-	UNUSED(closure);
 	if (fwrite(text, 1, textlen, stdout) != (size_t)textlen) {
+		isc_result_t *result = closure;
 		perror("fwrite");
-		exit(EXIT_FAILURE);
+		*result = ISC_R_FAILURE;
 	}
 }
 
@@ -585,20 +589,21 @@ main(int argc, char **argv) {
 	cfg_obj_t *config = NULL;
 	const char *conffile = NULL;
 	isc_mem_t *mctx = NULL;
-	isc_result_t result;
-	int exit_status = 0;
+	isc_result_t result = ISC_R_SUCCESS;
 	bool load_zones = false;
 	bool list_zones = false;
 	bool print = false;
 	bool nodeprecate = false;
+	bool allconfigs = false;
 	unsigned int flags = 0;
+	unsigned int checkflags = BIND_CHECK_PLUGINS | BIND_CHECK_ALGORITHMS;
 
 	isc_commandline_errprint = false;
 
 	/*
 	 * Process memory debugging argument first.
 	 */
-#define CMDLINE_FLAGS "cdhijlm:t:pvxz"
+#define CMDLINE_FLAGS "acdhijlm:nt:pvxz"
 	while ((c = isc_commandline_parse(argc, argv, CMDLINE_FLAGS)) != -1) {
 		switch (c) {
 		case 'm':
@@ -625,8 +630,12 @@ main(int argc, char **argv) {
 
 	while ((c = isc_commandline_parse(argc, argv, CMDLINE_FLAGS)) != EOF) {
 		switch (c) {
+		case 'a':
+			checkflags &= ~BIND_CHECK_ALGORITHMS;
+			break;
+
 		case 'c':
-			loadplugins = false;
+			checkflags &= ~BIND_CHECK_PLUGINS;
 			break;
 
 		case 'd':
@@ -648,12 +657,16 @@ main(int argc, char **argv) {
 		case 'm':
 			break;
 
+		case 'n':
+			allconfigs = true;
+			break;
+
 		case 't':
 			result = isc_dir_chroot(isc_commandline_argument);
 			if (result != ISC_R_SUCCESS) {
 				fprintf(stderr, "isc_dir_chroot: %s\n",
 					isc_result_totext(result));
-				exit(EXIT_FAILURE);
+				CHECK(result);
 			}
 			break;
 
@@ -663,7 +676,8 @@ main(int argc, char **argv) {
 
 		case 'v':
 			printf("%s\n", PACKAGE_VERSION);
-			exit(EXIT_SUCCESS);
+			result = ISC_R_SUCCESS;
+			goto cleanup;
 
 		case 'x':
 			flags |= CFG_PRINTER_XKEY;
@@ -683,25 +697,27 @@ main(int argc, char **argv) {
 			}
 			FALLTHROUGH;
 		case 'h':
+			isc_mem_detach(&mctx);
 			usage();
 
 		default:
 			fprintf(stderr, "%s: unhandled option -%c\n", program,
 				isc_commandline_option);
-			exit(EXIT_FAILURE);
+			CHECK(ISC_R_FAILURE);
 		}
 	}
 
 	if (((flags & CFG_PRINTER_XKEY) != 0) && !print) {
 		fprintf(stderr, "%s: -x cannot be used without -p\n", program);
-		exit(EXIT_FAILURE);
+		CHECK(ISC_R_FAILURE);
 	}
 	if (print && list_zones) {
 		fprintf(stderr, "%s: -l cannot be used with -p\n", program);
-		exit(EXIT_FAILURE);
+		CHECK(ISC_R_FAILURE);
 	}
 
 	if (isc_commandline_index + 1 < argc) {
+		isc_mem_detach(&mctx);
 		usage();
 	}
 	if (argv[isc_commandline_index] != NULL) {
@@ -711,44 +727,40 @@ main(int argc, char **argv) {
 		conffile = NAMED_CONFFILE;
 	}
 
-	RUNTIME_CHECK(setup_logging(mctx, stdout, &logc) == ISC_R_SUCCESS);
+	CHECK(setup_logging(stdout));
 
-	RUNTIME_CHECK(cfg_parser_create(mctx, logc, &parser) == ISC_R_SUCCESS);
+	CHECK(cfg_parser_create(mctx, &parser));
 
 	if (nodeprecate) {
 		cfg_parser_setflags(parser, CFG_PCTX_NODEPRECATED, true);
 	}
+	if (allconfigs) {
+		cfg_parser_setflags(parser, CFG_PCTX_ALLCONFIGS, true);
+	}
 	cfg_parser_setcallback(parser, directory_callback, NULL);
 
-	if (cfg_parse_file(parser, conffile, &cfg_type_namedconf, &config) !=
-	    ISC_R_SUCCESS)
-	{
-		exit(EXIT_FAILURE);
+	CHECK(cfg_parse_file(parser, conffile, &cfg_type_namedconf, &config));
+	CHECK(isccfg_check_namedconf(config, checkflags, mctx));
+	if (load_zones || list_zones) {
+		CHECK(load_zones_fromconfig(config, mctx, list_zones));
 	}
 
-	result = bind9_check_namedconf(config, loadplugins, nodeprecate, logc,
-				       mctx);
-	if (result != ISC_R_SUCCESS) {
-		exit_status = 1;
+	if (print) {
+		cfg_printx(config, flags, output, &result);
 	}
 
-	if (result == ISC_R_SUCCESS && (load_zones || list_zones)) {
-		result = load_zones_fromconfig(config, mctx, list_zones);
-		if (result != ISC_R_SUCCESS) {
-			exit_status = 1;
-		}
+cleanup:
+	if (config != NULL) {
+		cfg_obj_destroy(parser, &config);
 	}
 
-	if (print && exit_status == 0) {
-		cfg_printx(config, flags, output, NULL);
+	if (parser != NULL) {
+		cfg_parser_destroy(&parser);
 	}
-	cfg_obj_destroy(parser, &config);
 
-	cfg_parser_destroy(&parser);
+	if (mctx != NULL) {
+		isc_mem_destroy(&mctx);
+	}
 
-	isc_log_destroy(&logc);
-
-	isc_mem_destroy(&mctx);
-
-	return (exit_status);
+	return result == ISC_R_SUCCESS ? 0 : 1;
 }
