@@ -53,7 +53,7 @@ const char *progname;
 static enum { progmode_keygen, progmode_confgen } progmode;
 bool verbose = false; /* needed by util.c but not used here */
 
-noreturn static void
+ISC_NORETURN static void
 usage(int status);
 
 static void
