@@ -21,6 +21,7 @@
 
 #include <isc/attributes.h>
 #include <isc/dir.h>
+#include <isc/fips.h>
 #include <isc/loop.h>
 #include <isc/netaddr.h>
 #include <isc/parseint.h>
@@ -69,6 +70,16 @@ static bool short_form = false, printcmd = true, plusquest = false,
 	    pluscomm = false, ipv4only = false, ipv6only = false, digrc = true;
 static uint32_t splitwidth = 0xffffffff;
 
+#include <openssl/opensslv.h>
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#include <openssl/err.h>
+#include <openssl/provider.h>
+#endif
+
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+static OSSL_PROVIDER *fips = NULL, *base = NULL;
+#endif
+
 /*% opcode text */
 static const char *const opcodetext[] = {
 	"QUERY",      "IQUERY",	    "STATUS",	  "RESERVED3",
@@ -111,7 +122,7 @@ usage(void) {
 	fprintf(stderr, "Press <Help> for complete list of options\n");
 }
 #else  /* if TARGET_OS_IPHONE */
-noreturn static void
+ISC_NORETURN static void
 usage(void);
 
 static void
@@ -188,9 +199,6 @@ help(void) {
 	       "                 +[no]cmd            (Control display of "
 	       "command line -\n"
 	       "                                      global option)\n"
-	       "                 +[no]coflag         (Set compact denial of "
-	       "existence ok flag)\n"
-	       "                                      in query)\n"
 	       "                 +[no]comments       (Control display of "
 	       "packet "
 	       "header\n"
@@ -208,8 +216,7 @@ help(void) {
 	       "                 +[no]dnssec         (Request DNSSEC records)\n"
 	       "                 +domain=###         (Set default domainname)\n"
 	       "                 +[no]edns[=###]     (Set EDNS version) [0]\n"
-	       "                 +ednsflags=###      (Set undefined EDNS flag "
-	       "bits)\n"
+	       "                 +ednsflags=###      (Set EDNS flag bits)\n"
 	       "                 +[no]ednsnegotiation (Set EDNS version "
 	       "negotiation)\n"
 	       "                 +ednsopt=###[:value] (Send specified EDNS "
@@ -225,14 +232,12 @@ help(void) {
 	       "                 +[no]https[=###]    (DNS-over-HTTPS mode) "
 	       "[/]\n"
 	       "                 +[no]https-get      (Use GET instead of "
-	       "default POST method\n"
-	       "                                      while using HTTPS)\n"
-	       "                 +[no]http-plain[=###] (DNS over plain HTTP "
-	       "mode) [/]\n"
-	       "                 +[no]http-plain-get (Use GET instead of "
-	       "default POST "
-	       "method\n"
-	       "                                      while using plain HTTP)\n"
+	       "default POST method while using HTTPS)\n"
+	       "                 +[no]http-plain[=###]    (DNS over plain HTTP "
+	       "mode) "
+	       "[/]\n"
+	       "                 +[no]http-plain-get      (Use GET instead of "
+	       "default POST method while using plain HTTP)\n"
 	       "                 +[no]identify       (ID responders in short "
 	       "answers)\n"
 #ifdef HAVE_LIBIDN2
@@ -259,23 +264,15 @@ help(void) {
 	       "                 +padding=###        (Set padding block size "
 	       "[0])\n"
 	       "                 "
-	       "+[no]proxy[=src_addr[#src_port]-dst_addr[#dst_port]]\n"
-	       "                                     (Add PROXYv2 headers to "
-	       "the queries. If\n"
-	       "                                      addresses are omitted, "
-	       "LOCAL PROXYv2\n"
-	       "                                      headers are added)\n"
+	       "+[no]proxy[=src_addr[#src_port]-dst_addr[#dst_port]]        "
+	       "(Add PROXYv2 headers to the queries. If addresses are omitted, "
+	       "LOCAL PROXYv2 headers are added)\n"
 	       "                 "
-	       "+[no]proxy-plain[=src_addr[#src_port]-dst_addr[#dst_port]]\n"
-	       "                                     (The same as'+[no]proxy', "
-	       "but send PROXYv2\n"
-	       "                                      headers ahead of any "
-	       "encryption if an\n"
-	       "                                      encrypted transport is "
-	       "used)\n"
+	       "+[no]proxy-plain[=src_addr[#src_port]-dst_addr[#dst_port]]  "
+	       "(The same as '+[no]proxy', but send PROXYv2 headers ahead of "
+	       "any encryption if an encrypted transport is used)\n"
 	       "                 +qid=###            (Specify the query ID to "
-	       "use when sending\n"
-	       "                                      queries)\n"
+	       "use when sending queries)\n"
 	       "                 +[no]qr             (Print question before "
 	       "sending)\n"
 	       "                 +[no]question       (Control display of "
@@ -298,7 +295,6 @@ help(void) {
 	       "                                      form of answers - global "
 	       "option)\n"
 	       "                 +[no]showbadcookie  (Show BADCOOKIE message)\n"
-	       "                 +[no]showbadvers    (Show BADVERS message)\n"
 	       "                 +[no]showsearch     (Search with intermediate "
 	       "results)\n"
 	       "                 +[no]split=##       (Split hex/base64 fields "
@@ -313,19 +309,15 @@ help(void) {
 	       "                 +timeout=###        (Set query timeout) [5]\n"
 	       "                 +[no]tls            (DNS-over-TLS mode)\n"
 	       "                 +[no]tls-ca[=file]  (Enable remote server's "
-	       "TLS certificate\n"
-	       "                                      validation)\n"
+	       "TLS certificate validation)\n"
 	       "                 +[no]tls-hostname=hostname (Explicitly set "
-	       "the expected TLS\n"
-	       "                                      hostname)\n"
+	       "the expected TLS hostname)\n"
 	       "                 +[no]tls-certfile=file (Load client TLS "
-	       "certificate chain from\n"
-	       "                                      file)\n"
+	       "certificate chain from file)\n"
 	       "                 +[no]tls-keyfile=file (Load client TLS "
 	       "private key from file)\n"
 	       "                 +[no]trace          (Trace delegation down "
-	       "from root [implies\n"
-	       "                                      +dnssec])\n"
+	       "from root [implies +dnssec])\n"
 	       "                 +tries=###          (Set number of UDP "
 	       "attempts) [3]\n"
 	       "                 +[no]ttlid          (Control display of ttls "
@@ -1652,11 +1644,6 @@ plus_option(char *option, bool is_batchfile, bool *need_clone,
 			break;
 		case 'o': /* comments */
 			switch (cmd[2]) {
-			case 'f':
-			case '\0': /* +co is a synonym for +coflag */
-				FULLCHECK("coflag");
-				lookup->coflag = state;
-				break;
 			case 'm':
 				FULLCHECK("comments");
 				lookup->comments = state;
@@ -1785,8 +1772,6 @@ plus_option(char *option, bool is_batchfile, bool *need_clone,
 						FULLCHECK("edns");
 						if (!state) {
 							lookup->edns = -1;
-							lookup->original_edns =
-								-1;
 							break;
 						}
 						if (value == NULL) {
@@ -1803,7 +1788,6 @@ plus_option(char *option, bool is_batchfile, bool *need_clone,
 							goto exit_or_usage;
 						}
 						lookup->edns = num;
-						lookup->original_edns = num;
 						break;
 					case 'f':
 						FULLCHECK("ednsflags");
@@ -2322,18 +2306,8 @@ plus_option(char *option, bool is_batchfile, bool *need_clone,
 			case 'w': /* showsearch */
 				switch (cmd[4]) {
 				case 'b':
-					switch (cmd[7]) {
-					case 'c':
-						FULLCHECK("showbadcookie");
-						lookup->showbadcookie = state;
-						break;
-					case 'v':
-						FULLCHECK("showbadvers");
-						lookup->showbadvers = state;
-						break;
-					default:
-						goto invalid_option;
-					}
+					FULLCHECK("showbadcookie");
+					lookup->showbadcookie = state;
 					break;
 				case 's':
 					FULLCHECK("showsearch");
@@ -2610,8 +2584,8 @@ exit_or_usage:
 /*%
  * #true returned if value was used
  */
-static const char *single_dash_opts = "46dhimnruv";
-static const char *dash_opts = "46bcdfhikmnpqrtvyx";
+static const char *single_dash_opts = "46dFhimnruv";
+static const char *dash_opts = "46bcdFfhikmnpqrtvyx";
 static bool
 dash_option(char *option, char *next, dig_lookup_t **lookup,
 	    bool *open_type_class, bool *need_clone, bool config_only, int argc,
@@ -2668,6 +2642,9 @@ dash_option(char *option, char *next, dig_lookup_t **lookup,
 				debugging = true;
 			}
 			break;
+		case 'F': /* FIPS */
+			/* FIPS is handled in preparse_args() */
+			break;
 		case 'h':
 			help();
 			exit(EXIT_SUCCESS);
@@ -2787,7 +2764,7 @@ dash_option(char *option, char *next, dig_lookup_t **lookup,
 				printgreeting(argc, argv, *lookup);
 				*firstarg = false;
 			}
-			ISC_LIST_APPEND(lookup_list, *lookup, link);
+			ISC_LIST_APPEND(lookup_list, (*lookup), link);
 			debug("looking up %s", (*lookup)->textname);
 		}
 		return value_from_next;
@@ -2941,6 +2918,28 @@ preparse_args(int argc, char **argv) {
 				/* For debugging early startup */
 				debugging = true;
 				break;
+			case 'F':
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+				fips = OSSL_PROVIDER_load(NULL, "fips");
+				if (fips == NULL) {
+					ERR_clear_error();
+					fatal("Failed to load FIPS provider");
+				}
+				base = OSSL_PROVIDER_load(NULL, "base");
+				if (base == NULL) {
+					OSSL_PROVIDER_unload(fips);
+					ERR_clear_error();
+					fatal("Failed to load base provider");
+				}
+#endif
+				/* Already in FIPS mode?  */
+				if (isc_fips_mode()) {
+					break;
+				}
+				if (isc_fips_set_mode(1) != ISC_R_SUCCESS) {
+					fatal("setting FIPS mode failed");
+				}
+				break;
 			case 'm':
 				memdebugging = true;
 				isc_mem_debugging = ISC_MEM_DEBUGTRACE |
@@ -3465,5 +3464,14 @@ main(int argc, char **argv) {
 	dig_startup();
 	dig_shutdown();
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+	if (base != NULL) {
+		OSSL_PROVIDER_unload(base);
+	}
+	if (fips != NULL) {
+		OSSL_PROVIDER_unload(fips);
+	}
+#endif
+
 	return exitcode;
 }
