@@ -40,9 +40,7 @@ const char *program = "dnssec-ksr";
 /*
  * Infrastructure
  */
-static isc_log_t *lctx = NULL;
 static isc_mem_t *mctx = NULL;
-const char *engine = NULL;
 /*
  * The domain we are working on
  */
@@ -97,15 +95,23 @@ static int min_dh = 128;
 
 #define READLINE(lex, opt, token)
 
-#define NEXTTOKEN(lex, opt, token)                        \
-	{                                                 \
-		CHECK(isc_lex_gettoken(lex, opt, token)); \
+#define NEXTTOKEN(lex, opt, token)                       \
+	{                                                \
+		ret = isc_lex_gettoken(lex, opt, token); \
+		if (ret != ISC_R_SUCCESS)                \
+			goto cleanup;                    \
 	}
 
-#define BADTOKEN()                              \
-	{                                       \
-		result = ISC_R_UNEXPECTEDTOKEN; \
-		goto cleanup;                   \
+#define BADTOKEN()                           \
+	{                                    \
+		ret = ISC_R_UNEXPECTEDTOKEN; \
+		goto cleanup;                \
+	}
+
+#define CHECK(r)                    \
+	ret = (r);                  \
+	if (ret != ISC_R_SUCCESS) { \
+		goto fail;          \
 	}
 
 isc_bufferlist_t cleanup_list = ISC_LIST_INITIALIZER;
@@ -118,7 +124,6 @@ usage(int ret) {
 	fprintf(stderr, "Version: %s\n", PACKAGE_VERSION);
 	fprintf(stderr, "\n");
 	fprintf(stderr, "Options:\n");
-	fprintf(stderr, "    -E <engine>: name of an OpenSSL engine to use\n");
 	fprintf(stderr, "    -e <date/offset>: end date\n");
 	fprintf(stderr, "    -F: FIPS mode\n");
 	fprintf(stderr, "    -f: KSR file to sign\n");
@@ -171,15 +176,14 @@ getkasp(ksr_ctx_t *ksr, dns_kasp_t **kasp) {
 	cfg_parser_t *parser = NULL;
 	cfg_obj_t *config = NULL;
 
-	RUNTIME_CHECK(cfg_parser_create(mctx, lctx, &parser) == ISC_R_SUCCESS);
+	RUNTIME_CHECK(cfg_parser_create(mctx, &parser) == ISC_R_SUCCESS);
 	if (cfg_parse_file(parser, ksr->configfile, &cfg_type_namedconf,
 			   &config) != ISC_R_SUCCESS)
 	{
 		fatal("unable to load dnssec-policy '%s' from '%s'",
 		      ksr->policy, ksr->configfile);
 	}
-	kasp_from_conf(config, mctx, lctx, ksr->policy, ksr->keydir, engine,
-		       kasp);
+	kasp_from_conf(config, mctx, ksr->policy, ksr->keydir, kasp);
 	if (*kasp == NULL) {
 		fatal("failed to load dnssec-policy '%s'", ksr->policy);
 	}
@@ -211,15 +215,15 @@ get_dnskeys(ksr_ctx_t *ksr, dns_dnsseckeylist_t *keys) {
 	dns_dnsseckeylist_t keys_read;
 	dns_dnsseckey_t **keys_sorted;
 	int i = 0, n = 0;
-	isc_result_t result;
+	isc_result_t ret;
 
 	ISC_LIST_INIT(*keys);
 	ISC_LIST_INIT(keys_read);
-	result = dns_dnssec_findmatchingkeys(name, NULL, ksr->keydir, NULL,
-					     ksr->now, false, mctx, &keys_read);
-	if (result != ISC_R_SUCCESS && result != ISC_R_NOTFOUND) {
+	ret = dns_dnssec_findmatchingkeys(name, NULL, ksr->keydir, NULL,
+					  ksr->now, mctx, &keys_read);
+	if (ret != ISC_R_SUCCESS && ret != ISC_R_NOTFOUND) {
 		fatal("failed to load existing keys from %s: %s", ksr->keydir,
-		      isc_result_totext(result));
+		      isc_result_totext(ret));
 	}
 	/* Sort on keytag. */
 	for (dns_dnsseckey_t *dk = ISC_LIST_HEAD(keys_read); dk != NULL;
@@ -336,7 +340,7 @@ create_key(ksr_ctx_t *ksr, dns_kasp_t *kasp, dns_kasp_key_t *kaspkey,
 	dst_key_t *key = NULL;
 	int options = (DST_TYPE_PRIVATE | DST_TYPE_PUBLIC | DST_TYPE_STATE);
 	isc_buffer_t buf;
-	isc_result_t result;
+	isc_result_t ret;
 	isc_stdtime_t prepub;
 	uint16_t flags = DNS_KEYOWNER_ZONE;
 
@@ -434,26 +438,26 @@ create_key(ksr_ctx_t *ksr, dns_kasp_t *kasp, dns_kasp_key_t *kaspkey,
 				"Generating key pair for bundle %s: ", timestr);
 		}
 		if (ksr->keystore != NULL && ksr->policy != NULL) {
-			result = dns_keystore_keygen(
+			ret = dns_keystore_keygen(
 				ksr->keystore, name, ksr->policy,
 				dns_rdataclass_in, mctx, ksr->alg, ksr->size,
 				flags, &key);
 		} else if (show_progress) {
-			result = dst_key_generate(name, ksr->alg, ksr->size, 0,
-						  flags, DNS_KEYPROTO_DNSSEC,
-						  dns_rdataclass_in, NULL, mctx,
-						  &key, &progress);
+			ret = dst_key_generate(name, ksr->alg, ksr->size, 0,
+					       flags, DNS_KEYPROTO_DNSSEC,
+					       dns_rdataclass_in, NULL, mctx,
+					       &key, &progress);
 			fflush(stderr);
 		} else {
-			result = dst_key_generate(name, ksr->alg, ksr->size, 0,
-						  flags, DNS_KEYPROTO_DNSSEC,
-						  dns_rdataclass_in, NULL, mctx,
-						  &key, NULL);
+			ret = dst_key_generate(name, ksr->alg, ksr->size, 0,
+					       flags, DNS_KEYPROTO_DNSSEC,
+					       dns_rdataclass_in, NULL, mctx,
+					       &key, NULL);
 		}
 
-		if (result != ISC_R_SUCCESS) {
+		if (ret != ISC_R_SUCCESS) {
 			fatal("failed to generate key %s/%s: %s\n", namestr,
-			      algstr, isc_result_totext(result));
+			      algstr, isc_result_totext(ret));
 		}
 
 		/* Do not overwrite an existing key. */
@@ -464,9 +468,9 @@ create_key(ksr_ctx_t *ksr, dns_kasp_t *kasp, dns_kasp_key_t *kaspkey,
 			conflict = true;
 			if (verbose > 0) {
 				isc_buffer_clear(&buf);
-				result = dst_key_buildfilename(
-					key, 0, ksr->keydir, &buf);
-				if (result == ISC_R_SUCCESS) {
+				ret = dst_key_buildfilename(key, 0, ksr->keydir,
+							    &buf);
+				if (ret == ISC_R_SUCCESS) {
 					fprintf(stderr,
 						"%s: %s already exists, or "
 						"might collide with another "
@@ -488,11 +492,11 @@ create_key(ksr_ctx_t *ksr, dns_kasp_t *kasp, dns_kasp_key_t *kaspkey,
 	dst_key_setbool(key, DST_BOOL_KSK, ksr->ksk);
 	dst_key_setbool(key, DST_BOOL_ZSK, !ksr->ksk);
 	dst_key_settime(key, DST_TIME_CREATED, ksr->now);
-	dst_key_settime(key, DST_TIME_PUBLISH, active - prepub);
+	dst_key_settime(key, DST_TIME_PUBLISH, (active - prepub));
 	dst_key_settime(key, DST_TIME_ACTIVATE, active);
 	if (ksr->ksk) {
 		dns_keymgr_settime_syncpublish(key, kasp,
-					       inception == ksr->start);
+					       (inception == ksr->start));
 	}
 
 	if (ksr->lifetime > 0) {
@@ -508,26 +512,26 @@ create_key(ksr_ctx_t *ksr, dns_kasp_t *kasp, dns_kasp_key_t *kaspkey,
 				 ksr->retiresafety + ksr->signdelay;
 		}
 		dst_key_settime(key, DST_TIME_INACTIVE, inactive);
-		dst_key_settime(key, DST_TIME_DELETE, inactive + remove);
+		dst_key_settime(key, DST_TIME_DELETE, (inactive + remove));
 		*expiration = inactive;
 	} else {
 		*expiration = 0;
 	}
 
-	result = dst_key_tofile(key, options, ksr->keydir);
-	if (result != ISC_R_SUCCESS) {
+	ret = dst_key_tofile(key, options, ksr->keydir);
+	if (ret != ISC_R_SUCCESS) {
 		char keystr[DST_KEY_FORMATSIZE];
 		dst_key_format(key, keystr, sizeof(keystr));
 		fatal("failed to write key %s: %s\n", keystr,
-		      isc_result_totext(result));
+		      isc_result_totext(ret));
 	}
 
 output:
 	isc_buffer_clear(&buf);
-	result = dst_key_buildfilename(key, 0, NULL, &buf);
-	if (result != ISC_R_SUCCESS) {
+	ret = dst_key_buildfilename(key, 0, NULL, &buf);
+	if (ret != ISC_R_SUCCESS) {
 		fatal("dst_key_buildfilename returned: %s\n",
-		      isc_result_totext(result));
+		      isc_result_totext(ret));
 	}
 	printf("%s\n", filename);
 	fflush(stdout);
@@ -540,12 +544,12 @@ static void
 print_rdata(dns_rdataset_t *rrset) {
 	isc_buffer_t target;
 	isc_region_t r;
-	isc_result_t result;
+	isc_result_t ret;
 	char buf[4096];
 
 	isc_buffer_init(&target, buf, sizeof(buf));
-	result = dns_rdataset_totext(rrset, name, false, false, &target);
-	if (result != ISC_R_SUCCESS) {
+	ret = dns_rdataset_totext(rrset, name, false, false, &target);
+	if (ret != ISC_R_SUCCESS) {
 		fatal("failed to print rdata");
 	}
 	isc_buffer_usedregion(&target, &r);
@@ -559,7 +563,7 @@ print_dnskeys(dns_kasp_key_t *kaspkey, dns_ttl_t ttl, dns_dnsseckeylist_t *keys,
 	char timestr[26]; /* Minimal buf as per ctime_r() spec. */
 	dns_rdatalist_t *rdatalist = NULL;
 	dns_rdataset_t rdataset = DNS_RDATASET_INIT;
-	isc_result_t result = ISC_R_SUCCESS;
+	isc_result_t ret = ISC_R_SUCCESS;
 	isc_stdtime_t next_bundle = next_inception;
 
 	isc_stdtime_tostring(inception, timestr, sizeof(timestr));
@@ -628,11 +632,11 @@ print_dnskeys(dns_kasp_key_t *kaspkey, dns_ttl_t ttl, dns_dnsseckeylist_t *keys,
 	dns_rdatalist_tordataset(rdatalist, &rdataset);
 	print_rdata(&rdataset);
 
-cleanup:
+fail:
 	/* Cleanup */
 	freerrset(&rdataset);
 
-	if (result != ISC_R_SUCCESS) {
+	if (ret != ISC_R_SUCCESS) {
 		fatal("failed to print %s/%s zsk key pair found for bundle %s",
 		      namestr, algstr, timestr);
 	}
@@ -645,7 +649,7 @@ sign_rrset(ksr_ctx_t *ksr, isc_stdtime_t inception, isc_stdtime_t expiration,
 	   dns_rdataset_t *rrset, dns_dnsseckeylist_t *keys) {
 	dns_rdatalist_t *rrsiglist = NULL;
 	dns_rdataset_t rrsigset = DNS_RDATASET_INIT;
-	isc_result_t result;
+	isc_result_t ret;
 	isc_stdtime_t next_bundle = expiration;
 
 	UNUSED(ksr);
@@ -660,10 +664,10 @@ sign_rrset(ksr_ctx_t *ksr, isc_stdtime_t inception, isc_stdtime_t expiration,
 		isc_buffer_init(&timebuf, timestr, sizeof(timestr));
 		isc_stdtime_tostring(inception, timestr, sizeof(timestr));
 		isc_buffer_init(&b, utc, sizeof(utc));
-		result = dns_time32_totext(inception, &b);
-		if (result != ISC_R_SUCCESS) {
+		ret = dns_time32_totext(inception, &b);
+		if (ret != ISC_R_SUCCESS) {
 			fatal("failed to convert bundle time32 to text: %s",
-			      isc_result_totext(result));
+			      isc_result_totext(ret));
 		}
 		isc_buffer_usedregion(&b, &r);
 		fprintf(stdout, ";; SignedKeyResponse 1.0 %.*s (%s)\n",
@@ -712,9 +716,9 @@ sign_rrset(ksr_ctx_t *ksr, isc_stdtime_t inception, isc_stdtime_t expiration,
 		rrsig = isc_mem_get(mctx, sizeof(*rrsig));
 		dns_rdata_init(rrsig);
 		isc_buffer_init(&buf, rdatabuf, sizeof(rdatabuf));
-		result = dns_dnssec_sign(name, rrset, dk->key, &clockskew,
-					 &expiration, mctx, &buf, &rdata);
-		if (result != ISC_R_SUCCESS) {
+		ret = dns_dnssec_sign(name, rrset, dk->key, &clockskew,
+				      &expiration, mctx, &buf, &rdata);
+		if (ret != ISC_R_SUCCESS) {
 			fatal("failed to sign KSR");
 		}
 		isc_buffer_usedregion(&buf, &rs);
@@ -747,7 +751,7 @@ get_keymaterial(ksr_ctx_t *ksr, dns_kasp_t *kasp, isc_stdtime_t inception,
 	dns_rdatalist_t *dnskeylist = isc_mem_get(mctx, sizeof(*dnskeylist));
 	dns_rdatalist_t *cdnskeylist = isc_mem_get(mctx, sizeof(*cdnskeylist));
 	dns_rdatalist_t *cdslist = isc_mem_get(mctx, sizeof(*cdslist));
-	isc_result_t result = ISC_R_SUCCESS;
+	isc_result_t ret = ISC_R_SUCCESS;
 	isc_stdtime_t next_bundle = next_inception;
 
 	dns_rdatalist_init(dnskeylist);
@@ -891,7 +895,7 @@ get_keymaterial(ksr_ctx_t *ksr, dns_kasp_t *kasp, isc_stdtime_t inception,
 
 	return next_bundle;
 
-cleanup:
+fail:
 	fatal("failed to create KSK/CDS/CDNSKEY");
 	return 0;
 }
@@ -988,7 +992,7 @@ parse_dnskey(isc_lex_t *lex, char *owner, isc_buffer_t *buf, dns_ttl_t *ttl) {
 	dns_name_t *dname = NULL;
 	dns_rdataclass_t rdclass = dns_rdataclass_in;
 	isc_buffer_t b;
-	isc_result_t result;
+	isc_result_t ret;
 	isc_token_t token;
 	unsigned int opt = ISC_LEXOPT_EOL;
 
@@ -1002,12 +1006,12 @@ parse_dnskey(isc_lex_t *lex, char *owner, isc_buffer_t *buf, dns_ttl_t *ttl) {
 	dname = dns_fixedname_initname(&dfname);
 	isc_buffer_init(&b, owner, strlen(owner));
 	isc_buffer_add(&b, strlen(owner));
-	result = dns_name_fromtext(dname, &b, dns_rootname, 0, NULL);
-	if (result != ISC_R_SUCCESS) {
-		goto cleanup;
+	ret = dns_name_fromtext(dname, &b, dns_rootname, 0, NULL);
+	if (ret != ISC_R_SUCCESS) {
+		return ret;
 	}
 	if (dns_name_compare(dname, name) != 0) {
-		CHECK(DNS_R_BADOWNERNAME);
+		return DNS_R_BADOWNERNAME;
 	}
 	isc_buffer_clear(&b);
 
@@ -1018,8 +1022,8 @@ parse_dnskey(isc_lex_t *lex, char *owner, isc_buffer_t *buf, dns_ttl_t *ttl) {
 	}
 
 	/* If it's a TTL, read the next one */
-	result = dns_ttl_fromtext(&token.value.as_textregion, ttl);
-	if (result == ISC_R_SUCCESS) {
+	ret = dns_ttl_fromtext(&token.value.as_textregion, ttl);
+	if (ret == ISC_R_SUCCESS) {
 		NEXTTOKEN(lex, opt, &token);
 	}
 	if (token.type != isc_tokentype_string) {
@@ -1027,8 +1031,8 @@ parse_dnskey(isc_lex_t *lex, char *owner, isc_buffer_t *buf, dns_ttl_t *ttl) {
 	}
 
 	/* If it's a class, read the next one */
-	result = dns_rdataclass_fromtext(&rdclass, &token.value.as_textregion);
-	if (result == ISC_R_SUCCESS) {
+	ret = dns_rdataclass_fromtext(&rdclass, &token.value.as_textregion);
+	if (ret == ISC_R_SUCCESS) {
 		NEXTTOKEN(lex, opt, &token);
 	}
 	if (token.type != isc_tokentype_string) {
@@ -1040,12 +1044,12 @@ parse_dnskey(isc_lex_t *lex, char *owner, isc_buffer_t *buf, dns_ttl_t *ttl) {
 		BADTOKEN();
 	}
 
-	result = dns_rdata_fromtext(NULL, rdclass, dns_rdatatype_dnskey, lex,
-				    name, 0, mctx, buf, NULL);
+	ret = dns_rdata_fromtext(NULL, rdclass, dns_rdatatype_dnskey, lex, name,
+				 0, mctx, buf, NULL);
 
 cleanup:
 	isc_lex_setcomments(lex, 0);
-	return result;
+	return ret;
 }
 
 static void
@@ -1118,14 +1122,14 @@ request(ksr_ctx_t *ksr) {
 		char utc[sizeof("YYYYMMDDHHSSMM")];
 		isc_buffer_t b;
 		isc_region_t r;
-		isc_result_t result;
+		isc_result_t ret;
 
 		isc_stdtime_tostring(inception, timestr, sizeof(timestr));
 		isc_buffer_init(&b, utc, sizeof(utc));
-		result = dns_time32_totext(inception, &b);
-		if (result != ISC_R_SUCCESS) {
+		ret = dns_time32_totext(inception, &b);
+		if (ret != ISC_R_SUCCESS) {
 			fatal("failed to convert bundle time32 to text: %s",
-			      isc_result_totext(result));
+			      isc_result_totext(ret));
 		}
 		isc_buffer_usedregion(&b, &r);
 		fprintf(stdout, ";; KeySigningRequest 1.0 %.*s (%s)\n",
@@ -1169,7 +1173,7 @@ sign(ksr_ctx_t *ksr) {
 	dns_dnsseckeylist_t keys;
 	dns_kasp_t *kasp = NULL;
 	dns_rdatalist_t *rdatalist = NULL;
-	isc_result_t result;
+	isc_result_t ret;
 	isc_stdtime_t inception;
 	isc_lex_t *lex = NULL;
 	isc_lexspecials_t specials;
@@ -1195,15 +1199,14 @@ sign(ksr_ctx_t *ksr) {
 	specials[')'] = 1;
 	specials['"'] = 1;
 	isc_lex_setspecials(lex, specials);
-	result = isc_lex_openfile(lex, ksr->file);
-	if (result != ISC_R_SUCCESS) {
+	ret = isc_lex_openfile(lex, ksr->file);
+	if (ret != ISC_R_SUCCESS) {
 		fatal("unable to open KSR file %s: %s", ksr->file,
-		      isc_result_totext(result));
+		      isc_result_totext(ret));
 	}
 
-	for (result = isc_lex_gettoken(lex, opt, &token);
-	     result == ISC_R_SUCCESS;
-	     result = isc_lex_gettoken(lex, opt, &token))
+	for (ret = isc_lex_gettoken(lex, opt, &token); ret == ISC_R_SUCCESS;
+	     ret = isc_lex_gettoken(lex, opt, &token))
 	{
 		if (token.type != isc_tokentype_string) {
 			fatal("bad KSR file %s(%lu): syntax error", ksr->file,
@@ -1269,13 +1272,13 @@ sign(ksr_ctx_t *ksr) {
 		readline:
 			/* Read remainder of header line */
 			do {
-				result = isc_lex_gettoken(lex, opt, &token);
-				if (result != ISC_R_SUCCESS) {
+				ret = isc_lex_gettoken(lex, opt, &token);
+				if (ret != ISC_R_SUCCESS) {
 					fatal("bad KSR file %s(%lu): bad "
 					      "header (%s)",
 					      ksr->file,
 					      isc_lex_getsourceline(lex),
-					      isc_result_totext(result));
+					      isc_result_totext(ret));
 				}
 			} while (token.type != isc_tokentype_eol);
 		} else {
@@ -1292,11 +1295,11 @@ sign(ksr_ctx_t *ksr) {
 			rdata = isc_mem_get(mctx, sizeof(*rdata));
 			dns_rdata_init(rdata);
 			isc_buffer_init(&buf, rdatabuf, sizeof(rdatabuf));
-			result = parse_dnskey(lex, STR(token), &buf, &ttl);
-			if (result != ISC_R_SUCCESS) {
+			ret = parse_dnskey(lex, STR(token), &buf, &ttl);
+			if (ret != ISC_R_SUCCESS) {
 				fatal("bad KSR file %s(%lu): bad DNSKEY (%s)",
 				      ksr->file, isc_lex_getsourceline(lex),
-				      isc_result_totext(result));
+				      isc_result_totext(ret));
 			}
 			isc_buffer_usedregion(&buf, &r);
 			isc_buffer_allocate(mctx, &newbuf, r.length);
@@ -1314,7 +1317,7 @@ sign(ksr_ctx_t *ksr) {
 		}
 	}
 
-	if (result != ISC_R_EOF) {
+	if (ret != ISC_R_EOF) {
 		fatal("bad KSR file %s(%lu): trailing garbage data", ksr->file,
 		      isc_lex_getsourceline(lex));
 	}
@@ -1332,19 +1335,19 @@ sign(ksr_ctx_t *ksr) {
 	fprintf(stdout, ";; SignedKeyResponse 1.0 generated at %s by %s\n",
 		timestr, PACKAGE_VERSION);
 
-cleanup:
+fail:
 	isc_lex_destroy(&lex);
 	cleanup(&keys, kasp);
 }
 
 int
 main(int argc, char *argv[]) {
-	isc_result_t result;
+	isc_result_t ret;
 	isc_buffer_t buf;
 	int ch;
 	char *endp;
 	bool set_fips_mode = false;
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L && OPENSSL_API_LEVEL >= 30000
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
 	OSSL_PROVIDER *fips = NULL, *base = NULL;
 #endif
 	ksr_ctx_t ksr = {
@@ -1359,7 +1362,7 @@ main(int argc, char *argv[]) {
 	while ((ch = isc_commandline_parse(argc, argv, OPTIONS)) != -1) {
 		switch (ch) {
 		case 'E':
-			engine = isc_commandline_argument;
+			fatal("%s", isc_result_totext(DST_R_NOENGINE));
 			break;
 		case 'e':
 			ksr.end = strtotime(isc_commandline_argument, ksr.now,
@@ -1380,10 +1383,10 @@ main(int argc, char *argv[]) {
 			break;
 		case 'K':
 			ksr.keydir = isc_commandline_argument;
-			result = try_dir(ksr.keydir);
-			if (result != ISC_R_SUCCESS) {
+			ret = try_dir(ksr.keydir);
+			if (ret != ISC_R_SUCCESS) {
 				fatal("cannot open directory %s: %s",
-				      ksr.keydir, isc_result_totext(result));
+				      ksr.keydir, isc_result_totext(ret));
 			}
 			break;
 		case 'k':
@@ -1416,24 +1419,18 @@ main(int argc, char *argv[]) {
 		fatal("must provide a command and zone name");
 	}
 
-	result = dst_lib_init(mctx, engine);
-	if (result != ISC_R_SUCCESS) {
-		fatal("could not initialize dst: %s",
-		      isc_result_totext(result));
-	}
-
 	/*
-	 * After dst_lib_init which will set FIPS mode if requested
-	 * at build time.  The minumums are both raised to 2048.
+	 * The DST subsystem will set FIPS mode if requested at build time.
+	 * The minimum sizes are both raised to 2048.
 	 */
 	if (isc_fips_mode()) {
 		min_rsa = min_dh = 2048;
 	}
 
-	setup_logging(mctx, &lctx);
+	setup_logging();
 
 	if (set_fips_mode) {
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L && OPENSSL_API_LEVEL >= 30000
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
 		fips = OSSL_PROVIDER_load(NULL, "fips");
 		if (fips == NULL) {
 			fatal("Failed to load FIPS provider");
@@ -1456,10 +1453,10 @@ main(int argc, char *argv[]) {
 	name = dns_fixedname_initname(&fname);
 	isc_buffer_init(&buf, argv[1], strlen(argv[1]));
 	isc_buffer_add(&buf, strlen(argv[1]));
-	result = dns_name_fromtext(name, &buf, dns_rootname, 0, NULL);
-	if (result != ISC_R_SUCCESS) {
+	ret = dns_name_fromtext(name, &buf, dns_rootname, 0, NULL);
+	if (ret != ISC_R_SUCCESS) {
 		fatal("invalid zone name %s: %s", argv[1],
-		      isc_result_totext(result));
+		      isc_result_totext(ret));
 	}
 
 	/* command */
