@@ -17,13 +17,13 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <isc/log.h>
 #include <isc/mem.h>
 #include <isc/result.h>
 #include <isc/util.h>
 #include <isc/uv.h>
 
 #include <dns/dlz_dlopen.h>
-#include <dns/log.h>
 
 #include <dlz/dlz_dlopen_driver.h>
 #include <named/globals.h>
@@ -80,7 +80,7 @@ static void
 dlopen_log(int level, const char *fmt, ...) {
 	va_list ap;
 	va_start(ap, fmt);
-	isc_log_vwrite(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+	isc_log_vwrite(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
 		       ISC_LOG_DEBUG(level), fmt, ap);
 	va_end(ap);
 }
@@ -223,7 +223,6 @@ dlopen_dlz_create(const char *dlzname, unsigned int argc, char *argv[],
 	}
 
 	isc_mem_create(&mctx);
-	isc_mem_setname(mctx, "named_dlz");
 	cd = isc_mem_get(mctx, sizeof(*cd));
 	*cd = (dlopen_data_t){
 		.mctx = mctx,
