@@ -18,6 +18,7 @@
 #include <stdbool.h>
 
 #include <isc/log.h>
+#include <isc/loop.h>
 #include <isc/net.h>
 #include <isc/netmgr.h>
 #include <isc/rwlock.h>
@@ -25,10 +26,11 @@
 #include <dns/acl.h>
 #include <dns/zone.h>
 
+#include <dst/dst.h>
+
 #include <isccfg/aclconf.h>
 #include <isccfg/cfg.h>
 
-#include <dst/dst.h>
 #include <named/fuzz.h>
 #include <named/types.h>
 
@@ -48,8 +50,9 @@
 
 EXTERN isc_mem_t *named_g_mctx		      INIT(NULL);
 EXTERN unsigned int named_g_cpus	      INIT(0);
-EXTERN unsigned int named_g_udpdisp	      INIT(0);
-EXTERN isc_taskmgr_t *named_g_taskmgr	      INIT(NULL);
+EXTERN isc_loop_t *named_g_mainloop	      INIT(NULL);
+EXTERN isc_loopmgr_t *named_g_loopmgr	      INIT(NULL);
+EXTERN bool named_g_loopmgr_running	      INIT(false);
 EXTERN dns_dispatchmgr_t *named_g_dispatchmgr INIT(NULL);
 EXTERN unsigned int named_g_cpus_detected     INIT(1);
 
@@ -61,20 +64,19 @@ EXTERN bool named_g_run_done INIT(false);
  *         for really short timers, another for client timers, and one
  *         for zone timers.
  */
-EXTERN isc_timermgr_t *named_g_timermgr INIT(NULL);
-EXTERN isc_nm_t *named_g_netmgr		INIT(NULL);
-EXTERN cfg_parser_t *named_g_parser	INIT(NULL);
-EXTERN cfg_parser_t *named_g_addparser	INIT(NULL);
-EXTERN const char *named_g_version	INIT(PACKAGE_VERSION);
-EXTERN const char *named_g_product	INIT(PACKAGE_NAME);
-EXTERN const char *named_g_description	INIT(PACKAGE_DESCRIPTION);
-EXTERN const char *named_g_srcid	INIT(PACKAGE_SRCID);
-EXTERN const char *named_g_configargs	INIT(PACKAGE_CONFIGARGS);
-EXTERN const char *named_g_builder	INIT(PACKAGE_BUILDER);
-EXTERN in_port_t named_g_port		INIT(0);
-EXTERN in_port_t named_g_tlsport	INIT(0);
-EXTERN in_port_t named_g_httpsport	INIT(0);
-EXTERN in_port_t named_g_httpport	INIT(0);
+EXTERN isc_nm_t *named_g_netmgr	       INIT(NULL);
+EXTERN cfg_parser_t *named_g_parser    INIT(NULL);
+EXTERN cfg_parser_t *named_g_addparser INIT(NULL);
+EXTERN const char *named_g_version     INIT(PACKAGE_VERSION);
+EXTERN const char *named_g_product     INIT(PACKAGE_NAME);
+EXTERN const char *named_g_description INIT(PACKAGE_DESCRIPTION);
+EXTERN const char *named_g_srcid       INIT(PACKAGE_SRCID);
+EXTERN const char *named_g_configargs  INIT(PACKAGE_CONFIGARGS);
+EXTERN const char *named_g_builder     INIT(PACKAGE_BUILDER);
+EXTERN in_port_t named_g_port	       INIT(0);
+EXTERN in_port_t named_g_tlsport       INIT(0);
+EXTERN in_port_t named_g_httpsport     INIT(0);
+EXTERN in_port_t named_g_httpport      INIT(0);
 
 EXTERN in_port_t named_g_http_listener_clients INIT(0);
 EXTERN in_port_t named_g_http_streams_per_conn INIT(0);
@@ -84,10 +86,8 @@ EXTERN named_server_t *named_g_server INIT(NULL);
 /*
  * Logging.
  */
-EXTERN isc_log_t *named_g_lctx		     INIT(NULL);
-EXTERN isc_logcategory_t *named_g_categories INIT(NULL);
-EXTERN isc_logmodule_t *named_g_modules	     INIT(NULL);
-EXTERN unsigned int named_g_debuglevel	     INIT(0);
+EXTERN bool named_g_logging	       INIT(false);
+EXTERN unsigned int named_g_debuglevel INIT(0);
 
 /*
  * Current configuration information.
@@ -95,22 +95,12 @@ EXTERN unsigned int named_g_debuglevel	     INIT(0);
 EXTERN cfg_obj_t *named_g_config	   INIT(NULL);
 EXTERN const cfg_obj_t *named_g_defaults   INIT(NULL);
 EXTERN const char *named_g_conffile	   INIT(NAMED_SYSCONFDIR "/named.conf");
-EXTERN const char *named_g_defaultbindkeys INIT(NAMED_SYSCONFDIR "/bind.keys");
+EXTERN const char *named_g_defaultbindkeys INIT(NULL);
 EXTERN const char *named_g_keyfile	   INIT(NAMED_SYSCONFDIR "/rndc.key");
 
-EXTERN dns_tsigkey_t *named_g_sessionkey    INIT(NULL);
-EXTERN dns_name_t			    named_g_sessionkeyname;
 EXTERN bool named_g_conffileset		    INIT(false);
 EXTERN cfg_aclconfctx_t *named_g_aclconfctx INIT(NULL);
 
-/*
- * Initial resource limits.
- */
-EXTERN isc_resourcevalue_t named_g_initstacksize INIT(0);
-EXTERN isc_resourcevalue_t named_g_initdatasize	 INIT(0);
-EXTERN isc_resourcevalue_t named_g_initcoresize	 INIT(0);
-EXTERN isc_resourcevalue_t named_g_initopenfiles INIT(0);
-
 /*
  * Misc.
  */
@@ -119,16 +109,12 @@ EXTERN const char *named_g_chrootdir INIT(NULL);
 EXTERN bool named_g_foreground	     INIT(false);
 EXTERN bool named_g_logstderr	     INIT(false);
 EXTERN bool named_g_nosyslog	     INIT(false);
+EXTERN unsigned int named_g_logflags INIT(0);
 EXTERN const char *named_g_logfile   INIT(NULL);
 
 EXTERN const char *named_g_defaultsessionkeyfile INIT(NAMED_LOCALSTATEDIR
 						      "/run/named/"
 						      "session.key");
-EXTERN const char *named_g_defaultlockfile INIT(NAMED_LOCALSTATEDIR "/run/"
-								    "named/"
-								    "named."
-								    "lock");
-EXTERN bool named_g_forcelock		   INIT(false);
 
 #if NAMED_RUN_PID_DIR
 EXTERN const char *named_g_defaultpidfile INIT(NAMED_LOCALSTATEDIR "/run/named/"
@@ -140,8 +126,6 @@ EXTERN const char *named_g_defaultpidfile INIT(NAMED_LOCALSTATEDIR "/run/"
 
 EXTERN const char *named_g_username INIT(NULL);
 
-EXTERN const char *named_g_engine INIT(NULL);
-
 EXTERN isc_time_t		  named_g_boottime;
 EXTERN isc_time_t		  named_g_configtime;
 EXTERN bool named_g_memstatistics INIT(false);
