@@ -120,22 +120,23 @@ add_doh_transports(const cfg_obj_t *transportlist, dns_transport_list_t *list) {
 					     dns_transport_set_tls_versions);
 		parse_transport_option(doh, transport, "ciphers",
 				       dns_transport_set_ciphers);
+		parse_transport_option(doh, transport, "cipher-suites",
+				       dns_transport_set_cipher_suites);
 		parse_transport_bool_option(
 			doh, transport, "prefer-server-ciphers",
-			dns_transport_set_prefer_server_ciphers)
-			parse_transport_option(doh, transport, "ca-file",
-					       dns_transport_set_cafile);
+			dns_transport_set_prefer_server_ciphers);
+		parse_transport_option(doh, transport, "ca-file",
+				       dns_transport_set_cafile);
 		parse_transport_option(doh, transport, "remote-hostname",
 				       dns_transport_set_remote_hostname);
 	}
 
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 failure:
-	cfg_obj_log(doh, named_g_lctx, ISC_LOG_ERROR,
-		    "configuring DoH '%s': %s", dohid,
+	cfg_obj_log(doh, ISC_LOG_ERROR, "configuring DoH '%s': %s", dohid,
 		    isc_result_totext(result));
 
-	return (result);
+	return result;
 }
 
 static isc_result_t
@@ -172,22 +173,23 @@ add_tls_transports(const cfg_obj_t *transportlist, dns_transport_list_t *list) {
 					     dns_transport_set_tls_versions);
 		parse_transport_option(tls, transport, "ciphers",
 				       dns_transport_set_ciphers);
+		parse_transport_option(tls, transport, "cipher-suites",
+				       dns_transport_set_cipher_suites);
 		parse_transport_bool_option(
 			tls, transport, "prefer-server-ciphers",
-			dns_transport_set_prefer_server_ciphers)
-			parse_transport_option(tls, transport, "ca-file",
-					       dns_transport_set_cafile);
+			dns_transport_set_prefer_server_ciphers);
+		parse_transport_option(tls, transport, "ca-file",
+				       dns_transport_set_cafile);
 		parse_transport_option(tls, transport, "remote-hostname",
 				       dns_transport_set_remote_hostname);
 	}
 
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 failure:
-	cfg_obj_log(tls, named_g_lctx, ISC_LOG_ERROR,
-		    "configuring tls '%s': %s", tlsid,
+	cfg_obj_log(tls, ISC_LOG_ERROR, "configuring tls '%s': %s", tlsid,
 		    isc_result_totext(result));
 
-	return (result);
+	return result;
 }
 
 #define CHECK(f)                             \
@@ -214,7 +216,7 @@ transport_list_fromconfig(const cfg_obj_t *config, dns_transport_list_t *list) {
 		obj = NULL;
 	}
 
-	return (result);
+	return result;
 }
 
 static void
@@ -256,8 +258,8 @@ named_transports_fromconfig(const cfg_obj_t *config, const cfg_obj_t *vconfig,
 	}
 
 	*listp = list;
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 failure:
 	dns_transport_list_detach(&list);
-	return (result);
+	return result;
 }
