@@ -44,10 +44,6 @@ for db in zones/bad*.db; do
     zones/bad-dns-sd-reverse.db | zones/bad-svcb-servername.db)
       $CHECKZONE -k fail -i local 0.0.0.0.in-addr.arpa $db >test.out.$n 2>&1 || v=$?
       ;;
-    bad-cname-and*.db)
-      $CHECKZONE -i local example $db >test.out.$n 2>&1 || v=$?
-      grep "CNAME and other data" test.out.$n >/dev/null || ret=1
-      ;;
     *)
       $CHECKZONE -i local example $db >test.out.$n 2>&1 || v=$?
       ;;
@@ -128,6 +124,16 @@ n=$((n + 1))
 if [ $ret != 0 ]; then echo_i "failed"; fi
 status=$((status + ret))
 
+echo_i "checking that log-report-channel zones fail if '*._er/TXT' is missing ($n)"
+ret=0
+$CHECKZONE -R fail example zones/er.db >test.out2.$n 2>&1 || ret=1
+grep -F "no '*._er/TXT' wildcard found" test.out4.$n >/dev/null && ret=1
+$CHECKZONE example zones/er-missing.db >test.out3.$n 2>&1 || ret=1
+grep -F "no '*._er/TXT' wildcard found" test.out4.$n >/dev/null && ret=1
+$CHECKZONE -R fail example zones/er-missing.db >test.out4.$n 2>&1 && ret=1
+grep -F "no '*._er/TXT' wildcard found" test.out4.$n >/dev/null || ret=1
+status=$((status + ret))
+
 echo_i "checking that raw zone with bad class is handled ($n)"
 ret=0
 $CHECKZONE -f raw example zones/bad-badclass.raw >test.out.$n 2>&1 && ret=1
