@@ -9,15 +9,16 @@
 # See the COPYRIGHT file distributed with this work for additional
 # information regarding copyright ownership.
 
+from functools import partial
 import filecmp
 import os
 from pathlib import Path
-from re import compile as Re
+import re
 import shutil
 import subprocess
 import tempfile
 import time
-from typing import Any
+from typing import Any, List, Optional
 
 import pytest
 
@@ -53,7 +54,7 @@ else:
 
 XDIST_WORKER = os.environ.get("PYTEST_XDIST_WORKER", "")
 FILE_DIR = os.path.abspath(Path(__file__).parent)
-ENV_RE = Re(b"([^=]+)=(.*)")
+ENV_RE = re.compile(b"([^=]+)=(.*)")
 PRIORITY_TESTS = [
     # Tests that are scheduled first. Speeds up parallel execution.
     "rpz/",
@@ -62,9 +63,9 @@ PRIORITY_TESTS = [
     "timeouts/",
     "upforwd/",
 ]
-PRIORITY_TESTS_RE = Re("|".join(PRIORITY_TESTS))
-SYSTEM_TEST_NAME_RE = Re(f"{SYSTEM_TEST_DIR_GIT_PATH}" + r"/([^/]+)")
-SYMLINK_REPLACEMENT_RE = Re(r"/tests(_.*)\.py")
+PRIORITY_TESTS_RE = re.compile("|".join(PRIORITY_TESTS))
+SYSTEM_TEST_NAME_RE = re.compile(f"{SYSTEM_TEST_DIR_GIT_PATH}" + r"/([^/]+)")
+SYMLINK_REPLACEMENT_RE = re.compile(r"/tests(_.*)\.py")
 
 # ----------------------- Global requirements ----------------------------
 
@@ -309,23 +310,13 @@ def logger(request, system_test_name):
 def expected_artifacts(request):
     common_artifacts = [
         ".libs/*",  # possible build artifacts, see GL #5055
-        "ns*/named*.conf",
-        "ns*/named.memstats",
         "ns*/named.run",
         "ns*/named.run.prev",
-        "core.[0-9]*-backtrace.txt",
-        "core.[0-9]*.gz",
+        "ns*/named.conf",
+        "ns*/named.memstats",
         "pytest.log.txt",
-        "tsan.*.[0-9]*",
     ]
 
-    if "USE_RR" in os.environ:
-        common_artifacts += [
-            "ns*/cpu_lock",
-            "ns*/latest-trace",
-            "ns*/named-[0-9]*",
-        ]
-
     try:
         test_specific_artifacts = request.node.get_closest_marker("extra_artifacts")
     except AttributeError:
@@ -334,7 +325,7 @@ def expected_artifacts(request):
     if test_specific_artifacts:
         return common_artifacts + test_specific_artifacts.args[0]
 
-    return None
+    return common_artifacts
 
 
 @pytest.fixture(scope="module")
@@ -363,13 +354,12 @@ def system_test_dir(request, system_test_name, expected_artifacts):
             if node.nodeid in all_test_results
         }
         assert len(test_results)
+        messages = []
         for node, result in test_results.items():
-            message = f"{result.outcome.upper()} {node}"
-            nonempty_extra = [msg for msg in result.messages.values() if msg.strip()]
-            if nonempty_extra:
-                message += "\n"
-                message += "\n\n".join(nonempty_extra)
-            isctest.log.debug(message)
+            isctest.log.debug("%s %s", result.outcome.upper(), node)
+            messages.extend(result.messages.values())
+        for message in messages:
+            isctest.log.debug("\n" + message)
         failed = any(res.outcome == "failed" for res in test_results.values())
         skipped = any(res.outcome == "skipped" for res in test_results.values())
         if failed:
@@ -396,14 +386,7 @@ def system_test_dir(request, system_test_name, expected_artifacts):
             # test must not remove any Git-tracked file, ignore libtool and gcov artifacts
             for name in dcmp.left_only:
                 assert name.startswith("lt-") or name.endswith(".gcda")
-            for name in dcmp.diff_files:
-                # There should be no diff_files. However, due to how the
-                # obsolete dnsrps tests are launched, this workaround is
-                # required to ignore the dnsrps-dnsrps.gcda file which may
-                # get detected here as git-tracked in some circumstances
-                assert name.endswith(
-                    ".gcda"
-                ), "test must not modify any Git-tracked file"
+            assert not dcmp.diff_files, "test must not modify any Git-tracked file"
 
             dir_path = Path(dcmp.left).relative_to(source_dir)
             for name in dcmp.right_only:
@@ -493,6 +476,46 @@ def templates(system_test_dir: Path):
     return isctest.template.TemplateEngine(system_test_dir)
 
 
+def _run_script(
+    system_test_dir: Path,
+    interpreter: str,
+    script: str,
+    args: Optional[List[str]] = None,
+):
+    """Helper function for the shell / perl script invocations (through fixtures below)."""
+    if args is None:
+        args = []
+    path = Path(script)
+    if not path.is_absolute():
+        # make sure relative paths are always relative to system_dir
+        path = system_test_dir.parent / path
+    script = str(path)
+    cwd = os.getcwd()
+    if not path.exists():
+        raise FileNotFoundError(f"script {script} not found in {cwd}")
+    isctest.log.debug("running script: %s %s %s", interpreter, script, " ".join(args))
+    isctest.log.debug("  workdir: %s", cwd)
+    returncode = 1
+
+    cmd = [interpreter, script] + args
+    with subprocess.Popen(
+        cmd,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
+        bufsize=1,
+        universal_newlines=True,
+        errors="backslashreplace",
+    ) as proc:
+        if proc.stdout:
+            for line in proc.stdout:
+                isctest.log.info("    %s", line.rstrip("\n"))
+        proc.communicate()
+        returncode = proc.returncode
+        if returncode:
+            raise subprocess.CalledProcessError(returncode, cmd)
+        isctest.log.debug("  exited with %d", returncode)
+
+
 def _get_node_path(node) -> Path:
     if isinstance(node.parent, pytest.Session):
         if _pytest_major_ver >= 8:
@@ -503,11 +526,23 @@ def _get_node_path(node) -> Path:
 
 
 @pytest.fixture(scope="module")
-def run_tests_sh(system_test_dir):
+def shell(system_test_dir):
+    """Function to call a shell script with arguments."""
+    return partial(_run_script, system_test_dir, os.environ["SHELL"])
+
+
+@pytest.fixture(scope="module")
+def perl(system_test_dir):
+    """Function to call a perl script with arguments."""
+    return partial(_run_script, system_test_dir, os.environ["PERL"])
+
+
+@pytest.fixture(scope="module")
+def run_tests_sh(system_test_dir, shell):
     """Utility function to execute tests.sh as a python test."""
 
     def run_tests():
-        isctest.run.shell(f"{system_test_dir}/tests.sh")
+        shell(f"{system_test_dir}/tests.sh")
 
     return run_tests
 
@@ -517,6 +552,8 @@ def system_test(
     request,
     system_test_dir,
     templates,
+    shell,
+    perl,
 ):
     """
     Driver of the test setup/teardown process. Used automatically for every test module.
@@ -542,56 +579,39 @@ def system_test(
 
     def check_net_interfaces():
         try:
-            isctest.run.perl(
-                f"{os.environ['srcdir']}/testsock.pl", ["-p", os.environ["PORT"]]
-            )
+            perl("testsock.pl", ["-p", os.environ["PORT"]])
         except subprocess.CalledProcessError as exc:
             isctest.log.error("testsock.pl: exited with code %d", exc.returncode)
             pytest.skip("Network interface aliases not set up.")
 
     def check_prerequisites():
         try:
-            isctest.run.shell(f"{system_test_dir}/prereq.sh")
+            shell(f"{system_test_dir}/prereq.sh")
         except FileNotFoundError:
             pass  # prereq.sh is optional
         except subprocess.CalledProcessError:
             pytest.skip("Prerequisites missing.")
 
     def setup_test():
-        template_data = None
-        bootstrap_fn = getattr(request.module, "bootstrap", None)
-        if bootstrap_fn:
-            isctest.log.debug("Running test bootstrap()")
-            try:
-                template_data = bootstrap_fn()
-            except Exception as exc:  # pylint: disable=broad-exception-caught
-                isctest.log.error("Failed to run test bootstrap()")
-                kind = type(exc).__name__
-                pytest.fail(f"bootstrap() failed with {kind}")
-
-        templates.render_auto(template_data)
-
-        setup_sh_path = f"{system_test_dir}/setup.sh"
-        if os.path.exists(setup_sh_path):
-            try:
-                isctest.run.shell(f"{system_test_dir}/setup.sh")
-            except subprocess.CalledProcessError as exc:
-                isctest.log.error("Failed to run test setup.sh")
-                pytest.fail(f"setup.sh exited with {exc.returncode}")
+        templates.render_auto()
+        try:
+            shell(f"{system_test_dir}/setup.sh")
+        except FileNotFoundError:
+            pass  # setup.sh is optional
+        except subprocess.CalledProcessError as exc:
+            isctest.log.error("Failed to run test setup")
+            pytest.fail(f"setup.sh exited with {exc.returncode}")
 
     def start_servers():
         try:
-            isctest.run.perl(
-                f"{os.environ['srcdir']}/start.pl",
-                ["--port", os.environ["PORT"], system_test_dir.name],
-            )
+            perl("start.pl", ["--port", os.environ["PORT"], system_test_dir.name])
         except subprocess.CalledProcessError as exc:
             isctest.log.error("Failed to start servers")
             pytest.fail(f"start.pl exited with {exc.returncode}")
 
     def stop_servers():
         try:
-            isctest.run.perl(f"{os.environ['srcdir']}/stop.pl", [system_test_dir.name])
+            perl("stop.pl", [system_test_dir.name])
         except subprocess.CalledProcessError as exc:
             isctest.log.error("Failed to stop servers")
             get_core_dumps()
@@ -599,9 +619,7 @@ def system_test(
 
     def get_core_dumps():
         try:
-            isctest.run.shell(
-                f"{os.environ['srcdir']}/get_core_dumps.sh", [system_test_dir.name]
-            )
+            shell("get_core_dumps.sh", [system_test_dir.name])
         except subprocess.CalledProcessError as exc:
             isctest.log.error("Found core dumps or sanitizer reports")
             pytest.fail(f"get_core_dumps.sh exited with {exc.returncode}")
@@ -637,7 +655,7 @@ def system_test(
         request.node.stash[FIXTURE_OK] = True
 
 
-@pytest.fixture(scope="module")
+@pytest.fixture
 def servers(system_test_dir):
     instances = {}
     for entry in system_test_dir.rglob("*"):
@@ -649,53 +667,3 @@ def servers(system_test_dir):
             except ValueError:
                 continue
     return instances
-
-
-@pytest.fixture(scope="module")
-def ns1(servers):
-    return servers["ns1"]
-
-
-@pytest.fixture(scope="module")
-def ns2(servers):
-    return servers["ns2"]
-
-
-@pytest.fixture(scope="module")
-def ns3(servers):
-    return servers["ns3"]
-
-
-@pytest.fixture(scope="module")
-def ns4(servers):
-    return servers["ns4"]
-
-
-@pytest.fixture(scope="module")
-def ns5(servers):
-    return servers["ns5"]
-
-
-@pytest.fixture(scope="module")
-def ns6(servers):
-    return servers["ns6"]
-
-
-@pytest.fixture(scope="module")
-def ns7(servers):
-    return servers["ns7"]
-
-
-@pytest.fixture(scope="module")
-def ns8(servers):
-    return servers["ns8"]
-
-
-@pytest.fixture(scope="module")
-def ns9(servers):
-    return servers["ns9"]
-
-
-@pytest.fixture(scope="module")
-def ns10(servers):
-    return servers["ns10"]
