@@ -19,7 +19,7 @@
 
 #include <openssl/crypto.h>
 #include <openssl/opensslv.h>
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L && OPENSSL_API_LEVEL >= 30000
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
 #include <openssl/provider.h>
 #endif
 
@@ -38,7 +38,6 @@ usage(void) {
 	fprintf(stderr, "usage: feature-test <arg>\n");
 	fprintf(stderr, "args:\n");
 	fprintf(stderr, "\t--edns-version\n");
-	fprintf(stderr, "\t--enable-dnsrps\n");
 	fprintf(stderr, "\t--enable-dnstap\n");
 	fprintf(stderr, "\t--enable-querytrace\n");
 	fprintf(stderr, "\t--fips-provider\n");
@@ -49,10 +48,11 @@ usage(void) {
 	fprintf(stderr, "\t--have-geoip2\n");
 	fprintf(stderr, "\t--have-json-c\n");
 	fprintf(stderr, "\t--have-libxml2\n");
-	fprintf(stderr, "\t--have-openssl-cipher-suites\n");
+	fprintf(stderr, "\t--ipv6only=no\n");
 	fprintf(stderr, "\t--md5\n");
 	fprintf(stderr, "\t--rsasha1\n");
 	fprintf(stderr, "\t--tsan\n");
+	fprintf(stderr, "\t--with-dlz-filesystem\n");
 	fprintf(stderr, "\t--with-libidn2\n");
 	fprintf(stderr, "\t--with-lmdb\n");
 	fprintf(stderr, "\t--with-libnghttp2\n");
@@ -75,14 +75,6 @@ main(int argc, char **argv) {
 		return 0;
 	}
 
-	if (strcmp(argv[1], "--enable-dnsrps") == 0) {
-#ifdef USE_DNSRPS
-		return 0;
-#else  /* ifdef USE_DNSRPS */
-		return 1;
-#endif /* ifdef USE_DNSRPS */
-	}
-
 	if (strcmp(argv[1], "--enable-dnstap") == 0) {
 #ifdef HAVE_DNSTAP
 		return 0;
@@ -100,7 +92,7 @@ main(int argc, char **argv) {
 	}
 
 	if (strcasecmp(argv[1], "--fips-provider") == 0) {
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L && OPENSSL_API_LEVEL >= 30000
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
 		OSSL_PROVIDER *fips = OSSL_PROVIDER_load(NULL, "fips");
 		if (fips != NULL) {
 			OSSL_PROVIDER_unload(fips);
@@ -134,14 +126,14 @@ main(int argc, char **argv) {
 
 	if (strcmp(argv[1], "--have-fips-dh") == 0) {
 #if defined(ENABLE_FIPS_MODE)
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L && OPENSSL_API_LEVEL >= 30000
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
 		return 0;
 #else
 		return 1;
 #endif
 #else
 		if (isc_fips_mode()) {
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L && OPENSSL_API_LEVEL >= 30000
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
 			return 0;
 #else
 			return 1;
@@ -183,14 +175,6 @@ main(int argc, char **argv) {
 #endif /* ifdef HAVE_LIBXML2 */
 	}
 
-	if (strcmp(argv[1], "--have-openssl-cipher-suites") == 0) {
-#ifdef HAVE_SSL_CTX_SET_CIPHERSUITES
-		return 0;
-#else  /* ifdef HAVE_SSL_CTX_SET_CIPHERSUITES */
-		return 1;
-#endif /* ifdef HAVE_SSL_CTX_SET_CIPHERSUITES */
-	}
-
 	if (strcmp(argv[1], "--tsan") == 0) {
 #if defined(__has_feature)
 #if __has_feature(thread_sanitizer)
@@ -205,26 +189,45 @@ main(int argc, char **argv) {
 	}
 
 	if (strcmp(argv[1], "--md5") == 0) {
-		isc_mem_t *mctx = NULL;
-		int answer;
+		if (!dst_algorithm_supported(DST_ALG_HMACMD5)) {
+			return 1;
+		}
+		return 0;
+	}
 
-		isc_mem_create(&mctx);
-		dst_lib_init(mctx, NULL);
-		answer = dst_algorithm_supported(DST_ALG_HMACMD5) ? 0 : 1;
-		dst_lib_destroy();
-		isc_mem_detach(&mctx);
-		return answer;
+	if (strcmp(argv[1], "--ipv6only=no") == 0) {
+#if defined(IPPROTO_IPV6) && defined(IPV6_V6ONLY)
+		int s;
+		int n = -1;
+		int v6only = -1;
+		socklen_t len = sizeof(v6only);
+
+		s = socket(AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
+		if (s >= 0) {
+			n = getsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY,
+				       (void *)&v6only, &len);
+			close(s);
+		}
+		return (n == 0 && v6only == 0) ? 0 : 1;
+#else  /* defined(IPPROTO_IPV6) && defined(IPV6_V6ONLY) */
+		return 1;
+#endif /* defined(IPPROTO_IPV6) && defined(IPV6_V6ONLY) */
 	}
 
 	if (strcasecmp(argv[1], "--rsasha1") == 0) {
-		int answer;
-		isc_mem_t *mctx = NULL;
-		isc_mem_create(&mctx);
-		dst_lib_init(mctx, NULL);
-		answer = dst_algorithm_supported(DST_ALG_RSASHA1) ? 0 : 1;
-		dst_lib_destroy();
-		isc_mem_detach(&mctx);
-		return answer;
+		if (!dst_algorithm_supported(DST_ALG_RSASHA1)) {
+			return 1;
+		}
+
+		return 0;
+	}
+
+	if (strcmp(argv[1], "--with-dlz-filesystem") == 0) {
+#ifdef DLZ_FILESYSTEM
+		return 0;
+#else  /* ifdef DLZ_FILESYSTEM */
+		return 1;
+#endif /* ifdef DLZ_FILESYSTEM */
 	}
 
 	if (strcmp(argv[1], "--with-libidn2") == 0) {
