@@ -21,10 +21,7 @@ dig_with_opts() (
 )
 
 sendcmd() (
-  SERVER="${1}"
-  COMMAND="${2}"
-  COMMAND_ARGS="${3}"
-  dig_with_opts "@${SERVER}" "${COMMAND_ARGS}.${COMMAND}._control." TXT +time=5 +tries=1 +tcp >/dev/null 2>&1
+  send "$1" "$EXTRAPORT1"
 )
 
 rndccmd() {
@@ -169,10 +166,11 @@ status=$((status + ret))
 
 # GL#1793
 n=$((n + 1))
-echo_i "checking that the 'serverquota' counter isn't increased because of the SERVFAIL in the previous check ($n)"
+echo_i "checking that the correct counter is increased because of the SERVFAIL in the previous check ($n)"
 ret=0
 "${CURL}" "http://10.53.0.4:${EXTRAPORT1}/json/v1" 2>/dev/null >statschannel.out.$n
-grep -F "ServerQuota" statschannel.out.$n >/dev/null && ret=1
+grep -F '"ServerQuota"' statschannel.out.$n >/dev/null && ret=1
+grep -F '"ForwardOnlyFail":1' statschannel.out.$n >/dev/null || ret=1
 if [ $ret != 0 ]; then echo_i "failed"; fi
 status=$((status + ret))
 
@@ -262,7 +260,7 @@ n=$((n + 1))
 echo_i "checking that a forwarder timeout prevents it from being reused in the same fetch context ($n)"
 ret=0
 # Make ans6 receive queries without responding to them.
-sendcmd 10.53.0.6 send-responses "disable"
+echo "//" | sendcmd 10.53.0.6
 # Query for a record in a zone which is forwarded to a non-responding forwarder
 # and is delegated from the root to check whether the forwarder will be retried
 # when a delegation is encountered after falling back to full recursive
@@ -273,7 +271,6 @@ dig_with_opts txt.example7. txt @$f1 >dig.out.$n.f1 || ret=1
 start_pattern="sending packet to 10\.53\.0\.6"
 retry_quiet 5 wait_for_log ns3/named.run "$start_pattern"
 check_sent 1 ns3/named.run "$start_pattern" ";txt\.example7\.[[:space:]]*IN[[:space:]]*TXT$" || ret=1
-sendcmd 10.53.0.6 send-responses "enable"
 if [ $ret != 0 ]; then echo_i "failed"; fi
 status=$((status + ret))
 
@@ -313,10 +310,19 @@ grep "status: SERVFAIL" dig.out.$n >/dev/null || ret=1
 if [ $ret != 0 ]; then echo_i "failed"; fi
 status=$((status + ret))
 
+# Prepare ans6 for the chasing DS tests.
+sendcmd 10.53.0.6 <<EOF
+/ns1.sld.tld/A/
+300 A 10.53.0.2
+/sld.tld/NS/
+300 NS ns1.sld.tld.
+/sld.tld/
+EOF
+
 n=$((n + 1))
 echo_i "checking switch from forwarding to normal resolution while chasing DS ($n)"
 ret=0
-cp ns3/named2.conf ns3/named.conf
+copy_setports ns3/named2.conf.in ns3/named.conf
 rndccmd 10.53.0.3 reconfig 2>&1 | sed 's/^/ns3 /' | cat_i
 sleep 1
 nextpart ns3/named.run >/dev/null
@@ -327,11 +333,13 @@ status=$((status + ret))
 
 # See [GL #3129].
 # Enable silent mode for ans11.
-sendcmd 10.53.0.11 send-responses "disable"
+echo "1" | sendcmd 10.53.0.11
 n=$((n + 1))
 echo_i "checking the handling of hung DS fetch while chasing DS ($n)"
 ret=0
-cp ns3/named2.conf ns3/named.conf
+copy_setports ns3/named2.conf.in ns3/tmp
+sed 's/root.db/root2.db/' ns3/tmp >ns3/named.conf
+rm -f ns3/tmp
 rndccmd 10.53.0.3 reconfig 2>&1 | sed 's/^/ns3 /' | cat_i
 rndccmd 10.53.0.3 flush 2>&1 | sed 's/^/ns3 /' | cat_i
 sleep 1
@@ -339,7 +347,7 @@ nextpart ns3/named.run >/dev/null
 dig_with_opts @$f1 xxx.yyy.sld.tld ds >dig.out.$n.f1 || ret=1
 grep "status: SERVFAIL" dig.out.$n.f1 >/dev/null || ret=1
 # Disable silent mode for ans11.
-sendcmd 10.53.0.11 send-responses "enable"
+echo "0" | sendcmd 10.53.0.11
 if [ $ret != 0 ]; then echo_i "failed"; fi
 status=$((status + ret))
 
@@ -398,7 +406,7 @@ run_spooftests() {
 echo_i "checking spoofed response scenarios with forward first zones"
 run_spooftests
 
-cp ns9/named2.conf ns9/named.conf
+copy_setports ns9/named2.conf.in ns9/named.conf
 rndccmd 10.53.0.9 reconfig 2>&1 | sed 's/^/ns3 /' | cat_i
 rndccmd 10.53.0.9 flush 2>&1 | sed 's/^/ns3 /' | cat_i
 sleep 1
@@ -411,7 +419,7 @@ run_spooftests
 # similar to the ones above, but not identical.
 #
 echo_i "rechecking spoofed response scenarios with 'forward only' set globally"
-cp ns9/named3.conf ns9/named.conf
+copy_setports ns9/named3.conf.in ns9/named.conf
 rndccmd 10.53.0.9 reconfig 2>&1 | sed 's/^/ns3 /' | cat_i
 rndccmd 10.53.0.9 flush 2>&1 | sed 's/^/ns3 /' | cat_i
 sleep 1
@@ -448,7 +456,7 @@ status=$((status + ret))
 # the namespace is defined by the zone content.
 #
 echo_i "rechecking spoofed response scenarios glue below local zone"
-cp ns9/named4.conf ns9/named.conf
+copy_setports ns9/named4.conf.in ns9/named.conf
 rndccmd 10.53.0.9 reconfig 2>&1 | sed 's/^/ns3 /' | cat_i
 rndccmd 10.53.0.9 flush 2>&1 | sed 's/^/ns3 /' | cat_i
 sleep 1
