@@ -16,14 +16,17 @@ pytestmark = pytest.mark.extra_artifacts(
         "dig.out.*",
         "statschannel.out.*",
         "ans*/ans.run",
+        "ans*/query.log",
         "ns*/trusted.conf",
         "ns1/K*",
         "ns1/dsset-*",
         "ns1/root.db",
         "ns1/root.db.signed",
         "ns2/named-tls.conf",
+        "ns2/options-tls.conf",
         "ns3/trusted.conf",
         "ns4/named-tls.conf",
+        "ns4/options-tls.conf",
         "ns9/named_dump.db",
     ]
 )
