@@ -11,11 +11,14 @@
 
 import itertools
 
-import dns.flags
-import dns.rrset
+import isctest
 import pytest
 
-import isctest
+import dns.message
+
+# Everything from getting a big answer to creating an RR set with thousands
+# of records takes minutes of CPU and real time with dnspython < 2.0.0.
+pytest.importorskip("dns", minversion="2.0.0")
 
 
 @pytest.mark.parametrize(
@@ -29,8 +32,8 @@ import isctest
     ],
 )
 def test_limits(name, limit):
-    msg_query = isctest.query.create(f"{name}.example.", "A")
-    res = isctest.query.tcp(msg_query, "10.53.0.1", log_response=False)
+    msg_query = dns.message.make_query(f"{name}.example.", "A")
+    res = isctest.query.tcp(msg_query, "10.53.0.1")
 
     iplist = [
         f"10.0.{x}.{y}"
@@ -43,7 +46,7 @@ def test_limits(name, limit):
 
 
 def test_limit_exceeded():
-    msg_query = isctest.query.create("5000.example.", "A")
-    res = isctest.query.tcp(msg_query, "10.53.0.1", log_response=False)
+    msg_query = dns.message.make_query("5000.example.", "A")
+    res = isctest.query.tcp(msg_query, "10.53.0.1")
 
     assert res.flags & dns.flags.TC, "TC flag was not set"
