@@ -130,7 +130,7 @@ sendquery(void) {
 
 	c = scanf("%255s", host);
 	if (c == EOF) {
-		return (ISC_R_NOMORE);
+		return ISC_R_NOMORE;
 	}
 
 	onfly++;
@@ -166,7 +166,7 @@ sendquery(void) {
 		isc_loop_main(loopmgr), recvresponse, message, &request);
 	CHECK("dns_request_create", result);
 
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 }
 
 static void
@@ -216,8 +216,6 @@ main(int argc, char *argv[]) {
 	isc_sockaddr_t bind_any;
 	struct in_addr inaddr;
 	isc_result_t result;
-	isc_log_t *lctx = NULL;
-	isc_logconfig_t *lcfg = NULL;
 	isc_nm_t *netmgr = NULL;
 	dns_dispatchmgr_t *dispatchmgr = NULL;
 	dns_dispatch_t *dispatchv4 = NULL;
@@ -273,10 +271,6 @@ main(int argc, char *argv[]) {
 
 	isc_managers_create(&mctx, 1, &loopmgr, &netmgr);
 
-	isc_log_create(mctx, &lctx, &lcfg);
-
-	RUNCHECK(dst_lib_init(mctx, NULL));
-
 	RUNCHECK(dns_dispatchmgr_create(mctx, loopmgr, netmgr, &dispatchmgr));
 
 	RUNCHECK(dns_dispatch_createudp(
@@ -284,7 +278,7 @@ main(int argc, char *argv[]) {
 	RUNCHECK(dns_requestmgr_create(mctx, loopmgr, dispatchmgr, dispatchv4,
 				       NULL, &requestmgr));
 
-	RUNCHECK(dns_view_create(mctx, NULL, 0, "_test", &view));
+	RUNCHECK(dns_view_create(mctx, loopmgr, NULL, 0, "_test", &view));
 
 	isc_loopmgr_setup(loopmgr, sendqueries, NULL);
 	isc_loopmgr_teardown(loopmgr, teardown_view, view);
@@ -294,11 +288,7 @@ main(int argc, char *argv[]) {
 
 	isc_loopmgr_run(loopmgr);
 
-	dst_lib_destroy();
-
-	isc_log_destroy(&lctx);
-
 	isc_managers_destroy(&mctx, &loopmgr, &netmgr);
 
-	return (0);
+	return 0;
 }
