@@ -11,8 +11,6 @@
 # See the COPYRIGHT file distributed with this work for additional
 # information regarding copyright ownership.
 
-# touch dnsrps-off to not test with DNSRPS
-
 set -e
 
 . ../conf.sh
@@ -22,8 +20,16 @@ for dir in ns*; do
   nextpart $dir/named.run >/dev/null
 done
 
-touch dnsrps.conf
-touch dnsrps.cache
+copy_setports ns1/named.conf.in ns1/named.conf
+copy_setports ns2/named.conf.in ns2/named.conf
+copy_setports ns3/named.conf.in ns3/named.conf
+copy_setports ns4/named.conf.in ns4/named.conf
+copy_setports ns5/named.conf.in ns5/named.conf
+copy_setports ns6/named.conf.in ns6/named.conf
+copy_setports ns7/named.conf.in ns7/named.conf
+copy_setports ns8/named.conf.in ns8/named.conf
+copy_setports ns9/named.conf.in ns9/named.conf
+copy_setports ns10/named.conf.in ns10/named.conf
 
 # set up test policy zones.
 #   bl is the main test zone
@@ -38,17 +44,8 @@ done
 cp ns3/manual-update-rpz.db.in ns3/manual-update-rpz.db
 cp ns8/manual-update-rpz.db.in ns8/manual-update-rpz.db
 
-cp ns3/evil-cname.db.in ns3/evil-cname.db
-cp ns3/wild-cname.db.in ns3/wild-cname.db
-
 cp ns3/mixed-case-rpz-1.db.in ns3/mixed-case-rpz.db
 
-cp ns3/include-rpz.db.in ns3/include-rpz.db
-cp ns3/include-rpz.inc-1.in ns3/include-rpz.inc
-
-# a "big" zone (tested with '-T rpzslow' enabled to slow down loading)
-cp ns3/slow-rpz.db.in ns3/slow-rpz.db
-
 # a zone that expires quickly and then can't be refreshed
 cp ns5/fast-expire.db.in ns5/fast-expire.db
 cp ns5/expire.conf.in ns5/expire.conf
