@@ -234,7 +234,7 @@ sub construct_ns_command {
 		$command = "taskset $taskset $NAMED ";
 	} elsif ($ENV{'USE_RR'}) {
 		$ENV{'_RR_TRACE_DIR'} = ".";
-		$command = "$ENV{'TOP_BUILDDIR'}/libtool --mode=execute rr record --chaos $NAMED ";
+		$command = "rr record --chaos $NAMED ";
 	} else {
 		$command = "$NAMED ";
 	}
@@ -264,8 +264,7 @@ sub construct_ns_command {
 
 		foreach my $t_option(
 			"dropedns", "ednsformerr", "ednsnotimp", "ednsrefused",
-			"cookiealwaysvalid", "noaa", "noedns", "nosoa",
-			"maxudp512", "maxudp1460",
+			"noaa", "noedns", "nosoa", "maxudp512", "maxudp1460",
 		    ) {
 			if (-e "$testdir/$server/named.$t_option") {
 				$command .= "-T $t_option "
@@ -324,8 +323,8 @@ sub construct_ans_command {
 	}
 
 	if (-e "$testdir/$server/ans.py") {
-		$ENV{'PYTHONPATH'} = $testdir . ":" . $builddir;
-		$command = "$PYTHON -u -m $test.$server.ans 10.53.0.$n $queryport";
+		$ENV{'PYTHONPATH'} = $testdir . ":" . $ENV{'srcdir'};
+		$command = "$PYTHON -u ans.py 10.53.0.$n $queryport";
 	} elsif (-e "$testdir/$server/ans.pl") {
 		$command = "$PERL ans.pl";
 	} else {
@@ -374,9 +373,9 @@ sub count_running_lines {
 	# the shell *ought* to have created the file immediately, but this
 	# logic allows the creation to be delayed without issues
 	if (open(my $fh, "<", $runfile)) {
-		# the two non-whitespace blobs should be the date and time
+		# the whitespace blob should be an ISO timestamp
 		# but we don't care about them really, only that they are there
-		return scalar(grep /^\S+ \S+ running\R/, <$fh>);
+		return scalar(grep /^\S+[ T]\S+ running\R/, <$fh>);
 	} else {
 		return 0;
 	}
