@@ -27,20 +27,28 @@ Limitations - untested properties:
     - special behavior of rdtypes like CNAME
 """
 
-# Silence incorrect warnings cause by hypothesis.assume()
-# https://github.com/pylint-dev/pylint/issues/10785#issuecomment-3677224217
-# pylint: disable=unreachable
-
-from hypothesis import assume, example, given, settings
+import pytest
 
+pytest.importorskip("dns", minversion="2.0.0")
+import dns.message
 import dns.name
+import dns.query
+import dns.rcode
 import dns.rdataclass
 import dns.rdatatype
 import dns.rrset
-import pytest
+
+# in FIPs mode md5 fails so we need 4.41.2 or later which does not use md5
+try:
+    import hashlib
+
+    hashlib.md5(b"1234")
+    pytest.importorskip("hypothesis")
+except ValueError:
+    pytest.importorskip("hypothesis", minversion="4.41.2")
+from hypothesis import assume, example, given, settings
 
 from isctest.hypothesis.strategies import dns_names, dns_rdatatypes_without_meta
-
 import isctest.check
 import isctest.name
 import isctest.query
@@ -94,7 +102,7 @@ def test_wildcard_rdtype_mismatch(
     # See RFC 4592 section 2.2.1.
     assume(name == SUFFIX or name.labels[-len(SUFFIX) - 1] != b"*")
 
-    query_msg = isctest.query.create(name, rdtype)
+    query_msg = dns.message.make_query(name, rdtype)
     response_msg = isctest.query.tcp(query_msg, IP_ADDR, named_port, timeout=TIMEOUT)
 
     isctest.check.is_response_to(response_msg, query_msg)
@@ -111,7 +119,7 @@ def test_wildcard_match(name: dns.name.Name, named_port: int) -> None:
     # See RFC 4592 section 2.2.1.
     assume(name.labels[-len(SUFFIX) - 1] != b"*")
 
-    query_msg = isctest.query.create(name, WILDCARD_RDTYPE)
+    query_msg = dns.message.make_query(name, WILDCARD_RDTYPE)
     response_msg = isctest.query.tcp(query_msg, IP_ADDR, named_port, timeout=TIMEOUT)
 
     isctest.check.is_response_to(response_msg, query_msg)
@@ -140,7 +148,7 @@ def test_wildcard_with_star_not_synthesized(
     name: dns.name.Name, named_port: int
 ) -> None:
     """RFC 4592 section 2.2.1 ghost.*.example."""
-    query_msg = isctest.query.create(name, WILDCARD_RDTYPE)
+    query_msg = dns.message.make_query(name, WILDCARD_RDTYPE)
     response_msg = isctest.query.tcp(query_msg, IP_ADDR, named_port, timeout=TIMEOUT)
 
     isctest.check.is_response_to(response_msg, query_msg)
@@ -170,7 +178,7 @@ def test_name_in_between_wildcards(name: dns.name.Name, named_port: int) -> None
         or name.labels[-len(NESTED_SUFFIX) - 1] != b"*"
     )
 
-    query_msg = isctest.query.create(name, WILDCARD_RDTYPE)
+    query_msg = dns.message.make_query(name, WILDCARD_RDTYPE)
     response_msg = isctest.query.tcp(query_msg, IP_ADDR, named_port, timeout=TIMEOUT)
 
     isctest.check.is_response_to(response_msg, query_msg)
@@ -201,7 +209,7 @@ def test_name_nested_wildcard_subdomains_not_synthesized(
 
     `foo.*.*.*.nestedwild.test. A` must not be synthesized.
     """
-    query_msg = isctest.query.create(name, WILDCARD_RDTYPE)
+    query_msg = dns.message.make_query(name, WILDCARD_RDTYPE)
     response_msg = isctest.query.tcp(query_msg, IP_ADDR, named_port, timeout=TIMEOUT)
 
     isctest.check.is_response_to(response_msg, query_msg)
