@@ -51,7 +51,7 @@
 #include <dns/types.h>
 #include <dns/view.h>
 
-#define CHECKM(str, x)                                                      \
+#define CHECK(str, x)                                                       \
 	{                                                                   \
 		if ((x) != ISC_R_SUCCESS) {                                 \
 			fprintf(stderr, "mdig: %s failed with %s\n", (str), \
@@ -222,7 +222,7 @@ recvresponse(void *arg) {
 
 	msgbuf = dns_request_getanswer(request);
 	result = dns_request_getresponse(request, response, parseflags);
-	CHECKM("dns_request_getresponse", result);
+	CHECK("dns_request_getresponse", result);
 
 	styleflags |= DNS_STYLEFLAG_REL_OWNER;
 	if (yaml) {
@@ -278,7 +278,7 @@ recvresponse(void *arg) {
 						48, 80, 8, display_splitwidth,
 						mctx);
 	}
-	CHECKM("dns_master_stylecreate2", result);
+	CHECK("dns_master_stylecreate2", result);
 
 	flags = 0;
 	if (!display_headers) {
@@ -342,7 +342,7 @@ recvresponse(void *arg) {
 
 		printf("    %s:\n", "response_message_data");
 		result = dns_message_headertotext(response, style, flags, buf);
-		CHECKM("dns_message_headertotext", result);
+		CHECK("dns_message_headertotext", result);
 	} else if (display_comments && !display_short_form) {
 		printf(";; Got answer:\n");
 
@@ -405,7 +405,7 @@ repopulate_buffer:
 			isc_buffer_allocate(mctx, &buf, len);
 			goto repopulate_buffer;
 		}
-		CHECKM("dns_message_pseudosectiontotext", result);
+		CHECK("dns_message_pseudosectiontotext", result);
 	}
 
 	if (display_question && display_headers && !display_short_form) {
@@ -414,7 +414,7 @@ repopulate_buffer:
 		if (result == ISC_R_NOSPACE) {
 			goto buftoosmall;
 		}
-		CHECKM("dns_message_sectiontotext", result);
+		CHECK("dns_message_sectiontotext", result);
 	}
 
 	if (display_answer && !display_short_form) {
@@ -423,7 +423,7 @@ repopulate_buffer:
 		if (result == ISC_R_NOSPACE) {
 			goto buftoosmall;
 		}
-		CHECKM("dns_message_sectiontotext", result);
+		CHECK("dns_message_sectiontotext", result);
 	} else if (display_answer) {
 		dns_name_t *name;
 		dns_rdataset_t *rdataset;
@@ -442,14 +442,14 @@ repopulate_buffer:
 		dns_name_init(&empty_name, NULL);
 		result = dns_message_firstname(response, DNS_SECTION_ANSWER);
 		if (result != ISC_R_NOMORE) {
-			CHECKM("dns_message_firstname", result);
+			CHECK("dns_message_firstname", result);
 		}
 
 		for (;;) {
 			if (result == ISC_R_NOMORE) {
 				break;
 			}
-			CHECKM("dns_message_nextname", result);
+			CHECK("dns_message_nextname", result);
 			name = NULL;
 			dns_message_currentname(response, DNS_SECTION_ANSWER,
 						&name);
@@ -467,7 +467,7 @@ repopulate_buffer:
 					if (result == ISC_R_NOSPACE) {
 						goto buftoosmall;
 					}
-					CHECKM("dns_rdata_tofmttext", result);
+					CHECK("dns_rdata_tofmttext", result);
 					loopresult =
 						dns_rdataset_next(rdataset);
 					dns_rdata_reset(&rdata);
@@ -490,7 +490,7 @@ repopulate_buffer:
 		if (result == ISC_R_NOSPACE) {
 			goto buftoosmall;
 		}
-		CHECKM("dns_message_sectiontotext", result);
+		CHECK("dns_message_sectiontotext", result);
 	}
 
 	if (display_additional && !display_short_form) {
@@ -499,7 +499,7 @@ repopulate_buffer:
 		if (result == ISC_R_NOSPACE) {
 			goto buftoosmall;
 		}
-		CHECKM("dns_message_sectiontotext", result);
+		CHECK("dns_message_sectiontotext", result);
 	}
 
 	if (display_additional && !display_short_form && display_headers) {
@@ -511,13 +511,13 @@ repopulate_buffer:
 		if (result == ISC_R_NOSPACE) {
 			goto buftoosmall;
 		}
-		CHECKM("dns_message_pseudosectiontotext", result);
+		CHECK("dns_message_pseudosectiontotext", result);
 		result = dns_message_pseudosectiontotext(
 			response, DNS_PSEUDOSECTION_SIG0, style, flags, buf);
 		if (result == ISC_R_NOSPACE) {
 			goto buftoosmall;
 		}
-		CHECKM("dns_message_pseudosectiontotext", result);
+		CHECK("dns_message_pseudosectiontotext", result);
 	}
 
 	if (display_headers && display_comments && !display_short_form && !yaml)
@@ -561,9 +561,9 @@ add_opt(dns_message_t *msg, uint16_t udpsize, uint16_t edns, unsigned int flags,
 
 	result = dns_message_buildopt(msg, &rdataset, edns, udpsize, flags,
 				      opts, count);
-	CHECKM("dns_message_buildopt", result);
+	CHECK("dns_message_buildopt", result);
 	result = dns_message_setopt(msg, rdataset);
-	CHECKM("dns_message_setopt", result);
+	CHECK("dns_message_setopt", result);
 }
 
 static void
@@ -591,7 +591,7 @@ sendquery(struct query *query) {
 	isc_buffer_add(&buf, strlen(query->textname));
 	result = dns_name_fromtext(dns_fixedname_name(&queryname), &buf,
 				   dns_rootname, 0, NULL);
-	CHECKM("dns_name_fromtext", result);
+	CHECK("dns_name_fromtext", result);
 
 	dns_message_create(mctx, NULL, NULL, DNS_MESSAGE_INTENTRENDER,
 			   &message);
@@ -666,7 +666,7 @@ sendquery(struct query *query) {
 			INSIST(i < DNS_EDNSOPTIONS);
 			opts[i].code = DNS_OPT_CLIENT_SUBNET;
 			opts[i].length = (uint16_t)addrl + 4;
-			CHECKM("isc_buffer_allocate", result);
+			CHECK("isc_buffer_allocate", result);
 			isc_buffer_init(&b, ecsbuf, sizeof(ecsbuf));
 			if (sa->sa_family == AF_INET) {
 				family = 1;
@@ -712,7 +712,7 @@ sendquery(struct query *query) {
 				isc_buffer_init(&b, cookie, sizeof(cookie));
 				result = isc_hex_decodestring(query->cookie,
 							      &b);
-				CHECKM("isc_hex_decodestring", result);
+				CHECK("isc_hex_decodestring", result);
 				opts[i].value = isc_buffer_base(&b);
 				opts[i].length = isc_buffer_usedlength(&b);
 			} else {
@@ -754,7 +754,7 @@ sendquery(struct query *query) {
 		NULL, options, NULL, query->timeout, query->udptimeout,
 		query->udpretries, isc_loop_main(loopmgr), recvresponse,
 		message, &request);
-	CHECKM("dns_request_create", result);
+	CHECK("dns_request_create", result);
 
 	return ISC_R_SUCCESS;
 }
@@ -775,7 +775,7 @@ sendqueries(void *arg) {
 	}
 }
 
-noreturn static void
+ISC_NORETURN static void
 usage(void);
 
 static void
@@ -883,7 +883,7 @@ help(void) {
 	       "Server ID)\n");
 }
 
-noreturn static void
+ISC_NORETURN static void
 fatal(const char *format, ...) ISC_FORMAT_PRINTF(1, 2);
 
 static void
@@ -968,7 +968,7 @@ save_opt(struct query *query, char *code, char *value) {
 		buf = isc_mem_allocate(mctx, strlen(value) / 2 + 1);
 		isc_buffer_init(&b, buf, strlen(value) / 2 + 1);
 		result = isc_hex_decodestring(value, &b);
-		CHECKM("isc_hex_decodestring", result);
+		CHECK("isc_hex_decodestring", result);
 		query->ednsopts[query->ednsoptscnt].value = isc_buffer_base(&b);
 		query->ednsopts[query->ednsoptscnt].length =
 			isc_buffer_usedlength(&b);
@@ -1065,9 +1065,9 @@ reverse_octets(const char *in, char **p, char *end) {
 	if (dot != NULL) {
 		isc_result_t result;
 		result = reverse_octets(dot + 1, p, end);
-		CHECKM("reverse_octets", result);
+		CHECK("reverse_octets", result);
 		result = append(".", 1, p, end);
-		CHECKM("append", result);
+		CHECK("append", result);
 		len = (int)(dot - in);
 	} else {
 		len = strlen(in);
@@ -1090,7 +1090,7 @@ get_reverse(char *reverse, size_t len, const char *value) {
 
 		name = dns_fixedname_initname(&fname);
 		result = dns_byaddr_createptrname(&addr, name);
-		CHECKM("dns_byaddr_createptrname", result);
+		CHECK("dns_byaddr_createptrname", result);
 		dns_name_format(name, reverse, (unsigned int)len);
 		return;
 	} else {
@@ -1104,10 +1104,10 @@ get_reverse(char *reverse, size_t len, const char *value) {
 		char *p = reverse;
 		char *end = reverse + len;
 		result = reverse_octets(value, &p, end);
-		CHECKM("reverse_octets", result);
+		CHECK("reverse_octets", result);
 		/* Append .in-addr.arpa. and a terminating NUL. */
 		result = append(".in-addr.arpa.", 15, &p, end);
-		CHECKM("append", result);
+		CHECK("append", result);
 		return;
 	}
 }
@@ -1224,7 +1224,7 @@ plus_option(char *option, struct query *query, bool global) {
 				}
 				result = parse_uint(&num, value, COMMSIZE,
 						    "buffer size");
-				CHECKM("parse_uint(buffer size)", result);
+				CHECK("parse_uint(buffer size)", result);
 				query->udpsize = num;
 				break;
 			case 'r': /* burst */
@@ -1334,8 +1334,8 @@ plus_option(char *option, struct query *query, bool global) {
 						result = parse_uint(&num, value,
 								    255,
 								    "edns");
-						CHECKM("parse_uint(edns)",
-						       result);
+						CHECK("parse_uint(edns)",
+						      result);
 						query->edns = num;
 						break;
 					case 'f':
@@ -1351,8 +1351,8 @@ plus_option(char *option, struct query *query, bool global) {
 						result = parse_xint(
 							&num, value, 0xffff,
 							"ednsflags");
-						CHECKM("parse_xint(ednsflags)",
-						       result);
+						CHECK("parse_xint(ednsflags)",
+						      result);
 						if (query->edns == -1) {
 							query->edns = 1;
 						}
@@ -1434,7 +1434,7 @@ plus_option(char *option, struct query *query, bool global) {
 				}
 				result = parse_uint(&query->udpretries, value,
 						    MAXTRIES - 1, "udpretries");
-				CHECKM("parse_uint(udpretries)", result);
+				CHECK("parse_uint(udpretries)", result);
 				break;
 			default:
 				goto invalid_option;
@@ -1498,7 +1498,7 @@ plus_option(char *option, struct query *query, bool global) {
 			if (display_splitwidth) {
 				display_splitwidth += 3;
 			}
-			CHECKM("parse_uint(split)", result);
+			CHECK("parse_uint(split)", result);
 			break;
 		case 'u': /* subnet */
 			FULLCHECK("subnet");
@@ -1516,7 +1516,7 @@ plus_option(char *option, struct query *query, bool global) {
 				query->edns = 0;
 			}
 			result = parse_netprefix(&query->ecs_addr, value);
-			CHECKM("parse_netprefix", result);
+			CHECK("parse_netprefix", result);
 			break;
 		default:
 			goto invalid_option;
@@ -1539,7 +1539,7 @@ plus_option(char *option, struct query *query, bool global) {
 			}
 			result = parse_uint(&query->timeout, value, MAXTIMEOUT,
 					    "timeout");
-			CHECKM("parse_uint(timeout)", result);
+			CHECK("parse_uint(timeout)", result);
 			if (query->timeout == 0) {
 				query->timeout = 1;
 			}
@@ -1554,7 +1554,7 @@ plus_option(char *option, struct query *query, bool global) {
 			}
 			result = parse_uint(&query->udpretries, value, MAXTRIES,
 					    "udpretries");
-			CHECKM("parse_uint(udpretries)", result);
+			CHECK("parse_uint(udpretries)", result);
 			if (query->udpretries > 0) {
 				query->udpretries -= 1;
 			}
@@ -1599,7 +1599,7 @@ plus_option(char *option, struct query *query, bool global) {
 			}
 			result = parse_uint(&query->udptimeout, value,
 					    MAXTIMEOUT, "udptimeout");
-			CHECKM("parse_uint(udptimeout)", result);
+			CHECK("parse_uint(udptimeout)", result);
 			break;
 		case 'n':
 			FULLCHECK("unknownformat");
@@ -1618,7 +1618,7 @@ plus_option(char *option, struct query *query, bool global) {
 		FULLCHECK("yaml");
 		yaml = state;
 		if (state) {
-			display_rrcomments = 1;
+			display_rrcomments = state;
 		}
 		break;
 	case 'z': /* zflag */
@@ -1646,7 +1646,7 @@ static bool
 dash_option(const char *option, char *next, struct query *query, bool global,
 	    bool *setname) {
 	char opt;
-	const char *value, *oldvalue;
+	const char *value;
 	isc_result_t result;
 	bool value_from_next;
 	isc_consttextregion_t tr;
@@ -1656,7 +1656,7 @@ dash_option(const char *option, char *next, struct query *query, bool global,
 	struct in_addr in4;
 	struct in6_addr in6;
 	in_port_t srcport;
-	const char *hash;
+	char *hash;
 	uint32_t num;
 
 	while (strpbrk(option, single_dash_opts) == &option[0]) {
@@ -1726,15 +1726,12 @@ dash_option(const char *option, char *next, struct query *query, bool global,
 	case 'b':
 		GLOBAL();
 		hash = strchr(value, '#');
-		oldvalue = value;
 		if (hash != NULL) {
 			result = parse_uint(&num, hash + 1, MAXPORT,
 					    "port number");
-			CHECKM("parse_uint(srcport)", result);
+			CHECK("parse_uint(srcport)", result);
 			srcport = num;
-			snprintf(textname, sizeof(textname), "%.*s",
-				 (int)(hash - value), value);
-			value = textname;
+			*hash = '\0';
 		} else {
 			srcport = 0;
 		}
@@ -1745,7 +1742,13 @@ dash_option(const char *option, char *next, struct query *query, bool global,
 			isc_sockaddr_fromin(&srcaddr, &in4, srcport);
 			isc_net_disableipv6();
 		} else {
-			fatal("invalid address %s", oldvalue);
+			if (hash != NULL) {
+				*hash = '#';
+			}
+			fatal("invalid address %s", value);
+		}
+		if (hash != NULL) {
+			*hash = '#';
 		}
 		have_src = true;
 		return value_from_next;
@@ -1754,7 +1757,7 @@ dash_option(const char *option, char *next, struct query *query, bool global,
 		tr.length = strlen(value);
 		result = dns_rdataclass_fromtext(&rdclass,
 						 (isc_textregion_t *)&tr);
-		CHECKM("dns_rdataclass_fromtext", result);
+		CHECK("dns_rdataclass_fromtext", result);
 		query->rdclass = rdclass;
 		return value_from_next;
 	case 'f':
@@ -1763,7 +1766,7 @@ dash_option(const char *option, char *next, struct query *query, bool global,
 	case 'p':
 		GLOBAL();
 		result = parse_uint(&num, value, MAXPORT, "port number");
-		CHECKM("parse_uint(port)", result);
+		CHECK("parse_uint(port)", result);
 		port = num;
 		return value_from_next;
 	case 't':
@@ -1771,7 +1774,7 @@ dash_option(const char *option, char *next, struct query *query, bool global,
 		tr.length = strlen(value);
 		result = dns_rdatatype_fromtext(&rdtype,
 						(isc_textregion_t *)&tr);
-		CHECKM("dns_rdatatype_fromtext", result);
+		CHECK("dns_rdatatype_fromtext", result);
 		query->rdtype = rdtype;
 		return value_from_next;
 	case 'x':
@@ -2108,8 +2111,6 @@ int
 main(int argc, char *argv[]) {
 	struct query *query = NULL;
 	isc_result_t result;
-	isc_log_t *lctx = NULL;
-	isc_logconfig_t *lcfg = NULL;
 	unsigned int i;
 	int ns;
 
@@ -2126,9 +2127,7 @@ main(int argc, char *argv[]) {
 	preparse_args(argc, argv);
 
 	isc_managers_create(&mctx, 1, &loopmgr, &netmgr);
-	isc_log_create(mctx, &lctx, &lcfg);
 
-	RUNCHECK(dst_lib_init(mctx, NULL));
 	isc_nonce_buf(cookie_secret, sizeof(cookie_secret));
 
 	ISC_LIST_INIT(queries);
@@ -2188,10 +2187,6 @@ main(int argc, char *argv[]) {
 
 	isc_loopmgr_run(loopmgr);
 
-	dst_lib_destroy();
-
-	isc_log_destroy(&lctx);
-
 	query = ISC_LIST_HEAD(queries);
 	while (query != NULL) {
 		struct query *next = ISC_LIST_NEXT(query, link);
