@@ -483,7 +483,7 @@ class TextProc(object):
             return TextProc(lambda text: value.fun(self.fun(text)))
         import inspect
 
-        _frame, filename, lineno, _function_name, lines, _index = inspect.stack()[1]
+        (_frame, filename, lineno, _function_name, lines, _index) = inspect.stack()[1]
         raise SyntaxError(
             "Invalid syntax in config file",
             (
@@ -1889,7 +1889,8 @@ def changelog(
     try:
         first_version = next(versions)
     except StopIteration:
-        die("Empty changelog. No commits were elected to be used as entry.")
+        warn("Empty changelog. No commits were elected to be used as entry.")
+        data["versions"] = []
     else:
         data["versions"] = itertools.chain([first_version], versions)
 
@@ -2084,7 +2085,10 @@ def safe_print(content):
             raise
         ## XXXvlab: should use $COLUMNS in bash and for windows:
         ## http://stackoverflow.com/questions/14978548
-        stderr(paragraph_wrap(textwrap.dedent("""\
+        stderr(
+            paragraph_wrap(
+                textwrap.dedent(
+                    """\
             UnicodeEncodeError:
               There was a problem outputing the resulting changelog to
               your console.
@@ -2092,7 +2096,11 @@ def safe_print(content):
               This probably means that the changelog contains characters
               that can't be translated to characters in your current charset
               (%s).
-            """) % sys.stdout.encoding))
+            """
+                )
+                % sys.stdout.encoding
+            )
+        )
         if WIN32 and PY_VERSION < 3.6 and sys.stdout.encoding != "utf-8":
             ## As of PY 3.6, encoding is now ``utf-8`` regardless of
             ## PYTHONIOENCODING
