@@ -462,7 +462,8 @@ static void
 dns__acl_destroy_port_transports(dns_acl_t *acl) {
 	dns_acl_port_transports_t *port_proto = NULL;
 	dns_acl_port_transports_t *next = NULL;
-	ISC_LIST_FOREACH_SAFE(acl->ports_and_transports, port_proto, link, next)
+	ISC_LIST_FOREACH_SAFE (acl->ports_and_transports, port_proto, link,
+			       next)
 	{
 		ISC_LIST_DEQUEUE(acl->ports_and_transports, port_proto, link);
 		isc_mem_put(acl->mctx, port_proto, sizeof(*port_proto));
