@@ -22,6 +22,7 @@
 #include <isc/hash.h>
 #include <isc/hashmap.h>
 #include <isc/list.h>
+#include <isc/log.h>
 #include <isc/loop.h>
 #include <isc/mutex.h>
 #include <isc/netaddr.h>
@@ -35,12 +36,9 @@
 
 #include <dns/adb.h>
 #include <dns/db.h>
-#include <dns/log.h>
 #include <dns/rdata.h>
-#include <dns/rdataset.h>
 #include <dns/rdatastruct.h>
 #include <dns/rdatatype.h>
-#include <dns/resolver.h>
 #include <dns/stats.h>
 #include <dns/transport.h>
 
@@ -60,8 +58,12 @@
 /*!
  * For type 3 negative cache entries, we will remember that the address is
  * broken for this long.  XXXMLG This is also used for actual addresses, too.
+ * The intent is to keep us from constantly asking about A/AAAA records
+ * if the zone has extremely low TTLs.
  */
+#define ADB_CACHE_MINIMUM 10	/*%< seconds */
 #define ADB_CACHE_MAXIMUM 86400 /*%< seconds (86400 = 24 hours) */
+#define ADB_ENTRY_WINDOW  60	/*%< seconds */
 
 #ifndef ADB_HASH_BITS
 #define ADB_HASH_BITS 12
@@ -135,7 +137,6 @@ struct dns_adbname {
 	dns_name_t *name;
 	unsigned int partial_result;
 	unsigned int flags;
-	unsigned int type;
 	dns_name_t target;
 	isc_stdtime_t expire_target;
 	isc_stdtime_t expire_v4;
@@ -260,17 +261,11 @@ ISC_REFCOUNT_TRACE_DECL(dns_adbentry);
 ISC_REFCOUNT_DECL(dns_adbentry);
 #endif
 
-/*
- * ADB settings that can be tweaked with named -T option
- */
-unsigned int dns_adb_entrywindow = 60;
-unsigned int dns_adb_cachemin = 10;
-
 /*
  * Internal functions (and prototypes).
  */
 static dns_adbname_t *
-new_adbname(dns_adb_t *adb, const dns_name_t *, unsigned int type);
+new_adbname(dns_adb_t *adb, const dns_name_t *, unsigned int flags);
 static void
 destroy_adbname(dns_adbname_t *);
 static bool
@@ -299,15 +294,11 @@ static void
 free_adbfetch(dns_adb_t *, dns_adbfetch_t **);
 static void
 purge_stale_names(dns_adb_t *adb, isc_stdtime_t now);
-static void
-purge_names_overmem(dns_adb_t *adb, size_t requested);
 static dns_adbname_t *
-get_attached_and_locked_name(dns_adb_t *, const dns_name_t *, unsigned int type,
-			     isc_stdtime_t now);
+get_attached_and_locked_name(dns_adb_t *, const dns_name_t *,
+			     unsigned int flags, isc_stdtime_t now);
 static void
 purge_stale_entries(dns_adb_t *adb, isc_stdtime_t now);
-static void
-purge_entries_overmem(dns_adb_t *adb, size_t requested);
 static dns_adbentry_t *
 get_attached_and_locked_entry(dns_adb_t *adb, isc_stdtime_t now,
 			      const isc_sockaddr_t *addr);
@@ -328,7 +319,7 @@ static void
 clean_finds_at_name(dns_adbname_t *, dns_adbstatus_t, unsigned int);
 static void
 maybe_expire_namehooks(dns_adbname_t *, isc_stdtime_t);
-static void
+static bool
 maybe_expire_name(dns_adbname_t *adbname, isc_stdtime_t now);
 static void
 expire_name(dns_adbname_t *adbname, dns_adbstatus_t astat);
@@ -342,7 +333,7 @@ static isc_result_t
 dbfind_name(dns_adbname_t *, isc_stdtime_t, dns_rdatatype_t);
 static isc_result_t
 fetch_name(dns_adbname_t *, bool, unsigned int, isc_counter_t *qc,
-	   isc_counter_t *gqc, fetchctx_t *parent, dns_rdatatype_t);
+	   dns_rdatatype_t);
 static void
 destroy(dns_adb_t *);
 static void
@@ -422,9 +413,7 @@ enum {
 #define FIND_HAS_ADDRS(fn)	(!ISC_LIST_EMPTY((fn)->list))
 #define FIND_NOFETCH(fn)	(((fn)->options & DNS_ADBFIND_NOFETCH) != 0)
 
-#define ADBNAME_TYPE_MASK (DNS_ADBFIND_STARTATZONE | DNS_ADBFIND_STATICSTUB)
-
-#define ADBNAME_TYPE(options) ((options) & ADBNAME_TYPE_MASK)
+#define ADBNAME_FLAGS_MASK (DNS_ADBFIND_STARTATZONE | DNS_ADBFIND_STATICSTUB)
 
 /*
  * These are currently used on simple unsigned ints, so they are
@@ -435,6 +424,15 @@ enum {
 
 #define EXPIRE_OK(exp, now) ((exp == INT_MAX) || (exp < now))
 
+/*
+ * Find out if the flags on a name (nf) indicate if it is a hint or
+ * glue, and compare this to the appropriate bits set in o, to see if
+ * this is ok.
+ */
+#define STARTATZONE_MATCHES(nf, o)                  \
+	(((nf)->flags & DNS_ADBFIND_STARTATZONE) == \
+	 ((o) & DNS_ADBFIND_STARTATZONE))
+
 #define ENTER_LEVEL  ISC_LOG_DEBUG(50)
 #define CLEAN_LEVEL  ISC_LOG_DEBUG(100)
 #define DEF_LEVEL    ISC_LOG_DEBUG(5)
@@ -448,10 +446,10 @@ enum {
  * Due to the ttlclamp(), the TTL is never 0 unless the trust is ultimate,
  * in which case we need to set the expiration to have immediate effect.
  */
-#define ADJUSTED_EXPIRE(expire, now, ttl)                                    \
-	((ttl != 0) ? ISC_MIN(expire,                                        \
-			      ISC_MAX(now + dns_adb_entrywindow, now + ttl)) \
-		    : INT_MAX)
+#define ADJUSTED_EXPIRE(expire, now, ttl)                                      \
+	((ttl != 0)                                                            \
+		 ? ISC_MIN(expire, ISC_MAX(now + ADB_ENTRY_WINDOW, now + ttl)) \
+		 : INT_MAX)
 
 /*
  * Error states.
@@ -483,8 +481,8 @@ DP(int level, const char *format, ...) {
 	va_list args;
 
 	va_start(args, format);
-	isc_log_vwrite(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_ADB,
-		       level, format, args);
+	isc_log_vwrite(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_ADB, level,
+		       format, args);
 	va_end(args);
 }
 
@@ -524,12 +522,8 @@ inc_adbstats(dns_adb_t *adb, isc_statscounter_t counter) {
 
 static dns_ttl_t
 ttlclamp(dns_ttl_t ttl) {
-	if (ttl < dns_adb_cachemin) {
-		/*
-		 * Avoid to constantly ask about A/AAAA records if the zone has
-		 * extremely low TTLs.
-		 */
-		ttl = dns_adb_cachemin;
+	if (ttl < ADB_CACHE_MINIMUM) {
+		ttl = ADB_CACHE_MINIMUM;
 	}
 	if (ttl > ADB_CACHE_MAXIMUM) {
 		ttl = ADB_CACHE_MAXIMUM;
@@ -561,7 +555,7 @@ import_rdataset(dns_adbname_t *adbname, dns_rdataset_t *rdataset,
 	switch (rdataset->trust) {
 	case dns_trust_glue:
 	case dns_trust_additional:
-		rdataset->ttl = dns_adb_cachemin;
+		rdataset->ttl = ADB_CACHE_MINIMUM;
 		break;
 	case dns_trust_ultimate:
 		rdataset->ttl = 0;
@@ -960,7 +954,7 @@ clean_finds_at_name(dns_adbname_t *name, dns_adbstatus_t astat,
 }
 
 static dns_adbname_t *
-new_adbname(dns_adb_t *adb, const dns_name_t *dnsname, unsigned int type) {
+new_adbname(dns_adb_t *adb, const dns_name_t *dnsname, unsigned int flags) {
 	dns_adbname_t *name = NULL;
 
 	name = isc_mem_get(adb->mctx, sizeof(*name));
@@ -975,7 +969,7 @@ new_adbname(dns_adb_t *adb, const dns_name_t *dnsname, unsigned int type) {
 		.v6 = ISC_LIST_INITIALIZER,
 		.finds = ISC_LIST_INITIALIZER,
 		.link = ISC_LINK_INITIALIZER,
-		.type = type,
+		.flags = flags & ADBNAME_FLAGS_MASK,
 		.magic = DNS_ADBNAME_MAGIC,
 	};
 
@@ -1065,7 +1059,7 @@ new_adbentry(dns_adb_t *adb, const isc_sockaddr_t *addr, isc_stdtime_t now) {
 		.quota = adb->quota,
 		.references = ISC_REFCOUNT_INITIALIZER(1),
 		.adb = dns_adb_ref(adb),
-		.expires = now + dns_adb_entrywindow,
+		.expires = now + ADB_ENTRY_WINDOW,
 		.magic = DNS_ADBENTRY_MAGIC,
 	};
 
@@ -1118,7 +1112,7 @@ static dns_adbfind_t *
 new_adbfind(dns_adb_t *adb, in_port_t port) {
 	dns_adbfind_t *find = NULL;
 
-	find = isc_mem_get(adb->hmctx, sizeof(*find));
+	find = isc_mem_get(adb->mctx, sizeof(*find));
 	*find = (dns_adbfind_t){
 		.port = port,
 		.result_v4 = ISC_R_UNEXPECTED,
@@ -1157,7 +1151,7 @@ free_adbfind(dns_adbfind_t **findp) {
 
 	isc_mutex_destroy(&find->lock);
 
-	isc_mem_put(adb->hmctx, find, sizeof(*find));
+	isc_mem_put(adb->mctx, find, sizeof(*find));
 	dns_adb_detach(&adb);
 }
 
@@ -1165,7 +1159,7 @@ static dns_adbfetch_t *
 new_adbfetch(dns_adb_t *adb) {
 	dns_adbfetch_t *fetch = NULL;
 
-	fetch = isc_mem_get(adb->hmctx, sizeof(*fetch));
+	fetch = isc_mem_get(adb->mctx, sizeof(*fetch));
 	*fetch = (dns_adbfetch_t){ 0 };
 	dns_rdataset_init(&fetch->rdataset);
 
@@ -1189,7 +1183,7 @@ free_adbfetch(dns_adb_t *adb, dns_adbfetch_t **fetchp) {
 		dns_rdataset_disassociate(&fetch->rdataset);
 	}
 
-	isc_mem_put(adb->hmctx, fetch, sizeof(*fetch));
+	isc_mem_put(adb->mctx, fetch, sizeof(*fetch));
 }
 
 /*
@@ -1200,7 +1194,7 @@ static dns_adbaddrinfo_t *
 new_adbaddrinfo(dns_adb_t *adb, dns_adbentry_t *entry, in_port_t port) {
 	dns_adbaddrinfo_t *ai = NULL;
 
-	ai = isc_mem_get(adb->hmctx, sizeof(*ai));
+	ai = isc_mem_get(adb->mctx, sizeof(*ai));
 	*ai = (dns_adbaddrinfo_t){
 		.srtt = atomic_load(&entry->srtt),
 		.flags = atomic_load(&entry->flags),
@@ -1233,7 +1227,7 @@ free_adbaddrinfo(dns_adb_t *adb, dns_adbaddrinfo_t **ainfo) {
 	}
 	dns_adbentry_detach(&ai->entry);
 
-	isc_mem_put(adb->hmctx, ai, sizeof(*ai));
+	isc_mem_put(adb->mctx, ai, sizeof(*ai));
 }
 
 static bool
@@ -1241,7 +1235,9 @@ match_adbname(void *node, const void *key) {
 	const dns_adbname_t *adbname0 = node;
 	const dns_adbname_t *adbname1 = key;
 
-	if (adbname0->type != adbname1->type) {
+	if ((adbname0->flags & ADBNAME_FLAGS_MASK) !=
+	    (adbname1->flags & ADBNAME_FLAGS_MASK))
+	{
 		return false;
 	}
 
@@ -1251,11 +1247,12 @@ match_adbname(void *node, const void *key) {
 static uint32_t
 hash_adbname(const dns_adbname_t *adbname) {
 	isc_hash32_t hash;
+	unsigned int flags = adbname->flags & ADBNAME_FLAGS_MASK;
 
 	isc_hash32_init(&hash);
 	isc_hash32_hash(&hash, adbname->name->ndata, adbname->name->length,
 			false);
-	isc_hash32_hash(&hash, &adbname->type, sizeof(adbname->type), true);
+	isc_hash32_hash(&hash, &flags, sizeof(flags), true);
 	return isc_hash32_finalize(&hash);
 }
 
@@ -1264,32 +1261,29 @@ hash_adbname(const dns_adbname_t *adbname) {
  */
 static dns_adbname_t *
 get_attached_and_locked_name(dns_adb_t *adb, const dns_name_t *name,
-			     unsigned int type, isc_stdtime_t now) {
+			     unsigned int flags, isc_stdtime_t now) {
 	isc_result_t result;
 	dns_adbname_t *adbname = NULL;
 	isc_time_t timenow;
 	isc_stdtime_t last_update;
 	dns_adbname_t key = {
 		.name = UNCONST(name),
-		.type = type,
+		.flags = flags & ADBNAME_FLAGS_MASK,
 	};
 	uint32_t hashval = hash_adbname(&key);
 	isc_rwlocktype_t locktype = isc_rwlocktype_read;
-	bool overmem = isc_mem_isovermem(adb->mctx);
 
 	isc_time_set(&timenow, now, 0);
 
 	RWLOCK(&adb->names_lock, locktype);
 	last_update = adb->names_last_update;
 
-	if (last_update + ADB_STALE_MARGIN >= now || overmem) {
+	if (last_update + ADB_STALE_MARGIN >= now ||
+	    isc_mem_isovermem(adb->mctx))
+	{
 		last_update = now;
 		UPGRADELOCK(&adb->names_lock, locktype);
-		if (overmem) {
-			purge_names_overmem(adb, 2 * sizeof(*adbname));
-		} else {
-			purge_stale_names(adb, now);
-		}
+		purge_stale_names(adb, now);
 		adb->names_last_update = last_update;
 	}
 
@@ -1300,7 +1294,7 @@ get_attached_and_locked_name(dns_adb_t *adb, const dns_name_t *name,
 		UPGRADELOCK(&adb->names_lock, locktype);
 
 		/* Allocate a new name and add it to the hash table. */
-		adbname = new_adbname(adb, name, key.type);
+		adbname = new_adbname(adb, name, key.flags);
 
 		void *found = NULL;
 		result = isc_hashmap_add(adb->names, hashval, match_adbname,
@@ -1326,7 +1320,7 @@ get_attached_and_locked_name(dns_adb_t *adb, const dns_name_t *name,
 	dns_adbname_ref(adbname);
 
 	LOCK(&adbname->lock); /* Must be unlocked by the caller */
-	if (adbname->last_used + dns_adb_cachemin <= last_update) {
+	if (adbname->last_used + ADB_CACHE_MINIMUM <= last_update) {
 		adbname->last_used = now;
 	}
 	if (locktype == isc_rwlocktype_write) {
@@ -1343,6 +1337,16 @@ get_attached_and_locked_name(dns_adb_t *adb, const dns_name_t *name,
 	return adbname;
 }
 
+static void
+upgrade_entries_lock(dns_adb_t *adb, isc_rwlocktype_t *locktypep,
+		     isc_stdtime_t now) {
+	if (*locktypep == isc_rwlocktype_read) {
+		UPGRADELOCK(&adb->entries_lock, *locktypep);
+		purge_stale_entries(adb, now);
+		adb->entries_last_update = now;
+	}
+}
+
 static bool
 match_adbentry(void *node, const void *key) {
 	dns_adbentry_t *adbentry = node;
@@ -1362,30 +1366,25 @@ get_attached_and_locked_entry(dns_adb_t *adb, isc_stdtime_t now,
 	isc_stdtime_t last_update;
 	uint32_t hashval = isc_sockaddr_hash(addr, true);
 	isc_rwlocktype_t locktype = isc_rwlocktype_read;
-	bool overmem = isc_mem_isovermem(adb->mctx);
 
 	isc_time_set(&timenow, now, 0);
 
 	RWLOCK(&adb->entries_lock, locktype);
 	last_update = adb->entries_last_update;
 
-	if (now - last_update > ADB_STALE_MARGIN || overmem) {
+	if (now - last_update > ADB_STALE_MARGIN ||
+	    isc_mem_isovermem(adb->mctx))
+	{
 		last_update = now;
 
-		UPGRADELOCK(&adb->entries_lock, locktype);
-		if (overmem) {
-			purge_entries_overmem(adb, 2 * sizeof(*adbentry));
-		} else {
-			purge_stale_entries(adb, now);
-		}
-		adb->entries_last_update = now;
+		upgrade_entries_lock(adb, &locktype, now);
 	}
 
 	result = isc_hashmap_find(adb->entries, hashval, match_adbentry,
 				  (const unsigned char *)addr,
 				  (void **)&adbentry);
 	if (result == ISC_R_NOTFOUND) {
-		UPGRADELOCK(&adb->entries_lock, locktype);
+		upgrade_entries_lock(adb, &locktype, now);
 
 	create:
 		INSIST(locktype == isc_rwlocktype_write);
@@ -1420,7 +1419,7 @@ get_attached_and_locked_entry(dns_adb_t *adb, isc_stdtime_t now,
 
 		/* We need to upgrade the LRU lock */
 		UNLOCK(&adbentry->lock);
-		UPGRADELOCK(&adb->entries_lock, locktype);
+		upgrade_entries_lock(adb, &locktype, now);
 		LOCK(&adbentry->lock);
 		FALLTHROUGH;
 	case isc_rwlocktype_write:
@@ -1435,7 +1434,7 @@ get_attached_and_locked_entry(dns_adb_t *adb, isc_stdtime_t now,
 	}
 
 	/* Did enough time pass to update the LRU? */
-	if (adbentry->last_used + dns_adb_cachemin <= last_update) {
+	if (adbentry->last_used + ADB_CACHE_MINIMUM <= last_update) {
 		adbentry->last_used = now;
 		if (locktype == isc_rwlocktype_write) {
 			ISC_LIST_UNLINK(adb->entries_lru, adbentry, link);
@@ -1462,8 +1461,7 @@ log_quota(dns_adbentry_t *entry, const char *fmt, ...) {
 	isc_netaddr_fromsockaddr(&netaddr, &entry->sockaddr);
 	isc_netaddr_format(&netaddr, addrbuf, sizeof(addrbuf));
 
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_ADB,
-		      ISC_LOG_INFO,
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_ADB, ISC_LOG_INFO,
 		      "adb: quota %s (%" PRIuFAST32 "/%" PRIuFAST32 "): %s",
 		      addrbuf, atomic_load_relaxed(&entry->active),
 		      atomic_load_relaxed(&entry->quota), msgbuf);
@@ -1526,18 +1524,18 @@ copy_namehook_lists(dns_adb_t *adb, dns_adbfind_t *find, dns_adbname_t *name) {
 /*
  * The name must be locked and write lock on adb->names_lock must be held.
  */
-static void
+static bool
 maybe_expire_name(dns_adbname_t *adbname, isc_stdtime_t now) {
 	REQUIRE(DNS_ADBNAME_VALID(adbname));
 
 	/* Leave this name alone if it still has active namehooks... */
 	if (NAME_HAS_V4(adbname) || NAME_HAS_V6(adbname)) {
-		return;
+		return false;
 	}
 
 	/* ...an active fetch in progres... */
 	if (NAME_FETCH(adbname)) {
-		return;
+		return false;
 	}
 
 	/* ... or is not yet expired. */
@@ -1545,10 +1543,12 @@ maybe_expire_name(dns_adbname_t *adbname, isc_stdtime_t now) {
 	    !EXPIRE_OK(adbname->expire_v6, now) ||
 	    !EXPIRE_OK(adbname->expire_target, now))
 	{
-		return;
+		return false;
 	}
 
 	expire_name(adbname, DNS_ADB_EXPIRED);
+
+	return true;
 }
 
 static void
@@ -1607,45 +1607,68 @@ maybe_expire_entry(dns_adbentry_t *adbentry, isc_stdtime_t now) {
  */
 static void
 purge_stale_names(dns_adb_t *adb, isc_stdtime_t now) {
-	dns_adbname_t *adbname = ISC_LIST_TAIL(adb->names_lru);
-
-	if (adbname == NULL) {
-		return;
-	}
-
-	dns_adbname_ref(adbname);
-	LOCK(&adbname->lock);
+	bool overmem = isc_mem_isovermem(adb->mctx);
+	int max_removed = overmem ? 2 : 1;
+	int scans = 0, removed = 0;
+	dns_adbname_t *prev = NULL;
 
 	/*
-	 * Remove the name if it's expired or unused, has no address data.
+	 * We limit the number of scanned entries to 10 (arbitrary choice)
+	 * in order to avoid examining too many entries when there are many
+	 * tail entries that have fetches (this should be rare, but could
+	 * happen).
 	 */
-	maybe_expire_namehooks(adbname, now);
-	maybe_expire_name(adbname, now);
-
-	UNLOCK(&adbname->lock);
-	dns_adbname_detach(&adbname);
-}
-
-static void
-purge_names_overmem(dns_adb_t *adb, size_t requested) {
-	dns_adbname_t *prev = NULL;
-	size_t expired = 0;
 
 	for (dns_adbname_t *adbname = ISC_LIST_TAIL(adb->names_lru);
-	     adbname != NULL && expired < requested; adbname = prev)
+	     adbname != NULL && removed < max_removed && scans < 10;
+	     adbname = prev)
 	{
 		prev = ISC_LIST_PREV(adbname, link);
 
 		dns_adbname_ref(adbname);
 		LOCK(&adbname->lock);
 
+		scans++;
+
 		/*
 		 * Remove the name if it's expired or unused,
 		 * has no address data.
 		 */
-		maybe_expire_namehooks(adbname, INT_MAX);
-		expire_name(adbname, DNS_ADB_CANCELED);
-		expired += sizeof(*adbname);
+		maybe_expire_namehooks(adbname, now);
+		if (maybe_expire_name(adbname, now)) {
+			removed++;
+			goto next;
+		}
+
+		/*
+		 * Make sure that we are not purging ADB names that has been
+		 * just created.
+		 */
+		if (adbname->last_used + ADB_CACHE_MINIMUM >= now) {
+			prev = NULL;
+			goto next;
+		}
+
+		if (overmem) {
+			expire_name(adbname, DNS_ADB_CANCELED);
+			removed++;
+			goto next;
+		}
+
+		if (adbname->last_used + ADB_STALE_MARGIN < now) {
+			expire_name(adbname, DNS_ADB_CANCELED);
+			removed++;
+			goto next;
+		}
+
+		/*
+		 * We won't expire anything on the LRU list as the
+		 * .last_used + ADB_STALE_MARGIN will always be bigger
+		 * than `now` for all previous entries, so we just stop
+		 * the scanning.
+		 */
+		prev = NULL;
+	next:
 		UNLOCK(&adbname->lock);
 		dns_adbname_detach(&adbname);
 	}
@@ -1670,7 +1693,7 @@ cleanup_names(dns_adb_t *adb, isc_stdtime_t now) {
 		 * fetches, we can remove this name from the bucket.
 		 */
 		maybe_expire_namehooks(adbname, now);
-		maybe_expire_name(adbname, now);
+		(void)maybe_expire_name(adbname, now);
 		UNLOCK(&adbname->lock);
 		dns_adbname_detach(&adbname);
 	}
@@ -1689,28 +1712,10 @@ cleanup_names(dns_adb_t *adb, isc_stdtime_t now) {
  */
 static void
 purge_stale_entries(dns_adb_t *adb, isc_stdtime_t now) {
-	dns_adbentry_t *adbentry = ISC_LIST_TAIL(adb->entries_lru);
-
-	if (adbentry == NULL) {
-		return;
-	}
-
-	dns_adbentry_ref(adbentry);
-	LOCK(&adbentry->lock);
-
-	/*
-	 * Remove the entry if it's expired and unused.
-	 */
-	(void)maybe_expire_entry(adbentry, now);
-
-	UNLOCK(&adbentry->lock);
-	dns_adbentry_detach(&adbentry);
-}
-
-static void
-purge_entries_overmem(dns_adb_t *adb, size_t requested) {
+	bool overmem = isc_mem_isovermem(adb->mctx);
+	int max_removed = overmem ? 2 : 1;
+	int scans = 0, removed = 0;
 	dns_adbentry_t *prev = NULL;
-	size_t expired = 0;
 
 	/*
 	 * We limit the number of scanned entries to 10 (arbitrary choice)
@@ -1720,16 +1725,53 @@ purge_entries_overmem(dns_adb_t *adb, size_t requested) {
 	 */
 
 	for (dns_adbentry_t *adbentry = ISC_LIST_TAIL(adb->entries_lru);
-	     adbentry != NULL && expired < requested; adbentry = prev)
+	     adbentry != NULL && removed < max_removed && scans < 10;
+	     adbentry = prev)
 	{
 		prev = ISC_LIST_PREV(adbentry, link);
 
 		dns_adbentry_ref(adbentry);
 		LOCK(&adbentry->lock);
 
-		(void)maybe_expire_entry(adbentry, INT_MAX);
-		expired += sizeof(*adbentry);
+		scans++;
 
+		/*
+		 * Remove the entry if it's expired and unused.
+		 */
+		if (maybe_expire_entry(adbentry, now)) {
+			removed++;
+			goto next;
+		}
+
+		/*
+		 * Make sure that we are not purging ADB entry that has been
+		 * just created.
+		 */
+		if (adbentry->last_used + ADB_CACHE_MINIMUM >= now) {
+			prev = NULL;
+			goto next;
+		}
+
+		if (overmem) {
+			maybe_expire_entry(adbentry, INT_MAX);
+			removed++;
+			goto next;
+		}
+
+		if (adbentry->last_used + ADB_STALE_MARGIN < now) {
+			maybe_expire_entry(adbentry, INT_MAX);
+			removed++;
+			goto next;
+		}
+
+		/*
+		 * We won't expire anything on the LRU list as the
+		 * .last_used + ADB_STALE_MARGIN will always be bigger
+		 * than `now` for all previous entries, so we just stop
+		 * the scanning
+		 */
+		prev = NULL;
+	next:
 		UNLOCK(&adbentry->lock);
 		dns_adbentry_detach(&adbentry);
 	}
@@ -1821,7 +1863,7 @@ dns_adb_create(isc_mem_t *mem, dns_view_t *view, dns_adb_t **newadb) {
 	isc_mem_attach(mem, &adb->mctx);
 
 	isc_mem_create(&adb->hmctx);
-	isc_mem_setname(adb->hmctx, "ADB_dynamic");
+	isc_mem_setname(adb->hmctx, "ADB_hashmaps");
 
 	isc_hashmap_create(adb->hmctx, ADB_HASH_BITS, &adb->names);
 	isc_rwlock_init(&adb->names_lock);
@@ -1881,8 +1923,8 @@ dns_adb_createfind(dns_adb_t *adb, isc_loop_t *loop, isc_job_cb cb, void *cbarg,
 		   const dns_name_t *name, const dns_name_t *qname,
 		   dns_rdatatype_t qtype ISC_ATTR_UNUSED, unsigned int options,
 		   isc_stdtime_t now, dns_name_t *target, in_port_t port,
-		   unsigned int depth, isc_counter_t *qc, isc_counter_t *gqc,
-		   fetchctx_t *parent, dns_adbfind_t **findp) {
+		   unsigned int depth, isc_counter_t *qc,
+		   dns_adbfind_t **findp) {
 	isc_result_t result = ISC_R_UNEXPECTED;
 	dns_adbfind_t *find = NULL;
 	dns_adbname_t *adbname = NULL;
@@ -1934,14 +1976,13 @@ dns_adb_createfind(dns_adb_t *adb, isc_loop_t *loop, isc_job_cb cb, void *cbarg,
 		REQUIRE(loop != NULL);
 	}
 
-	if (isc_log_wouldlog(dns_lctx, DEF_LEVEL)) {
+	if (isc_log_wouldlog(DEF_LEVEL)) {
 		dns_name_format(name, namebuf, sizeof(namebuf));
 	}
 
 again:
 	/* Try to see if we know anything about this name at all. */
-	adbname = get_attached_and_locked_name(
-		adb, name, ADBNAME_TYPE(find->options), now);
+	adbname = get_attached_and_locked_name(adb, name, find->options, now);
 
 	if (NAME_DEAD(adbname)) {
 		UNLOCK(&adbname->lock);
@@ -2092,7 +2133,7 @@ fetch:
 		 * Start V4.
 		 */
 		if (WANT_INET(wanted_fetches) &&
-		    fetch_name(adbname, start_at_zone, depth, qc, gqc, parent,
+		    fetch_name(adbname, start_at_zone, depth, qc,
 			       dns_rdatatype_a) == ISC_R_SUCCESS)
 		{
 			DP(DEF_LEVEL,
@@ -2105,7 +2146,7 @@ fetch:
 		 * Start V6.
 		 */
 		if (WANT_INET6(wanted_fetches) &&
-		    fetch_name(adbname, start_at_zone, depth, qc, gqc, parent,
+		    fetch_name(adbname, start_at_zone, depth, qc,
 			       dns_rdatatype_aaaa) == ISC_R_SUCCESS)
 		{
 			DP(DEF_LEVEL,
@@ -2193,10 +2234,6 @@ post_copy:
 		find->cbarg = cbarg;
 	}
 
-	if (wanted_fetches) {
-		find->options |= DNS_ADBFIND_STARTEDFETCH;
-	}
-
 	*findp = find;
 
 	UNLOCK(&adbname->lock);
@@ -2630,11 +2667,11 @@ dbfind_name(dns_adbname_t *adbname, isc_stdtime_t now, dns_rdatatype_t rdtype) {
 	 * any matching static-stub zone without looking into the cache to honor
 	 * the configuration on which server we should send queries to.
 	 */
-	result = dns_view_find(adb->view, adbname->name, rdtype, now,
-			       DNS_DBFIND_GLUEOK | DNS_DBFIND_ADDITIONALOK,
-			       true,
-			       (adbname->type & DNS_ADBFIND_STARTATZONE) != 0,
-			       NULL, NULL, fname, &rdataset, NULL);
+	result =
+		dns_view_find(adb->view, adbname->name, rdtype, now,
+			      DNS_DBFIND_GLUEOK | DNS_DBFIND_ADDITIONALOK, true,
+			      ((adbname->flags & DNS_ADBFIND_STARTATZONE) != 0),
+			      NULL, NULL, fname, &rdataset, NULL);
 
 	switch (result) {
 	case DNS_R_GLUE:
@@ -2904,7 +2941,7 @@ check_result:
 out:
 	dns_resolver_destroyfetch(&fetch->fetch);
 	free_adbfetch(adb, &fetch);
-	dns_resolver_freefresp(&resp);
+	isc_mem_putanddetach(&resp->mctx, resp, sizeof(*resp));
 	if (astat != DNS_ADB_CANCELED) {
 		clean_finds_at_name(name, astat, address_type);
 	}
@@ -2915,8 +2952,7 @@ out:
 
 static isc_result_t
 fetch_name(dns_adbname_t *adbname, bool start_at_zone, unsigned int depth,
-	   isc_counter_t *qc, isc_counter_t *gqc, fetchctx_t *parent,
-	   dns_rdatatype_t type) {
+	   isc_counter_t *qc, dns_rdatatype_t type) {
 	isc_result_t result;
 	dns_adbfetch_t *fetch = NULL;
 	dns_adb_t *adb = NULL;
@@ -2970,18 +3006,18 @@ fetch_name(dns_adbname_t *adbname, bool start_at_zone, unsigned int depth,
 	 * createfetch to find deepest cached name when we're providing
 	 * domain and nameservers.
 	 */
-	dns_adbname_ref(adbname);
 	result = dns_resolver_createfetch(
 		adb->res, adbname->name, type, name, nameservers, NULL, NULL, 0,
-		options, depth, qc, gqc, parent, isc_loop(), fetch_callback,
-		adbname, NULL, &fetch->rdataset, NULL, &fetch->fetch);
+		options, depth, qc, isc_loop(), fetch_callback, adbname,
+		&fetch->rdataset, NULL, &fetch->fetch);
 	if (result != ISC_R_SUCCESS) {
 		DP(ENTER_LEVEL, "fetch_name: createfetch failed with %s",
 		   isc_result_totext(result));
-		dns_adbname_unref(adbname);
 		goto cleanup;
 	}
 
+	dns_adbname_ref(adbname);
+
 	if (type == dns_rdatatype_a) {
 		adbname->fetch_a = fetch;
 		inc_resstats(adb, dns_resstatscounter_gluefetchv4);
@@ -3390,8 +3426,8 @@ again:
 	 * Delete all entries - with and without DNS_ADBFIND_STARTATZONE set
 	 * and with and without DNS_ADBFIND_STATICSTUB set.
 	 */
-	key.type = ((static_stub) ? DNS_ADBFIND_STATICSTUB : 0) |
-		   ((start_at_zone) ? DNS_ADBFIND_STARTATZONE : 0);
+	key.flags = ((static_stub) ? DNS_ADBFIND_STATICSTUB : 0) |
+		    ((start_at_zone) ? DNS_ADBFIND_STARTATZONE : 0);
 
 	result = isc_hashmap_find(adb->names, hash_adbname(&key), match_adbname,
 				  (void *)&key, (void **)&adbname);
