@@ -22,6 +22,7 @@
 #include <isc/async.h>
 #include <isc/hash.h>
 #include <isc/hashmap.h>
+#include <isc/log.h>
 #include <isc/loop.h>
 #include <isc/mem.h>
 #include <isc/mutex.h>
@@ -39,7 +40,6 @@
 
 #include <dns/acl.h>
 #include <dns/dispatch.h>
-#include <dns/log.h>
 #include <dns/message.h>
 #include <dns/stats.h>
 #include <dns/transport.h>
@@ -251,7 +251,7 @@ mgr_log(dns_dispatchmgr_t *mgr, int level, const char *fmt, ...) {
 	char msgbuf[2048];
 	va_list ap;
 
-	if (!isc_log_wouldlog(dns_lctx, level)) {
+	if (!isc_log_wouldlog(level)) {
 		return;
 	}
 
@@ -259,9 +259,8 @@ mgr_log(dns_dispatchmgr_t *mgr, int level, const char *fmt, ...) {
 	vsnprintf(msgbuf, sizeof(msgbuf), fmt, ap);
 	va_end(ap);
 
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DISPATCH,
-		      DNS_LOGMODULE_DISPATCH, level, "dispatchmgr %p: %s", mgr,
-		      msgbuf);
+	isc_log_write(DNS_LOGCATEGORY_DISPATCH, DNS_LOGMODULE_DISPATCH, level,
+		      "dispatchmgr %p: %s", mgr, msgbuf);
 }
 
 static void
@@ -288,7 +287,7 @@ dispatch_log(dns_dispatch_t *disp, int level, const char *fmt, ...) {
 	va_list ap;
 	int r;
 
-	if (!isc_log_wouldlog(dns_lctx, level)) {
+	if (!isc_log_wouldlog(level)) {
 		return;
 	}
 
@@ -302,9 +301,8 @@ dispatch_log(dns_dispatch_t *disp, int level, const char *fmt, ...) {
 	}
 	va_end(ap);
 
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DISPATCH,
-		      DNS_LOGMODULE_DISPATCH, level, "dispatch %p: %s", disp,
-		      msgbuf);
+	isc_log_write(DNS_LOGCATEGORY_DISPATCH, DNS_LOGMODULE_DISPATCH, level,
+		      "dispatch %p: %s", disp, msgbuf);
 }
 
 static void
@@ -317,7 +315,7 @@ dispentry_log(dns_dispentry_t *resp, int level, const char *fmt, ...) {
 	va_list ap;
 	int r;
 
-	if (!isc_log_wouldlog(dns_lctx, level)) {
+	if (!isc_log_wouldlog(level)) {
 		return;
 	}
 
@@ -537,7 +535,7 @@ udp_recv(isc_nmhandle_t *handle, isc_result_t eresult, isc_region_t *region,
 			  NULL) == ISC_R_SUCCESS &&
 	    match > 0)
 	{
-		if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(10))) {
+		if (isc_log_wouldlog(ISC_LOG_DEBUG(10))) {
 			char netaddrstr[ISC_NETADDR_FORMATSIZE];
 			isc_netaddr_format(&netaddr, netaddrstr,
 					   sizeof(netaddrstr));
@@ -564,7 +562,7 @@ udp_recv(isc_nmhandle_t *handle, isc_result_t eresult, isc_region_t *region,
 
 	dispentry_log(resp, ISC_LOG_DEBUG(92),
 		      "got valid DNS message header, /QR %c, id %u",
-		      ((flags & DNS_MESSAGEFLAG_QR) != 0) ? '1' : '0', id);
+		      (((flags & DNS_MESSAGEFLAG_QR) != 0) ? '1' : '0'), id);
 
 	/*
 	 * Look at the message flags.  If it's a query, ignore it.
@@ -668,7 +666,7 @@ tcp_recv_success(dns_dispatch_t *disp, isc_region_t *region,
 
 	dispatch_log(disp, ISC_LOG_DEBUG(92),
 		     "got valid DNS message header, /QR %c, id %u",
-		     ((flags & DNS_MESSAGEFLAG_QR) != 0) ? '1' : '0', id);
+		     (((flags & DNS_MESSAGEFLAG_QR) != 0) ? '1' : '0'), id);
 
 	/*
 	 * Look at the message flags.  If it's a query, ignore it and keep
@@ -1214,7 +1212,7 @@ dns_dispatch_createtcp(dns_dispatchmgr_t *mgr, const isc_sockaddr_t *localaddr,
 		rcu_read_unlock();
 	}
 
-	if (isc_log_wouldlog(dns_lctx, 90)) {
+	if (isc_log_wouldlog(90)) {
 		char addrbuf[ISC_SOCKADDR_FORMATSIZE];
 
 		isc_sockaddr_format(&disp->local, addrbuf,
@@ -1352,7 +1350,7 @@ dispatch_createudp(dns_dispatchmgr_t *mgr, const isc_sockaddr_t *localaddr,
 
 	dispatch_allocate(mgr, isc_socktype_udp, tid, &disp);
 
-	if (isc_log_wouldlog(dns_lctx, 90)) {
+	if (isc_log_wouldlog(90)) {
 		char addrbuf[ISC_SOCKADDR_FORMATSIZE];
 
 		isc_sockaddr_format(localaddr, addrbuf,
@@ -1823,7 +1821,7 @@ tcp_connected(isc_nmhandle_t *handle, isc_result_t eresult, void *arg) {
 	dns_dispentry_t *next = NULL;
 	dns_displist_t resps = ISC_LIST_INITIALIZER;
 
-	if (isc_log_wouldlog(dns_lctx, 90)) {
+	if (isc_log_wouldlog(90)) {
 		char localbuf[ISC_SOCKADDR_FORMATSIZE];
 		char peerbuf[ISC_SOCKADDR_FORMATSIZE];
 		if (handle != NULL) {
@@ -1960,25 +1958,6 @@ udp_dispatch_connect(dns_dispatch_t *disp, dns_dispentry_t *resp) {
 			  udp_connected, resp, resp->timeout);
 }
 
-static inline const char *
-get_tls_sni_hostname(dns_dispentry_t *resp) {
-	char *hostname = NULL;
-
-	if (resp->transport != NULL) {
-		hostname = dns_transport_get_remote_hostname(resp->transport);
-	}
-
-	if (hostname == NULL) {
-		return NULL;
-	}
-
-	if (isc_tls_valid_sni_hostname(hostname)) {
-		return hostname;
-	}
-
-	return NULL;
-}
-
 static isc_result_t
 tcp_dispatch_connect(dns_dispatch_t *disp, dns_dispentry_t *resp) {
 	dns_transport_type_t transport_type = DNS_TRANSPORT_TCP;
@@ -2026,12 +2005,10 @@ tcp_dispatch_connect(dns_dispatch_t *disp, dns_dispentry_t *resp) {
 			      "connecting from %s to %s, timeout %u", localbuf,
 			      peerbuf, resp->timeout);
 
-		const char *hostname = get_tls_sni_hostname(resp);
-
 		isc_nm_streamdnsconnect(disp->mgr->nm, &disp->local,
 					&disp->peer, tcp_connected, disp,
-					resp->timeout, tlsctx, hostname,
-					sess_cache, ISC_NM_PROXY_NONE, NULL);
+					resp->timeout, tlsctx, sess_cache,
+					ISC_NM_PROXY_NONE, NULL);
 		break;
 
 	case DNS_DISPATCHSTATE_CONNECTING:
