@@ -56,9 +56,12 @@
 
 #include <isc/buffer.h>
 #include <isc/commandline.h>
+#include <isc/log.h>
 #include <isc/magic.h>
 #include <isc/mem.h>
+#include <isc/netmgr.h>
 #include <isc/once.h>
+#include <isc/random.h>
 #include <isc/rwlock.h>
 #include <isc/string.h>
 #include <isc/util.h>
@@ -66,7 +69,6 @@
 #include <dns/db.h>
 #include <dns/dlz.h>
 #include <dns/fixedname.h>
-#include <dns/log.h>
 #include <dns/master.h>
 #include <dns/ssu.h>
 #include <dns/zone.h>
@@ -81,7 +83,7 @@ static isc_once_t once = ISC_ONCE_INIT;
 
 static void
 dlz_initialize(void) {
-	isc_rwlock_init(&dlz_implock, 0, 0);
+	isc_rwlock_init(&dlz_implock);
 	ISC_LIST_INIT(dlz_implementations);
 }
 
@@ -164,7 +166,7 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 	 * initialize the dlz_implementations list, this is guaranteed
 	 * to only really happen once.
 	 */
-	RUNTIME_CHECK(isc_once_do(&once, dlz_initialize) == ISC_R_SUCCESS);
+	isc_once_do(&once, dlz_initialize);
 
 	/*
 	 * Performs checks to make sure data is as we expect it to be.
@@ -175,9 +177,8 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 	REQUIRE(mctx != NULL);
 
 	/* write log message */
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
-		      ISC_LOG_INFO, "Loading '%s' using driver %s", dlzname,
-		      drivername);
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ, ISC_LOG_INFO,
+		      "Loading '%s' using driver %s", dlzname, drivername);
 
 	/* lock the dlz_implementations list so we can search it. */
 	RWLOCK(&dlz_implock, isc_rwlocktype_read);
@@ -187,8 +188,8 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 	if (impinfo == NULL) {
 		RWUNLOCK(&dlz_implock, isc_rwlocktype_read);
 
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DLZ, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+			      ISC_LOG_ERROR,
 			      "unsupported DLZ database driver '%s'."
 			      "  %s not loaded.",
 			      drivername, dlzname);
@@ -197,13 +198,12 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 	}
 
 	/* Allocate memory to hold the DLZ database driver */
-	db = isc_mem_get(mctx, sizeof(dns_dlzdb_t));
-
-	/* Make sure memory region is set to all 0's */
-	memset(db, 0, sizeof(dns_dlzdb_t));
+	db = isc_mem_get(mctx, sizeof(*db));
+	*db = (dns_dlzdb_t){
+		.implementation = impinfo,
+	};
 
 	ISC_LINK_INIT(db, link);
-	db->implementation = impinfo;
 	if (dlzname != NULL) {
 		db->dlzname = isc_mem_strdup(mctx, dlzname);
 	}
@@ -212,26 +212,25 @@ dns_dlzcreate(isc_mem_t *mctx, const char *dlzname, const char *drivername,
 	result = ((impinfo->methods->create)(mctx, dlzname, argc, argv,
 					     impinfo->driverarg, &db->dbdata));
 
+	RWUNLOCK(&dlz_implock, isc_rwlocktype_read);
 	/* mark the DLZ driver as valid */
-	if (result == ISC_R_SUCCESS) {
-		RWUNLOCK(&dlz_implock, isc_rwlocktype_read);
-		db->magic = DNS_DLZ_MAGIC;
-		isc_mem_attach(mctx, &db->mctx);
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DLZ, ISC_LOG_DEBUG(2),
-			      "DLZ driver loaded successfully.");
-		*dbp = db;
-		return ISC_R_SUCCESS;
-	} else {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DLZ, ISC_LOG_ERROR,
-			      "DLZ driver failed to load.");
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
 	}
 
+	db->magic = DNS_DLZ_MAGIC;
+	isc_mem_attach(mctx, &db->mctx);
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+		      ISC_LOG_DEBUG(2), "DLZ driver loaded successfully.");
+	*dbp = db;
+	return ISC_R_SUCCESS;
+failure:
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+		      ISC_LOG_ERROR, "DLZ driver failed to load.");
+
 	/* impinfo->methods->create failed. */
-	RWUNLOCK(&dlz_implock, isc_rwlocktype_read);
 	isc_mem_free(mctx, db->dlzname);
-	isc_mem_put(mctx, db, sizeof(dns_dlzdb_t));
+	isc_mem_put(mctx, db, sizeof(*db));
 	return result;
 }
 
@@ -241,7 +240,7 @@ dns_dlzdestroy(dns_dlzdb_t **dbp) {
 	dns_dlzdb_t *db;
 
 	/* Write debugging message to log */
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
 		      ISC_LOG_DEBUG(2), "Unloading DLZ driver.");
 
 	/*
@@ -263,7 +262,7 @@ dns_dlzdestroy(dns_dlzdb_t **dbp) {
 	destroy = db->implementation->methods->destroy;
 	(*destroy)(db->implementation->driverarg, db->dbdata);
 	/* return memory and detach */
-	isc_mem_putanddetach(&db->mctx, db, sizeof(dns_dlzdb_t));
+	isc_mem_putanddetach(&db->mctx, db, sizeof(*db));
 }
 
 /*%
@@ -277,7 +276,7 @@ dns_dlzregister(const char *drivername, const dns_dlzmethods_t *methods,
 	dns_dlzimplementation_t *dlz_imp;
 
 	/* Write debugging message to log */
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
 		      ISC_LOG_DEBUG(2), "Registering DLZ driver '%s'",
 		      drivername);
 
@@ -296,7 +295,7 @@ dns_dlzregister(const char *drivername, const dns_dlzmethods_t *methods,
 	 * initialize the dlz_implementations list, this is guaranteed
 	 * to only really happen once.
 	 */
-	RUNTIME_CHECK(isc_once_do(&once, dlz_initialize) == ISC_R_SUCCESS);
+	isc_once_do(&once, dlz_initialize);
 
 	/* lock the dlz_implementations list so we can modify it. */
 	RWLOCK(&dlz_implock, isc_rwlocktype_write);
@@ -307,8 +306,8 @@ dns_dlzregister(const char *drivername, const dns_dlzmethods_t *methods,
 	 */
 	dlz_imp = dlz_impfind(drivername);
 	if (dlz_imp != NULL) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DLZ, ISC_LOG_DEBUG(2),
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+			      ISC_LOG_DEBUG(2),
 			      "DLZ Driver '%s' already registered", drivername);
 		RWUNLOCK(&dlz_implock, isc_rwlocktype_write);
 		return ISC_R_EXISTS;
@@ -318,16 +317,12 @@ dns_dlzregister(const char *drivername, const dns_dlzmethods_t *methods,
 	 * Allocate memory for a dlz_implementation object.  Error if
 	 * we cannot.
 	 */
-	dlz_imp = isc_mem_get(mctx, sizeof(dns_dlzimplementation_t));
-
-	/* Make sure memory region is set to all 0's */
-	memset(dlz_imp, 0, sizeof(dns_dlzimplementation_t));
-
-	/* Store the data passed into this method */
-	dlz_imp->name = drivername;
-	dlz_imp->methods = methods;
-	dlz_imp->mctx = NULL;
-	dlz_imp->driverarg = driverarg;
+	dlz_imp = isc_mem_get(mctx, sizeof(*dlz_imp));
+	*dlz_imp = (dns_dlzimplementation_t){
+		.name = drivername,
+		.methods = methods,
+		.driverarg = driverarg,
+	};
 
 	/* attach the new dlz_implementation object to a memory context */
 	isc_mem_attach(mctx, &dlz_imp->mctx);
@@ -369,7 +364,7 @@ dns_dlzunregister(dns_dlzimplementation_t **dlzimp) {
 	dns_dlzimplementation_t *dlz_imp;
 
 	/* Write debugging message to log */
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
 		      ISC_LOG_DEBUG(2), "Unregistering DLZ driver.");
 
 	/*
@@ -381,7 +376,7 @@ dns_dlzunregister(dns_dlzimplementation_t **dlzimp) {
 	 * initialize the dlz_implementations list, this is guaranteed
 	 * to only really happen once.
 	 */
-	RUNTIME_CHECK(isc_once_do(&once, dlz_initialize) == ISC_R_SUCCESS);
+	isc_once_do(&once, dlz_initialize);
 
 	dlz_imp = *dlzimp;
 
@@ -435,8 +430,8 @@ dns_dlz_writeablezone(dns_view_t *view, dns_dlzdb_t *dlzdb,
 	origin = dns_fixedname_name(&fixorigin);
 
 	if (!dlzdb->search) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DLZ, ISC_LOG_WARNING,
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+			      ISC_LOG_WARNING,
 			      "DLZ %s has 'search no;', but attempted to "
 			      "register writeable zone %s.",
 			      dlzdb->dlzname, zone_name);
@@ -445,7 +440,7 @@ dns_dlz_writeablezone(dns_view_t *view, dns_dlzdb_t *dlzdb,
 	}
 
 	/* See if the zone already exists */
-	result = dns_view_findzone(view, origin, &dupzone);
+	result = dns_view_findzone(view, origin, DNS_ZTFIND_EXACT, &dupzone);
 	if (result == ISC_R_SUCCESS) {
 		dns_zone_detach(&dupzone);
 		result = ISC_R_EXISTS;
@@ -454,10 +449,7 @@ dns_dlz_writeablezone(dns_view_t *view, dns_dlzdb_t *dlzdb,
 	INSIST(dupzone == NULL);
 
 	/* Create it */
-	result = dns_zone_create(&zone, view->mctx);
-	if (result != ISC_R_SUCCESS) {
-		goto cleanup;
-	}
+	dns_zone_create(&zone, view->mctx, 0);
 	result = dns_zone_setorigin(zone, origin);
 	if (result != ISC_R_SUCCESS) {
 		goto cleanup;
@@ -525,8 +517,8 @@ dns_dlz_ssumatch(dns_dlzdb_t *dlzdatabase, const dns_name_t *signer,
 	impl = dlzdatabase->implementation;
 
 	if (impl->methods->ssumatch == NULL) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DLZ, ISC_LOG_INFO,
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DLZ,
+			      ISC_LOG_INFO,
 			      "No ssumatch method for DLZ database");
 		return false;
 	}
