@@ -20,6 +20,7 @@
 
 #include <isc/buffer.h>
 #include <isc/dir.h>
+#include <isc/log.h>
 #include <isc/mem.h>
 #include <isc/result.h>
 #include <isc/serial.h>
@@ -32,11 +33,8 @@
 #include <dns/fixedname.h>
 #include <dns/kasp.h>
 #include <dns/keyvalues.h>
-#include <dns/log.h>
-#include <dns/message.h>
 #include <dns/rdata.h>
 #include <dns/rdatalist.h>
-#include <dns/rdataset.h>
 #include <dns/rdatastruct.h>
 #include <dns/stats.h>
 #include <dns/tsig.h> /* for DNS_TSIG_FUDGE */
@@ -193,6 +191,7 @@ dns_dnssec_sign(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 	isc_result_t ret;
 	isc_buffer_t *databuf = NULL;
 	char data[256 + 8];
+	uint32_t flags;
 	unsigned int sigsize;
 	dns_fixedname_t fnewname;
 	dns_fixedname_t fsigner;
@@ -210,6 +209,17 @@ dns_dnssec_sign(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 		return DNS_R_INVALIDTIME;
 	}
 
+	/*
+	 * Is the key allowed to sign data?
+	 */
+	flags = dst_key_flags(key);
+	if ((flags & DNS_KEYTYPE_NOAUTH) != 0) {
+		return DNS_R_KEYUNAUTHORIZED;
+	}
+	if ((flags & DNS_KEYFLAG_OWNERMASK) != DNS_KEYOWNER_ZONE) {
+		return DNS_R_KEYUNAUTHORIZED;
+	}
+
 	sig.mctx = mctx;
 	sig.common.rdclass = set->rdclass;
 	sig.common.rdtype = dns_rdatatype_rrsig;
@@ -371,6 +381,7 @@ dns_dnssec_verify(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 	unsigned char data[300];
 	dst_context_t *ctx = NULL;
 	int labels = 0;
+	uint32_t flags;
 	bool downcase = false;
 
 	REQUIRE(name != NULL);
@@ -409,7 +420,7 @@ dns_dnssec_verify(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 	}
 
 	/*
-	 * NS, SOA and DNSKEY records are signed by their owner.
+	 * NS, SOA and DNSSKEY records are signed by their owner.
 	 * DS records are signed by the parent.
 	 */
 	switch (set->type) {
@@ -435,6 +446,19 @@ dns_dnssec_verify(const dns_name_t *name, dns_rdataset_t *set, dst_key_t *key,
 		break;
 	}
 
+	/*
+	 * Is the key allowed to sign data?
+	 */
+	flags = dst_key_flags(key);
+	if ((flags & DNS_KEYTYPE_NOAUTH) != 0) {
+		inc_stat(dns_dnssecstats_fail);
+		return DNS_R_KEYUNAUTHORIZED;
+	}
+	if ((flags & DNS_KEYFLAG_OWNERMASK) != DNS_KEYOWNER_ZONE) {
+		inc_stat(dns_dnssecstats_fail);
+		return DNS_R_KEYUNAUTHORIZED;
+	}
+
 again:
 	ret = dst_context_create(key, mctx, DNS_LOGCATEGORY_DNSSEC, false,
 				 maxbits, &ctx);
@@ -534,8 +558,8 @@ again:
 	if (ret == ISC_R_SUCCESS && downcase) {
 		char namebuf[DNS_NAME_FORMATSIZE];
 		dns_name_format(&sig.signer, namebuf, sizeof(namebuf));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
+		isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_DEBUG(1),
 			      "successfully validated after lower casing "
 			      "signer '%s'",
 			      namebuf);
@@ -1097,7 +1121,6 @@ dns_dnsseckey_create(isc_mem_t *mctx, dst_key_t **dstkey,
 	dk->hint_remove = false;
 	dk->first_sign = false;
 	dk->is_active = false;
-	dk->pubkey = false;
 	dk->purge = false;
 	dk->prepublish = 0;
 	dk->source = dns_keysource_unknown;
@@ -1280,7 +1303,7 @@ findmatchingkeys(const char *directory, char *namebuf, unsigned int len,
 		}
 
 		if (result != ISC_R_SUCCESS) {
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_WARNING,
 				      "dns_dnssec_findmatchingkeys: "
 				      "error reading key file %s: %s",
@@ -1386,7 +1409,7 @@ failure:
  */
 static void
 addkey(dns_dnsseckeylist_t *keylist, dst_key_t **newkey, bool savekeys,
-       bool pubkey_only, isc_mem_t *mctx) {
+       isc_mem_t *mctx) {
 	dns_dnsseckey_t *key = NULL;
 
 	/* Skip duplicates */
@@ -1404,35 +1427,28 @@ addkey(dns_dnsseckeylist_t *keylist, dst_key_t **newkey, bool savekeys,
 
 	if (key != NULL) {
 		/*
-		 * Found a match. If we already had a private key, then
-		 * the new key can't be an improvement. If the existing
-		 * key was public-only but the new key is too, then it's
-		 * still not an improvement. Mark the old key as having
-		 * been found in the zone and stop.
+		 * Found a match.  If the old key was only public and the
+		 * new key is private, replace the old one; otherwise
+		 * leave it.  But either way, mark the key as having
+		 * been found in the zone.
 		 */
-		if (dst_key_isprivate(key->key) || !dst_key_isprivate(*newkey))
-		{
-			key->source = dns_keysource_zoneapex;
-			return;
+		if (dst_key_isprivate(key->key)) {
+			dst_key_free(newkey);
+		} else if (dst_key_isprivate(*newkey)) {
+			dst_key_free(&key->key);
+			key->key = *newkey;
 		}
 
-		/*
-		 * However, if the old key was public-only, and the new key
-		 * is private, then we're throwing away the old key.
-		 */
-		dst_key_free(&key->key);
-		ISC_LIST_UNLINK(*keylist, key, link);
-		dns_dnsseckey_destroy(mctx, &key);
+		key->source = dns_keysource_zoneapex;
+		return;
 	}
 
-	/* Store the new key. */
 	dns_dnsseckey_create(mctx, newkey, &key);
-	key->source = dns_keysource_zoneapex;
-	key->pubkey = pubkey_only;
 	if (key->legacy || savekeys) {
 		key->force_publish = true;
 		key->force_sign = dst_key_isprivate(key->key);
 	}
+	key->source = dns_keysource_zoneapex;
 	ISC_LIST_APPEND(*keylist, key, link);
 	*newkey = NULL;
 }
@@ -1556,7 +1572,9 @@ dns_dnssec_keylistfromrdataset(const dns_name_t *origin, dns_kasp_t *kasp,
 		RETERR(dns_dnssec_keyfromrdata(origin, &rdata, mctx, &dnskey));
 		dst_key_setttl(dnskey, keys.ttl);
 
-		if (!is_zone_key(dnskey)) {
+		if (!is_zone_key(dnskey) ||
+		    (dst_key_flags(dnskey) & DNS_KEYTYPE_NOAUTH) != 0)
+		{
 			goto skip;
 		}
 
@@ -1566,13 +1584,13 @@ dns_dnssec_keylistfromrdataset(const dns_name_t *origin, dns_kasp_t *kasp,
 		}
 
 		if (publickey) {
-			addkey(keylist, &dnskey, savekeys, true, mctx);
+			addkey(keylist, &dnskey, savekeys, mctx);
 			goto skip;
 		}
 
 		/* Try to read the public key. */
 		result = keyfromfile(kasp, directory, dnskey,
-				     DST_TYPE_PUBLIC | DST_TYPE_STATE, mctx,
+				     (DST_TYPE_PUBLIC | DST_TYPE_STATE), mctx,
 				     &pubkey);
 		if (result == ISC_R_FILENOTFOUND || result == ISC_R_NOPERM) {
 			result = ISC_R_SUCCESS;
@@ -1587,10 +1605,10 @@ dns_dnssec_keylistfromrdataset(const dns_name_t *origin, dns_kasp_t *kasp,
 		}
 
 		/* Now read the private key. */
-		result = keyfromfile(kasp, directory, dnskey,
-				     DST_TYPE_PUBLIC | DST_TYPE_PRIVATE |
-					     DST_TYPE_STATE,
-				     mctx, &privkey);
+		result = keyfromfile(
+			kasp, directory, dnskey,
+			(DST_TYPE_PUBLIC | DST_TYPE_PRIVATE | DST_TYPE_STATE),
+			mctx, &privkey);
 
 		/*
 		 * If the key was revoked and the private file
@@ -1604,9 +1622,9 @@ dns_dnssec_keylistfromrdataset(const dns_name_t *origin, dns_kasp_t *kasp,
 				dst_key_setflags(dnskey,
 						 flags & ~DNS_KEYFLAG_REVOKE);
 				result = keyfromfile(kasp, directory, dnskey,
-						     DST_TYPE_PUBLIC |
-							     DST_TYPE_PRIVATE |
-							     DST_TYPE_STATE,
+						     (DST_TYPE_PUBLIC |
+						      DST_TYPE_PRIVATE |
+						      DST_TYPE_STATE),
 						     mctx, &privkey);
 				if (result == ISC_R_SUCCESS &&
 				    dst_key_pubcompare(dnskey, privkey, false))
@@ -1628,8 +1646,8 @@ dns_dnssec_keylistfromrdataset(const dns_name_t *origin, dns_kasp_t *kasp,
 			result2 = dst_key_getfilename(
 				dst_key_name(dnskey), dst_key_id(dnskey),
 				dst_key_alg(dnskey),
-				DST_TYPE_PUBLIC | DST_TYPE_PRIVATE |
-					DST_TYPE_STATE,
+				(DST_TYPE_PUBLIC | DST_TYPE_PRIVATE |
+				 DST_TYPE_STATE),
 				NULL, mctx, &buf);
 			if (result2 != ISC_R_SUCCESS) {
 				char namebuf[DNS_NAME_FORMATSIZE];
@@ -1644,7 +1662,7 @@ dns_dnssec_keylistfromrdataset(const dns_name_t *origin, dns_kasp_t *kasp,
 					 algbuf, dst_key_id(dnskey));
 			}
 
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_WARNING,
 				      "dns_dnssec_keylistfromrdataset: error "
 				      "reading %s: %s",
@@ -1654,21 +1672,26 @@ dns_dnssec_keylistfromrdataset(const dns_name_t *origin, dns_kasp_t *kasp,
 	addkey:
 		if (result == ISC_R_FILENOTFOUND || result == ISC_R_NOPERM) {
 			if (pubkey != NULL) {
-				addkey(keylist, &pubkey, savekeys, true, mctx);
+				addkey(keylist, &pubkey, savekeys, mctx);
 			} else {
-				addkey(keylist, &dnskey, savekeys, false, mctx);
+				addkey(keylist, &dnskey, savekeys, mctx);
 			}
 			goto skip;
 		}
 		RETERR(result);
 
+		/* This should never happen. */
+		if ((dst_key_flags(privkey) & DNS_KEYTYPE_NOAUTH) != 0) {
+			goto skip;
+		}
+
 		/*
 		 * Whatever the key's default TTL may have
 		 * been, the rdataset TTL takes priority.
 		 */
 		dst_key_setttl(privkey, dst_key_getttl(dnskey));
 
-		addkey(keylist, &privkey, savekeys, false, mctx);
+		addkey(keylist, &privkey, savekeys, mctx);
 	skip:
 		if (dnskey != NULL) {
 			dst_key_free(&dnskey);
@@ -1731,39 +1754,29 @@ dns_dnssec_make_dnskey(dst_key_t *key, unsigned char *buf, int bufsize,
 	return ISC_R_SUCCESS;
 }
 
-static isc_result_t
+static void
 addrdata(dns_rdata_t *rdata, dns_diff_t *diff, const dns_name_t *origin,
 	 dns_ttl_t ttl, isc_mem_t *mctx) {
-	isc_result_t result;
 	dns_difftuple_t *tuple = NULL;
 
-	RETERR(dns_difftuple_create(mctx, DNS_DIFFOP_ADD, origin, ttl, rdata,
-				    &tuple));
+	dns_difftuple_create(mctx, DNS_DIFFOP_ADD, origin, ttl, rdata, &tuple);
 	dns_diff_appendminimal(diff, &tuple);
-
-failure:
-	return result;
 }
 
-static isc_result_t
+static void
 delrdata(dns_rdata_t *rdata, dns_diff_t *diff, const dns_name_t *origin,
 	 dns_ttl_t ttl, isc_mem_t *mctx) {
-	isc_result_t result;
 	dns_difftuple_t *tuple = NULL;
 
-	RETERR(dns_difftuple_create(mctx, DNS_DIFFOP_DEL, origin, ttl, rdata,
-				    &tuple));
+	dns_difftuple_create(mctx, DNS_DIFFOP_DEL, origin, ttl, rdata, &tuple);
 	dns_diff_appendminimal(diff, &tuple);
-
-failure:
-	return result;
 }
 
 static isc_result_t
 publish_key(dns_diff_t *diff, dns_dnsseckey_t *key, const dns_name_t *origin,
 	    dns_ttl_t ttl, isc_mem_t *mctx,
 	    void (*report)(const char *, ...) ISC_FORMAT_PRINTF(1, 2)) {
-	isc_result_t result;
+	isc_result_t result = ISC_R_SUCCESS;
 	unsigned char buf[DST_KEY_MAXSIZE];
 	char keystr[DST_KEY_FORMATSIZE];
 	dns_rdata_t dnskey = DNS_RDATA_INIT;
@@ -1788,7 +1801,7 @@ publish_key(dns_diff_t *diff, dns_dnsseckey_t *key, const dns_name_t *origin,
 	}
 
 	/* publish key */
-	result = addrdata(&dnskey, diff, origin, ttl, mctx);
+	addrdata(&dnskey, diff, origin, ttl, mctx);
 
 failure:
 	return result;
@@ -1798,7 +1811,7 @@ static isc_result_t
 remove_key(dns_diff_t *diff, dns_dnsseckey_t *key, const dns_name_t *origin,
 	   dns_ttl_t ttl, isc_mem_t *mctx, const char *reason,
 	   void (*report)(const char *, ...) ISC_FORMAT_PRINTF(1, 2)) {
-	isc_result_t result;
+	isc_result_t result = ISC_R_SUCCESS;
 	unsigned char buf[DST_KEY_MAXSIZE];
 	dns_rdata_t dnskey = DNS_RDATA_INIT;
 	char alg[80];
@@ -1810,7 +1823,7 @@ remove_key(dns_diff_t *diff, dns_dnsseckey_t *key, const dns_name_t *origin,
 	       dst_key_id(key->key), alg);
 
 	RETERR(dns_dnssec_make_dnskey(key->key, buf, sizeof(buf), &dnskey));
-	result = delrdata(&dnskey, diff, origin, ttl, mctx);
+	delrdata(&dnskey, diff, origin, ttl, mctx);
 
 failure:
 	return result;
@@ -1842,7 +1855,7 @@ static isc_result_t
 add_cds(dns_dnsseckey_t *key, dns_rdata_t *keyrdata, const char *keystr,
 	dns_rdataset_t *cds, unsigned int digesttype, dns_ttl_t ttl,
 	dns_diff_t *diff, isc_mem_t *mctx) {
-	isc_result_t r = ISC_R_SUCCESS;
+	isc_result_t r;
 	unsigned char dsbuf[DNS_DS_BUFFERSIZE];
 	dns_rdata_t cdsrdata = DNS_RDATA_INIT;
 	dns_name_t *origin = dst_key_name(key->key);
@@ -1852,8 +1865,8 @@ add_cds(dns_dnsseckey_t *key, dns_rdata_t *keyrdata, const char *keystr,
 		char algbuf[DNS_DSDIGEST_FORMATSIZE];
 		dns_dsdigest_format(digesttype, algbuf,
 				    DNS_DSDIGEST_FORMATSIZE);
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_ERROR,
 			      "build rdata CDS (%s) for key %s failed", algbuf,
 			      keystr);
 		return r;
@@ -1864,20 +1877,20 @@ add_cds(dns_dnsseckey_t *key, dns_rdata_t *keyrdata, const char *keystr,
 		char algbuf[DNS_DSDIGEST_FORMATSIZE];
 		dns_dsdigest_format(digesttype, algbuf,
 				    DNS_DSDIGEST_FORMATSIZE);
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_INFO,
 			      "CDS (%s) for key %s is now published", algbuf,
 			      keystr);
-		r = addrdata(&cdsrdata, diff, origin, ttl, mctx);
+		addrdata(&cdsrdata, diff, origin, ttl, mctx);
 	}
-	return r;
+	return ISC_R_SUCCESS;
 }
 
 static isc_result_t
 delete_cds(dns_dnsseckey_t *key, dns_rdata_t *keyrdata, const char *keystr,
 	   dns_rdataset_t *cds, unsigned int digesttype, dns_diff_t *diff,
 	   isc_mem_t *mctx) {
-	isc_result_t r = ISC_R_SUCCESS;
+	isc_result_t r;
 	unsigned char dsbuf[DNS_DS_BUFFERSIZE];
 	dns_rdata_t cdsrdata = DNS_RDATA_INIT;
 	dns_name_t *origin = dst_key_name(key->key);
@@ -1892,13 +1905,13 @@ delete_cds(dns_dnsseckey_t *key, dns_rdata_t *keyrdata, const char *keystr,
 		char algbuf[DNS_DSDIGEST_FORMATSIZE];
 		dns_dsdigest_format(digesttype, algbuf,
 				    DNS_DSDIGEST_FORMATSIZE);
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_INFO,
 			      "CDS (%s) for key %s is now deleted", algbuf,
 			      keystr);
-		r = delrdata(&cdsrdata, diff, origin, cds->ttl, mctx);
+		delrdata(&cdsrdata, diff, origin, cds->ttl, mctx);
 	}
-	return r;
+	return ISC_R_SUCCESS;
 }
 
 isc_result_t
@@ -1953,12 +1966,12 @@ dns_dnssec_syncupdate(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *rmkeys,
 			     !exists(cdnskey, &cdnskeyrdata)))
 			{
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_GENERAL,
+					DNS_LOGCATEGORY_GENERAL,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 					"CDNSKEY for key %s is now published",
 					keystr);
-				RETERR(addrdata(&cdnskeyrdata, diff, origin,
-						cdnskeyttl, mctx));
+				addrdata(&cdnskeyrdata, diff, origin,
+					 cdnskeyttl, mctx);
 			}
 		}
 
@@ -1981,16 +1994,14 @@ dns_dnssec_syncupdate(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *rmkeys,
 
 			if (dns_rdataset_isassociated(cdnskey)) {
 				if (exists(cdnskey, &cdnskeyrdata)) {
-					isc_log_write(dns_lctx,
-						      DNS_LOGCATEGORY_GENERAL,
+					isc_log_write(DNS_LOGCATEGORY_GENERAL,
 						      DNS_LOGMODULE_DNSSEC,
 						      ISC_LOG_INFO,
 						      "CDNSKEY for key %s is "
 						      "now deleted",
 						      keystr);
-					RETERR(delrdata(&cdnskeyrdata, diff,
-							origin, cdnskey->ttl,
-							mctx));
+					delrdata(&cdnskeyrdata, diff, origin,
+						 cdnskey->ttl, mctx);
 				}
 			}
 		}
@@ -2029,12 +2040,12 @@ dns_dnssec_syncupdate(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *rmkeys,
 		if (dns_rdataset_isassociated(cdnskey)) {
 			if (exists(cdnskey, &cdnskeyrdata)) {
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_GENERAL,
+					DNS_LOGCATEGORY_GENERAL,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 					"CDNSKEY for key %s is now deleted",
 					keystr);
-				RETERR(delrdata(&cdnskeyrdata, diff, origin,
-						cdnskey->ttl, mctx));
+				delrdata(&cdnskeyrdata, diff, origin,
+					 cdnskey->ttl, mctx);
 			}
 		}
 	}
@@ -2056,7 +2067,6 @@ dns_dnssec_syncdelete(dns_rdataset_t *cds, dns_rdataset_t *cdnskey,
 	dns_rdata_t cds_delete = DNS_RDATA_INIT;
 	dns_rdata_t cdnskey_delete = DNS_RDATA_INIT;
 	isc_region_t r;
-	isc_result_t result;
 
 	r.base = keybuf;
 	r.length = sizeof(keybuf);
@@ -2073,23 +2083,22 @@ dns_dnssec_syncdelete(dns_rdataset_t *cds, dns_rdataset_t *cdnskey,
 		if (!dns_rdataset_isassociated(cds) ||
 		    !exists(cds, &cds_delete))
 		{
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 				      "CDS (DELETE) for zone %s is now "
 				      "published",
 				      namebuf);
-			RETERR(addrdata(&cds_delete, diff, origin, ttl, mctx));
+			addrdata(&cds_delete, diff, origin, ttl, mctx);
 		}
 	} else {
 		if (dns_rdataset_isassociated(cds) && exists(cds, &cds_delete))
 		{
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 				      "CDS (DELETE) for zone %s is now "
 				      "deleted",
 				      namebuf);
-			RETERR(delrdata(&cds_delete, diff, origin, cds->ttl,
-					mctx));
+			delrdata(&cds_delete, diff, origin, cds->ttl, mctx);
 		}
 	}
 
@@ -2097,32 +2106,28 @@ dns_dnssec_syncdelete(dns_rdataset_t *cds, dns_rdataset_t *cdnskey,
 		if (!dns_rdataset_isassociated(cdnskey) ||
 		    !exists(cdnskey, &cdnskey_delete))
 		{
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 				      "CDNSKEY (DELETE) for zone %s is now "
 				      "published",
 				      namebuf);
-			RETERR(addrdata(&cdnskey_delete, diff, origin, ttl,
-					mctx));
+			addrdata(&cdnskey_delete, diff, origin, ttl, mctx);
 		}
 	} else {
 		if (dns_rdataset_isassociated(cdnskey) &&
 		    exists(cdnskey, &cdnskey_delete))
 		{
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 				      "CDNSKEY (DELETE) for zone %s is now "
 				      "deleted",
 				      namebuf);
-			RETERR(delrdata(&cdnskey_delete, diff, origin,
-					cdnskey->ttl, mctx));
+			delrdata(&cdnskey_delete, diff, origin, cdnskey->ttl,
+				 mctx);
 		}
 	}
 
-	result = ISC_R_SUCCESS;
-
-failure:
-	return result;
+	return ISC_R_SUCCESS;
 }
 
 /*
@@ -2234,7 +2239,7 @@ dns_dnssec_updatekeys(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *newkeys,
 				RETERR(publish_key(diff, key1, origin, ttl,
 						   mctx, report));
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 					"DNSKEY %s (%s) is now published",
 					keystr1,
@@ -2243,7 +2248,6 @@ dns_dnssec_updatekeys(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *newkeys,
 				if (key1->hint_sign || key1->force_sign) {
 					key1->first_sign = true;
 					isc_log_write(
-						dns_lctx,
 						DNS_LOGCATEGORY_DNSSEC,
 						DNS_LOGMODULE_DNSSEC,
 						ISC_LOG_INFO,
@@ -2274,7 +2278,7 @@ dns_dnssec_updatekeys(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *newkeys,
 			if (removed != NULL) {
 				ISC_LIST_APPEND(*removed, key2, link);
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 					"DNSKEY %s (%s) is now deleted",
 					keystr2,
@@ -2297,7 +2301,7 @@ dns_dnssec_updatekeys(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *newkeys,
 			if (removed != NULL) {
 				ISC_LIST_APPEND(*removed, key2, link);
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 					"DNSKEY %s (%s) is now revoked; "
 					"new ID is %05d",
@@ -2330,7 +2334,7 @@ dns_dnssec_updatekeys(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *newkeys,
 			{
 				key2->first_sign = true;
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 					"DNSKEY %s (%s) is now active", keystr1,
 					key1->ksk ? (key1->zsk ? "CSK" : "KSK")
@@ -2339,7 +2343,7 @@ dns_dnssec_updatekeys(dns_dnsseckeylist_t *keys, dns_dnsseckeylist_t *newkeys,
 				   !key1->force_sign)
 			{
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 					"DNSKEY %s (%s) is now inactive",
 					keystr1,
