@@ -1,34 +1,30 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
  */
 
-#if HAVE_DLFCN_H
-#include <dlfcn.h>
-#elif _WIN32
-#include <windows.h>
-#endif /* if HAVE_DLFCN_H */
-
 #include <string.h>
 
 #include <isc/buffer.h>
+#include <isc/log.h>
 #include <isc/mem.h>
 #include <isc/mutex.h>
 #include <isc/once.h>
 #include <isc/region.h>
 #include <isc/result.h>
-#include <isc/task.h>
 #include <isc/types.h>
 #include <isc/util.h>
+#include <isc/uv.h>
 
 #include <dns/dyndb.h>
-#include <dns/log.h>
 #include <dns/types.h>
 #include <dns/view.h>
 #include <dns/zone.h>
@@ -43,7 +39,7 @@
 typedef struct dyndb_implementation dyndb_implementation_t;
 struct dyndb_implementation {
 	isc_mem_t *mctx;
-	void *handle;
+	uv_lib_t handle;
 	dns_dyndb_register_t *register_func;
 	dns_dyndb_destroy_t *destroy_func;
 	char *name;
@@ -77,237 +73,114 @@ impfind(const char *name) {
 	     imp = ISC_LIST_NEXT(imp, link))
 	{
 		if (strcasecmp(name, imp->name) == 0) {
-			return (imp);
+			return imp;
 		}
 	}
-	return (NULL);
+	return NULL;
 }
 
-#if HAVE_DLFCN_H && HAVE_DLOPEN
 static isc_result_t
-load_symbol(void *handle, const char *filename, const char *symbol_name,
+load_symbol(uv_lib_t *handle, const char *filename, const char *symbol_name,
 	    void **symbolp) {
-	const char *errmsg;
 	void *symbol;
+	int r;
 
 	REQUIRE(handle != NULL);
 	REQUIRE(symbolp != NULL && *symbolp == NULL);
 
-	symbol = dlsym(handle, symbol_name);
-	if (symbol == NULL) {
-		errmsg = dlerror();
+	r = uv_dlsym(handle, symbol_name, &symbol);
+	if (r != 0) {
+		const char *errmsg = uv_dlerror(handle);
 		if (errmsg == NULL) {
 			errmsg = "returned function pointer is NULL";
 		}
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DYNDB, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
+			      ISC_LOG_ERROR,
 			      "failed to lookup symbol %s in "
-			      "dyndb module '%s': %s",
+			      "DynDB module '%s': %s",
 			      symbol_name, filename, errmsg);
-		return (ISC_R_FAILURE);
+		return ISC_R_FAILURE;
 	}
-	dlerror();
 
 	*symbolp = symbol;
 
-	return (ISC_R_SUCCESS);
-}
-
-static isc_result_t
-load_library(isc_mem_t *mctx, const char *filename, const char *instname,
-	     dyndb_implementation_t **impp) {
-	isc_result_t result;
-	void *handle = NULL;
-	dyndb_implementation_t *imp = NULL;
-	dns_dyndb_register_t *register_func = NULL;
-	dns_dyndb_destroy_t *destroy_func = NULL;
-	dns_dyndb_version_t *version_func = NULL;
-	int version, flags;
-
-	REQUIRE(impp != NULL && *impp == NULL);
-
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
-		      ISC_LOG_INFO, "loading DynDB instance '%s' driver '%s'",
-		      instname, filename);
-
-	flags = RTLD_NOW | RTLD_LOCAL;
-#if defined(RTLD_DEEPBIND) && !__SANITIZE_ADDRESS__
-	flags |= RTLD_DEEPBIND;
-#endif /* if defined(RTLD_DEEPBIND) && !__SANITIZE_ADDRESS__ */
-
-	handle = dlopen(filename, flags);
-	if (handle == NULL) {
-		CHECK(ISC_R_FAILURE);
-	}
-
-	/* Clear dlerror */
-	dlerror();
-
-	CHECK(load_symbol(handle, filename, "dyndb_version",
-			  (void **)&version_func));
-
-	version = version_func(NULL);
-	if (version < (DNS_DYNDB_VERSION - DNS_DYNDB_AGE) ||
-	    version > DNS_DYNDB_VERSION)
-	{
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DYNDB, ISC_LOG_ERROR,
-			      "driver API version mismatch: %d/%d", version,
-			      DNS_DYNDB_VERSION);
-		CHECK(ISC_R_FAILURE);
-	}
-
-	CHECK(load_symbol(handle, filename, "dyndb_init",
-			  (void **)&register_func));
-	CHECK(load_symbol(handle, filename, "dyndb_destroy",
-			  (void **)&destroy_func));
-
-	imp = isc_mem_get(mctx, sizeof(dyndb_implementation_t));
-
-	imp->mctx = NULL;
-	isc_mem_attach(mctx, &imp->mctx);
-	imp->handle = handle;
-	imp->register_func = register_func;
-	imp->destroy_func = destroy_func;
-	imp->name = isc_mem_strdup(mctx, instname);
-
-	imp->inst = NULL;
-	INIT_LINK(imp, link);
-
-	*impp = imp;
-	imp = NULL;
-
-cleanup:
-	if (result != ISC_R_SUCCESS) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DYNDB, ISC_LOG_ERROR,
-			      "failed to dynamically load instance '%s' "
-			      "driver '%s': %s (%s)",
-			      instname, filename, dlerror(),
-			      isc_result_totext(result));
-	}
-	if (imp != NULL) {
-		isc_mem_putanddetach(&imp->mctx, imp,
-				     sizeof(dyndb_implementation_t));
-	}
-	if (result != ISC_R_SUCCESS && handle != NULL) {
-		dlclose(handle);
-	}
-
-	return (result);
+	return ISC_R_SUCCESS;
 }
 
 static void
-unload_library(dyndb_implementation_t **impp) {
-	dyndb_implementation_t *imp;
-
-	REQUIRE(impp != NULL && *impp != NULL);
-
-	imp = *impp;
-	*impp = NULL;
-
-	isc_mem_free(imp->mctx, imp->name);
-	isc_mem_putanddetach(&imp->mctx, imp, sizeof(dyndb_implementation_t));
-}
-#elif _WIN32
-static isc_result_t
-load_symbol(HMODULE handle, const char *filename, const char *symbol_name,
-	    void **symbolp) {
-	void *symbol;
-
-	REQUIRE(handle != NULL);
-	REQUIRE(symbolp != NULL && *symbolp == NULL);
-
-	symbol = GetProcAddress(handle, symbol_name);
-	if (symbol == NULL) {
-		int errstatus = GetLastError();
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DYNDB, ISC_LOG_ERROR,
-			      "failed to lookup symbol %s in "
-			      "dyndb module '%s': %d",
-			      symbol_name, filename, errstatus);
-		return (ISC_R_FAILURE);
-	}
-
-	*symbolp = symbol;
-
-	return (ISC_R_SUCCESS);
-}
+unload_library(dyndb_implementation_t **impp);
 
 static isc_result_t
 load_library(isc_mem_t *mctx, const char *filename, const char *instname,
 	     dyndb_implementation_t **impp) {
 	isc_result_t result;
-	HMODULE handle;
 	dyndb_implementation_t *imp = NULL;
-	dns_dyndb_register_t *register_func = NULL;
-	dns_dyndb_destroy_t *destroy_func = NULL;
 	dns_dyndb_version_t *version_func = NULL;
 	int version;
+	int r;
 
 	REQUIRE(impp != NULL && *impp == NULL);
 
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
 		      ISC_LOG_INFO, "loading DynDB instance '%s' driver '%s'",
 		      instname, filename);
 
-	handle = LoadLibraryA(filename);
-	if (handle == NULL) {
+	imp = isc_mem_get(mctx, sizeof(*imp));
+	*imp = (dyndb_implementation_t){
+		.name = isc_mem_strdup(mctx, instname),
+	};
+
+	isc_mem_attach(mctx, &imp->mctx);
+
+	INIT_LINK(imp, link);
+
+	r = uv_dlopen(filename, &imp->handle);
+	if (r != 0) {
+		const char *errmsg = uv_dlerror(&imp->handle);
+		if (errmsg == NULL) {
+			errmsg = "unknown error";
+		}
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
+			      ISC_LOG_ERROR,
+			      "failed to dlopen() DynDB instance '%s' driver "
+			      "'%s': %s",
+			      instname, filename, errmsg);
 		CHECK(ISC_R_FAILURE);
 	}
 
-	CHECK(load_symbol(handle, filename, "dyndb_version",
+	CHECK(load_symbol(&imp->handle, filename, "dyndb_version",
 			  (void **)&version_func));
 
 	version = version_func(NULL);
 	if (version < (DNS_DYNDB_VERSION - DNS_DYNDB_AGE) ||
 	    version > DNS_DYNDB_VERSION)
 	{
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DYNDB, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
+			      ISC_LOG_ERROR,
 			      "driver API version mismatch: %d/%d", version,
 			      DNS_DYNDB_VERSION);
 		CHECK(ISC_R_FAILURE);
 	}
 
-	CHECK(load_symbol(handle, filename, "dyndb_init",
-			  (void **)&register_func));
-	CHECK(load_symbol(handle, filename, "dyndb_destroy",
-			  (void **)&destroy_func));
-
-	imp = isc_mem_get(mctx, sizeof(dyndb_implementation_t));
-
-	imp->mctx = NULL;
-	isc_mem_attach(mctx, &imp->mctx);
-	imp->handle = handle;
-	imp->register_func = register_func;
-	imp->destroy_func = destroy_func;
-	imp->name = isc_mem_strdup(mctx, instname);
-
-	imp->inst = NULL;
-	INIT_LINK(imp, link);
+	CHECK(load_symbol(&imp->handle, filename, "dyndb_init",
+			  (void **)&imp->register_func));
+	CHECK(load_symbol(&imp->handle, filename, "dyndb_destroy",
+			  (void **)&imp->destroy_func));
 
 	*impp = imp;
-	imp = NULL;
+
+	return ISC_R_SUCCESS;
 
 cleanup:
-	if (result != ISC_R_SUCCESS) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DYNDB, ISC_LOG_ERROR,
-			      "failed to dynamically load instance '%s' "
-			      "driver '%s': %d (%s)",
-			      instname, filename, GetLastError(),
-			      isc_result_totext(result));
-	}
-	if (imp != NULL) {
-		isc_mem_putanddetach(&imp->mctx, imp,
-				     sizeof(dyndb_implementation_t));
-	}
-	if (result != ISC_R_SUCCESS && handle != NULL) {
-		FreeLibrary(handle);
-	}
+	isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
+		      ISC_LOG_ERROR,
+		      "failed to dynamically load DynDB instance '%s' driver "
+		      "'%s': %s",
+		      instname, filename, isc_result_totext(result));
 
-	return (result);
+	unload_library(&imp);
+
+	return result;
 }
 
 static void
@@ -319,30 +192,14 @@ unload_library(dyndb_implementation_t **impp) {
 	imp = *impp;
 	*impp = NULL;
 
+	/*
+	 * This is a resource leak, but there is nothing we can currently do
+	 * about it due to how configuration loading/reloading is designed.
+	 */
+	/* uv_dlclose(&imp->handle); */
 	isc_mem_free(imp->mctx, imp->name);
-	isc_mem_putanddetach(&imp->mctx, imp, sizeof(dyndb_implementation_t));
+	isc_mem_putanddetach(&imp->mctx, imp, sizeof(*imp));
 }
-#else  /* HAVE_DLFCN_H || _WIN32 */
-static isc_result_t
-load_library(isc_mem_t *mctx, const char *filename, const char *instname,
-	     dyndb_implementation_t **impp) {
-	UNUSED(mctx);
-	UNUSED(filename);
-	UNUSED(instname);
-	UNUSED(impp);
-
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
-		      ISC_LOG_ERROR,
-		      "dynamic database support is not implemented");
-
-	return (ISC_R_NOTIMPLEMENTED);
-}
-
-static void
-unload_library(dyndb_implementation_t **impp) {
-	UNUSED(impp);
-}
-#endif /* HAVE_DLFCN_H */
 
 isc_result_t
 dns_dyndb_load(const char *libname, const char *name, const char *parameters,
@@ -354,7 +211,7 @@ dns_dyndb_load(const char *libname, const char *name, const char *parameters,
 	REQUIRE(DNS_DYNDBCTX_VALID(dctx));
 	REQUIRE(name != NULL);
 
-	RUNTIME_CHECK(isc_once_do(&once, dyndb_initialize) == ISC_R_SUCCESS);
+	isc_once_do(&once, dyndb_initialize);
 
 	LOCK(&dyndb_lock);
 
@@ -378,7 +235,7 @@ cleanup:
 	}
 
 	UNLOCK(&dyndb_lock);
-	return (result);
+	return result;
 }
 
 void
@@ -386,16 +243,16 @@ dns_dyndb_cleanup(bool exiting) {
 	dyndb_implementation_t *elem;
 	dyndb_implementation_t *prev;
 
-	RUNTIME_CHECK(isc_once_do(&once, dyndb_initialize) == ISC_R_SUCCESS);
+	isc_once_do(&once, dyndb_initialize);
 
 	LOCK(&dyndb_lock);
 	elem = TAIL(dyndb_implementations);
 	while (elem != NULL) {
 		prev = PREV(elem, link);
 		UNLINK(dyndb_implementations, elem, link);
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DATABASE,
-			      DNS_LOGMODULE_DYNDB, ISC_LOG_INFO,
-			      "unloading DynDB instance '%s'", elem->name);
+		isc_log_write(DNS_LOGCATEGORY_DATABASE, DNS_LOGMODULE_DYNDB,
+			      ISC_LOG_INFO, "unloading DynDB instance '%s'",
+			      elem->name);
 		elem->destroy_func(&elem->inst);
 		ENSURE(elem->inst == NULL);
 		unload_library(&elem);
@@ -403,42 +260,38 @@ dns_dyndb_cleanup(bool exiting) {
 	}
 	UNLOCK(&dyndb_lock);
 
-	if (exiting == true) {
+	if (exiting) {
 		isc_mutex_destroy(&dyndb_lock);
 	}
 }
 
 isc_result_t
-dns_dyndb_createctx(isc_mem_t *mctx, const void *hashinit, isc_log_t *lctx,
-		    dns_view_t *view, dns_zonemgr_t *zmgr, isc_task_t *task,
-		    isc_timermgr_t *tmgr, dns_dyndbctx_t **dctxp) {
+dns_dyndb_createctx(isc_mem_t *mctx, const void *hashinit, dns_view_t *view,
+		    dns_zonemgr_t *zmgr, isc_loopmgr_t *loopmgr,
+		    dns_dyndbctx_t **dctxp) {
 	dns_dyndbctx_t *dctx;
 
 	REQUIRE(dctxp != NULL && *dctxp == NULL);
 
 	dctx = isc_mem_get(mctx, sizeof(*dctx));
+	*dctx = (dns_dyndbctx_t){
+		.loopmgr = loopmgr,
+		.hashinit = hashinit,
+	};
 
-	memset(dctx, 0, sizeof(*dctx));
 	if (view != NULL) {
 		dns_view_attach(view, &dctx->view);
 	}
 	if (zmgr != NULL) {
 		dns_zonemgr_attach(zmgr, &dctx->zmgr);
 	}
-	if (task != NULL) {
-		isc_task_attach(task, &dctx->task);
-	}
-	dctx->timermgr = tmgr;
-	dctx->hashinit = hashinit;
-	dctx->lctx = lctx;
-	dctx->refvar = &isc_bind9;
 
 	isc_mem_attach(mctx, &dctx->mctx);
 	dctx->magic = DNS_DYNDBCTX_MAGIC;
 
 	*dctxp = dctx;
 
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 }
 
 void
@@ -458,11 +311,7 @@ dns_dyndb_destroyctx(dns_dyndbctx_t **dctxp) {
 	if (dctx->zmgr != NULL) {
 		dns_zonemgr_detach(&dctx->zmgr);
 	}
-	if (dctx->task != NULL) {
-		isc_task_detach(&dctx->task);
-	}
-	dctx->timermgr = NULL;
-	dctx->lctx = NULL;
+	dctx->loopmgr = NULL;
 
 	isc_mem_putanddetach(&dctx->mctx, dctx, sizeof(*dctx));
 }
