@@ -40,6 +40,7 @@
 #include <isc/dir.h>
 #include <isc/file.h>
 #include <isc/lex.h>
+#include <isc/log.h>
 #include <isc/mem.h>
 #include <isc/once.h>
 #include <isc/random.h>
@@ -50,8 +51,6 @@
 
 #include <dns/fixedname.h>
 #include <dns/keyvalues.h>
-#include <dns/log.h>
-#include <dns/name.h>
 #include <dns/rdata.h>
 #include <dns/rdataclass.h>
 #include <dns/types.h>
@@ -92,6 +91,13 @@ static gss_OID_desc __gss_spnego_mechanism_oid_desc = {
 		(r).base = (gb).value;                  \
 	} while (0)
 
+#define RETERR(x)                            \
+	do {                                 \
+		result = (x);                \
+		if (result != ISC_R_SUCCESS) \
+			goto out;            \
+	} while (0)
+
 static void
 name_to_gbuffer(const dns_name_t *name, isc_buffer_t *buffer,
 		gss_buffer_desc *gbuffer) {
@@ -585,7 +591,8 @@ dst_gssapi_initctx(const dns_name_t *name, isc_buffer_t *intoken,
 	gret = gss_import_name(&minor, &gnamebuf, GSS_C_NO_OID, &gname);
 	if (gret != GSS_S_COMPLETE) {
 		gss_err_message(mctx, gret, minor, err_message);
-		CHECK(ISC_R_FAILURE);
+		result = ISC_R_FAILURE;
+		goto out;
 	}
 
 	if (intoken != NULL) {
@@ -616,7 +623,8 @@ dst_gssapi_initctx(const dns_name_t *name, isc_buffer_t *intoken,
 			gss_log(3, "Failure initiating security context");
 		}
 
-		CHECK(ISC_R_FAILURE);
+		result = ISC_R_FAILURE;
+		goto out;
 	}
 
 	/*
@@ -629,7 +637,7 @@ dst_gssapi_initctx(const dns_name_t *name, isc_buffer_t *intoken,
 	 */
 	if (gouttoken.length != 0U) {
 		GBUFFER_TO_REGION(gouttoken, r);
-		CHECK(isc_buffer_copyregion(outtoken, &r));
+		RETERR(isc_buffer_copyregion(outtoken, &r));
 	}
 
 	if (gret == GSS_S_COMPLETE) {
@@ -638,7 +646,7 @@ dst_gssapi_initctx(const dns_name_t *name, isc_buffer_t *intoken,
 		result = DNS_R_CONTINUE;
 	}
 
-cleanup:
+out:
 	if (gouttoken.length != 0U) {
 		(void)gss_release_buffer(&minor, &gouttoken);
 	}
@@ -743,7 +751,7 @@ dst_gssapi_acceptctx(dns_gss_cred_id_t cred, const char *gssapi_keytab,
 		isc_buffer_allocate(mctx, outtoken,
 				    (unsigned int)gouttoken.length);
 		GBUFFER_TO_REGION(gouttoken, r);
-		CHECK(isc_buffer_copyregion(*outtoken, &r));
+		RETERR(isc_buffer_copyregion(*outtoken, &r));
 		(void)gss_release_buffer(&minor, &gouttoken);
 	}
 
@@ -753,7 +761,7 @@ dst_gssapi_acceptctx(dns_gss_cred_id_t cred, const char *gssapi_keytab,
 			gss_log(3, "failed gss_display_name: %s",
 				gss_error_tostring(gret, minor, buf,
 						   sizeof(buf)));
-			CHECK(ISC_R_FAILURE);
+			RETERR(ISC_R_FAILURE);
 		}
 
 		/*
@@ -775,8 +783,8 @@ dst_gssapi_acceptctx(dns_gss_cred_id_t cred, const char *gssapi_keytab,
 		isc_buffer_init(&namebuf, r.base, r.length);
 		isc_buffer_add(&namebuf, r.length);
 
-		CHECK(dns_name_fromtext(principal, &namebuf, dns_rootname, 0,
-					NULL));
+		RETERR(dns_name_fromtext(principal, &namebuf, dns_rootname, 0,
+					 NULL));
 
 		if (gnamebuf.length != 0U) {
 			gret = gss_release_buffer(&minor, &gnamebuf);
@@ -792,7 +800,7 @@ dst_gssapi_acceptctx(dns_gss_cred_id_t cred, const char *gssapi_keytab,
 
 	*ctxout = context;
 
-cleanup:
+out:
 	if (gname != NULL) {
 		gret = gss_release_name(&minor, &gname);
 		if (gret != GSS_S_COMPLETE) {
@@ -950,7 +958,7 @@ gss_log(int level, const char *fmt, ...) {
 	va_list ap;
 
 	va_start(ap, fmt);
-	isc_log_vwrite(dns_lctx, DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_TKEY,
+	isc_log_vwrite(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_TKEY,
 		       ISC_LOG_DEBUG(level), fmt, ap);
 	va_end(ap);
 }
