@@ -94,16 +94,15 @@ typedef enum {
 					      */
 	DNS_ZONEOPT_NOTIFYTOSOA = 1 << 21,   /*%< Notify the SOA MNAME */
 	DNS_ZONEOPT_NSEC3TESTZONE = 1 << 22, /*%< nsec3-test-zone */
-	/* DNS_ZONEOPT_SECURETOINSECURE = 1 << 23, */
-	DNS_ZONEOPT_DNSKEYKSKONLY = 1 << 24,  /*%< dnssec-dnskey-kskonly */
-	DNS_ZONEOPT_CHECKDUPRR = 1 << 25,     /*%< check-dup-records */
+	DNS_ZONEOPT_LOGREPORTS = 1 << 23,    /* Log error-reporting queries */
+	DNS_ZONEOPT_DNSKEYKSKONLY = 1 << 24, /*%< dnssec-dnskey-kskonly */
+	DNS_ZONEOPT_CHECKDUPRR = 1 << 25,    /*%< check-dup-records */
 	DNS_ZONEOPT_CHECKDUPRRFAIL = 1 << 26, /*%< fatal check-dup-records
 					       * failures */
 	DNS_ZONEOPT_CHECKSPF = 1 << 27,	      /*%< check SPF records */
 	DNS_ZONEOPT_CHECKTTL = 1 << 28,	      /*%< check max-zone-ttl */
 	DNS_ZONEOPT_AUTOEMPTY = 1 << 29,      /*%< automatic empty zone */
 	DNS_ZONEOPT_CHECKSVCB = 1 << 30,      /*%< check SVBC records */
-	DNS_ZONEOPT_FORCEKEYMGR = 1ULL << 31, /*%< force keymgr step */
 	DNS_ZONEOPT___MAX = UINT64_MAX, /* trick to make the ENUM 64-bit wide */
 } dns_zoneopt_t;
 
@@ -850,7 +849,7 @@ dns_zone_setmaxretrytime(dns_zone_t *zone, uint32_t val);
  *	val > 0.
  */
 
-void
+isc_result_t
 dns_zone_setxfrsource4(dns_zone_t *zone, const isc_sockaddr_t *xfrsource);
 /*%<
  * 	Set the source address to be used in IPv4 zone transfers.
@@ -858,20 +857,22 @@ dns_zone_setxfrsource4(dns_zone_t *zone, const isc_sockaddr_t *xfrsource);
  * Require:
  *\li	'zone' to be a valid zone.
  *\li	'xfrsource' to contain the address.
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS
  */
 
-void
-dns_zone_getxfrsource4(dns_zone_t *zone, isc_sockaddr_t *xfrsource);
+isc_sockaddr_t *
+dns_zone_getxfrsource4(dns_zone_t *zone);
 /*%<
  *	Returns the source address set by a previous dns_zone_setxfrsource4
  *	call, or the default of inaddr_any, port 0.
  *
  * Require:
  *\li	'zone' to be a valid zone.
- *\li	'xfrsource' to not be NULL
  */
 
-void
+isc_result_t
 dns_zone_setxfrsource6(dns_zone_t *zone, const isc_sockaddr_t *xfrsource);
 /*%<
  * 	Set the source address to be used in IPv6 zone transfers.
@@ -879,20 +880,22 @@ dns_zone_setxfrsource6(dns_zone_t *zone, const isc_sockaddr_t *xfrsource);
  * Require:
  *\li	'zone' to be a valid zone.
  *\li	'xfrsource' to contain the address.
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS
  */
 
-void
-dns_zone_getxfrsource6(dns_zone_t *zone, isc_sockaddr_t *xfrsource);
+isc_sockaddr_t *
+dns_zone_getxfrsource6(dns_zone_t *zone);
 /*%<
  *	Returns the source address set by a previous dns_zone_setxfrsource6
  *	call, or the default of in6addr_any, port 0.
  *
  * Require:
  *\li	'zone' to be a valid zone.
- *\li	'xfrsource' to not be NULL
  */
 
-void
+isc_result_t
 dns_zone_setparentalsrc4(dns_zone_t *zone, const isc_sockaddr_t *parentalsrc);
 /*%<
  * 	Set the source address to be used with IPv4 parental DS queries.
@@ -900,20 +903,22 @@ dns_zone_setparentalsrc4(dns_zone_t *zone, const isc_sockaddr_t *parentalsrc);
  * Require:
  *\li	'zone' to be a valid zone.
  *\li	'parentalsrc' to contain the address.
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS
  */
 
-void
-dns_zone_getparentalsrc4(dns_zone_t *zone, isc_sockaddr_t *parentalsrc);
+isc_sockaddr_t *
+dns_zone_getparentalsrc4(dns_zone_t *zone);
 /*%<
  *	Returns the source address set by a previous dns_zone_setparentalsrc4
  *	call, or the default of inaddr_any, port 0.
  *
  * Require:
  *\li	'zone' to be a valid zone.
- *\li	'parentalsrc' to be non NULL.
  */
 
-void
+isc_result_t
 dns_zone_setparentalsrc6(dns_zone_t *zone, const isc_sockaddr_t *parentalsrc);
 /*%<
  * 	Set the source address to be used with IPv6 parental DS queries.
@@ -921,20 +926,22 @@ dns_zone_setparentalsrc6(dns_zone_t *zone, const isc_sockaddr_t *parentalsrc);
  * Require:
  *\li	'zone' to be a valid zone.
  *\li	'parentalsrc' to contain the address.
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS
  */
 
-void
-dns_zone_getparentalsrc6(dns_zone_t *zone, isc_sockaddr_t *parentalsrc);
+isc_sockaddr_t *
+dns_zone_getparentalsrc6(dns_zone_t *zone);
 /*%<
  *	Returns the source address set by a previous dns_zone_setparentalsrc6
  *	call, or the default of in6addr_any, port 0.
  *
  * Require:
  *\li	'zone' to be a valid zone.
- *\li	'parentalsrc' to be non NULL.
  */
 
-void
+isc_result_t
 dns_zone_setnotifysrc4(dns_zone_t *zone, const isc_sockaddr_t *notifysrc);
 /*%<
  * 	Set the source address to be used with IPv4 NOTIFY messages.
@@ -942,20 +949,22 @@ dns_zone_setnotifysrc4(dns_zone_t *zone, const isc_sockaddr_t *notifysrc);
  * Require:
  *\li	'zone' to be a valid zone.
  *\li	'notifysrc' to contain the address.
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS
  */
 
-void
-dns_zone_getnotifysrc4(dns_zone_t *zone, isc_sockaddr_t *notifysrc);
+isc_sockaddr_t *
+dns_zone_getnotifysrc4(dns_zone_t *zone);
 /*%<
  *	Returns the source address set by a previous dns_zone_setnotifysrc4
  *	call, or the default of inaddr_any, port 0.
  *
  * Require:
  *\li	'zone' to be a valid zone.
- *\li	'notifysrc' to be non NULL.
  */
 
-void
+isc_result_t
 dns_zone_setnotifysrc6(dns_zone_t *zone, const isc_sockaddr_t *notifysrc);
 /*%<
  * 	Set the source address to be used with IPv6 NOTIFY messages.
@@ -963,17 +972,19 @@ dns_zone_setnotifysrc6(dns_zone_t *zone, const isc_sockaddr_t *notifysrc);
  * Require:
  *\li	'zone' to be a valid zone.
  *\li	'notifysrc' to contain the address.
+ *
+ * Returns:
+ *\li	#ISC_R_SUCCESS
  */
 
-void
-dns_zone_getnotifysrc6(dns_zone_t *zone, isc_sockaddr_t *notifysrc);
+isc_sockaddr_t *
+dns_zone_getnotifysrc6(dns_zone_t *zone);
 /*%<
  *	Returns the source address set by a previous dns_zone_setnotifysrc6
  *	call, or the default of in6addr_any, port 0.
  *
  * Require:
  *\li	'zone' to be a valid zone.
- *\li	'notifysrc' to be non NULL.
  */
 
 void
@@ -1253,36 +1264,6 @@ dns_zone_notifyreceive(dns_zone_t *zone, isc_sockaddr_t *from,
  *\li	DNS_R_SUCCESS
  */
 
-void
-dns_zone_setminxfrratein(dns_zone_t *zone, uint32_t bytes, uint32_t seconds);
-/*%<
- * Set the minumum traffic rate (in bytes per seconds) that a zone transfer in
- * (AXFR/IXFR) of this zone will use before being aborted.
- *
- * Requires:
- * \li	'zone' to be valid initialised zone.
- */
-
-uint32_t
-dns_zone_getminxfrratebytesin(dns_zone_t *zone);
-/*%<
- * Returns the 'bytes' portion of the minimum traffic rate for the transfer in
- * for this zone.
- *
- * Requires:
- *\li	'zone' to be valid initialised zone.
- */
-
-uint32_t
-dns_zone_getminxfrratesecondsin(dns_zone_t *zone);
-/*%<
- * Returns the 'seconds' portion of the minimum traffic rate for the transfer in
- * for this zone.
- *
- * Requires:
- *\li	'zone' to be valid initialised zone.
- */
-
 void
 dns_zone_setmaxxfrin(dns_zone_t *zone, uint32_t maxxfrin);
 /*%<
@@ -1376,10 +1357,9 @@ dns_zone_getredirecttype(dns_zone_t *zone);
  */
 
 void
-dns_zone_notify(dns_zone_t *zone, bool nodefer);
+dns_zone_notify(dns_zone_t *zone);
 /*%<
- * Generate notify events for this zone. If 'nodefer' is true, the
- * 'notify-defer' configuration option is ingored.
+ * Generate notify events for this zone.
  *
  * Requires:
  *\li	'zone' to be a valid zone.
@@ -1553,8 +1533,8 @@ dns_zone_getsigresigninginterval(dns_zone_t *zone);
  * \li	'zone' to be a valid zone.
  */
 
-void
-dns_zone_getsourceaddr(dns_zone_t *zone, isc_sockaddr_t *sourceaddr);
+isc_sockaddr_t
+dns_zone_getsourceaddr(dns_zone_t *zone);
 /*%<
  * Get the zone's source address from which it has last contacted the current
  * primary server.
@@ -1562,22 +1542,16 @@ dns_zone_getsourceaddr(dns_zone_t *zone, isc_sockaddr_t *sourceaddr);
  * Requires:
  * \li	'zone' to be a valid zone.
  * \li	'zone' has a non-empty primaries list.
- * \li	'sourceaddr' to be non-NULL.
  */
 
-isc_result_t
-dns_zone_getprimaryaddr(dns_zone_t *zone, isc_sockaddr_t *primaryaddr);
+isc_sockaddr_t
+dns_zone_getprimaryaddr(dns_zone_t *zone);
 /*%<
- * Get the zone's current primary server into '*primaryaddr'.
+ * Get the zone's current primary server.
  *
  * Requires:
  * \li	'zone' to be a valid zone.
  * \li	'zone' has a non-empty primaries list.
- * \li	'primaryaddr' to be non-NULL.
- *
- * Returns:
- *\li	#ISC_R_SUCCESS if the current primary server was found
- *\li	#ISC_R_NOMORE if all the primaries were already iterated over
  */
 
 isc_time_t
@@ -1758,20 +1732,6 @@ dns_zone_findkeys(dns_zone_t *zone, dns_db_t *db, dns_dbversion_t *ver,
  *\li	Error
  */
 
-void
-dns_zone_prepare_shutdown(dns_zone_t *zone);
-/*%<
- * Prepare a zone for shutdown by setting the DNS_ZONEFLG_EXITING flag even
- * before the final reference is detached. Useful, because the zone object can
- * be kept around with a valid reference from the zonetable until qp garbage
- * collector runs, and we don't want, for example, zone maintenance to happen
- * while waiting for it. Note that the zone can not be used normally again after
- * this function is called.
- *
- * Requires:
- *\li	'zone' to be a valid initialised zone.
- */
-
 void
 dns_zonemgr_create(isc_mem_t *mctx, isc_nm_t *netmgr, dns_zonemgr_t **zmgrp);
 /*%<
@@ -1810,12 +1770,6 @@ dns_zonemgr_forcemaint(dns_zonemgr_t *zmgr);
  * to take place at the system's earliest convenience.
  */
 
-void
-dns_zonemgr_resumexfrs(dns_zonemgr_t *zmgr);
-/*%<
- * Attempt to start any stalled zone transfers.
- */
-
 void
 dns_zonemgr_shutdown(dns_zonemgr_t *zmgr);
 /*%<
@@ -2142,23 +2096,7 @@ dns_zone_getdnssecsignstats(dns_zone_t *zone);
  */
 
 void
-dns_zone_dialup(dns_zone_t *zone);
-/*%<
- * Perform dialup-time maintenance on 'zone'.
- */
-
-void
-dns_zone_setdialup(dns_zone_t *zone, dns_dialuptype_t dialup);
-/*%<
- * Set the dialup type of 'zone' to 'dialup'.
- *
- * Requires:
- * \li	'zone' to be valid initialised zone.
- *\li	'dialup' to be a valid dialup type.
- */
-
-void
-dns_zone_logv(dns_zone_t *zone, isc_logcategory_t *category, int level,
+dns_zone_logv(dns_zone_t *zone, isc_logcategory_t category, int level,
 	      const char *prefix, const char *msg, va_list ap);
 /*%<
  * Log the message 'msg...' at 'level' using log category 'category', including
@@ -2175,7 +2113,7 @@ dns_zone_log(dns_zone_t *zone, int level, const char *msg, ...)
  */
 
 void
-dns_zone_logc(dns_zone_t *zone, isc_logcategory_t *category, int level,
+dns_zone_logc(dns_zone_t *zone, isc_logcategory_t category, int level,
 	      const char *msg, ...) ISC_FORMAT_PRINTF(4, 5);
 /*%<
  * Log the message 'msg...' at 'level', including text that identifies
@@ -2250,19 +2188,18 @@ dns_zone_setcheckns(dns_zone_t *zone, dns_checknsfunc_t checkns);
  */
 
 void
-dns_zone_setnotifydefer(dns_zone_t *zone, uint32_t defer);
+dns_zone_setnotifydelay(dns_zone_t *zone, uint32_t delay);
 /*%<
- * Set the wait/defer time (in seconds) before notify messages are sent when
- * they are ready.
+ * Set the minimum delay between sets of notify messages.
  *
  * Requires:
  *	'zone' to be valid.
  */
 
-void
-dns_zone_setnotifydelay(dns_zone_t *zone, uint32_t delay);
+uint32_t
+dns_zone_getnotifydelay(dns_zone_t *zone);
 /*%<
- * Set the minimum delay (in seconds) between sets of notify messages.
+ * Get the minimum delay between sets of notify messages.
  *
  * Requires:
  *	'zone' to be valid.
@@ -2301,6 +2238,14 @@ dns_zone_getsignatures(dns_zone_t *zone);
  * Get the number of signatures that will be generated per quantum.
  */
 
+isc_result_t
+dns_zone_signwithkey(dns_zone_t *zone, dns_secalg_t algorithm, uint16_t keyid,
+		     bool deleteit);
+/*%<
+ * Initiate/resume signing of the entire zone with the zone DNSKEY(s)
+ * that match the given algorithm and keyid.
+ */
+
 isc_result_t
 dns_zone_addnsec3chain(dns_zone_t *zone, dns_rdata_nsec3param_t *nsec3param);
 /*%<
@@ -2317,7 +2262,7 @@ dns_zone_getprivatetype(dns_zone_t *zone);
  */
 
 void
-dns_zone_rekey(dns_zone_t *zone, bool fullsign, bool forcekeymgr);
+dns_zone_rekey(dns_zone_t *zone, bool fullsign);
 /*%<
  * Update the zone's DNSKEY set from the key repository.
  *
@@ -2325,9 +2270,6 @@ dns_zone_rekey(dns_zone_t *zone, bool fullsign, bool forcekeymgr);
  * the zone with the new key.  Otherwise, if there are no keys or
  * if the new keys are for algorithms that have already signed the
  * zone, then the zone can be re-signed incrementally.
- *
- * If 'forcekeymgr' is true, trigger a rekey event and allow the
- * next steps in the run to happen.
  */
 
 isc_result_t
@@ -2478,6 +2420,25 @@ dns_zone_setrequestixfr(dns_zone_t *zone, bool flag);
  * \li	'zone' to be valid.
  */
 
+bool
+dns_zone_getrequestixfrmaxdiffs(dns_zone_t *zone);
+/*%
+ * Returns the value of the request-ixfr-max-diffs option in the zone.
+ *
+ * Requires:
+ * \li	'zone' to be valid.
+ */
+
+void
+dns_zone_setrequestixfrmaxdiffs(dns_zone_t *zone, uint32_t maxmsgs);
+/*%
+ * Sets the request-ixfr-max-diffs option for the zone. 0 means unlimited. The
+ * default value is determined by the setting of this option in the view.
+ *
+ * Requires:
+ * \li	'zone' to be valid.
+ */
+
 uint32_t
 dns_zone_getixfrratio(dns_zone_t *zone);
 /*%
@@ -2809,6 +2770,26 @@ dns_zone_import_skr(dns_zone_t *zone, const char *file);
  * \li  ISC_R_SUCCESS if there were no errors loading the SKR.
  */
 
+void
+dns_zone_setrad(dns_zone_t *zone, dns_name_t *name);
+/**<
+ * \brief Set the per zone RAD
+ *
+ * Requires:
+ * \li	'zone' to be a valid zone.
+ * \li	'name' is NULL or a valid name.
+ */
+
+isc_result_t
+dns_zone_getrad(dns_zone_t *zone, dns_name_t *name);
+/**<
+ * \brief get the per zone RAD
+ *
+ * Requires:
+ * \li	'zone' to be a valid zone.
+ * \li	'name' is a valid name with a buffer.
+ */
+
 #if DNS_ZONE_TRACE
 #define dns_zone_ref(ptr)   dns_zone__ref(ptr, __func__, __FILE__, __LINE__)
 #define dns_zone_unref(ptr) dns_zone__unref(ptr, __func__, __FILE__, __LINE__)
