@@ -19,11 +19,11 @@
 #include <stdbool.h>
 
 #include <isc/lang.h>
+#include <isc/log.h>
 #include <isc/stdtime.h>
 
 #include <dns/ds.h>
 #include <dns/dsdigest.h>
-#include <dns/log.h>
 #include <dns/name.h>
 #include <dns/secalg.h>
 #include <dns/types.h>
@@ -37,9 +37,10 @@ ISC_LANG_BEGINDECLS
  ***/
 
 /*%
- * The dst_key structure is opaque.  Applications should use the accessor
- * functions provided to retrieve key attributes.  If an application needs
- * to set attributes, new accessor functions will be written.
+ * The dst_key structure is opaque.  Applications should use the
+ * accessor functions provided to retrieve key attributes.  If an
+ * application needs to set attributes, new accessor functions will be
+ * written.
  */
 
 typedef struct dst_key	   dst_key_t;
@@ -197,28 +198,6 @@ typedef enum dst_algorithm {
 /***
  *** Functions
  ***/
-isc_result_t
-dst_lib_init(isc_mem_t *mctx, const char *engine);
-/*%<
- * Initializes the DST subsystem.
- *
- * Requires:
- * \li 	"mctx" is a valid memory context
- *
- * Returns:
- * \li	ISC_R_SUCCESS
- * \li	ISC_R_NOMEMORY
- * \li	DST_R_NOENGINE
- *
- * Ensures:
- * \li	DST is properly initialized.
- */
-
-void
-dst_lib_destroy(void);
-/*%<
- * Releases all resources allocated by DST.
- */
 
 bool
 dst_algorithm_supported(unsigned int alg);
@@ -241,7 +220,7 @@ dst_ds_digest_supported(unsigned int digest_type);
  */
 
 isc_result_t
-dst_context_create(dst_key_t *key, isc_mem_t *mctx, isc_logcategory_t *category,
+dst_context_create(dst_key_t *key, isc_mem_t *mctx, isc_logcategory_t category,
 		   bool useforsigning, int maxbits, dst_context_t **dctxp);
 /*%<
  * Creates a context to be used for a sign or verify operation.
@@ -627,8 +606,8 @@ dst_key_buildinternal(const dns_name_t *name, unsigned int alg,
 isc_result_t
 dst_key_fromlabel(const dns_name_t *name, int alg, unsigned int flags,
 		  unsigned int protocol, dns_rdataclass_t rdclass,
-		  const char *engine, const char *label, const char *pin,
-		  isc_mem_t *mctx, dst_key_t **keyp);
+		  const char *label, const char *pin, isc_mem_t *mctx,
+		  dst_key_t **keyp);
 
 isc_result_t
 dst_key_generate(const dns_name_t *name, unsigned int alg, unsigned int bits,
@@ -640,10 +619,8 @@ dst_key_generate(const dns_name_t *name, unsigned int alg, unsigned int bits,
  * Generate a DST key (or keypair) with the supplied parameters.  The
  * interpretation of the "param" field depends on the algorithm:
  * \code
- * 	RSA:	exponent
- * 		0	use exponent 3
- * 		!0	use Fermat4 (2^16 + 1)
- * 	DSA:	unused
+ * 	RSA:	unused
+ * 	ECDSA:	unused
  * 	HMACMD5: entropy
  *		0	default - require good entropy
  *		!0	lack of good entropy is ok
