@@ -19,6 +19,7 @@
 
 #include <isc/dir.h>
 #include <isc/file.h>
+#include <isc/log.h>
 #include <isc/mem.h>
 #include <isc/overflow.h>
 #include <isc/result.h>
@@ -30,10 +31,8 @@
 #include <dns/compress.h>
 #include <dns/db.h>
 #include <dns/dbiterator.h>
-#include <dns/diff.h>
 #include <dns/fixedname.h>
 #include <dns/journal.h>
-#include <dns/log.h>
 #include <dns/rdataset.h>
 #include <dns/rdatasetiter.h>
 #include <dns/soa.h>
@@ -81,10 +80,24 @@
  * Miscellaneous utilities.
  */
 
-#define JOURNAL_COMMON_LOGARGS \
-	dns_lctx, DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL
+/*%
+ * It would be non-sensical (or at least obtuse) to use FAIL() with an
+ * ISC_R_SUCCESS code, but the test is there to keep the Solaris compiler
+ * from complaining about "end-of-loop code not reached".
+ */
+#define FAIL(code)                           \
+	do {                                 \
+		result = (code);             \
+		if (result != ISC_R_SUCCESS) \
+			goto failure;        \
+	} while (0)
 
-#define JOURNAL_DEBUG_LOGARGS(n) JOURNAL_COMMON_LOGARGS, ISC_LOG_DEBUG(n)
+#define CHECK(op)                            \
+	do {                                 \
+		result = (op);               \
+		if (result != ISC_R_SUCCESS) \
+			goto failure;        \
+	} while (0)
 
 #define JOURNAL_SERIALSET 0x01U
 
@@ -139,8 +152,7 @@ dns_db_createsoatuple(dns_db_t *db, dns_dbversion_t *ver, isc_mem_t *mctx,
 	dns_rdataset_current(&rdataset, &rdata);
 	dns_rdataset_getownercase(&rdataset, zonename);
 
-	result = dns_difftuple_create(mctx, op, zonename, rdataset.ttl, &rdata,
-				      tp);
+	dns_difftuple_create(mctx, op, zonename, rdataset.ttl, &rdata, tp);
 
 	dns_rdataset_disassociate(&rdataset);
 	dns_db_detachnode(db, &node);
@@ -399,8 +411,8 @@ journal_seek(dns_journal_t *j, uint32_t offset) {
 
 	result = isc_stdio_seek(j->fp, (off_t)offset, SEEK_SET);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: seek: %s", j->filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: seek: %s", j->filename,
 			      isc_result_totext(result));
 		return ISC_R_UNEXPECTED;
 	}
@@ -417,8 +429,8 @@ journal_read(dns_journal_t *j, void *mem, size_t nbytes) {
 		if (result == ISC_R_EOF) {
 			return ISC_R_NOMORE;
 		}
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: read: %s", j->filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: read: %s", j->filename,
 			      isc_result_totext(result));
 		return ISC_R_UNEXPECTED;
 	}
@@ -432,8 +444,8 @@ journal_write(dns_journal_t *j, void *mem, size_t nbytes) {
 
 	result = isc_stdio_write(mem, 1, nbytes, j->fp, NULL);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: write: %s", j->filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: write: %s", j->filename,
 			      isc_result_totext(result));
 		return ISC_R_UNEXPECTED;
 	}
@@ -447,15 +459,15 @@ journal_fsync(dns_journal_t *j) {
 
 	result = isc_stdio_flush(j->fp);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: flush: %s", j->filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: flush: %s", j->filename,
 			      isc_result_totext(result));
 		return ISC_R_UNEXPECTED;
 	}
 	result = isc_stdio_sync(j->fp);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: fsync: %s", j->filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: fsync: %s", j->filename,
 			      isc_result_totext(result));
 		return ISC_R_UNEXPECTED;
 	}
@@ -555,8 +567,8 @@ journal_file_create(isc_mem_t *mctx, bool downgrade, const char *filename) {
 
 	result = isc_stdio_open(filename, "wb", &fp);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: create: %s", filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: create: %s", filename,
 			      isc_result_totext(result));
 		return ISC_R_UNEXPECTED;
 	}
@@ -577,8 +589,8 @@ journal_file_create(isc_mem_t *mctx, bool downgrade, const char *filename) {
 
 	result = isc_stdio_write(mem, 1, (size_t)size, fp, NULL);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: write: %s", filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: write: %s", filename,
 			      isc_result_totext(result));
 		(void)isc_stdio_close(fp);
 		(void)isc_file_remove(filename);
@@ -589,8 +601,8 @@ journal_file_create(isc_mem_t *mctx, bool downgrade, const char *filename) {
 
 	result = isc_stdio_close(fp);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: close: %s", filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: close: %s", filename,
 			      isc_result_totext(result));
 		(void)isc_file_remove(filename);
 		return ISC_R_UNEXPECTED;
@@ -618,7 +630,8 @@ journal_open(isc_mem_t *mctx, const char *filename, bool writable, bool create,
 	result = isc_stdio_open(j->filename, writable ? "rb+" : "rb", &fp);
 	if (result == ISC_R_FILENOTFOUND) {
 		if (create) {
-			isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_DEBUG(1),
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_DEBUG(1),
 				      "journal file %s does not exist, "
 				      "creating it",
 				      j->filename);
@@ -628,14 +641,14 @@ journal_open(isc_mem_t *mctx, const char *filename, bool writable, bool create,
 			 */
 			result = isc_stdio_open(j->filename, "rb+", &fp);
 		} else {
-			CHECK(ISC_R_NOTFOUND);
+			FAIL(ISC_R_NOTFOUND);
 		}
 	}
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: open: %s", j->filename,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: open: %s", j->filename,
 			      isc_result_totext(result));
-		CHECK(ISC_R_UNEXPECTED);
+		FAIL(ISC_R_UNEXPECTED);
 	}
 
 	j->fp = fp;
@@ -671,9 +684,10 @@ journal_open(isc_mem_t *mctx, const char *filename, bool writable, bool create,
 		 */
 		j->header_ver1 = false;
 	} else {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "%s: journal format not recognized", j->filename);
-		CHECK(ISC_R_UNEXPECTED);
+		FAIL(ISC_R_UNEXPECTED);
 	}
 	journal_header_decode(&rawheader, &j->header);
 
@@ -726,7 +740,7 @@ journal_open(isc_mem_t *mctx, const char *filename, bool writable, bool create,
 	*journalp = j;
 	return ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	j->magic = 0;
 	if (j->rawindex != NULL) {
 		isc_mem_cput(j->mctx, j->rawindex, j->header.index_size,
@@ -852,7 +866,8 @@ maybe_fixup_xhdr(dns_journal_t *j, journal_xhdr_t *xhdr, uint32_t serial,
 		if (j->xhdr_version == XHDR_VERSION1 && xhdr->serial1 == serial)
 		{
 			isc_log_write(
-				JOURNAL_COMMON_LOGARGS, ISC_LOG_DEBUG(3),
+				DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+				ISC_LOG_DEBUG(3),
 				"%s: XHDR_VERSION1 -> XHDR_VERSION2 at %u",
 				j->filename, serial);
 			j->xhdr_version = XHDR_VERSION2;
@@ -863,7 +878,8 @@ maybe_fixup_xhdr(dns_journal_t *j, journal_xhdr_t *xhdr, uint32_t serial,
 			   xhdr->count == serial)
 		{
 			isc_log_write(
-				JOURNAL_COMMON_LOGARGS, ISC_LOG_DEBUG(3),
+				DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+				ISC_LOG_DEBUG(3),
 				"%s: XHDR_VERSION2 -> XHDR_VERSION1 at %u",
 				j->filename, serial);
 			j->xhdr_version = XHDR_VERSION1;
@@ -883,7 +899,8 @@ maybe_fixup_xhdr(dns_journal_t *j, journal_xhdr_t *xhdr, uint32_t serial,
 		if (value != 0L) {
 			CHECK(journal_seek(j, offset + 12));
 		} else {
-			isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_DEBUG(3),
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_DEBUG(3),
 				      "%s: XHDR_VERSION1 count zero at %u",
 				      j->filename, serial);
 			j->xhdr_version = XHDR_VERSION2;
@@ -893,7 +910,8 @@ maybe_fixup_xhdr(dns_journal_t *j, journal_xhdr_t *xhdr, uint32_t serial,
 		   xhdr->serial1 == 0U &&
 		   isc_serial_gt(xhdr->serial0, xhdr->count))
 	{
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_DEBUG(3),
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_DEBUG(3),
 			      "%s: XHDR_VERSION2 count zero at %u", j->filename,
 			      serial);
 		xhdr->serial1 = xhdr->serial0;
@@ -902,7 +920,7 @@ maybe_fixup_xhdr(dns_journal_t *j, journal_xhdr_t *xhdr, uint32_t serial,
 		j->recovered = true;
 	}
 
-cleanup:
+failure:
 	return result;
 }
 
@@ -958,7 +976,8 @@ journal_next(dns_journal_t *j, journal_pos_t *pos) {
 	if (xhdr.serial0 != pos->serial ||
 	    isc_serial_le(xhdr.serial1, xhdr.serial0))
 	{
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "%s: journal file corrupt: "
 			      "expected serial %u, got %u",
 			      j->filename, pos->serial, xhdr.serial0);
@@ -973,8 +992,9 @@ journal_next(dns_journal_t *j, journal_pos_t *pos) {
 			  : sizeof(journal_rawxhdr_ver1_t);
 
 	if ((off_t)(pos->offset + hdrsize + xhdr.size) < pos->offset) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "%s: offset too large", j->filename);
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "%s: offset too large",
+			      j->filename);
 		return ISC_R_UNEXPECTED;
 	}
 
@@ -982,7 +1002,7 @@ journal_next(dns_journal_t *j, journal_pos_t *pos) {
 	pos->serial = xhdr.serial1;
 	return ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	return result;
 }
 
@@ -1163,7 +1183,7 @@ dns_journal_begin_transaction(dns_journal_t *j) {
 
 	j->state = JOURNAL_STATE_TRANSACTION;
 	result = ISC_R_SUCCESS;
-cleanup:
+failure:
 	return result;
 }
 
@@ -1180,7 +1200,8 @@ dns_journal_writediff(dns_journal_t *j, dns_diff_t *diff) {
 	REQUIRE(DNS_DIFF_VALID(diff));
 	REQUIRE(j->state == JOURNAL_STATE_TRANSACTION);
 
-	isc_log_write(JOURNAL_DEBUG_LOGARGS(3), "writing to journal");
+	isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+		      ISC_LOG_DEBUG(3), "writing to journal");
 	(void)dns_diff_print(diff, NULL);
 
 	/*
@@ -1204,7 +1225,8 @@ dns_journal_writediff(dns_journal_t *j, dns_diff_t *diff) {
 	}
 
 	if (size >= DNS_JOURNAL_SIZE_MAX) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "dns_journal_writediff: %s: journal entry "
 			      "too big to be stored: %" PRIu64 " bytes",
 			      j->filename, size);
@@ -1253,7 +1275,7 @@ dns_journal_writediff(dns_journal_t *j, dns_diff_t *diff) {
 
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	if (mem != NULL) {
 		isc_mem_put(j->mctx, mem, size);
 	}
@@ -1287,13 +1309,15 @@ dns_journal_commit(dns_journal_t *j) {
 	 * Perform some basic consistency checks.
 	 */
 	if (j->x.n_soa != 2) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "%s: malformed transaction: %d SOAs", j->filename,
 			      j->x.n_soa);
 		return ISC_R_UNEXPECTED;
 	}
 	if (!DNS_SERIAL_GT(j->x.pos[1].serial, j->x.pos[0].serial)) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "%s: malformed transaction: serial number "
 			      "did not increase",
 			      j->filename);
@@ -1301,7 +1325,8 @@ dns_journal_commit(dns_journal_t *j) {
 	}
 	if (!JOURNAL_EMPTY(&j->header)) {
 		if (j->x.pos[0].serial != j->header.end.serial) {
-			isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_ERROR,
 				      "malformed transaction: "
 				      "%s last serial %u != "
 				      "transaction first serial %u",
@@ -1316,7 +1341,8 @@ dns_journal_commit(dns_journal_t *j) {
 	 */
 	total = j->x.pos[1].offset - j->x.pos[0].offset;
 	if (total >= DNS_JOURNAL_SIZE_MAX) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "transaction too big to be stored in journal: "
 			      "%" PRIu64 "b (max is %" PRIu64 "b)",
 			      total, (uint64_t)DNS_JOURNAL_SIZE_MAX);
@@ -1396,7 +1422,7 @@ dns_journal_commit(dns_journal_t *j) {
 
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	return result;
 }
 
@@ -1409,7 +1435,7 @@ dns_journal_write_transaction(dns_journal_t *j, dns_diff_t *diff) {
 	CHECK(dns_journal_writediff(j, diff));
 	CHECK(dns_journal_commit(j));
 	result = ISC_R_SUCCESS;
-cleanup:
+failure:
 	return result;
 }
 
@@ -1542,11 +1568,12 @@ dns_journal_rollforward(dns_journal_t *j, dns_db_t *db, unsigned int options) {
 			n_soa = 1;
 		}
 		if (n_soa == 0) {
-			isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_ERROR,
 				      "%s: journal file corrupt: missing "
 				      "initial SOA",
 				      j->filename);
-			CHECK(ISC_R_UNEXPECTED);
+			FAIL(ISC_R_UNEXPECTED);
 		}
 		if ((options & DNS_JOURNALOPT_RESIGN) != 0) {
 			op = (n_soa == 1) ? DNS_DIFFOP_DELRESIGN
@@ -1555,12 +1582,12 @@ dns_journal_rollforward(dns_journal_t *j, dns_db_t *db, unsigned int options) {
 			op = (n_soa == 1) ? DNS_DIFFOP_DEL : DNS_DIFFOP_ADD;
 		}
 
-		CHECK(dns_difftuple_create(diff.mctx, op, name, ttl, rdata,
-					   &tuple));
+		dns_difftuple_create(diff.mctx, op, name, ttl, rdata, &tuple);
 		dns_diff_append(&diff, &tuple);
 
 		if (++n_put > 100) {
-			isc_log_write(JOURNAL_DEBUG_LOGARGS(3),
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_DEBUG(3),
 				      "%s: applying diff to database (%u)",
 				      j->filename, db_serial);
 			(void)dns_diff_print(&diff, NULL);
@@ -1575,7 +1602,8 @@ dns_journal_rollforward(dns_journal_t *j, dns_db_t *db, unsigned int options) {
 	CHECK(result);
 
 	if (n_put != 0) {
-		isc_log_write(JOURNAL_DEBUG_LOGARGS(3),
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_DEBUG(3),
 			      "%s: applying final diff to database (%u)",
 			      j->filename, db_serial);
 		(void)dns_diff_print(&diff, NULL);
@@ -1583,7 +1611,7 @@ dns_journal_rollforward(dns_journal_t *j, dns_db_t *db, unsigned int options) {
 		dns_diff_clear(&diff);
 	}
 
-cleanup:
+failure:
 	if (ver != NULL) {
 		dns_db_closeversion(db, &ver,
 				    result == ISC_R_SUCCESS ? true : false);
@@ -1621,11 +1649,12 @@ dns_journal_print(isc_mem_t *mctx, uint32_t flags, const char *filename,
 
 	result = dns_journal_open(mctx, filename, DNS_JOURNAL_READ, &j);
 	if (result == ISC_R_NOTFOUND) {
-		isc_log_write(JOURNAL_DEBUG_LOGARGS(3), "no journal file");
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_DEBUG(3), "no journal file");
 		return DNS_R_NOJOURNAL;
 	} else if (result != ISC_R_SUCCESS) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "journal open failure: %s: %s",
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR, "journal open failure: %s: %s",
 			      isc_result_totext(result), filename);
 		return result;
 	}
@@ -1685,11 +1714,12 @@ dns_journal_print(isc_mem_t *mctx, uint32_t flags, const char *filename,
 			}
 		}
 		if (n_soa == 0) {
-			isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_ERROR,
 				      "%s: journal file corrupt: missing "
 				      "initial SOA",
 				      j->filename);
-			CHECK(ISC_R_UNEXPECTED);
+			FAIL(ISC_R_UNEXPECTED);
 		}
 
 		if (print) {
@@ -1708,9 +1738,9 @@ dns_journal_print(isc_mem_t *mctx, uint32_t flags, const char *filename,
 				i++;
 			}
 		}
-		CHECK(dns_difftuple_create(
+		dns_difftuple_create(
 			diff.mctx, n_soa == 1 ? DNS_DIFFOP_DEL : DNS_DIFFOP_ADD,
-			name, ttl, rdata, &tuple));
+			name, ttl, rdata, &tuple);
 		dns_diff_append(&diff, &tuple);
 
 		if (++n_put > 100 || printxhdr) {
@@ -1731,13 +1761,14 @@ dns_journal_print(isc_mem_t *mctx, uint32_t flags, const char *filename,
 		result = dns_diff_print(&diff, file);
 		dns_diff_clear(&diff);
 	}
-	goto done;
+	goto cleanup;
+
+failure:
+	isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+		      ISC_LOG_ERROR, "%s: cannot print: journal file corrupt",
+		      j->filename);
 
 cleanup:
-	isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
-		      "%s: cannot print: journal file corrupt", j->filename);
-
-done:
 	if (source.base != NULL) {
 		isc_mem_put(j->mctx, source.base, source.length);
 	}
@@ -1902,7 +1933,7 @@ dns_journal_iter_init(dns_journal_t *j, uint32_t begin_serial,
 	}
 
 	result = ISC_R_SUCCESS;
-cleanup:
+failure:
 	j->it.result = result;
 	return j->it.result;
 }
@@ -1923,7 +1954,7 @@ dns_journal_first_rr(dns_journal_t *j) {
 
 	return read_one_rr(j);
 
-cleanup:
+failure:
 	return result;
 }
 
@@ -1939,7 +1970,8 @@ read_one_rr(dns_journal_t *j) {
 	dns_journal_t save = *j;
 
 	if (j->offset > j->it.epos.offset) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "%s: journal corrupt: possible integer overflow",
 			      j->filename);
 		return ISC_R_UNEXPECTED;
@@ -1954,10 +1986,11 @@ read_one_rr(dns_journal_t *j) {
 		 */
 		CHECK(journal_read_xhdr(j, &xhdr));
 		if (xhdr.size == 0) {
-			isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_ERROR,
 				      "%s: journal corrupt: empty transaction",
 				      j->filename);
-			CHECK(ISC_R_UNEXPECTED);
+			FAIL(ISC_R_UNEXPECTED);
 		}
 
 		if (j->header_ver1) {
@@ -1968,12 +2001,13 @@ read_one_rr(dns_journal_t *j) {
 		if (xhdr.serial0 != j->it.current_serial ||
 		    isc_serial_le(xhdr.serial1, xhdr.serial0))
 		{
-			isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_ERROR,
 				      "%s: journal file corrupt: "
 				      "expected serial %u, got %u",
 				      j->filename, j->it.current_serial,
 				      xhdr.serial0);
-			CHECK(ISC_R_UNEXPECTED);
+			FAIL(ISC_R_UNEXPECTED);
 		}
 
 		j->it.xsize = xhdr.size;
@@ -1991,11 +2025,12 @@ read_one_rr(dns_journal_t *j) {
 	 * size owner name, well below 70 k total.
 	 */
 	if (rrhdr.size < 1 + 10 || rrhdr.size > 70000) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "%s: journal corrupt: impossible RR size "
 			      "(%d bytes)",
 			      j->filename, rrhdr.size);
-		CHECK(ISC_R_UNEXPECTED);
+		FAIL(ISC_R_UNEXPECTED);
 	}
 
 	CHECK(size_buffer(j->mctx, &j->it.source, rrhdr.size));
@@ -2024,7 +2059,7 @@ read_one_rr(dns_journal_t *j) {
 	 * Check that the RR header is there, and parse it.
 	 */
 	if (isc_buffer_remaininglength(&j->it.source) < 10) {
-		CHECK(DNS_R_FORMERR);
+		FAIL(DNS_R_FORMERR);
 	}
 
 	rdtype = isc_buffer_getuint16(&j->it.source);
@@ -2033,18 +2068,19 @@ read_one_rr(dns_journal_t *j) {
 	rdlen = isc_buffer_getuint16(&j->it.source);
 
 	if (rdlen > DNS_RDATA_MAXLENGTH) {
-		isc_log_write(JOURNAL_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_JOURNAL,
+			      ISC_LOG_ERROR,
 			      "%s: journal corrupt: impossible rdlen "
 			      "(%u bytes)",
 			      j->filename, rdlen);
-		CHECK(ISC_R_FAILURE);
+		FAIL(ISC_R_FAILURE);
 	}
 
 	/*
 	 * Parse the rdata.
 	 */
 	if (isc_buffer_remaininglength(&j->it.source) != rdlen) {
-		CHECK(DNS_R_FORMERR);
+		FAIL(DNS_R_FORMERR);
 	}
 	isc_buffer_setactive(&j->it.source, rdlen);
 	dns_rdata_reset(&j->it.rdata);
@@ -2060,7 +2096,7 @@ read_one_rr(dns_journal_t *j) {
 
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	j->it.result = result;
 	return result;
 }
@@ -2127,13 +2163,8 @@ get_name_diff(dns_db_t *db, dns_dbversion_t *ver, isc_stdtime_t now,
 		{
 			dns_rdata_t rdata = DNS_RDATA_INIT;
 			dns_rdataset_current(&rdataset, &rdata);
-			result = dns_difftuple_create(diff->mctx, op, name,
-						      rdataset.ttl, &rdata,
-						      &tuple);
-			if (result != ISC_R_SUCCESS) {
-				dns_rdataset_disassociate(&rdataset);
-				goto cleanup_iterator;
-			}
+			dns_difftuple_create(diff->mctx, op, name, rdataset.ttl,
+					     &rdata, &tuple);
 			dns_diff_append(diff, &tuple);
 		}
 		dns_rdataset_disassociate(&rdataset);
@@ -2236,7 +2267,7 @@ dns_diff_subtract(dns_diff_t diff[2], dns_diff_t *r) {
 	ISC_LIST_APPENDLIST(r->tuples, del, link);
 	ISC_LIST_APPENDLIST(r->tuples, add, link);
 	result = ISC_R_SUCCESS;
-cleanup:
+failure:
 	return result;
 }
 
@@ -2328,16 +2359,16 @@ diff_namespace(dns_db_t *dba, dns_dbversion_t *dbvera, dns_db_t *dbb,
 	next:;
 	}
 	if (itresult[0] != ISC_R_NOMORE) {
-		CHECK(itresult[0]);
+		FAIL(itresult[0]);
 	}
 	if (itresult[1] != ISC_R_NOMORE) {
-		CHECK(itresult[1]);
+		FAIL(itresult[1]);
 	}
 
 	INSIST(ISC_LIST_EMPTY(diff[0].tuples));
 	INSIST(ISC_LIST_EMPTY(diff[1].tuples));
 
-cleanup:
+failure:
 	dns_dbiterator_destroy(&dbit[1]);
 
 cleanup_iterator:
@@ -2387,13 +2418,15 @@ dns_db_diffx(dns_diff_t *diff, dns_db_t *dba, dns_dbversion_t *dbvera,
 
 	if (journal != NULL) {
 		if (ISC_LIST_EMPTY(diff->tuples)) {
-			isc_log_write(JOURNAL_DEBUG_LOGARGS(3), "no changes");
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
+				      DNS_LOGMODULE_JOURNAL, ISC_LOG_DEBUG(3),
+				      "no changes");
 		} else {
 			CHECK(dns_journal_write_transaction(journal, diff));
 		}
 	}
 
-cleanup:
+failure:
 	if (journal != NULL) {
 		dns_journal_destroy(&journal);
 	}
@@ -2616,7 +2649,8 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, uint32_t serial,
 
 			size = xhdr.size;
 			if (size > len) {
-				isc_log_write(JOURNAL_COMMON_LOGARGS,
+				isc_log_write(DNS_LOGCATEGORY_GENERAL,
+					      DNS_LOGMODULE_JOURNAL,
 					      ISC_LOG_ERROR,
 					      "%s: journal file corrupt, "
 					      "transaction too large",
@@ -2650,7 +2684,8 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, uint32_t serial,
 				size = xhdr.size;
 				if (size > len) {
 					isc_log_write(
-						JOURNAL_COMMON_LOGARGS,
+						DNS_LOGCATEGORY_GENERAL,
+						DNS_LOGMODULE_JOURNAL,
 						ISC_LOG_ERROR,
 						"%s: journal file corrupt, "
 						"transaction too large",
@@ -2778,7 +2813,7 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, uint32_t serial,
 			if (result != ISC_R_SUCCESS &&
 			    result != ISC_R_FILENOTFOUND)
 			{
-				CHECK(result);
+				goto failure;
 			}
 			if (rename(filename, backup) == -1) {
 				goto maperrno;
@@ -2789,13 +2824,14 @@ dns_journal_compact(isc_mem_t *mctx, char *filename, uint32_t serial,
 			(void)isc_file_remove(backup);
 		} else {
 		maperrno:
-			CHECK(ISC_R_FAILURE);
+			result = ISC_R_FAILURE;
+			goto failure;
 		}
 	}
 
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	(void)isc_file_remove(newname);
 	if (buf != NULL) {
 		isc_mem_put(mctx, buf, size);
@@ -2833,6 +2869,6 @@ index_to_disk(dns_journal_t *j) {
 		CHECK(journal_seek(j, sizeof(journal_rawheader_t)));
 		CHECK(journal_write(j, j->rawindex, rawbytes));
 	}
-cleanup:
+failure:
 	return result;
 }
