@@ -16,6 +16,7 @@
 #include <stddef.h>
 #include <stdlib.h>
 
+#include <isc/mem.h>
 #include <isc/region.h>
 #include <isc/util.h>
 
@@ -123,10 +124,19 @@ dst_key_class(const dst_key_t *key) {
 	return key->key_class;
 }
 
+const char *
+dst_key_directory(const dst_key_t *key) {
+	REQUIRE(VALID_KEY(key));
+	return key->directory;
+}
+
 bool
 dst_key_iszonekey(const dst_key_t *key) {
 	REQUIRE(VALID_KEY(key));
 
+	if ((key->key_flags & DNS_KEYTYPE_NOAUTH) != 0) {
+		return false;
+	}
 	if ((key->key_flags & DNS_KEYFLAG_OWNERMASK) != DNS_KEYOWNER_ZONE) {
 		return false;
 	}
@@ -156,6 +166,64 @@ dst_key_isnullkey(const dst_key_t *key) {
 	return true;
 }
 
+#define REVOKE(x) ((dst_key_flags(x) & DNS_KEYFLAG_REVOKE) != 0)
+#define KSK(x)	  ((dst_key_flags(x) & DNS_KEYFLAG_KSK) != 0)
+#define ID(x)	  dst_key_id(x)
+#define ALG(x)	  dst_key_alg(x)
+
+bool
+dst_key_have_ksk_and_zsk(dst_key_t **keys, unsigned int nkeys, unsigned int i,
+			 bool check_offline, bool ksk, bool zsk, bool *have_ksk,
+			 bool *have_zsk) {
+	bool hksk = ksk;
+	bool hzsk = zsk;
+	isc_result_t result;
+
+	REQUIRE(keys != NULL);
+
+	for (unsigned int j = 0; j < nkeys && !(hksk && hzsk); j++) {
+		if (j == i || ALG(keys[i]) != ALG(keys[j])) {
+			continue;
+		}
+		/*
+		 * Don't consider inactive keys.
+		 */
+		if (dst_key_inactive(keys[j])) {
+			continue;
+		}
+		/*
+		 * Don't consider offline keys.
+		 */
+		if (check_offline && !dst_key_isprivate(keys[j])) {
+			continue;
+		}
+		if (REVOKE(keys[j])) {
+			continue;
+		}
+
+		if (!hksk) {
+			result = dst_key_getbool(keys[j], DST_BOOL_KSK, &hksk);
+			if (result != ISC_R_SUCCESS) {
+				if (KSK(keys[j])) {
+					hksk = true;
+				}
+			}
+		}
+		if (!hzsk) {
+			result = dst_key_getbool(keys[j], DST_BOOL_ZSK, &hzsk);
+			if (result != ISC_R_SUCCESS) {
+				if (!KSK(keys[j])) {
+					hzsk = dst_key_isprivate(keys[j]);
+				}
+			}
+		}
+	}
+
+	SET_IF_NOT_NULL(have_ksk, hksk);
+	SET_IF_NOT_NULL(have_zsk, hzsk);
+	return hksk && hzsk;
+}
+
 void
 dst_key_setbits(dst_key_t *key, uint16_t bits) {
 	unsigned int maxbits;
@@ -186,4 +254,12 @@ dst_key_getttl(const dst_key_t *key) {
 	return key->key_ttl;
 }
 
+void
+dst_key_setdirectory(dst_key_t *key, const char *dir) {
+	if (key->directory != NULL) {
+		isc_mem_free(key->mctx, key->directory);
+	}
+	key->directory = isc_mem_strdup(key->mctx, dir);
+}
+
 /*! \file */
