@@ -20,6 +20,7 @@
 
 #include <isc/buffer.h>
 #include <isc/dir.h>
+#include <isc/log.h>
 #include <isc/mem.h>
 #include <isc/result.h>
 #include <isc/string.h>
@@ -30,7 +31,6 @@
 #include <dns/kasp.h>
 #include <dns/keymgr.h>
 #include <dns/keyvalues.h>
-#include <dns/log.h>
 
 #include <dst/dst.h>
 
@@ -53,15 +53,15 @@
 			dst_key_setstate((key), (state), (target));            \
 			dst_key_settime((key), (timing), time);                \
                                                                                \
-			if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {    \
+			if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {              \
 				dst_key_format((key), keystr, sizeof(keystr)); \
 				isc_log_write(                                 \
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,      \
+					DNS_LOGCATEGORY_DNSSEC,                \
 					DNS_LOGMODULE_DNSSEC,                  \
 					ISC_LOG_DEBUG(3),                      \
 					"keymgr: DNSKEY %s (%s) initialize "   \
 					"%s state to %s (policy %s)",          \
-					keystr, keymgr_keyrole(key),           \
+					keystr, keymgr_keyrole((key)),         \
 					keystatetags[state],                   \
 					keystatestrings[target],               \
 					dns_kasp_getname(kasp));               \
@@ -90,10 +90,9 @@ static void
 log_key_overflow(dst_key_t *key, const char *what) {
 	char keystr[DST_KEY_FORMATSIZE];
 	dst_key_format(key, keystr, sizeof(keystr));
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
-		      ISC_LOG_WARNING,
-		      "keymgr: DNSKEY %s (%s) calculation overflowed", keystr,
-		      what);
+	isc_log_write(
+		DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC, ISC_LOG_WARNING,
+		"keymgr: DNSKEY %s (%s) calculation overflowed", keystr, what);
 }
 
 /*
@@ -190,19 +189,13 @@ dns_keymgr_settime_syncpublish(dst_key_t *key, dns_kasp_t *kasp, bool first) {
 		isc_stdtime_t zrrsig_present;
 		dns_ttl_t ttlsig = dns_kasp_zonemaxttl(kasp, true);
 		zrrsig_present = published + ttlsig +
-				 dns_kasp_zonepropagationdelay(kasp);
+				 dns_kasp_zonepropagationdelay(kasp) +
+				 dns_kasp_publishsafety(kasp);
 		if (zrrsig_present > syncpublish) {
 			syncpublish = zrrsig_present;
 		}
 	}
 	dst_key_settime(key, DST_TIME_SYNCPUBLISH, syncpublish);
-
-	uint32_t lifetime = 0;
-	ret = dst_key_getnum(key, DST_NUM_LIFETIME, &lifetime);
-	if (ret == ISC_R_SUCCESS && lifetime > 0) {
-		dst_key_settime(key, DST_TIME_SYNCDELETE,
-				syncpublish + lifetime);
-	}
 }
 
 /*
@@ -250,17 +243,6 @@ keymgr_prepublication_time(dns_dnsseckey_t *key, dns_kasp_t *kasp,
 		pub = now;
 	}
 
-	/*
-	 * To calculate phase out times ("Retired", "Removed", ...),
-	 * the key lifetime is required.
-	 */
-	uint32_t klifetime = 0;
-	ret = dst_key_getnum(key->key, DST_NUM_LIFETIME, &klifetime);
-	if (ret != ISC_R_SUCCESS) {
-		dst_key_setnum(key->key, DST_NUM_LIFETIME, lifetime);
-		klifetime = lifetime;
-	}
-
 	/*
 	 * Calculate prepublication time.
 	 */
@@ -290,16 +272,13 @@ keymgr_prepublication_time(dns_dnsseckey_t *key, dns_kasp_t *kasp,
 				dns_ttl_t ttlsig = dns_kasp_zonemaxttl(kasp,
 								       true);
 				syncpub2 = pub + ttlsig +
+					   dns_kasp_publishsafety(kasp) +
 					   dns_kasp_zonepropagationdelay(kasp);
 			}
 
 			syncpub = ISC_MAX(syncpub1, syncpub2);
 			dst_key_settime(key->key, DST_TIME_SYNCPUBLISH,
 					syncpub);
-			if (klifetime > 0) {
-				dst_key_settime(key->key, DST_TIME_SYNCDELETE,
-						syncpub + klifetime);
-			}
 		}
 	}
 
@@ -312,6 +291,13 @@ keymgr_prepublication_time(dns_dnsseckey_t *key, dns_kasp_t *kasp,
 
 	ret = dst_key_gettime(key->key, DST_TIME_INACTIVE, &retire);
 	if (ret != ISC_R_SUCCESS) {
+		uint32_t klifetime = 0;
+
+		ret = dst_key_getnum(key->key, DST_NUM_LIFETIME, &klifetime);
+		if (ret != ISC_R_SUCCESS) {
+			dst_key_setnum(key->key, DST_NUM_LIFETIME, lifetime);
+			klifetime = lifetime;
+		}
 		if (klifetime == 0) {
 			/*
 			 * No inactive time and no lifetime,
@@ -394,7 +380,7 @@ keymgr_key_retire(dns_dnsseckey_t *key, dns_kasp_t *kasp, isc_stdtime_t now) {
 	}
 
 	dst_key_format(key->key, keystr, sizeof(keystr));
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+	isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
 		      ISC_LOG_INFO, "keymgr: retire DNSKEY %s (%s)", keystr,
 		      keymgr_keyrole(key->key));
 }
@@ -412,7 +398,7 @@ keymgr_key_update_lifetime(dns_dnsseckey_t *key, dns_kasp_t *kasp,
 	/* Initialize lifetime. */
 	if (r != ISC_R_SUCCESS) {
 		dst_key_setnum(key->key, DST_NUM_LIFETIME, lifetime);
-		l = lifetime - 1;
+		return;
 	}
 	/* Skip keys that are still hidden or already retiring. */
 	if (g != OMNIPRESENT) {
@@ -434,7 +420,6 @@ keymgr_key_update_lifetime(dns_dnsseckey_t *key, dns_kasp_t *kasp,
 		} else {
 			dst_key_unsettime(key->key, DST_TIME_INACTIVE);
 			dst_key_unsettime(key->key, DST_TIME_DELETE);
-			dst_key_unsettime(key->key, DST_TIME_SYNCDELETE);
 		}
 	}
 }
@@ -514,7 +499,7 @@ keymgr_createkey(dns_kasp_key_t *kkey, const dns_name_t *origin,
 		}
 		if (conflict) {
 			/* Try again. */
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_WARNING,
 				      "keymgr: key collision id %d",
 				      dst_key_id(newkey));
@@ -608,7 +593,7 @@ keymgr_desiredstate(dns_dnsseckey_t *key, dst_key_state_t state) {
  *
  */
 static bool
-keymgr_key_match_state(const dst_key_t *key, const dst_key_t *subject, int type,
+keymgr_key_match_state(dst_key_t *key, dst_key_t *subject, int type,
 		       dst_key_state_t next_state,
 		       dst_key_state_t states[NUM_KEYSTATES]) {
 	REQUIRE(key != NULL);
@@ -1240,7 +1225,7 @@ keymgr_transition_allowed(dns_dnsseckeylist_t *keyring, dns_dnsseckey_t *key,
 			  int type, dst_key_state_t next_state,
 			  bool secure_to_insecure) {
 	/* Debug logging. */
-	if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {
+	if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 		bool rule1a, rule1b, rule2a, rule2b, rule3a, rule3b;
 		char keystr[DST_KEY_FORMATSIZE];
 		dst_key_format(key->key, keystr, sizeof(keystr));
@@ -1253,7 +1238,7 @@ keymgr_transition_allowed(dns_dnsseckeylist_t *keyring, dns_dnsseckey_t *key,
 		rule3a = keymgr_have_rrsig(keyring, key, type, NA);
 		rule3b = keymgr_have_rrsig(keyring, key, type, next_state);
 		isc_log_write(
-			dns_lctx, DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
 			ISC_LOG_DEBUG(1),
 			"keymgr: dnssec evaluation of %s %s record %s: "
 			"rule1=(~%s or %s) rule2=(~%s or %s) "
@@ -1301,7 +1286,6 @@ keymgr_transition_time(dns_dnsseckey_t *key, int type,
 	isc_result_t ret;
 	isc_stdtime_t lastchange, dstime, nexttime = now;
 	dns_ttl_t ttlsig = dns_kasp_zonemaxttl(kasp, true);
-	uint32_t dsstate;
 
 	/*
 	 * No need to wait if we move things into an uncertain state.
@@ -1371,12 +1355,15 @@ keymgr_transition_time(dns_dnsseckey_t *key, int type,
 			 * records.  This translates to:
 			 *
 			 *     Dsgn + zone-propagation-delay + max-zone-ttl.
+			 *
+			 * We will also add the retire-safety interval.
 			 */
 			nexttime = lastchange + ttlsig +
-				   dns_kasp_zonepropagationdelay(kasp);
+				   dns_kasp_zonepropagationdelay(kasp) +
+				   dns_kasp_retiresafety(kasp);
 			/*
-			 * Only add the sign delay Dsgn and retire-safety if
-			 * there is an actual predecessor or successor key.
+			 * Only add the sign delay Dsgn if there is an actual
+			 * predecessor or successor key.
 			 */
 			uint32_t tag;
 			ret = dst_key_getnum(key->key, DST_NUM_PREDECESSOR,
@@ -1386,8 +1373,7 @@ keymgr_transition_time(dns_dnsseckey_t *key, int type,
 						     DST_NUM_SUCCESSOR, &tag);
 			}
 			if (ret == ISC_R_SUCCESS) {
-				nexttime += dns_kasp_signdelay(kasp) +
-					    dns_kasp_retiresafety(kasp);
+				nexttime += dns_kasp_signdelay(kasp);
 			}
 			break;
 		default:
@@ -1413,36 +1399,35 @@ keymgr_transition_time(dns_dnsseckey_t *key, int type,
 		 * This translates to:
 		 *
 		 *      parent-propagation-delay + parent-ds-ttl.
+		 *
+		 * We will also add the retire-safety interval.
 		 */
 		case OMNIPRESENT:
-		case HIDDEN:
-			/* Make sure DS has been seen in/withdrawn from the
-			 * parent. */
-			dsstate = next_state == HIDDEN ? DST_TIME_DSDELETE
-						       : DST_TIME_DSPUBLISH;
-			ret = dst_key_gettime(key->key, dsstate, &dstime);
+			/* Make sure DS has been seen in the parent. */
+			ret = dst_key_gettime(key->key, DST_TIME_DSPUBLISH,
+					      &dstime);
 			if (ret != ISC_R_SUCCESS || dstime > now) {
 				/* Not yet, try again in an hour. */
 				nexttime = now + 3600;
 			} else {
 				nexttime =
 					dstime + dns_kasp_dsttl(kasp) +
-					dns_kasp_parentpropagationdelay(kasp);
-				/*
-				 * Only add the retire-safety if there is an
-				 * actual predecessor or successor key.
-				 */
-				uint32_t tag;
-				ret = dst_key_getnum(key->key,
-						     DST_NUM_PREDECESSOR, &tag);
-				if (ret != ISC_R_SUCCESS) {
-					ret = dst_key_getnum(key->key,
-							     DST_NUM_SUCCESSOR,
-							     &tag);
-				}
-				if (ret == ISC_R_SUCCESS) {
-					nexttime += dns_kasp_retiresafety(kasp);
-				}
+					dns_kasp_parentpropagationdelay(kasp) +
+					dns_kasp_retiresafety(kasp);
+			}
+			break;
+		case HIDDEN:
+			/* Make sure DS has been withdrawn from the parent. */
+			ret = dst_key_gettime(key->key, DST_TIME_DSDELETE,
+					      &dstime);
+			if (ret != ISC_R_SUCCESS || dstime > now) {
+				/* Not yet, try again in an hour. */
+				nexttime = now + 3600;
+			} else {
+				nexttime =
+					dstime + dns_kasp_dsttl(kasp) +
+					dns_kasp_parentpropagationdelay(kasp) +
+					dns_kasp_retiresafety(kasp);
 			}
 			break;
 		default:
@@ -1499,7 +1484,7 @@ transition:
 				continue;
 			}
 
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 				      "keymgr: examine %s %s type %s "
 				      "in state %s",
@@ -1514,7 +1499,7 @@ transition:
 				 * No change needed, continue with the next
 				 * record type.
 				 */
-				isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+				isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 					      DNS_LOGMODULE_DNSSEC,
 					      ISC_LOG_DEBUG(1),
 					      "keymgr: %s %s type %s in "
@@ -1525,7 +1510,7 @@ transition:
 				continue;
 			}
 
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 				      "keymgr: can we transition %s %s type %s "
 				      "state %s to state %s?",
@@ -1539,7 +1524,7 @@ transition:
 			{
 				/* No, please respect rollover methods. */
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 					"keymgr: policy says no to %s %s type "
 					"%s "
@@ -1558,7 +1543,7 @@ transition:
 			{
 				/* No, this would make the zone bogus. */
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 					"keymgr: dnssec says no to %s %s type "
 					"%s "
@@ -1576,7 +1561,7 @@ transition:
 			if (when > now) {
 				/* Not yet. */
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 					"keymgr: time says no to %s %s type %s "
 					"state %s to state %s (wait %u "
@@ -1591,7 +1576,7 @@ transition:
 				continue;
 			}
 
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 				      "keymgr: transition %s %s type %s "
 				      "state %s to state %s!",
@@ -1640,12 +1625,12 @@ keymgr_key_init(dns_dnsseckey_t *key, dns_kasp_t *kasp, isc_stdtime_t now,
 	ret = dst_key_getbool(key->key, DST_BOOL_KSK, &ksk);
 	if (ret != ISC_R_SUCCESS) {
 		ksk = ((dst_key_flags(key->key) & DNS_KEYFLAG_KSK) != 0);
-		dst_key_setbool(key->key, DST_BOOL_KSK, ksk || csk);
+		dst_key_setbool(key->key, DST_BOOL_KSK, (ksk || csk));
 	}
 	ret = dst_key_getbool(key->key, DST_BOOL_ZSK, &zsk);
 	if (ret != ISC_R_SUCCESS) {
 		zsk = ((dst_key_flags(key->key) & DNS_KEYFLAG_KSK) == 0);
-		dst_key_setbool(key->key, DST_BOOL_ZSK, zsk || csk);
+		dst_key_setbool(key->key, DST_BOOL_ZSK, (zsk || csk));
 	}
 
 	/* Get time metadata. */
@@ -1745,11 +1730,11 @@ keymgr_key_rollover(dns_kasp_key_t *kaspkey, dns_dnsseckey_t *active_key,
 
 	/* Do we need to create a successor for the active key? */
 	if (active_key != NULL) {
-		if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {
+		if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 			dst_key_format(active_key->key, keystr, sizeof(keystr));
 			isc_log_write(
-				dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-				DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
+				DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+				ISC_LOG_DEBUG(1),
 				"keymgr: DNSKEY %s (%s) is active in policy %s",
 				keystr, keymgr_keyrole(active_key->key),
 				dns_kasp_getname(kasp));
@@ -1762,36 +1747,34 @@ keymgr_key_rollover(dns_kasp_key_t *kaspkey, dns_dnsseckey_t *active_key,
 		prepub = keymgr_prepublication_time(active_key, kasp, lifetime,
 						    now);
 		if (prepub > now) {
-			if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {
+			if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 				dst_key_format(active_key->key, keystr,
 					       sizeof(keystr));
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 					"keymgr: new successor needed for "
 					"DNSKEY %s (%s) (policy %s) in %u "
 					"seconds",
 					keystr, keymgr_keyrole(active_key->key),
-					dns_kasp_getname(kasp), prepub - now);
+					dns_kasp_getname(kasp), (prepub - now));
 			}
 		}
 		if (prepub == 0 || prepub > now) {
 			/* No need to start rollover now. */
 			if (*nexttime == 0 || prepub < *nexttime) {
-				if (prepub > 0) {
-					*nexttime = prepub;
-				}
+				*nexttime = prepub;
 			}
 			return ISC_R_SUCCESS;
 		}
 
 		if (keymgr_key_has_successor(active_key, keyring)) {
 			/* Key already has successor. */
-			if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {
+			if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 				dst_key_format(active_key->key, keystr,
 					       sizeof(keystr));
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+					DNS_LOGCATEGORY_DNSSEC,
 					DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 					"keymgr: key DNSKEY %s (%s) (policy "
 					"%s) already has successor",
@@ -1801,9 +1784,9 @@ keymgr_key_rollover(dns_kasp_key_t *kaspkey, dns_dnsseckey_t *active_key,
 			return ISC_R_SUCCESS;
 		}
 
-		if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {
+		if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 			dst_key_format(active_key->key, keystr, sizeof(keystr));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 				      "keymgr: need successor for DNSKEY %s "
 				      "(%s) (policy %s)",
@@ -1816,7 +1799,7 @@ keymgr_key_rollover(dns_kasp_key_t *kaspkey, dns_dnsseckey_t *active_key,
 		 */
 		if (!rollover) {
 			dst_key_format(active_key->key, keystr, sizeof(keystr));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_WARNING,
 				      "keymgr: DNSKEY %s (%s) is offline in "
 				      "policy %s, cannot start rollover",
@@ -1824,11 +1807,11 @@ keymgr_key_rollover(dns_kasp_key_t *kaspkey, dns_dnsseckey_t *active_key,
 				      dns_kasp_getname(kasp));
 			return ISC_R_SUCCESS;
 		}
-	} else if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {
+	} else if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 		char namestr[DNS_NAME_FORMATSIZE];
 		dns_name_format(origin, namestr, sizeof(namestr));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
+		isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_DEBUG(1),
 			      "keymgr: no active key found for %s (policy %s)",
 			      namestr, dns_kasp_getname(kasp));
 	}
@@ -1943,7 +1926,7 @@ keymgr_key_rollover(dns_kasp_key_t *kaspkey, dns_dnsseckey_t *active_key,
 
 	/* Logging. */
 	dst_key_format(new_key->key, keystr, sizeof(keystr));
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+	isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
 		      ISC_LOG_INFO, "keymgr: DNSKEY %s (%s) %s for policy %s",
 		      keystr, keymgr_keyrole(new_key->key),
 		      (candidate != NULL) ? "selected" : "created",
@@ -1951,9 +1934,8 @@ keymgr_key_rollover(dns_kasp_key_t *kaspkey, dns_dnsseckey_t *active_key,
 	return ISC_R_SUCCESS;
 }
 
-bool
-dns_keymgr_key_may_be_purged(const dst_key_t *key, uint32_t after,
-			     isc_stdtime_t now) {
+static bool
+keymgr_key_may_be_purged(dst_key_t *key, uint32_t after, isc_stdtime_t now) {
 	bool ksk = false;
 	bool zsk = false;
 	dst_key_state_t hidden[NUM_KEYSTATES] = { HIDDEN, NA, NA, NA };
@@ -2020,8 +2002,8 @@ keymgr_purge_keyfile(dst_key_t *key, int type) {
 	if (ret != ISC_R_SUCCESS) {
 		char keystr[DST_KEY_FORMATSIZE];
 		dst_key_format(key, keystr, sizeof(keystr));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_WARNING,
+		isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_WARNING,
 			      "keymgr: failed to purge DNSKEY %s (%s): cannot "
 			      "build filename (%s)",
 			      keystr, keymgr_keyrole(key),
@@ -2032,28 +2014,14 @@ keymgr_purge_keyfile(dst_key_t *key, int type) {
 	if (unlink(filename) < 0) {
 		char keystr[DST_KEY_FORMATSIZE];
 		dst_key_format(key, keystr, sizeof(keystr));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_WARNING,
+		isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_WARNING,
 			      "keymgr: failed to purge DNSKEY %s (%s): unlink "
 			      "'%s' failed",
 			      keystr, keymgr_keyrole(key), filename);
 	}
 }
 
-static bool
-dst_key_doublematch(dns_dnsseckey_t *key, dns_kasp_t *kasp) {
-	int matches = 0;
-
-	for (dns_kasp_key_t *kkey = ISC_LIST_HEAD(dns_kasp_keys(kasp));
-	     kkey != NULL; kkey = ISC_LIST_NEXT(kkey, link))
-	{
-		if (dns_kasp_key_match(kkey, key)) {
-			matches++;
-		}
-	}
-	return matches > 1;
-}
-
 /*
  * Examine 'keys' and match 'kasp' policy.
  *
@@ -2082,11 +2050,11 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 	*nexttime = 0;
 
 	/* Debug logging: what keys are available in the keyring? */
-	if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(1))) {
+	if (isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 		if (ISC_LIST_EMPTY(*keyring)) {
 			char namebuf[DNS_NAME_FORMATSIZE];
 			dns_name_format(origin, namebuf, sizeof(namebuf));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 				      "keymgr: keyring empty (zone %s policy "
 				      "%s)",
@@ -2097,7 +2065,7 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 		     dkey != NULL; dkey = ISC_LIST_NEXT(dkey, link))
 		{
 			dst_key_format(dkey->key, keystr, sizeof(keystr));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 				      "keymgr: keyring: %s (policy %s)", keystr,
 				      dns_kasp_getname(kasp));
@@ -2106,7 +2074,7 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 		     dkey != NULL; dkey = ISC_LIST_NEXT(dkey, link))
 		{
 			dst_key_format(dkey->key, keystr, sizeof(keystr));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(1),
 				      "keymgr: dnskeys: %s (policy %s)", keystr,
 				      dns_kasp_getname(kasp));
@@ -2125,7 +2093,7 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 	{
 		bool found_match = false;
 
-		keymgr_key_init(dkey, kasp, now, numkeys == 1);
+		keymgr_key_init(dkey, kasp, now, (numkeys == 1));
 
 		for (kkey = ISC_LIST_HEAD(dns_kasp_keys(kasp)); kkey != NULL;
 		     kkey = ISC_LIST_NEXT(kkey, link))
@@ -2142,11 +2110,11 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 		}
 
 		/* Check purge-keys interval. */
-		if (dns_keymgr_key_may_be_purged(dkey->key,
-						 dns_kasp_purgekeys(kasp), now))
+		if (keymgr_key_may_be_purged(dkey->key,
+					     dns_kasp_purgekeys(kasp), now))
 		{
 			dst_key_format(dkey->key, keystr, sizeof(keystr));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_INFO,
 				      "keymgr: purge DNSKEY %s (%s) according "
 				      "to policy %s",
@@ -2176,7 +2144,7 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 				/* Found a match. */
 				dst_key_format(dkey->key, keystr,
 					       sizeof(keystr));
-				isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+				isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 					      DNS_LOGMODULE_DNSSEC,
 					      ISC_LOG_DEBUG(1),
 					      "keymgr: DNSKEY %s (%s) matches "
@@ -2193,7 +2161,6 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 					 * matches the kasp policy.
 					 */
 					if (!dst_key_is_unused(dkey->key) &&
-					    !dst_key_doublematch(dkey, kasp) &&
 					    (dst_key_goal(dkey->key) ==
 					     OMNIPRESENT) &&
 					    !keymgr_dep(dkey->key, keyring,
@@ -2240,7 +2207,6 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 					dst_key_format(dnskey->key, keystr,
 						       sizeof(keystr));
 					isc_log_write(
-						dns_lctx,
 						DNS_LOGCATEGORY_DNSSEC,
 						DNS_LOGMODULE_DNSSEC,
 						ISC_LOG_DEBUG(1),
@@ -2296,11 +2262,11 @@ dns_keymgr_run(const dns_name_t *origin, dns_rdataclass_t rdclass,
 			RETERR(dst_key_tofile(dkey->key, options, directory));
 			dst_key_setmodified(dkey->key, false);
 
-			if (!isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(3))) {
+			if (!isc_log_wouldlog(ISC_LOG_DEBUG(3))) {
 				continue;
 			}
 			dst_key_format(dkey->key, keystr, sizeof(keystr));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(3),
 				      "keymgr: DNSKEY %s (%s) "
 				      "saved to directory %s, policy %s",
@@ -2322,12 +2288,11 @@ failure:
 		}
 	}
 
-	if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(3))) {
+	if (isc_log_wouldlog(ISC_LOG_DEBUG(3))) {
 		char namebuf[DNS_NAME_FORMATSIZE];
 		dns_name_format(origin, namebuf, sizeof(namebuf));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(3),
-			      "keymgr: %s done", namebuf);
+		isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_DEBUG(3), "keymgr: %s done", namebuf);
 	}
 	return result;
 }
@@ -2390,14 +2355,14 @@ keymgr_checkds(dns_kasp_t *kasp, dns_dnsseckeylist_t *keyring,
 		}
 	}
 
-	if (isc_log_wouldlog(dns_lctx, ISC_LOG_NOTICE)) {
+	if (isc_log_wouldlog(ISC_LOG_NOTICE)) {
 		char keystr[DST_KEY_FORMATSIZE];
 		char timestr[26]; /* Minimal buf as per ctime_r() spec. */
 
 		dst_key_format(ksk_key->key, keystr, sizeof(keystr));
 		isc_stdtime_tostring(when, timestr, sizeof(timestr));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_NOTICE,
+		isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_NOTICE,
 			      "keymgr: checkds DS for key %s seen %s at %s",
 			      keystr, dspublish ? "published" : "withdrawn",
 			      timestr);
@@ -2432,38 +2397,36 @@ dns_keymgr_checkds_id(dns_kasp_t *kasp, dns_dnsseckeylist_t *keyring,
 			      true);
 }
 
-static isc_result_t
+static void
 keytime_status(dst_key_t *key, isc_stdtime_t now, isc_buffer_t *buf,
 	       const char *pre, int ks, int kt) {
 	char timestr[26]; /* Minimal buf as per ctime_r() spec. */
-	isc_result_t result = ISC_R_SUCCESS;
+	isc_result_t ret;
 	isc_stdtime_t when = 0;
 	dst_key_state_t state = NA;
 
-	RETERR(isc_buffer_printf(buf, "%s", pre));
+	isc_buffer_printf(buf, "%s", pre);
 	(void)dst_key_getstate(key, ks, &state);
-	isc_result_t r = dst_key_gettime(key, kt, &when);
+	ret = dst_key_gettime(key, kt, &when);
 	if (state == RUMOURED || state == OMNIPRESENT) {
-		RETERR(isc_buffer_printf(buf, "yes - since "));
+		isc_buffer_printf(buf, "yes - since ");
 	} else if (now < when) {
-		RETERR(isc_buffer_printf(buf, "no  - scheduled "));
+		isc_buffer_printf(buf, "no  - scheduled ");
 	} else {
-		return isc_buffer_printf(buf, "no\n");
+		isc_buffer_printf(buf, "no\n");
+		return;
 	}
-	if (r == ISC_R_SUCCESS) {
+	if (ret == ISC_R_SUCCESS) {
 		isc_stdtime_tostring(when, timestr, sizeof(timestr));
-		RETERR(isc_buffer_printf(buf, "%s\n", timestr));
+		isc_buffer_printf(buf, "%s\n", timestr);
 	}
-
-failure:
-	return result;
 }
 
-static isc_result_t
+static void
 rollover_status(dns_dnsseckey_t *dkey, dns_kasp_t *kasp, isc_stdtime_t now,
 		isc_buffer_t *buf, bool zsk) {
 	char timestr[26]; /* Minimal buf as per ctime_r() spec. */
-	isc_result_t result = ISC_R_SUCCESS;
+	isc_result_t ret = ISC_R_SUCCESS;
 	isc_stdtime_t active_time = 0;
 	dst_key_state_t state = NA, goal = NA;
 	int rrsig, active, retire;
@@ -2479,14 +2442,14 @@ rollover_status(dns_dnsseckey_t *dkey, dns_kasp_t *kasp, isc_stdtime_t now,
 		retire = DST_TIME_DELETE;
 	}
 
-	RETERR(isc_buffer_printf(buf, "\n"));
+	isc_buffer_printf(buf, "\n");
 
 	(void)dst_key_getstate(key, DST_KEY_GOAL, &goal);
 	(void)dst_key_getstate(key, rrsig, &state);
 	(void)dst_key_gettime(key, active, &active_time);
 	if (active_time == 0) {
 		// only interested in keys that were once active.
-		return ISC_R_SUCCESS;
+		return;
 	}
 
 	if (goal == HIDDEN && (state == UNRETENTIVE || state == HIDDEN)) {
@@ -2495,88 +2458,79 @@ rollover_status(dns_dnsseckey_t *dkey, dns_kasp_t *kasp, isc_stdtime_t now,
 		state = NA;
 		(void)dst_key_getstate(key, DST_KEY_DNSKEY, &state);
 		if (state == RUMOURED || state == OMNIPRESENT) {
-			result = dst_key_gettime(key, DST_TIME_DELETE,
-						 &remove_time);
-			if (result == ISC_R_SUCCESS) {
-				RETERR(isc_buffer_printf(
-					buf, "  Key is retired, will be "
-					     "removed on "));
+			ret = dst_key_gettime(key, DST_TIME_DELETE,
+					      &remove_time);
+			if (ret == ISC_R_SUCCESS) {
+				isc_buffer_printf(buf, "  Key is retired, will "
+						       "be removed on ");
 				isc_stdtime_tostring(remove_time, timestr,
 						     sizeof(timestr));
-				RETERR(isc_buffer_printf(buf, "%s", timestr));
+				isc_buffer_printf(buf, "%s", timestr);
 			}
 		} else {
-			RETERR(isc_buffer_printf(buf, "  Key has been removed "
-						      "from the zone"));
+			isc_buffer_printf(
+				buf, "  Key has been removed from the zone");
 		}
 	} else {
 		isc_stdtime_t retire_time = 0;
-		result = dst_key_gettime(key, retire, &retire_time);
-		if (result == ISC_R_SUCCESS) {
+		ret = dst_key_gettime(key, retire, &retire_time);
+		if (ret == ISC_R_SUCCESS) {
 			if (now < retire_time) {
 				if (goal == OMNIPRESENT) {
-					RETERR(isc_buffer_printf(
-						buf, "  Next rollover "
-						     "scheduled on "));
+					isc_buffer_printf(buf,
+							  "  Next rollover "
+							  "scheduled on ");
 					retire_time = keymgr_prepublication_time(
 						dkey, kasp,
-						retire_time - active_time, now);
+						(retire_time - active_time),
+						now);
 				} else {
-					RETERR(isc_buffer_printf(
-						buf, "  Key will retire on "));
+					isc_buffer_printf(
+						buf, "  Key will retire on ");
 				}
 			} else {
-				RETERR(isc_buffer_printf(buf, "  Rollover is "
-							      "due since "));
+				isc_buffer_printf(buf,
+						  "  Rollover is due since ");
 			}
 			isc_stdtime_tostring(retire_time, timestr,
 					     sizeof(timestr));
-			RETERR(isc_buffer_printf(buf, "%s", timestr));
+			isc_buffer_printf(buf, "%s", timestr);
 		} else {
-			RETERR(isc_buffer_printf(buf,
-						 "  No rollover scheduled"));
+			isc_buffer_printf(buf, "  No rollover scheduled");
 		}
 	}
-	RETERR(isc_buffer_printf(buf, "\n"));
-
-failure:
-	return result;
+	isc_buffer_printf(buf, "\n");
 }
 
-static isc_result_t
+static void
 keystate_status(dst_key_t *key, isc_buffer_t *buf, const char *pre, int ks) {
 	dst_key_state_t state = NA;
-	isc_result_t result = ISC_R_SUCCESS;
 
 	(void)dst_key_getstate(key, ks, &state);
 	switch (state) {
 	case HIDDEN:
-		RETERR(isc_buffer_printf(buf, "  - %shidden\n", pre));
+		isc_buffer_printf(buf, "  - %shidden\n", pre);
 		break;
 	case RUMOURED:
-		RETERR(isc_buffer_printf(buf, "  - %srumoured\n", pre));
+		isc_buffer_printf(buf, "  - %srumoured\n", pre);
 		break;
 	case OMNIPRESENT:
-		RETERR(isc_buffer_printf(buf, "  - %somnipresent\n", pre));
+		isc_buffer_printf(buf, "  - %somnipresent\n", pre);
 		break;
 	case UNRETENTIVE:
-		RETERR(isc_buffer_printf(buf, "  - %sunretentive\n", pre));
+		isc_buffer_printf(buf, "  - %sunretentive\n", pre);
 		break;
 	case NA:
 	default:
 		/* print nothing */
 		break;
 	}
-
-failure:
-	return result;
 }
 
-isc_result_t
+void
 dns_keymgr_status(dns_kasp_t *kasp, dns_dnsseckeylist_t *keyring,
 		  isc_stdtime_t now, char *out, size_t out_len) {
 	isc_buffer_t buf;
-	isc_result_t result = ISC_R_SUCCESS;
 	char timestr[26]; /* Minimal buf as per ctime_r() spec. */
 
 	REQUIRE(DNS_KASP_VALID(kasp));
@@ -2586,17 +2540,17 @@ dns_keymgr_status(dns_kasp_t *kasp, dns_dnsseckeylist_t *keyring,
 	isc_buffer_init(&buf, out, out_len);
 
 	// policy name
-	RETERR(isc_buffer_printf(&buf, "dnssec-policy: %s\n",
-				 dns_kasp_getname(kasp)));
-	RETERR(isc_buffer_printf(&buf, "current time:  "));
+	isc_buffer_printf(&buf, "dnssec-policy: %s\n", dns_kasp_getname(kasp));
+	isc_buffer_printf(&buf, "current time:  ");
 	isc_stdtime_tostring(now, timestr, sizeof(timestr));
-	RETERR(isc_buffer_printf(&buf, "%s\n", timestr));
+	isc_buffer_printf(&buf, "%s\n", timestr);
 
 	for (dns_dnsseckey_t *dkey = ISC_LIST_HEAD(*keyring); dkey != NULL;
 	     dkey = ISC_LIST_NEXT(dkey, link))
 	{
 		char algstr[DNS_NAME_FORMATSIZE];
 		bool ksk = false, zsk = false;
+		isc_result_t ret;
 
 		if (dst_key_is_unused(dkey->key)) {
 			continue;
@@ -2605,48 +2559,44 @@ dns_keymgr_status(dns_kasp_t *kasp, dns_dnsseckeylist_t *keyring,
 		// key data
 		dns_secalg_format((dns_secalg_t)dst_key_alg(dkey->key), algstr,
 				  sizeof(algstr));
-		RETERR(isc_buffer_printf(&buf, "\nkey: %d (%s), %s\n",
-					 dst_key_id(dkey->key), algstr,
-					 keymgr_keyrole(dkey->key)));
+		isc_buffer_printf(&buf, "\nkey: %d (%s), %s\n",
+				  dst_key_id(dkey->key), algstr,
+				  keymgr_keyrole(dkey->key));
 
 		// publish status
-		RETERR(keytime_status(dkey->key, now, &buf,
-				      "  published:      ", DST_KEY_DNSKEY,
-				      DST_TIME_PUBLISH));
+		keytime_status(dkey->key, now, &buf,
+			       "  published:      ", DST_KEY_DNSKEY,
+			       DST_TIME_PUBLISH);
 
 		// signing status
-		result = dst_key_getbool(dkey->key, DST_BOOL_KSK, &ksk);
-		if (result == ISC_R_SUCCESS && ksk) {
-			RETERR(keytime_status(
-				dkey->key, now, &buf, "  key signing:    ",
-				DST_KEY_KRRSIG, DST_TIME_PUBLISH));
+		ret = dst_key_getbool(dkey->key, DST_BOOL_KSK, &ksk);
+		if (ret == ISC_R_SUCCESS && ksk) {
+			keytime_status(dkey->key, now, &buf,
+				       "  key signing:    ", DST_KEY_KRRSIG,
+				       DST_TIME_PUBLISH);
 		}
-		result = dst_key_getbool(dkey->key, DST_BOOL_ZSK, &zsk);
-		if (result == ISC_R_SUCCESS && zsk) {
-			RETERR(keytime_status(
-				dkey->key, now, &buf, "  zone signing:   ",
-				DST_KEY_ZRRSIG, DST_TIME_ACTIVATE));
+		ret = dst_key_getbool(dkey->key, DST_BOOL_ZSK, &zsk);
+		if (ret == ISC_R_SUCCESS && zsk) {
+			keytime_status(dkey->key, now, &buf,
+				       "  zone signing:   ", DST_KEY_ZRRSIG,
+				       DST_TIME_ACTIVATE);
 		}
 
 		// rollover status
-		RETERR(rollover_status(dkey, kasp, now, &buf, zsk));
+		rollover_status(dkey, kasp, now, &buf, zsk);
 
 		// key states
-		RETERR(keystate_status(dkey->key, &buf,
-				       "goal:           ", DST_KEY_GOAL));
-		RETERR(keystate_status(dkey->key, &buf,
-				       "dnskey:         ", DST_KEY_DNSKEY));
-		RETERR(keystate_status(dkey->key, &buf,
-				       "ds:             ", DST_KEY_DS));
-		RETERR(keystate_status(dkey->key, &buf,
-				       "zone rrsig:     ", DST_KEY_ZRRSIG));
-		RETERR(keystate_status(dkey->key, &buf,
-				       "key rrsig:      ", DST_KEY_KRRSIG));
+		keystate_status(dkey->key, &buf,
+				"goal:           ", DST_KEY_GOAL);
+		keystate_status(dkey->key, &buf,
+				"dnskey:         ", DST_KEY_DNSKEY);
+		keystate_status(dkey->key, &buf,
+				"ds:             ", DST_KEY_DS);
+		keystate_status(dkey->key, &buf,
+				"zone rrsig:     ", DST_KEY_ZRRSIG);
+		keystate_status(dkey->key, &buf,
+				"key rrsig:      ", DST_KEY_KRRSIG);
 	}
-
-failure:
-
-	return result;
 }
 
 isc_result_t
@@ -2873,11 +2823,11 @@ dns_keymgr_offline(const dns_name_t *origin, dns_dnsseckeylist_t *keyring,
 			RETERR(dst_key_tofile(dkey->key, options, directory));
 			dst_key_setmodified(dkey->key, false);
 
-			if (!isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(3))) {
+			if (!isc_log_wouldlog(ISC_LOG_DEBUG(3))) {
 				continue;
 			}
 			dst_key_format(dkey->key, keystr, sizeof(keystr));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC,
 				      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(3),
 				      "keymgr: DNSKEY %s (%s) "
 				      "saved to directory %s, policy %s",
@@ -2890,12 +2840,12 @@ dns_keymgr_offline(const dns_name_t *origin, dns_dnsseckeylist_t *keyring,
 	result = ISC_R_SUCCESS;
 
 failure:
-	if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(3))) {
+	if (isc_log_wouldlog(ISC_LOG_DEBUG(3))) {
 		char namebuf[DNS_NAME_FORMATSIZE];
 		dns_name_format(origin, namebuf, sizeof(namebuf));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-			      DNS_LOGMODULE_DNSSEC, ISC_LOG_DEBUG(3),
-			      "keymgr: %s (offline-ksk) done", namebuf);
+		isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_DNSSEC,
+			      ISC_LOG_DEBUG(3), "keymgr: %s (offline-ksk) done",
+			      namebuf);
 	}
 	return result;
 }
