@@ -21,6 +21,7 @@
 #include <isc/atomic.h>
 #include <isc/buffer.h>
 #include <isc/file.h>
+#include <isc/log.h>
 #include <isc/loop.h>
 #include <isc/magic.h>
 #include <isc/mem.h>
@@ -36,13 +37,11 @@
 #include <dns/db.h>
 #include <dns/dbiterator.h>
 #include <dns/fixedname.h>
-#include <dns/log.h>
 #include <dns/master.h>
 #include <dns/masterdump.h>
 #include <dns/ncache.h>
 #include <dns/rdata.h>
 #include <dns/rdataclass.h>
-#include <dns/rdataset.h>
 #include <dns/rdatasetiter.h>
 #include <dns/rdatatype.h>
 #include <dns/time.h>
@@ -769,17 +768,10 @@ rdataset_totext(dns_rdataset_t *rdataset, const dns_name_t *owner_name,
 		INDENT_TO(rdata_column);
 		if ((rdataset->attributes & DNS_RDATASETATTR_NEGATIVE) != 0) {
 			if (NXDOMAIN(rdataset)) {
-				RETERR(str_totext(";-$NXDOMAIN", target));
+				RETERR(str_totext(";-$NXDOMAIN\n", target));
 			} else {
-				RETERR(str_totext(";-$NXRRSET", target));
+				RETERR(str_totext(";-$NXRRSET\n", target));
 			}
-			if (start != NULL) {
-				RETERR(yaml_stringify(target, start));
-				RETERR(str_totext("'\n", target));
-			} else {
-				RETERR(str_totext("\n", target));
-			}
-
 			/*
 			 * Print a summary of the cached records which make
 			 * up the negative response.
@@ -1162,14 +1154,17 @@ again:
 		} else {
 			isc_result_t result;
 			if (STALE(rds)) {
+				fprintf(f, "; stale\n");
+			} else if (ANCIENT(rds)) {
 				isc_buffer_t b;
 				char buf[sizeof("YYYYMMDDHHMMSS")];
 				memset(buf, 0, sizeof(buf));
 				isc_buffer_init(&b, buf, sizeof(buf) - 1);
-				dns_time64_totext((uint64_t)rds->expire, &b);
-				fprintf(f, "; stale since %s\n", buf);
-			} else if (ANCIENT(rds)) {
-				fprintf(f, "; expired (awaiting cleanup)\n");
+				dns_time64_totext((uint64_t)rds->ttl, &b);
+				fprintf(f,
+					"; expired since %s "
+					"(awaiting cleanup)\n",
+					buf);
 			}
 			result = dump_rdataset(mctx, name, rds, ctx, buffer, f);
 			if (result != ISC_R_SUCCESS) {
@@ -1452,12 +1447,12 @@ flushandsync(FILE *f, isc_result_t result, const char *temp) {
 	}
 	if (result != ISC_R_SUCCESS && logit) {
 		if (temp != NULL) {
-			isc_log_write(dns_lctx, ISC_LOGCATEGORY_GENERAL,
+			isc_log_write(ISC_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTERDUMP, ISC_LOG_ERROR,
 				      "dumping to master file: %s: flush: %s",
 				      temp, isc_result_totext(result));
 		} else {
-			isc_log_write(dns_lctx, ISC_LOGCATEGORY_GENERAL,
+			isc_log_write(ISC_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTERDUMP, ISC_LOG_ERROR,
 				      "dumping to stream: flush: %s",
 				      isc_result_totext(result));
@@ -1470,12 +1465,12 @@ flushandsync(FILE *f, isc_result_t result, const char *temp) {
 	}
 	if (result != ISC_R_SUCCESS && logit) {
 		if (temp != NULL) {
-			isc_log_write(dns_lctx, ISC_LOGCATEGORY_GENERAL,
+			isc_log_write(ISC_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTERDUMP, ISC_LOG_ERROR,
 				      "dumping to master file: %s: fsync: %s",
 				      temp, isc_result_totext(result));
 		} else {
-			isc_log_write(dns_lctx, ISC_LOGCATEGORY_GENERAL,
+			isc_log_write(ISC_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTERDUMP, ISC_LOG_ERROR,
 				      "dumping to stream: fsync: %s",
 				      isc_result_totext(result));
@@ -1500,8 +1495,8 @@ closeandrename(FILE *f, isc_result_t result, const char *temp,
 		result = tresult;
 	}
 	if (result != ISC_R_SUCCESS && logit) {
-		isc_log_write(dns_lctx, ISC_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_MASTERDUMP, ISC_LOG_ERROR,
+		isc_log_write(ISC_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTERDUMP,
+			      ISC_LOG_ERROR,
 			      "dumping master file: %s: fclose: %s", temp,
 			      isc_result_totext(result));
 		logit = false;
@@ -1512,8 +1507,8 @@ closeandrename(FILE *f, isc_result_t result, const char *temp,
 		(void)isc_file_remove(temp);
 	}
 	if (result != ISC_R_SUCCESS && logit) {
-		isc_log_write(dns_lctx, ISC_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_MASTERDUMP, ISC_LOG_ERROR,
+		isc_log_write(ISC_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTERDUMP,
+			      ISC_LOG_ERROR,
 			      "dumping master file: rename: %s: %s", file,
 			      isc_result_totext(result));
 	}
@@ -1856,8 +1851,8 @@ opentmp(isc_mem_t *mctx, const char *file, char **tempp, FILE **fp) {
 
 	result = isc_file_openunique(tempname, &f);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(dns_lctx, ISC_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_MASTERDUMP, ISC_LOG_ERROR,
+		isc_log_write(ISC_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTERDUMP,
+			      ISC_LOG_ERROR,
 			      "dumping master file: %s: open: %s", tempname,
 			      isc_result_totext(result));
 		goto cleanup;
