@@ -41,6 +41,13 @@
 
 #include <dst/dst.h>
 
+#define CHECK(x)                             \
+	do {                                 \
+		result = (x);                \
+		if (result != ISC_R_SUCCESS) \
+			goto failure;        \
+	} while (0)
+
 #define OPTOUT(x)  (((x) & DNS_NSEC3FLAG_OPTOUT) != 0)
 #define CREATE(x)  (((x) & DNS_NSEC3FLAG_CREATE) != 0)
 #define INITIAL(x) (((x) & DNS_NSEC3FLAG_INITIAL) != 0)
@@ -435,16 +442,19 @@ delnsec3(dns_db_t *db, dns_dbversion_t *version, const dns_name_t *name,
 			continue;
 		}
 
-		CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_DEL, name,
-					   rdataset.ttl, &rdata, &tuple));
-		CHECK(do_one_tuple(&tuple, db, version, diff));
+		dns_difftuple_create(diff->mctx, DNS_DIFFOP_DEL, name,
+				     rdataset.ttl, &rdata, &tuple);
+		result = do_one_tuple(&tuple, db, version, diff);
+		if (result != ISC_R_SUCCESS) {
+			goto failure;
+		}
 	}
 	if (result != ISC_R_NOMORE) {
-		goto cleanup;
+		goto failure;
 	}
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	dns_rdataset_disassociate(&rdataset);
 cleanup_node:
 	dns_db_detachnode(db, &node);
@@ -517,7 +527,7 @@ find_nsec3(dns_rdata_nsec3_t *nsec3, dns_rdataset_t *rdataset,
 			break;
 		}
 	}
-cleanup:
+failure:
 	return result;
 }
 
@@ -589,24 +599,7 @@ dns_nsec3_addnsec3(dns_db_t *db, dns_dbversion_t *version,
 	 * Create the node if it doesn't exist and hold
 	 * a reference to it until we have added the NSEC3.
 	 */
-	result = dns_db_findnsec3node(db, hashname, false, &newnode);
-	if (result != ISC_R_SUCCESS) {
-		isc_result_t tresult;
-
-		CHECK(dns_db_createiterator(db, DNS_DB_NSEC3ONLY, &dbit));
-		tresult = dns_dbiterator_seek3(dbit, hashname);
-		CHECK(dns_dbiterator_pause(dbit));
-
-		if (tresult != ISC_R_SUCCESS) {
-			/* Nothing in the NSEC3 space yet. */
-			if (!unsecure) {
-				goto addnsec3;
-			}
-			goto cleanup;
-		}
-
-		goto find_previous;
-	}
+	CHECK(dns_db_findnsec3node(db, hashname, true, &newnode));
 
 	/*
 	 * Seek the iterator to the 'newnode'.
@@ -642,19 +635,18 @@ dns_nsec3_addnsec3(dns_db_t *db, dns_dbversion_t *version,
 			} else if (CREATE(nsec3param->flags) && OPTOUT(flags)) {
 				result = dns_nsec3_delnsec3(db, version, name,
 							    nsec3param, diff);
-				goto cleanup;
+				goto failure;
 			} else {
 				maybe_remove_unsecure = true;
 			}
 		} else {
 			dns_rdataset_disassociate(&rdataset);
 			if (result != ISC_R_NOMORE) {
-				goto cleanup;
+				goto failure;
 			}
 		}
 	}
 
-find_previous:
 	/*
 	 * Find the previous NSEC3 (if any) and update it if required.
 	 */
@@ -680,7 +672,9 @@ find_previous:
 			dns_rdataset_disassociate(&rdataset);
 			continue;
 		}
-		CHECK(result);
+		if (result != ISC_R_SUCCESS) {
+			goto failure;
+		}
 
 		if (maybe_remove_unsecure) {
 			dns_rdataset_disassociate(&rdataset);
@@ -692,7 +686,7 @@ find_previous:
 			if (OPTOUT(nsec3.flags)) {
 				result = dns_nsec3_delnsec3(db, version, name,
 							    nsec3param, diff);
-				goto cleanup;
+				goto failure;
 			}
 			goto addnsec3;
 		} else {
@@ -702,7 +696,7 @@ find_previous:
 			 */
 			if (OPTOUT(nsec3.flags) && unsecure) {
 				dns_rdataset_disassociate(&rdataset);
-				goto cleanup;
+				goto failure;
 			}
 		}
 
@@ -723,8 +717,8 @@ find_previous:
 		CHECK(dns_rdata_fromstruct(&rdata, rdataset.rdclass,
 					   dns_rdatatype_nsec3, &nsec3,
 					   &buffer));
-		CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, prev,
-					   rdataset.ttl, &rdata, &tuple));
+		dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, prev,
+				     rdataset.ttl, &rdata, &tuple);
 		CHECK(do_one_tuple(&tuple, db, version, diff));
 		INSIST(old_length <= sizeof(nexthash));
 		memmove(nexthash, old_next, old_length);
@@ -740,10 +734,6 @@ addnsec3:
 	/*
 	 * Create the NSEC3 RDATA.
 	 */
-	if (newnode == NULL) {
-		CHECK(dns_db_findnsec3node(db, hashname, true, &newnode));
-	}
-
 	CHECK(dns_db_findnode(db, name, false, &node));
 	CHECK(dns_nsec3_buildrdata(db, version, node, hash, flags, iterations,
 				   salt, salt_length, nexthash, next_length,
@@ -757,8 +747,8 @@ addnsec3:
 	/*
 	 * Add the new NSEC3 and record the change.
 	 */
-	CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, hashname,
-				   nsecttl, &rdata, &tuple));
+	dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, hashname, nsecttl,
+			     &rdata, &tuple);
 	CHECK(do_one_tuple(&tuple, db, version, diff));
 	INSIST(tuple == NULL);
 	dns_rdata_reset(&rdata);
@@ -786,7 +776,7 @@ addnsec3:
 		/*
 		 * Create the node if it doesn't exist and hold
 		 * a reference to it until we have added the NSEC3
-		 * or we discover we don't need to make a change.
+		 * or we discover we don't need to add make a change.
 		 */
 		CHECK(dns_db_findnsec3node(db, hashname, true, &newnode));
 		result = dns_db_findrdataset(db, newnode, version,
@@ -800,19 +790,8 @@ addnsec3:
 				break;
 			}
 			if (result != ISC_R_NOMORE) {
-				goto cleanup;
+				goto failure;
 			}
-		} else if (result == ISC_R_NOTFOUND) {
-			/*
-			 * If we didn't find an NSEC3 in the node,
-			 * then the node must have been newly created
-			 * by dns_db_findnsec3node(). The iterator
-			 * needs to be updated so we can seek for
-			 * the node's predecessor.
-			 */
-			dns_dbiterator_destroy(&dbit);
-			CHECK(dns_db_createiterator(db, DNS_DB_NSEC3ONLY,
-						    &dbit));
 		}
 
 		/*
@@ -840,7 +819,9 @@ addnsec3:
 				dns_rdataset_disassociate(&rdataset);
 				continue;
 			}
-			CHECK(result);
+			if (result != ISC_R_SUCCESS) {
+				goto failure;
+			}
 
 			old_next = nsec3.next;
 			old_length = nsec3.next_length;
@@ -859,9 +840,8 @@ addnsec3:
 			CHECK(dns_rdata_fromstruct(&rdata, rdataset.rdclass,
 						   dns_rdatatype_nsec3, &nsec3,
 						   &buffer));
-			CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD,
-						   prev, rdataset.ttl, &rdata,
-						   &tuple));
+			dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, prev,
+					     rdataset.ttl, &rdata, &tuple);
 			CHECK(do_one_tuple(&tuple, db, version, diff));
 			INSIST(old_length <= sizeof(nexthash));
 			memmove(nexthash, old_next, old_length);
@@ -889,8 +869,8 @@ addnsec3:
 		/*
 		 * Add the new NSEC3 and record the change.
 		 */
-		CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, hashname,
-					   nsecttl, &rdata, &tuple));
+		dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, hashname,
+				     nsecttl, &rdata, &tuple);
 		CHECK(do_one_tuple(&tuple, db, version, diff));
 		INSIST(tuple == NULL);
 		dns_rdata_reset(&rdata);
@@ -900,7 +880,7 @@ addnsec3:
 	/* result cannot be ISC_R_NOMORE here */
 	INSIST(result != ISC_R_NOMORE);
 
-cleanup:
+failure:
 	if (dbit != NULL) {
 		dns_dbiterator_destroy(&dbit);
 	}
@@ -974,7 +954,7 @@ dns_nsec3_addnsec3s(dns_db_t *db, dns_dbversion_t *version,
 		result = ISC_R_SUCCESS;
 	}
 
-cleanup:
+failure:
 	if (dns_rdataset_isassociated(&rdataset)) {
 		dns_rdataset_disassociate(&rdataset);
 	}
@@ -1047,7 +1027,7 @@ rr_exists(dns_db_t *db, dns_dbversion_t *ver, const dns_name_t *name,
 	if (result == ISC_R_NOTFOUND) {
 		*flag = false;
 		result = ISC_R_SUCCESS;
-		goto cleanup;
+		goto failure;
 	}
 
 	for (result = dns_rdataset_first(&rdataset); result == ISC_R_SUCCESS;
@@ -1067,7 +1047,7 @@ rr_exists(dns_db_t *db, dns_dbversion_t *ver, const dns_name_t *name,
 		result = ISC_R_SUCCESS;
 	}
 
-cleanup:
+failure:
 	if (node != NULL) {
 		dns_db_detachnode(db, &node);
 	}
@@ -1139,7 +1119,9 @@ dns_nsec3param_deletechains(dns_db_t *db, dns_dbversion_t *ver,
 	if (result == ISC_R_NOTFOUND) {
 		goto try_private;
 	}
-	CHECK(result);
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
+	}
 
 	for (result = dns_rdataset_first(&rdataset); result == ISC_R_SUCCESS;
 	     result = dns_rdataset_next(&rdataset))
@@ -1157,32 +1139,31 @@ dns_nsec3param_deletechains(dns_db_t *db, dns_dbversion_t *ver,
 		CHECK(rr_exists(db, ver, origin, &private, &flag));
 
 		if (!flag) {
-			CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD,
-						   origin, 0, &private,
-						   &tuple));
+			dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, origin,
+					     0, &private, &tuple);
 			CHECK(do_one_tuple(&tuple, db, ver, diff));
 			INSIST(tuple == NULL);
 		}
 		dns_rdata_reset(&rdata);
 	}
 	if (result != ISC_R_NOMORE) {
-		goto cleanup;
+		goto failure;
 	}
 
 	dns_rdataset_disassociate(&rdataset);
 
 try_private:
 	if (privatetype == 0) {
-		result = ISC_R_SUCCESS;
-		goto cleanup;
+		goto success;
 	}
 	result = dns_db_findrdataset(db, node, ver, privatetype, 0,
 				     (isc_stdtime_t)0, &rdataset, NULL);
 	if (result == ISC_R_NOTFOUND) {
-		result = ISC_R_SUCCESS;
-		goto cleanup;
+		goto success;
+	}
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
 	}
-	CHECK(result);
 
 	for (result = dns_rdataset_first(&rdataset); result == ISC_R_SUCCESS;
 	     result = dns_rdataset_next(&rdataset))
@@ -1203,8 +1184,8 @@ try_private:
 			continue;
 		}
 
-		CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_DEL, origin,
-					   0, &rdata, &tuple));
+		dns_difftuple_create(diff->mctx, DNS_DIFFOP_DEL, origin, 0,
+				     &rdata, &tuple);
 		CHECK(do_one_tuple(&tuple, db, ver, diff));
 		INSIST(tuple == NULL);
 
@@ -1217,19 +1198,19 @@ try_private:
 		CHECK(rr_exists(db, ver, origin, &rdata, &flag));
 
 		if (!flag) {
-			CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD,
-						   origin, 0, &rdata, &tuple));
+			dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, origin,
+					     0, &rdata, &tuple);
 			CHECK(do_one_tuple(&tuple, db, ver, diff));
 			INSIST(tuple == NULL);
 		}
 	}
 	if (result != ISC_R_NOMORE) {
-		goto cleanup;
+		goto failure;
 	}
-
+success:
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	if (dns_rdataset_isassociated(&rdataset)) {
 		dns_rdataset_disassociate(&rdataset);
 	}
@@ -1261,7 +1242,7 @@ dns_nsec3_addnsec3sx(dns_db_t *db, dns_dbversion_t *version,
 	result = dns_db_findrdataset(db, node, version, type, 0, 0, &prdataset,
 				     NULL);
 	if (result != ISC_R_SUCCESS && result != ISC_R_NOTFOUND) {
-		CHECK(result);
+		goto failure;
 	}
 
 	result = dns_db_findrdataset(db, node, version,
@@ -1270,7 +1251,9 @@ dns_nsec3_addnsec3sx(dns_db_t *db, dns_dbversion_t *version,
 	if (result == ISC_R_NOTFOUND) {
 		goto try_private;
 	}
-	CHECK(result);
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
+	}
 
 	/*
 	 * Update each active NSEC3 chain.
@@ -1294,17 +1277,15 @@ dns_nsec3_addnsec3sx(dns_db_t *db, dns_dbversion_t *version,
 					 nsecttl, unsecure, diff));
 	}
 	if (result != ISC_R_NOMORE) {
-		goto cleanup;
+		goto failure;
 	}
 
 	dns_rdataset_disassociate(&rdataset);
 
 try_private:
 	if (!dns_rdataset_isassociated(&prdataset)) {
-		result = ISC_R_SUCCESS;
-		goto cleanup;
+		goto success;
 	}
-
 	/*
 	 * Update each active NSEC3 chain.
 	 */
@@ -1337,10 +1318,10 @@ try_private:
 					 nsecttl, unsecure, diff));
 	}
 	if (result == ISC_R_NOMORE) {
+	success:
 		result = ISC_R_SUCCESS;
 	}
-
-cleanup:
+failure:
 	if (dns_rdataset_isassociated(&rdataset)) {
 		dns_rdataset_disassociate(&rdataset);
 	}
@@ -1448,7 +1429,9 @@ dns_nsec3_delnsec3(dns_db_t *db, dns_dbversion_t *version,
 	if (result == ISC_R_NOTFOUND || result == DNS_R_PARTIALMATCH) {
 		goto cleanup_orphaned_ents;
 	}
-	CHECK(result);
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
+	}
 
 	CHECK(dns_dbiterator_current(dbit, &node, NULL));
 	CHECK(dns_dbiterator_pause(dbit));
@@ -1458,7 +1441,9 @@ dns_nsec3_delnsec3(dns_db_t *db, dns_dbversion_t *version,
 	if (result == ISC_R_NOTFOUND) {
 		goto cleanup_orphaned_ents;
 	}
-	CHECK(result);
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
+	}
 
 	/*
 	 * If we find a existing NSEC3 for this chain then save the
@@ -1472,9 +1457,11 @@ dns_nsec3_delnsec3(dns_db_t *db, dns_dbversion_t *version,
 	}
 	dns_rdataset_disassociate(&rdataset);
 	if (result == ISC_R_NOMORE) {
-		result = ISC_R_SUCCESS;
+		goto success;
+	}
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
 	}
-	CHECK(result);
 
 	/*
 	 * Find the previous NSEC3 and update it.
@@ -1500,7 +1487,9 @@ dns_nsec3_delnsec3(dns_db_t *db, dns_dbversion_t *version,
 			dns_rdataset_disassociate(&rdataset);
 			continue;
 		}
-		CHECK(result);
+		if (result != ISC_R_SUCCESS) {
+			goto failure;
+		}
 
 		/*
 		 * Delete the old previous NSEC3.
@@ -1519,8 +1508,8 @@ dns_nsec3_delnsec3(dns_db_t *db, dns_dbversion_t *version,
 		CHECK(dns_rdata_fromstruct(&rdata, rdataset.rdclass,
 					   dns_rdatatype_nsec3, &nsec3,
 					   &buffer));
-		CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, prev,
-					   rdataset.ttl, &rdata, &tuple));
+		dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, prev,
+				     rdataset.ttl, &rdata, &tuple);
 		CHECK(do_one_tuple(&tuple, db, version, diff));
 		dns_rdata_reset(&rdata);
 		dns_rdataset_disassociate(&rdataset);
@@ -1554,10 +1543,11 @@ cleanup_orphaned_ents:
 					 salt_length));
 		result = dns_dbiterator_seek(dbit, hashname);
 		if (result == ISC_R_NOTFOUND || result == DNS_R_PARTIALMATCH) {
-			result = ISC_R_SUCCESS;
-			goto cleanup;
+			goto success;
+		}
+		if (result != ISC_R_SUCCESS) {
+			goto failure;
 		}
-		CHECK(result);
 
 		CHECK(dns_dbiterator_current(dbit, &node, NULL));
 		CHECK(dns_dbiterator_pause(dbit));
@@ -1566,10 +1556,11 @@ cleanup_orphaned_ents:
 					     (isc_stdtime_t)0, &rdataset, NULL);
 		dns_db_detachnode(db, &node);
 		if (result == ISC_R_NOTFOUND) {
-			result = ISC_R_SUCCESS;
-			goto cleanup;
+			goto success;
+		}
+		if (result != ISC_R_SUCCESS) {
+			goto failure;
 		}
-		CHECK(result);
 
 		result = find_nsec3(&nsec3, &rdataset, nsec3param);
 		if (result == ISC_R_SUCCESS) {
@@ -1579,9 +1570,11 @@ cleanup_orphaned_ents:
 		}
 		dns_rdataset_disassociate(&rdataset);
 		if (result == ISC_R_NOMORE) {
-			result = ISC_R_SUCCESS;
+			goto success;
+		}
+		if (result != ISC_R_SUCCESS) {
+			goto failure;
 		}
-		CHECK(result);
 
 		pass = 0;
 		do {
@@ -1604,7 +1597,9 @@ cleanup_orphaned_ents:
 				dns_rdataset_disassociate(&rdataset);
 				continue;
 			}
-			CHECK(result);
+			if (result != ISC_R_SUCCESS) {
+				goto failure;
+			}
 
 			/*
 			 * Delete the old previous NSEC3.
@@ -1620,9 +1615,8 @@ cleanup_orphaned_ents:
 			CHECK(dns_rdata_fromstruct(&rdata, rdataset.rdclass,
 						   dns_rdatatype_nsec3, &nsec3,
 						   &buffer));
-			CHECK(dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD,
-						   prev, rdataset.ttl, &rdata,
-						   &tuple));
+			dns_difftuple_create(diff->mctx, DNS_DIFFOP_ADD, prev,
+					     rdataset.ttl, &rdata, &tuple);
 			CHECK(do_one_tuple(&tuple, db, version, diff));
 			dns_rdata_reset(&rdata);
 			dns_rdataset_disassociate(&rdataset);
@@ -1637,9 +1631,10 @@ cleanup_orphaned_ents:
 		CHECK(delnsec3(db, version, hashname, nsec3param, diff));
 	} while (1);
 
+success:
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	if (dbit != NULL) {
 		dns_dbiterator_destroy(&dbit);
 	}
@@ -1683,7 +1678,9 @@ dns_nsec3_delnsec3sx(dns_db_t *db, dns_dbversion_t *version,
 	if (result == ISC_R_NOTFOUND) {
 		goto try_private;
 	}
-	CHECK(result);
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
+	}
 
 	/*
 	 * Update each active NSEC3 chain.
@@ -1708,16 +1705,16 @@ dns_nsec3_delnsec3sx(dns_db_t *db, dns_dbversion_t *version,
 
 try_private:
 	if (privatetype == 0) {
-		result = ISC_R_SUCCESS;
-		goto cleanup;
+		goto success;
 	}
 	result = dns_db_findrdataset(db, node, version, privatetype, 0, 0,
 				     &rdataset, NULL);
 	if (result == ISC_R_NOTFOUND) {
-		result = ISC_R_SUCCESS;
-		goto cleanup;
+		goto success;
+	}
+	if (result != ISC_R_SUCCESS) {
+		goto failure;
 	}
-	CHECK(result);
 
 	/*
 	 * Update each NSEC3 chain being built.
@@ -1750,10 +1747,11 @@ try_private:
 		CHECK(dns_nsec3_delnsec3(db, version, name, &nsec3param, diff));
 	}
 	if (result == ISC_R_NOMORE) {
+	success:
 		result = ISC_R_SUCCESS;
 	}
 
-cleanup:
+failure:
 	if (dns_rdataset_isassociated(&rdataset)) {
 		dns_rdataset_disassociate(&rdataset);
 	}
@@ -2163,9 +2161,9 @@ dns_nsec3_noexistnodata(dns_rdatatype_t type, const dns_name_t *name,
 				*optout = ((nsec3.flags &
 					    DNS_NSEC3FLAG_OPTOUT) != 0);
 				(*logit)(arg, ISC_LOG_DEBUG(3),
-					 *optout ? "NSEC3 indicates optout"
-						 : "NSEC3 indicates secure "
-						   "range");
+					 (*optout ? "NSEC3 indicates optout"
+						  : "NSEC3 indicates secure "
+						    "range"));
 			}
 			answer = ISC_R_SUCCESS;
 		}
