@@ -37,10 +37,10 @@
 #include "dst_parse.h"
 #include "openssl_shim.h"
 
-#define DST_RET(a)            \
-	{                     \
-		result = a;   \
-		goto cleanup; \
+#define DST_RET(a)        \
+	{                 \
+		ret = a;  \
+		goto err; \
 	}
 
 typedef struct rsa_components {
@@ -73,37 +73,35 @@ opensslrsa_components_get(const dst_key_t *key, rsa_components_t *c,
 	 */
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
 	if (EVP_PKEY_get_bn_param(pub, OSSL_PKEY_PARAM_RSA_E,
-				  (BIGNUM **)&c->e) == 1)
+				  (BIGNUM **)&c->e) != 1)
 	{
-		c->bnfree = true;
-		if (EVP_PKEY_get_bn_param(pub, OSSL_PKEY_PARAM_RSA_N,
-					  (BIGNUM **)&c->n) != 1)
-		{
-			return dst__openssl_toresult(DST_R_OPENSSLFAILURE);
-		}
-		if (!private) {
-			return ISC_R_SUCCESS;
-		}
-		(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_D,
-					    (BIGNUM **)&c->d);
-		(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_FACTOR1,
-					    (BIGNUM **)&c->p);
-		(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_FACTOR2,
-					    (BIGNUM **)&c->q);
-		(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_EXPONENT1,
-					    (BIGNUM **)&c->dmp1);
-		(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_EXPONENT2,
-					    (BIGNUM **)&c->dmq1);
-		(void)EVP_PKEY_get_bn_param(priv,
-					    OSSL_PKEY_PARAM_RSA_COEFFICIENT1,
-					    (BIGNUM **)&c->iqmp);
-		ERR_clear_error();
-		return ISC_R_SUCCESS;
-	} else {
-		ERR_clear_error();
+		return dst__openssl_toresult(DST_R_OPENSSLFAILURE);
 	}
-#endif
-#if OPENSSL_VERSION_NUMBER < 0x30000000L || OPENSSL_API_LEVEL < 30000
+
+	c->bnfree = true;
+	if (EVP_PKEY_get_bn_param(pub, OSSL_PKEY_PARAM_RSA_N,
+				  (BIGNUM **)&c->n) != 1)
+	{
+		return dst__openssl_toresult(DST_R_OPENSSLFAILURE);
+	}
+	if (!private) {
+		return ISC_R_SUCCESS;
+	}
+	(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_D,
+				    (BIGNUM **)&c->d);
+	(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_FACTOR1,
+				    (BIGNUM **)&c->p);
+	(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_FACTOR2,
+				    (BIGNUM **)&c->q);
+	(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_EXPONENT1,
+				    (BIGNUM **)&c->dmp1);
+	(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_EXPONENT2,
+				    (BIGNUM **)&c->dmq1);
+	(void)EVP_PKEY_get_bn_param(priv, OSSL_PKEY_PARAM_RSA_COEFFICIENT1,
+				    (BIGNUM **)&c->iqmp);
+	ERR_clear_error();
+	return ISC_R_SUCCESS;
+#else
 	const RSA *rsa = EVP_PKEY_get0_RSA(pub);
 	if (rsa == NULL) {
 		return dst__openssl_toresult(DST_R_OPENSSLFAILURE);
@@ -122,8 +120,6 @@ opensslrsa_components_get(const dst_key_t *key, rsa_components_t *c,
 	RSA_get0_factors(rsa, &c->p, &c->q);
 	RSA_get0_crt_params(rsa, &c->dmp1, &c->dmq1, &c->iqmp);
 	return ISC_R_SUCCESS;
-#else
-	return DST_R_OPENSSLFAILURE;
 #endif
 }
 
@@ -206,13 +202,13 @@ opensslrsa_createctx(dst_key_t *key, dst_context_t *dctx) {
 	switch (dctx->key->key_alg) {
 	case DST_ALG_RSASHA1:
 	case DST_ALG_NSEC3RSASHA1:
-		type = EVP_sha1(); /* SHA1 + RSA */
+		type = isc__crypto_sha1; /* SHA1 + RSA */
 		break;
 	case DST_ALG_RSASHA256:
-		type = EVP_sha256(); /* SHA256 + RSA */
+		type = isc__crypto_sha256; /* SHA256 + RSA */
 		break;
 	case DST_ALG_RSASHA512:
-		type = EVP_sha512();
+		type = isc__crypto_sha512;
 		break;
 	default:
 		UNREACHABLE();
@@ -300,9 +296,7 @@ opensslrsa_check_exponent_bits(EVP_PKEY *pkey, int maxbits) {
 		BN_free(e);
 		return bits < maxbits;
 	}
-#endif
-	/* Use old API for the OpenSSL ENGINE support, even with OpenSSL 3.x */
-#if OPENSSL_VERSION_NUMBER < 0x30000000L || OPENSSL_API_LEVEL < 30000
+#else
 	const RSA *rsa = EVP_PKEY_get0_RSA(pkey);
 	if (rsa != NULL) {
 		const BIGNUM *ce = NULL;
@@ -350,7 +344,7 @@ opensslrsa_verify(dst_context_t *dctx, const isc_region_t *sig) {
 	return opensslrsa_verify2(dctx, 0, sig);
 }
 
-#if OPENSSL_VERSION_NUMBER < 0x30000000L || OPENSSL_API_LEVEL < 30000
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
 static int
 progress_cb(int p, int n, BN_GENCB *cb) {
 	void (*fptr)(int);
@@ -370,7 +364,7 @@ opensslrsa_generate_pkey(unsigned int key_size, const char *label, BIGNUM *e,
 	RSA *rsa = NULL;
 	EVP_PKEY *pkey = NULL;
 	BN_GENCB *cb = NULL;
-	isc_result_t result;
+	isc_result_t ret;
 
 	UNUSED(label);
 
@@ -398,18 +392,18 @@ opensslrsa_generate_pkey(unsigned int key_size, const char *label, BIGNUM *e,
 	}
 	*retkey = pkey;
 	pkey = NULL;
-	result = ISC_R_SUCCESS;
+	ret = ISC_R_SUCCESS;
 
-cleanup:
+err:
 	EVP_PKEY_free(pkey);
 	RSA_free(rsa);
 	BN_GENCB_free(cb);
-	return result;
+	return ret;
 }
 
 static isc_result_t
 opensslrsa_build_pkey(bool private, rsa_components_t *c, EVP_PKEY **retpkey) {
-	isc_result_t result;
+	isc_result_t ret;
 	EVP_PKEY *pkey = NULL;
 	RSA *rsa = RSA_new();
 	int status;
@@ -474,13 +468,13 @@ opensslrsa_build_pkey(bool private, rsa_components_t *c, EVP_PKEY **retpkey) {
 
 	*retpkey = pkey;
 	pkey = NULL;
-	result = ISC_R_SUCCESS;
+	ret = ISC_R_SUCCESS;
 
-cleanup:
+err:
 	EVP_PKEY_free(pkey);
 	RSA_free(rsa);
 	opensslrsa_components_free(c);
-	return result;
+	return ret;
 }
 #else
 static int
@@ -501,7 +495,7 @@ opensslrsa_generate_pkey_with_uri(size_t key_size, const char *label,
 	EVP_PKEY_CTX *ctx = NULL;
 	OSSL_PARAM params[4];
 	char *uri = UNCONST(label);
-	isc_result_t result;
+	isc_result_t ret;
 	int status;
 
 	params[0] = OSSL_PARAM_construct_utf8_string("pkcs11_uri", uri, 0);
@@ -534,17 +528,17 @@ opensslrsa_generate_pkey_with_uri(size_t key_size, const char *label,
 					       DST_R_OPENSSLFAILURE));
 	}
 
-	result = ISC_R_SUCCESS;
-cleanup:
+	ret = ISC_R_SUCCESS;
+err:
 	EVP_PKEY_CTX_free(ctx);
-	return result;
+	return ret;
 }
 
 static isc_result_t
 opensslrsa_generate_pkey(unsigned int key_size, const char *label, BIGNUM *e,
 			 void (*callback)(int), EVP_PKEY **retkey) {
 	EVP_PKEY_CTX *ctx;
-	isc_result_t result;
+	isc_result_t ret;
 
 	if (label != NULL) {
 		return opensslrsa_generate_pkey_with_uri(key_size, label,
@@ -577,15 +571,15 @@ opensslrsa_generate_pkey(unsigned int key_size, const char *label, BIGNUM *e,
 		DST_RET(dst__openssl_toresult2("EVP_PKEY_keygen",
 					       DST_R_OPENSSLFAILURE));
 	}
-	result = ISC_R_SUCCESS;
-cleanup:
+	ret = ISC_R_SUCCESS;
+err:
 	EVP_PKEY_CTX_free(ctx);
-	return result;
+	return ret;
 }
 
 static isc_result_t
 opensslrsa_build_pkey(bool private, rsa_components_t *c, EVP_PKEY **retpkey) {
-	isc_result_t result;
+	isc_result_t ret;
 	int status;
 	OSSL_PARAM_BLD *bld = NULL;
 	OSSL_PARAM *params = NULL;
@@ -666,22 +660,24 @@ opensslrsa_build_pkey(bool private, rsa_components_t *c, EVP_PKEY **retpkey) {
 		DST_RET(dst__openssl_toresult2("EVP_PKEY_fromdata",
 					       DST_R_OPENSSLFAILURE));
 	}
-	result = ISC_R_SUCCESS;
+	ret = ISC_R_SUCCESS;
 
-cleanup:
+err:
 	EVP_PKEY_CTX_free(ctx);
 	OSSL_PARAM_free(params);
 	OSSL_PARAM_BLD_free(bld);
-	return result;
+	return ret;
 }
-#endif /* OPENSSL_VERSION_NUMBER < 0x30000000L || OPENSSL_API_LEVEL < 30000 */
+#endif /* OPENSSL_VERSION_NUMBER < 0x30000000L */
 
 static isc_result_t
-opensslrsa_generate(dst_key_t *key, int exp, void (*callback)(int)) {
-	isc_result_t result;
+opensslrsa_generate(dst_key_t *key, int unused, void (*callback)(int)) {
+	isc_result_t ret;
 	BIGNUM *e = BN_new();
 	EVP_PKEY *pkey = NULL;
 
+	UNUSED(unused);
+
 	if (e == NULL) {
 		DST_RET(dst__openssl_toresult(DST_R_OPENSSLFAILURE));
 	}
@@ -713,28 +709,25 @@ opensslrsa_generate(dst_key_t *key, int exp, void (*callback)(int)) {
 		UNREACHABLE();
 	}
 
-	if (exp == 0) {
-		/* RSA_F4 0x10001 */
-		BN_set_bit(e, 0);
-		BN_set_bit(e, 16);
-	} else {
-		/* (phased-out) F5 0x100000001 */
-		BN_set_bit(e, 0);
-		BN_set_bit(e, 32);
-	}
+	/* e = 65537 (0x10001, F4) */
+	BN_set_bit(e, 0);
+	BN_set_bit(e, 16);
 
-	CHECK(opensslrsa_generate_pkey(key->key_size, key->label, e, callback,
-				       &pkey));
+	ret = opensslrsa_generate_pkey(key->key_size, key->label, e, callback,
+				       &pkey);
+	if (ret != ISC_R_SUCCESS) {
+		goto err;
+	}
 
 	key->keydata.pkeypair.pub = pkey;
 	key->keydata.pkeypair.priv = pkey;
 	pkey = NULL;
-	result = ISC_R_SUCCESS;
+	ret = ISC_R_SUCCESS;
 
-cleanup:
+err:
 	EVP_PKEY_free(pkey);
 	BN_free(e);
-	return result;
+	return ret;
 }
 
 static isc_result_t
@@ -742,14 +735,17 @@ opensslrsa_todns(const dst_key_t *key, isc_buffer_t *data) {
 	isc_region_t r;
 	unsigned int e_bytes;
 	unsigned int mod_bytes;
-	isc_result_t result;
+	isc_result_t ret;
 	rsa_components_t c = { 0 };
 
 	REQUIRE(key->keydata.pkeypair.pub != NULL);
 
 	isc_buffer_availableregion(data, &r);
 
-	CHECK(opensslrsa_components_get(key, &c, false));
+	ret = opensslrsa_components_get(key, &c, false);
+	if (ret != ISC_R_SUCCESS) {
+		goto err;
+	}
 
 	mod_bytes = BN_num_bytes(c.n);
 	e_bytes = BN_num_bytes(c.e);
@@ -780,15 +776,15 @@ opensslrsa_todns(const dst_key_t *key, isc_buffer_t *data) {
 
 	isc_buffer_add(data, e_bytes + mod_bytes);
 
-	result = ISC_R_SUCCESS;
-cleanup:
+	ret = ISC_R_SUCCESS;
+err:
 	opensslrsa_components_free(&c);
-	return result;
+	return ret;
 }
 
 static isc_result_t
 opensslrsa_fromdns(dst_key_t *key, isc_buffer_t *data) {
-	isc_result_t result;
+	isc_result_t ret;
 	isc_region_t r;
 	unsigned int e_bytes;
 	unsigned int length;
@@ -830,16 +826,16 @@ opensslrsa_fromdns(dst_key_t *key, isc_buffer_t *data) {
 	isc_buffer_forward(data, length);
 
 	key->key_size = BN_num_bits(c.n);
-	result = opensslrsa_build_pkey(false, &c, &key->keydata.pkeypair.pub);
+	ret = opensslrsa_build_pkey(false, &c, &key->keydata.pkeypair.pub);
 
-cleanup:
+err:
 	opensslrsa_components_free(&c);
-	return result;
+	return ret;
 }
 
 static isc_result_t
 opensslrsa_tofile(const dst_key_t *key, const char *directory) {
-	isc_result_t result;
+	isc_result_t ret;
 	dst_private_t priv = { 0 };
 	unsigned char *bufs[8] = { NULL };
 	unsigned short i = 0;
@@ -849,7 +845,10 @@ opensslrsa_tofile(const dst_key_t *key, const char *directory) {
 		return dst__privstruct_writefile(key, &priv, directory);
 	}
 
-	CHECK(opensslrsa_components_get(key, &c, true));
+	ret = opensslrsa_components_get(key, &c, true);
+	if (ret != ISC_R_SUCCESS) {
+		goto err;
+	}
 
 	priv.elements[i].tag = TAG_RSA_MODULUS;
 	priv.elements[i].length = BN_num_bytes(c.n);
@@ -925,14 +924,6 @@ opensslrsa_tofile(const dst_key_t *key, const char *directory) {
 		i++;
 	}
 
-	if (key->engine != NULL) {
-		priv.elements[i].tag = TAG_RSA_ENGINE;
-		priv.elements[i].length = (unsigned short)strlen(key->engine) +
-					  1;
-		priv.elements[i].data = (unsigned char *)key->engine;
-		i++;
-	}
-
 	if (key->label != NULL) {
 		priv.elements[i].tag = TAG_RSA_LABEL;
 		priv.elements[i].length = (unsigned short)strlen(key->label) +
@@ -942,9 +933,9 @@ opensslrsa_tofile(const dst_key_t *key, const char *directory) {
 	}
 
 	priv.nelements = i;
-	result = dst__privstruct_writefile(key, &priv, directory);
+	ret = dst__privstruct_writefile(key, &priv, directory);
 
-cleanup:
+err:
 	for (i = 0; i < ARRAY_SIZE(bufs); i++) {
 		if (bufs[i] != NULL) {
 			isc_mem_put(key->mctx, bufs[i],
@@ -953,20 +944,19 @@ cleanup:
 	}
 	opensslrsa_components_free(&c);
 
-	return result;
+	return ret;
 }
 
 static isc_result_t
-opensslrsa_fromlabel(dst_key_t *key, const char *engine, const char *label,
-		     const char *pin);
+opensslrsa_fromlabel(dst_key_t *key, const char *label, const char *pin);
 
 static isc_result_t
 opensslrsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	dst_private_t priv;
-	isc_result_t result;
+	isc_result_t ret;
 	int i;
 	isc_mem_t *mctx = NULL;
-	const char *engine = NULL, *label = NULL;
+	const char *label = NULL;
 	EVP_PKEY *pkey = NULL;
 	rsa_components_t c = { .bnfree = true };
 
@@ -976,7 +966,10 @@ opensslrsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	mctx = key->mctx;
 
 	/* read private key file */
-	CHECK(dst__privstruct_parse(key, DST_ALG_RSA, lexer, mctx, &priv));
+	ret = dst__privstruct_parse(key, DST_ALG_RSA, lexer, mctx, &priv);
+	if (ret != ISC_R_SUCCESS) {
+		goto err;
+	}
 
 	if (key->external) {
 		if (priv.nelements != 0 || pub == NULL) {
@@ -993,7 +986,7 @@ opensslrsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	for (i = 0; i < priv.nelements; i++) {
 		switch (priv.elements[i].tag) {
 		case TAG_RSA_ENGINE:
-			engine = (char *)priv.elements[i].data;
+			/* The Engine: tag is explicitly ignored */
 			break;
 		case TAG_RSA_LABEL:
 			label = (char *)priv.elements[i].data;
@@ -1008,9 +1001,9 @@ opensslrsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	 * See if we can fetch it.
 	 */
 	if (label != NULL) {
-		result = opensslrsa_fromlabel(key, engine, label, NULL);
-		if (result != ISC_R_SUCCESS) {
-			DST_RET(result);
+		ret = opensslrsa_fromlabel(key, label, NULL);
+		if (ret != ISC_R_SUCCESS) {
+			DST_RET(ret);
 		}
 		/* Check that the public component matches if given */
 		if (pub != NULL && EVP_PKEY_eq(key->keydata.pkeypair.pub,
@@ -1074,7 +1067,10 @@ opensslrsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	}
 
 	key->key_size = BN_num_bits(c.n);
-	CHECK(opensslrsa_build_pkey(true, &c, &pkey));
+	ret = opensslrsa_build_pkey(true, &c, &pkey);
+	if (ret != ISC_R_SUCCESS) {
+		goto err;
+	}
 
 	/* Check that the public component matches if given */
 	if (pub != NULL && EVP_PKEY_eq(pkey, pub->keydata.pkeypair.pub) != 1) {
@@ -1085,35 +1081,34 @@ opensslrsa_parse(dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {
 	key->keydata.pkeypair.priv = pkey;
 	pkey = NULL;
 
-cleanup:
+err:
 	opensslrsa_components_free(&c);
 	EVP_PKEY_free(pkey);
-	if (result != ISC_R_SUCCESS) {
+	if (ret != ISC_R_SUCCESS) {
 		key->keydata.generic = NULL;
 	}
 
 	dst__privstruct_free(&priv, mctx);
 	isc_safe_memwipe(&priv, sizeof(priv));
 
-	return result;
+	return ret;
 }
 
 static isc_result_t
-opensslrsa_fromlabel(dst_key_t *key, const char *engine, const char *label,
-		     const char *pin) {
+opensslrsa_fromlabel(dst_key_t *key, const char *label, const char *pin) {
 	EVP_PKEY *privpkey = NULL, *pubpkey = NULL;
-	isc_result_t result;
+	isc_result_t ret;
 
-	CHECK(dst__openssl_fromlabel(EVP_PKEY_RSA, engine, label, pin, &pubpkey,
-				     &privpkey));
+	ret = dst__openssl_fromlabel(EVP_PKEY_RSA, label, pin, &pubpkey,
+				     &privpkey);
+	if (ret != ISC_R_SUCCESS) {
+		goto err;
+	}
 
 	if (!opensslrsa_check_exponent_bits(pubpkey, RSA_MAX_PUBEXP_BITS)) {
 		DST_RET(ISC_R_RANGE);
 	}
 
-	if (engine != NULL) {
-		key->engine = isc_mem_strdup(key->mctx, engine);
-	}
 	key->label = isc_mem_strdup(key->mctx, label);
 	key->key_size = EVP_PKEY_bits(privpkey);
 	key->keydata.pkeypair.priv = privpkey;
@@ -1121,10 +1116,10 @@ opensslrsa_fromlabel(dst_key_t *key, const char *engine, const char *label,
 	privpkey = NULL;
 	pubpkey = NULL;
 
-cleanup:
+err:
 	EVP_PKEY_free(privpkey);
 	EVP_PKEY_free(pubpkey);
-	return result;
+	return ret;
 }
 
 static dst_func_t opensslrsa_functions = {
@@ -1234,27 +1229,23 @@ check_algorithm(unsigned char algorithm) {
 	EVP_PKEY *pkey = NULL;
 	const EVP_MD *type = NULL;
 	const unsigned char *sig = NULL;
-	isc_result_t result = ISC_R_SUCCESS;
+	isc_result_t ret = ISC_R_SUCCESS;
 	size_t len;
 
-	if (evp_md_ctx == NULL) {
-		DST_RET(ISC_R_NOMEMORY);
-	}
-
 	switch (algorithm) {
 	case DST_ALG_RSASHA1:
 	case DST_ALG_NSEC3RSASHA1:
-		type = EVP_sha1(); /* SHA1 + RSA */
+		type = isc__crypto_sha1; /* SHA1 + RSA */
 		sig = sha1_sig;
 		len = sizeof(sha1_sig) - 1;
 		break;
 	case DST_ALG_RSASHA256:
-		type = EVP_sha256(); /* SHA256 + RSA */
+		type = isc__crypto_sha256; /* SHA256 + RSA */
 		sig = sha256_sig;
 		len = sizeof(sha256_sig) - 1;
 		break;
 	case DST_ALG_RSASHA512:
-		type = EVP_sha512();
+		type = isc__crypto_sha512;
 		sig = sha512_sig;
 		len = sizeof(sha512_sig) - 1;
 		break;
@@ -1262,20 +1253,14 @@ check_algorithm(unsigned char algorithm) {
 		DST_RET(ISC_R_NOTIMPLEMENTED);
 	}
 
-	if (type == NULL) {
-		DST_RET(ISC_R_NOTIMPLEMENTED);
-	}
-
 	/*
 	 * Construct pkey.
 	 */
 	c.e = BN_bin2bn(e_bytes, sizeof(e_bytes) - 1, NULL);
 	c.n = BN_bin2bn(n_bytes, sizeof(n_bytes) - 1, NULL);
-	if (c.e == NULL || c.n == NULL) {
-		DST_RET(ISC_R_NOMEMORY);
-	}
 
-	CHECK(opensslrsa_build_pkey(false, &c, &pkey));
+	ret = opensslrsa_build_pkey(false, &c, &pkey);
+	INSIST(ret == ISC_R_SUCCESS);
 
 	/*
 	 * Check that we can verify the signature.
@@ -1287,29 +1272,21 @@ check_algorithm(unsigned char algorithm) {
 		DST_RET(ISC_R_NOTIMPLEMENTED);
 	}
 
-cleanup:
+err:
 	opensslrsa_components_free(&c);
 	EVP_PKEY_free(pkey);
 	EVP_MD_CTX_destroy(evp_md_ctx);
 	ERR_clear_error();
-	return result;
+	return ret;
 }
 
-isc_result_t
+void
 dst__opensslrsa_init(dst_func_t **funcp, unsigned char algorithm) {
-	isc_result_t result;
-
 	REQUIRE(funcp != NULL);
 
-	result = check_algorithm(algorithm);
-
-	if (result == ISC_R_SUCCESS) {
-		if (*funcp == NULL) {
+	if (*funcp == NULL) {
+		if (check_algorithm(algorithm) == ISC_R_SUCCESS) {
 			*funcp = &opensslrsa_functions;
 		}
-	} else if (result == ISC_R_NOTIMPLEMENTED) {
-		result = ISC_R_SUCCESS;
 	}
-
-	return result;
 }
