@@ -1,16 +1,17 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
  */
 
-#ifndef GENERIC_GPOS_27_H
-#define GENERIC_GPOS_27_H 1
+#pragma once
 
 /*!
  *  \brief per RFC1712 */
@@ -25,5 +26,3 @@ typedef struct dns_rdata_gpos {
 	uint8_t lat_len;
 	uint8_t alt_len;
 } dns_rdata_gpos_t;
-
-#endif /* GENERIC_GPOS_27_H */
