@@ -39,12 +39,13 @@ fromtext_isdn(ARGS_FROMTEXT) {
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_qstring,
 				      true));
 	if (token.type != isc_tokentype_string &&
-	    token.type != isc_tokentype_qstring) {
+	    token.type != isc_tokentype_qstring)
+	{
 		isc_lex_ungettoken(lexer, &token);
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 	RETTOK(txt_fromtext(&token.value.as_textregion, target));
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 }
 
 static isc_result_t
@@ -59,10 +60,10 @@ totext_isdn(ARGS_TOTEXT) {
 	dns_rdata_toregion(rdata, &region);
 	RETERR(txt_totext(&region, true, target));
 	if (region.length == 0) {
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 	RETERR(str_totext(" ", target));
-	return (txt_totext(&region, true, target));
+	return txt_totext(&region, true, target);
 }
 
 static isc_result_t
@@ -72,13 +73,12 @@ fromwire_isdn(ARGS_FROMWIRE) {
 	UNUSED(type);
 	UNUSED(dctx);
 	UNUSED(rdclass);
-	UNUSED(options);
 
 	RETERR(txt_fromwire(source, target));
 	if (buffer_empty(source)) {
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
-	return (txt_fromwire(source, target));
+	return txt_fromwire(source, target);
 }
 
 static isc_result_t
@@ -88,7 +88,7 @@ towire_isdn(ARGS_TOWIRE) {
 	REQUIRE(rdata->type == dns_rdatatype_isdn);
 	REQUIRE(rdata->length != 0);
 
-	return (mem_tobuffer(target, rdata->data, rdata->length));
+	return mem_tobuffer(target, rdata->data, rdata->length);
 }
 
 static int
@@ -104,7 +104,7 @@ compare_isdn(ARGS_COMPARE) {
 
 	dns_rdata_toregion(rdata1, &r1);
 	dns_rdata_toregion(rdata2, &r2);
-	return (isc_region_compare(&r1, &r2));
+	return isc_region_compare(&r1, &r2);
 }
 
 static isc_result_t
@@ -122,10 +122,10 @@ fromstruct_isdn(ARGS_FROMSTRUCT) {
 	RETERR(uint8_tobuffer(isdn->isdn_len, target));
 	RETERR(mem_tobuffer(target, isdn->isdn, isdn->isdn_len));
 	if (isdn->subaddress == NULL) {
-		return (ISC_R_SUCCESS);
+		return ISC_R_SUCCESS;
 	}
 	RETERR(uint8_tobuffer(isdn->subaddress_len, target));
-	return (mem_tobuffer(target, isdn->subaddress, isdn->subaddress_len));
+	return mem_tobuffer(target, isdn->subaddress, isdn->subaddress_len);
 }
 
 static isc_result_t
@@ -146,9 +146,6 @@ tostruct_isdn(ARGS_TOSTRUCT) {
 	isdn->isdn_len = uint8_fromregion(&r);
 	isc_region_consume(&r, 1);
 	isdn->isdn = mem_maybedup(mctx, r.base, isdn->isdn_len);
-	if (isdn->isdn == NULL) {
-		return (ISC_R_NOMEMORY);
-	}
 	isc_region_consume(&r, isdn->isdn_len);
 
 	if (r.length == 0) {
@@ -159,19 +156,10 @@ tostruct_isdn(ARGS_TOSTRUCT) {
 		isc_region_consume(&r, 1);
 		isdn->subaddress = mem_maybedup(mctx, r.base,
 						isdn->subaddress_len);
-		if (isdn->subaddress == NULL) {
-			goto cleanup;
-		}
 	}
 
 	isdn->mctx = mctx;
-	return (ISC_R_SUCCESS);
-
-cleanup:
-	if (mctx != NULL && isdn->isdn != NULL) {
-		isc_mem_free(mctx, isdn->isdn);
-	}
-	return (ISC_R_NOMEMORY);
+	return ISC_R_SUCCESS;
 }
 
 static void
@@ -202,7 +190,7 @@ additionaldata_isdn(ARGS_ADDLDATA) {
 	UNUSED(add);
 	UNUSED(arg);
 
-	return (ISC_R_SUCCESS);
+	return ISC_R_SUCCESS;
 }
 
 static isc_result_t
@@ -213,7 +201,7 @@ digest_isdn(ARGS_DIGEST) {
 
 	dns_rdata_toregion(rdata, &r);
 
-	return ((digest)(arg, &r));
+	return (digest)(arg, &r);
 }
 
 static bool
@@ -225,7 +213,7 @@ checkowner_isdn(ARGS_CHECKOWNER) {
 	UNUSED(rdclass);
 	UNUSED(wildcard);
 
-	return (true);
+	return true;
 }
 
 static bool
@@ -236,12 +224,12 @@ checknames_isdn(ARGS_CHECKNAMES) {
 	UNUSED(owner);
 	UNUSED(bad);
 
-	return (true);
+	return true;
 }
 
 static int
 casecompare_isdn(ARGS_COMPARE) {
-	return (compare_isdn(rdata1, rdata2));
+	return compare_isdn(rdata1, rdata2);
 }
 
 #endif /* RDATA_GENERIC_ISDN_20_C */
