@@ -159,7 +159,9 @@ tostruct_in_a(ARGS_TOSTRUCT) {
 
 	UNUSED(mctx);
 
-	DNS_RDATACOMMON_INIT(a, rdata->type, rdata->rdclass);
+	a->common.rdclass = rdata->rdclass;
+	a->common.rdtype = rdata->type;
+	ISC_LINK_INIT(&a->common, link);
 
 	dns_rdata_toregion(rdata, &region);
 	n = uint32_fromregion(&region);
