@@ -29,7 +29,7 @@ fromtext_in_nimloc(ARGS_FROMTEXT) {
 	UNUSED(rdclass);
 	UNUSED(callbacks);
 
-	return isc_hex_tobuffer(lexer, target, isc_one_or_more);
+	return isc_hex_tobuffer(lexer, target, -2);
 }
 
 static isc_result_t
@@ -133,7 +133,9 @@ tostruct_in_nimloc(ARGS_TOSTRUCT) {
 	REQUIRE(nimloc != NULL);
 	REQUIRE(rdata->length != 0);
 
-	DNS_RDATACOMMON_INIT(nimloc, rdata->type, rdata->rdclass);
+	nimloc->common.rdclass = rdata->rdclass;
+	nimloc->common.rdtype = rdata->type;
+	ISC_LINK_INIT(&nimloc->common, link);
 
 	dns_rdata_toregion(rdata, &r);
 	nimloc->nimloc_len = r.length;
