@@ -17,6 +17,7 @@
 #include <stdbool.h>
 
 #include <isc/async.h>
+#include <isc/log.h>
 #include <isc/loop.h>
 #include <isc/magic.h>
 #include <isc/mem.h>
@@ -29,7 +30,6 @@
 #include <dns/acl.h>
 #include <dns/compress.h>
 #include <dns/dispatch.h>
-#include <dns/log.h>
 #include <dns/message.h>
 #include <dns/rdata.h>
 #include <dns/rdatastruct.h>
@@ -143,7 +143,7 @@ dns_requestmgr_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr,
 	for (size_t i = 0; i < nloops; i++) {
 		ISC_LIST_INIT(requestmgr->requests[i]);
 
-		/* unreferenced in requests_cancel() */
+		/* unreferenced in requests_shutdown() */
 		isc_loop_ref(isc_loop_get(requestmgr->loopmgr, i));
 	}
 
@@ -169,19 +169,19 @@ dns_requestmgr_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr,
 }
 
 static void
-requests_cancel(void *arg) {
+requests_shutdown(void *arg) {
 	dns_requestmgr_t *requestmgr = arg;
 	dns_request_t *request = NULL, *next = NULL;
 	uint32_t tid = isc_tid();
 
-	ISC_LIST_FOREACH_SAFE(requestmgr->requests[tid], request, link, next) {
+	ISC_LIST_FOREACH_SAFE (requestmgr->requests[tid], request, link, next) {
 		req_log(ISC_LOG_DEBUG(3), "%s(%" PRIu32 ": request %p",
 			__func__, tid, request);
 		if (DNS_REQUEST_COMPLETE(request)) {
 			/* The callback has been already scheduled */
 			continue;
 		}
-		req_sendevent(request, ISC_R_CANCELED);
+		req_sendevent(request, ISC_R_SHUTTINGDOWN);
 	}
 
 	isc_loop_unref(isc_loop_get(requestmgr->loopmgr, tid));
@@ -217,12 +217,12 @@ dns_requestmgr_shutdown(dns_requestmgr_t *requestmgr) {
 
 		if (i == tid) {
 			/* Run the current loop synchronously */
-			requests_cancel(requestmgr);
+			requests_shutdown(requestmgr);
 			continue;
 		}
 
 		isc_loop_t *loop = isc_loop_get(requestmgr->loopmgr, i);
-		isc_async_run(loop, requests_cancel, requestmgr);
+		isc_async_run(loop, requests_shutdown, requestmgr);
 	}
 }
 
@@ -649,7 +649,6 @@ again:
 
 cleanup:
 	if (result != ISC_R_SUCCESS) {
-		dns_message_settsigkey(message, NULL);
 		req_cleanup(request);
 		dns_request_detach(&request);
 		req_log(ISC_LOG_DEBUG(3), "%s: failed %s", __func__,
@@ -1056,7 +1055,7 @@ req_log(int level, const char *fmt, ...) {
 	va_list ap;
 
 	va_start(ap, fmt);
-	isc_log_vwrite(dns_lctx, DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_REQUEST,
-		       level, fmt, ap);
+	isc_log_vwrite(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_REQUEST, level,
+		       fmt, ap);
 	va_end(ap);
 }
