@@ -20,6 +20,7 @@
 
 #include <isc/async.h>
 #include <isc/buffer.h>
+#include <isc/log.h>
 #include <isc/loop.h>
 #include <isc/magic.h>
 #include <isc/mem.h>
@@ -34,9 +35,7 @@
 
 #include <dns/db.h>
 #include <dns/dbiterator.h>
-#include <dns/dnsrps.h>
 #include <dns/fixedname.h>
-#include <dns/log.h>
 #include <dns/qp.h>
 #include <dns/rdata.h>
 #include <dns/rdataset.h>
@@ -610,7 +609,7 @@ fix_qname_skip_recurse(dns_rpz_zones_t *rpzs) {
 	}
 
 set:
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB,
+	isc_log_write(DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB,
 		      DNS_RPZ_DEBUG_QUIET,
 		      "computed RPZ qname_skip_recurse mask=0x%" PRIx64,
 		      (uint64_t)mask);
@@ -719,11 +718,10 @@ badname(int level, const dns_name_t *name, const char *str1, const char *str2) {
 	/*
 	 * bin/tests/system/rpz/tests.sh looks for "invalid rpz".
 	 */
-	if (level < DNS_RPZ_DEBUG_QUIET && isc_log_wouldlog(dns_lctx, level)) {
+	if (level < DNS_RPZ_DEBUG_QUIET && isc_log_wouldlog(level)) {
 		char namebuf[DNS_NAME_FORMATSIZE];
 		dns_name_format(name, namebuf, sizeof(namebuf));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_RPZ,
-			      DNS_LOGMODULE_RBTDB, level,
+		isc_log_write(DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB, level,
 			      "invalid rpz IP address \"%s\"%s%s", namebuf,
 			      str1, str2);
 	}
@@ -1012,9 +1010,7 @@ name2ipkey(int log_level, dns_rpz_zone_t *rpz, dns_rpz_type_t rpz_type,
 	/*
 	 * Complain about bad names but be generous and accept them.
 	 */
-	if (log_level < DNS_RPZ_DEBUG_QUIET &&
-	    isc_log_wouldlog(dns_lctx, log_level))
-	{
+	if (log_level < DNS_RPZ_DEBUG_QUIET && isc_log_wouldlog(log_level)) {
 		/*
 		 * Convert the address back to a canonical domain name
 		 * to ensure that the original name is in canonical form.
@@ -1027,8 +1023,8 @@ name2ipkey(int log_level, dns_rpz_zone_t *rpz, dns_rpz_type_t rpz_type,
 		{
 			char ip2_str[DNS_NAME_FORMATSIZE];
 			dns_name_format(ip_name2, ip2_str, sizeof(ip2_str));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_RPZ,
-				      DNS_LOGMODULE_RBTDB, log_level,
+			isc_log_write(DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB,
+				      log_level,
 				      "rpz IP address \"%s\""
 				      " is not the canonical \"%s\"",
 				      ip_str, ip2_str);
@@ -1388,8 +1384,8 @@ add_cidr(dns_rpz_zone_t *rpz, dns_rpz_type_t rpz_type,
 		 * bin/tests/system/rpz/tests.sh looks for "rpz.*failed".
 		 */
 		dns_name_format(src_name, namebuf, sizeof(namebuf));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_RPZ,
-			      DNS_LOGMODULE_RBTDB, DNS_RPZ_ERROR_LEVEL,
+		isc_log_write(DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB,
+			      DNS_RPZ_ERROR_LEVEL,
 			      "rpz add_cidr(%s) failed: %s", namebuf,
 			      isc_result_totext(result));
 		goto done;
@@ -1498,13 +1494,10 @@ add_name(dns_rpz_zone_t *rpz, dns_rpz_type_t rpz_type,
  * Get ready for a new set of policy zones for a view.
  */
 isc_result_t
-dns_rpz_new_zones(dns_view_t *view, isc_loopmgr_t *loopmgr, char *rps_cstr,
-		  size_t rps_cstr_size, dns_rpz_zones_t **rpzsp) {
+dns_rpz_new_zones(dns_view_t *view, isc_loopmgr_t *loopmgr,
+		  dns_rpz_zones_t **rpzsp) {
 	dns_rpz_zones_t *rpzs = NULL;
 	isc_mem_t *mctx = NULL;
-#ifdef USE_DNSRPS
-	isc_result_t result = ISC_R_SUCCESS;
-#endif
 
 	REQUIRE(rpzsp != NULL && *rpzsp == NULL);
 	REQUIRE(view != NULL);
@@ -1513,8 +1506,6 @@ dns_rpz_new_zones(dns_view_t *view, isc_loopmgr_t *loopmgr, char *rps_cstr,
 
 	rpzs = isc_mem_get(mctx, sizeof(*rpzs));
 	*rpzs = (dns_rpz_zones_t){
-		.rps_cstr = rps_cstr,
-		.rps_cstr_size = rps_cstr_size,
 		.loopmgr = loopmgr,
 		.magic = DNS_RPZ_ZONES_MAGIC,
 	};
@@ -1523,36 +1514,12 @@ dns_rpz_new_zones(dns_view_t *view, isc_loopmgr_t *loopmgr, char *rps_cstr,
 	isc_mutex_init(&rpzs->maint_lock);
 	isc_refcount_init(&rpzs->references, 1);
 
-#ifdef USE_DNSRPS
-	if (rps_cstr != NULL) {
-		result = dns_dnsrps_view_init(rpzs, rps_cstr);
-		if (result != ISC_R_SUCCESS) {
-			goto cleanup;
-		}
-	}
-#else  /* ifdef USE_DNSRPS */
-	INSIST(!rpzs->p.dnsrps_enabled);
-#endif /* ifdef USE_DNSRPS */
-	if (!rpzs->p.dnsrps_enabled) {
-		dns_qpmulti_create(mctx, &qpmethods, view, &rpzs->table);
-	}
+	dns_qpmulti_create(mctx, &qpmethods, view, &rpzs->table);
 
 	isc_mem_attach(mctx, &rpzs->mctx);
 
 	*rpzsp = rpzs;
 	return ISC_R_SUCCESS;
-
-#ifdef USE_DNSRPS
-	/* Only if DNSRPS is in use can this function fail */
-cleanup:
-	isc_refcount_decrementz(&rpzs->references);
-	isc_refcount_destroy(&rpzs->references);
-	isc_mutex_destroy(&rpzs->maint_lock);
-	isc_rwlock_destroy(&rpzs->search_lock);
-	isc_mem_put(mctx, rpzs, sizeof(*rpzs));
-
-	return result;
-#endif /* ifdef USE_DNSRPS */
 }
 
 isc_result_t
@@ -1647,8 +1614,8 @@ dns_rpz_dbupdate_callback(dns_db_t *db, void *fn_arg) {
 		rpz->updatepending = true;
 
 		dns_name_format(&rpz->origin, dname, DNS_NAME_FORMATSIZE);
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_MASTER, ISC_LOG_DEBUG(3),
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
+			      ISC_LOG_DEBUG(3),
 			      "rpz: %s: update already queued or running",
 			      dname);
 		if (rpz->dbversion != NULL) {
@@ -1693,8 +1660,8 @@ dns__rpz_timer_start(dns_rpz_zone_t *rpz) {
 		char dname[DNS_NAME_FORMATSIZE];
 
 		dns_name_format(&rpz->origin, dname, DNS_NAME_FORMATSIZE);
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_MASTER, ISC_LOG_INFO,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
+			      ISC_LOG_INFO,
 			      "rpz: %s: new zone version came "
 			      "too soon, deferring update for "
 			      "%" PRIu64 " seconds",
@@ -1744,7 +1711,7 @@ update_rpz_done_cb(void *data) {
 
 	UNLOCK(&rpz->rpzs->maint_lock);
 
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
+	isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
 		      ISC_LOG_INFO, "rpz: %s: reload done: %s", dname,
 		      isc_result_totext(rpz->updateresult));
 
@@ -1765,8 +1732,8 @@ update_nodes(dns_rpz_zone_t *rpz, isc_ht_t *newnodes) {
 
 	result = dns_db_createiterator(rpz->updb, DNS_DB_NONSEC3, &updbit);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_MASTER, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
+			      ISC_LOG_ERROR,
 			      "rpz: %s: failed to create DB iterator - %s",
 			      domain, isc_result_totext(result));
 		return result;
@@ -1774,8 +1741,8 @@ update_nodes(dns_rpz_zone_t *rpz, isc_ht_t *newnodes) {
 
 	result = dns_dbiterator_first(updbit);
 	if (result != ISC_R_SUCCESS && result != ISC_R_NOMORE) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      DNS_LOGMODULE_MASTER, ISC_LOG_ERROR,
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
+			      ISC_LOG_ERROR,
 			      "rpz: %s: failed to get db iterator - %s", domain,
 			      isc_result_totext(result));
 		goto cleanup;
@@ -1793,7 +1760,7 @@ update_nodes(dns_rpz_zone_t *rpz, isc_ht_t *newnodes) {
 
 		result = dns_dbiterator_current(updbit, &node, name);
 		if (result != ISC_R_SUCCESS) {
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTER, ISC_LOG_ERROR,
 				      "rpz: %s: failed to get dbiterator - %s",
 				      domain, isc_result_totext(result));
@@ -1806,7 +1773,7 @@ update_nodes(dns_rpz_zone_t *rpz, isc_ht_t *newnodes) {
 		result = dns_db_allrdatasets(rpz->updb, node, rpz->updbversion,
 					     0, 0, &rdsiter);
 		if (result != ISC_R_SUCCESS) {
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTER, ISC_LOG_ERROR,
 				      "rpz: %s: failed to fetch "
 				      "rrdatasets - %s",
@@ -1823,7 +1790,7 @@ update_nodes(dns_rpz_zone_t *rpz, isc_ht_t *newnodes) {
 		if (result != ISC_R_SUCCESS) { /* skip empty non-terminal */
 			if (result != ISC_R_NOMORE) {
 				isc_log_write(
-					dns_lctx, DNS_LOGCATEGORY_GENERAL,
+					DNS_LOGCATEGORY_GENERAL,
 					DNS_LOGMODULE_MASTER, ISC_LOG_ERROR,
 					"rpz: %s: error %s while creating "
 					"rdatasetiter",
@@ -1838,7 +1805,7 @@ update_nodes(dns_rpz_zone_t *rpz, isc_ht_t *newnodes) {
 		result = isc_ht_add(newnodes, name->ndata, name->length, rpz);
 		if (result != ISC_R_SUCCESS) {
 			dns_name_format(name, namebuf, sizeof(namebuf));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTER, ISC_LOG_ERROR,
 				      "rpz: %s, adding node %s to HT error %s",
 				      domain, namebuf,
@@ -1865,15 +1832,15 @@ update_nodes(dns_rpz_zone_t *rpz, isc_ht_t *newnodes) {
 
 		if (result != ISC_R_SUCCESS) {
 			dns_name_format(name, namebuf, sizeof(namebuf));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTER, ISC_LOG_ERROR,
 				      "rpz: %s: adding node %s "
 				      "to RPZ error %s",
 				      domain, namebuf,
 				      isc_result_totext(result));
-		} else if (isc_log_wouldlog(dns_lctx, ISC_LOG_DEBUG(3))) {
+		} else if (isc_log_wouldlog(ISC_LOG_DEBUG(3))) {
 			dns_name_format(name, namebuf, sizeof(namebuf));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      DNS_LOGMODULE_MASTER, ISC_LOG_DEBUG(3),
 				      "rpz: %s: adding node %s", domain,
 				      namebuf);
@@ -2011,7 +1978,7 @@ dns__rpz_timer_cb(void *arg) {
 	rpz->dbversion = NULL;
 
 	dns_name_format(&rpz->origin, domain, DNS_NAME_FORMATSIZE);
-	isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
+	isc_log_write(DNS_LOGCATEGORY_GENERAL, DNS_LOGMODULE_MASTER,
 		      ISC_LOG_INFO, "rpz: %s: reload start", domain);
 
 	dns_rpz_zones_ref(rpz->rpzs);
@@ -2136,13 +2103,6 @@ dns__rpz_zones_destroy(dns_rpz_zones_t *rpzs) {
 		dns_rpz_zone_destroy(&rpzs->zones[rpz_num]);
 	}
 
-	if (rpzs->rps_cstr_size != 0) {
-#ifdef USE_DNSRPS
-		librpz->client_detach(&rpzs->rps_client);
-#endif /* ifdef USE_DNSRPS */
-		isc_mem_put(rpzs->mctx, rpzs->rps_cstr, rpzs->rps_cstr_size);
-	}
-
 	cidr_free(rpzs);
 	if (rpzs->table != NULL) {
 		dns_qpmulti_destroy(&rpzs->table);
@@ -2363,8 +2323,8 @@ del_name(dns_rpz_zone_t *rpz, dns_rpz_type_t rpz_type,
 			 * "rpz.*failed".
 			 */
 			dns_name_format(src_name, namebuf, sizeof(namebuf));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_RPZ,
-				      DNS_LOGMODULE_RBTDB, DNS_RPZ_ERROR_LEVEL,
+			isc_log_write(DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB,
+				      DNS_RPZ_ERROR_LEVEL,
 				      "rpz del_name(%s) node delete "
 				      "failed: %s",
 				      namebuf, isc_result_totext(result));
@@ -2528,9 +2488,8 @@ dns_rpz_find_ip(dns_rpz_zones_t *rpzs, dns_rpz_type_t rpz_type,
 		/*
 		 * bin/tests/system/rpz/tests.sh looks for "rpz.*failed".
 		 */
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_RPZ,
-			      DNS_LOGMODULE_RBTDB, DNS_RPZ_ERROR_LEVEL,
-			      "rpz ip2name() failed: %s",
+		isc_log_write(DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB,
+			      DNS_RPZ_ERROR_LEVEL, "rpz ip2name() failed: %s",
 			      isc_result_totext(result));
 		return DNS_RPZ_INVALID_NUM;
 	}
@@ -2592,8 +2551,8 @@ dns_rpz_find_name(dns_rpz_zones_t *rpzs, dns_rpz_type_t rpz_type,
 		 * bin/tests/system/rpz/tests.sh looks for "rpz.*failed".
 		 */
 		dns_name_format(trig_name, namebuf, sizeof(namebuf));
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_RPZ,
-			      DNS_LOGMODULE_RBTDB, DNS_RPZ_ERROR_LEVEL,
+		isc_log_write(DNS_LOGCATEGORY_RPZ, DNS_LOGMODULE_RBTDB,
+			      DNS_RPZ_ERROR_LEVEL,
 			      "dns_rpz_find_name(%s) failed: %s", namebuf,
 			      isc_result_totext(result));
 		break;
