@@ -20,6 +20,7 @@
 #include <isc/counter.h>
 #include <isc/helper.h>
 #include <isc/job.h>
+#include <isc/log.h>
 #include <isc/md.h>
 #include <isc/mem.h>
 #include <isc/refcount.h>
@@ -33,10 +34,8 @@
 #include <dns/db.h>
 #include <dns/dnssec.h>
 #include <dns/ds.h>
-#include <dns/ede.h>
 #include <dns/keytable.h>
 #include <dns/keyvalues.h>
-#include <dns/log.h>
 #include <dns/message.h>
 #include <dns/ncache.h>
 #include <dns/nsec.h>
@@ -78,7 +77,8 @@ enum valattr {
 	VALATTR_COMPLETE = 1 << 3,	     /*%< Completion event sent. */
 	VALATTR_INSECURITY = 1 << 4,	     /*%< Attempting proveunsecure. */
 	VALATTR_MAXVALIDATIONS = 1 << 5,     /*%< Max validations quota */
-	VALATTR_MAXVALIDATIONFAILS = 1 << 6, /*%< Max validation fails quota */
+	VALATTR_MAXVALIDATIONFAILS = 1 << 6, /*%< Max validation fails
+						quota */
 	VALATTR_OFFLOADED = 1 << 7, /*%< The ownership has been passed to
 				       offloaded thread */
 
@@ -148,8 +148,8 @@ static isc_result_t
 proveunsecure(dns_validator_t *val, bool have_ds, bool resume);
 
 static void
-validator_logv(dns_validator_t *val, isc_logcategory_t *category,
-	       isc_logmodule_t *module, int level, const char *fmt, va_list ap)
+validator_logv(dns_validator_t *val, isc_logcategory_t category,
+	       isc_logmodule_t module, int level, const char *fmt, va_list ap)
 	ISC_FORMAT_PRINTF(5, 0);
 
 static void
@@ -161,10 +161,6 @@ validator_logcreate(dns_validator_t *val, dns_name_t *name,
 		    dns_rdatatype_t type, const char *caller,
 		    const char *operation);
 
-static isc_result_t
-create_fetch(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
-	     isc_job_cb callback, const char *caller);
-
 /*%
  * Ensure the validator's rdatasets are marked as expired.
  */
@@ -178,12 +174,6 @@ expire_rdatasets(dns_validator_t *val) {
 	}
 }
 
-static void
-validate_extendederror(dns_validator_t *val);
-
-static void
-validator_addede(dns_validator_t *val, uint16_t code, const char *extra);
-
 /*%
  * Ensure the validator's rdatasets are disassociated.
  */
@@ -251,8 +241,6 @@ validator_done(dns_validator_t *val, isc_result_t result) {
 	val->attributes |= VALATTR_COMPLETE;
 	val->result = result;
 
-	dns_ede_copy(val->cb_edectx, &val->edectx);
-
 	isc_async_run(val->loop, val->cb, val);
 }
 
@@ -378,12 +366,6 @@ trynsec3:
 static void
 resume_answer_with_key_done(void *arg);
 
-static bool
-over_max_fails(dns_validator_t *val);
-
-static void
-consume_validation_fail(dns_validator_t *val);
-
 static void
 resume_answer_with_key(void *arg) {
 	dns_validator_t *val = arg;
@@ -392,13 +374,6 @@ resume_answer_with_key(void *arg) {
 	isc_result_t result = select_signing_key(val, rdataset);
 	if (result == ISC_R_SUCCESS) {
 		val->keyset = &val->frdataset;
-	} else if (result != ISC_R_NOTFOUND) {
-		val->result = result;
-		if (over_max_fails(val)) {
-			INSIST(val->key == NULL);
-			val->result = ISC_R_QUOTA;
-		}
-		consume_validation_fail(val);
 	}
 
 	(void)validate_async_run(val, resume_answer_with_key_done);
@@ -408,16 +383,6 @@ static void
 resume_answer_with_key_done(void *arg) {
 	dns_validator_t *val = arg;
 
-	switch (val->result) {
-	case ISC_R_CANCELED:	 /* Validation was canceled */
-	case ISC_R_SHUTTINGDOWN: /* Server shutting down */
-	case ISC_R_QUOTA:	 /* Validation fails quota reached */
-		dns_validator_cancel(val);
-		break;
-	default:
-		break;
-	}
-
 	resume_answer(val);
 }
 
@@ -446,7 +411,6 @@ fetch_callback_dnskey(void *arg) {
 	}
 
 	validator_log(val, ISC_LOG_DEBUG(3), "in fetch_callback_dnskey");
-
 	dns_resolver_destroyfetch(&val->fetch);
 
 	if (CANCELED(val) || CANCELING(val)) {
@@ -485,7 +449,7 @@ fetch_callback_dnskey(void *arg) {
 	}
 
 cleanup:
-	dns_resolver_freefresp(&resp);
+	isc_mem_putanddetach(&resp->mctx, resp, sizeof(*resp));
 	validate_async_done(val, result);
 	dns_validator_detach(&val);
 }
@@ -521,7 +485,6 @@ fetch_callback_ds(void *arg) {
 	}
 
 	validator_log(val, ISC_LOG_DEBUG(3), "in fetch_callback_ds");
-
 	dns_resolver_destroyfetch(&val->fetch);
 
 	if (CANCELED(val) || CANCELING(val)) {
@@ -616,7 +579,7 @@ fetch_callback_ds(void *arg) {
 	}
 
 cleanup:
-	dns_resolver_freefresp(&resp);
+	isc_mem_putanddetach(&resp->mctx, resp, sizeof(*resp));
 	validate_async_done(val, result);
 	dns_validator_detach(&val);
 }
@@ -653,19 +616,13 @@ validator_callback_dnskey(void *arg) {
 			result = validate_async_run(val, resume_answer);
 		}
 	} else {
+		if (result != DNS_R_BROKENCHAIN) {
+			expire_rdatasets(val);
+		}
 		validator_log(val, ISC_LOG_DEBUG(3),
 			      "validator_callback_dnskey: got %s",
 			      isc_result_totext(result));
-		if (result != DNS_R_BROKENCHAIN) {
-			expire_rdatasets(val);
-			result = create_fetch(val, &val->siginfo->signer,
-					      dns_rdatatype_dnskey,
-					      fetch_callback_dnskey,
-					      "validator_callback_dnskey");
-			if (result == ISC_R_SUCCESS) {
-				result = DNS_R_WAIT;
-			}
-		}
+		result = DNS_R_BROKENCHAIN;
 	}
 
 cleanup:
@@ -684,7 +641,8 @@ static void
 validator_callback_ds(void *arg) {
 	dns_validator_t *subvalidator = (dns_validator_t *)arg;
 	dns_validator_t *val = subvalidator->parent;
-	isc_result_t result = subvalidator->result;
+	isc_result_t result;
+	isc_result_t eresult = subvalidator->result;
 
 	val->subvalidator = NULL;
 
@@ -694,7 +652,7 @@ validator_callback_ds(void *arg) {
 	}
 
 	validator_log(val, ISC_LOG_DEBUG(3), "in validator_callback_ds");
-	if (result == ISC_R_SUCCESS) {
+	if (eresult == ISC_R_SUCCESS) {
 		bool have_dsset;
 		dns_name_t *name;
 		validator_log(val, ISC_LOG_DEBUG(3), "%s with trust %s",
@@ -717,18 +675,13 @@ validator_callback_ds(void *arg) {
 			result = validate_async_run(val, validate_dnskey);
 		}
 	} else {
+		if (eresult != DNS_R_BROKENCHAIN) {
+			expire_rdatasets(val);
+		}
 		validator_log(val, ISC_LOG_DEBUG(3),
 			      "validator_callback_ds: got %s",
-			      isc_result_totext(result));
-		if (result != DNS_R_BROKENCHAIN) {
-			expire_rdatasets(val);
-			result = create_fetch(val, val->name, dns_rdatatype_ds,
-					      fetch_callback_ds,
-					      "validator_callback_ds");
-			if (result == ISC_R_SUCCESS) {
-				result = DNS_R_WAIT;
-			}
-		}
+			      isc_result_totext(eresult));
+		result = DNS_R_BROKENCHAIN;
 	}
 
 cleanup:
@@ -986,8 +939,8 @@ create_fetch(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
 	dns_validator_ref(val);
 	result = dns_resolver_createfetch(
 		val->view->resolver, name, type, NULL, NULL, NULL, NULL, 0,
-		fopts, 0, val->qc, val->gqc, val->loop, callback, val,
-		&val->edectx, &val->frdataset, &val->fsigrdataset, &val->fetch);
+		fopts, 0, NULL, val->loop, callback, val, &val->frdataset,
+		&val->fsigrdataset, &val->fetch);
 	if (result != ISC_R_SUCCESS) {
 		dns_validator_detach(&val);
 	}
@@ -1021,10 +974,10 @@ create_validator(dns_validator_t *val, dns_name_t *name, dns_rdatatype_t type,
 		  (DNS_VALIDATOR_NOCDFLAG | DNS_VALIDATOR_NONTA));
 
 	validator_logcreate(val, name, type, caller, "validator");
-	result = dns_validator_create(
-		val->view, name, type, rdataset, sig, NULL, vopts, val->loop,
-		cb, val, val->nvalidations, val->nfails, val->qc, val->gqc,
-		&val->edectx, &val->subvalidator);
+	result = dns_validator_create(val->view, name, type, rdataset, sig,
+				      NULL, vopts, val->loop, cb, val,
+				      val->nvalidations, val->nfails, val->qc,
+				      &val->subvalidator);
 	if (result == ISC_R_SUCCESS) {
 		dns_validator_attach(val, &val->subvalidator->parent);
 		val->subvalidator->depth = val->depth + 1;
@@ -1093,8 +1046,6 @@ select_signing_key(dns_validator_t *val, dns_rdataset_t *rdataset) {
 				goto done;
 			}
 			dst_key_free(&val->key);
-		} else {
-			break;
 		}
 		dns_rdata_reset(&rdata);
 		result = dns_rdataset_next(rdataset);
@@ -1181,13 +1132,14 @@ seek_dnskey(dns_validator_t *val) {
 		 * We have an rrset for the given keyname.
 		 */
 		val->keyset = &val->frdataset;
-		if (DNS_TRUST_PENDING(val->frdataset.trust) ||
-		    DNS_TRUST_ANSWER(val->frdataset.trust))
+		if ((DNS_TRUST_PENDING(val->frdataset.trust) ||
+		     DNS_TRUST_ANSWER(val->frdataset.trust)) &&
+		    dns_rdataset_isassociated(&val->fsigrdataset))
 		{
 			/*
-			 * We know the key but haven't validated it yet, or
-			 * we had a key with trust level "answer" and
-			 * a DS record for the zone has now been added.
+			 * We know the key but haven't validated it yet or
+			 * we have a key of trust answer but a DS
+			 * record for the zone may have been added.
 			 */
 			result = create_validator(
 				val, &siginfo->signer, dns_rdatatype_dnskey,
@@ -1197,6 +1149,12 @@ seek_dnskey(dns_validator_t *val) {
 				return result;
 			}
 			return DNS_R_WAIT;
+		} else if (DNS_TRUST_PENDING(val->frdataset.trust)) {
+			/*
+			 * Having a pending key with no signature means that
+			 * something is broken.
+			 */
+			result = DNS_R_CONTINUE;
 		} else if (val->frdataset.trust < dns_trust_secure) {
 			/*
 			 * The key is legitimately insecure.  There's no
@@ -1283,10 +1241,7 @@ compute_keytag(dns_rdata_t *rdata) {
 
 static bool
 over_max_validations(dns_validator_t *val) {
-	if (val->nvalidations == NULL ||
-	    isc_counter_used(val->nvalidations) <
-		    isc_counter_getlimit(val->nvalidations))
-	{
+	if (val->nvalidations == NULL || (*val->nvalidations) > 0) {
 		return false;
 	}
 
@@ -1300,14 +1255,14 @@ consume_validation(dns_validator_t *val) {
 	if (val->nvalidations == NULL) {
 		return;
 	}
-	(void)isc_counter_increment(val->nvalidations);
+	INSIST((*val->nvalidations) > 0);
+
+	(*val->nvalidations)--;
 }
 
 static bool
 over_max_fails(dns_validator_t *val) {
-	if (val->nfails == NULL ||
-	    isc_counter_used(val->nfails) < isc_counter_getlimit(val->nfails))
-	{
+	if (val->nfails == NULL || (*val->nfails) > 0) {
 		return false;
 	}
 
@@ -1321,7 +1276,9 @@ consume_validation_fail(dns_validator_t *val) {
 	if (val->nfails == NULL) {
 		return;
 	}
-	(void)isc_counter_increment(val->nfails);
+	INSIST((*val->nfails) > 0);
+
+	(*val->nfails)--;
 }
 
 /*%
@@ -1334,7 +1291,6 @@ selfsigned_dnskey(dns_validator_t *val) {
 	dns_name_t *name = val->name;
 	isc_result_t result;
 	isc_mem_t *mctx = val->view->mctx;
-	bool match = false;
 
 	if (rdataset->type != dns_rdatatype_dnskey) {
 		return DNS_R_NOKEYMATCH;
@@ -1375,16 +1331,17 @@ selfsigned_dnskey(dns_validator_t *val) {
 
 			/*
 			 * If the REVOKE bit is not set we have a
-			 * theoretically self-signed DNSKEY RRset;
-			 * this will be verified later.
-			 *
-			 * We don't return the answer yet, though,
-			 * because we need to check the remaining keys
-			 * and possbly remove them if they're revoked.
+			 * theoretically self signed DNSKEY RRset.
+			 * This will be verified later.
 			 */
 			if ((key.flags & DNS_KEYFLAG_REVOKE) == 0) {
-				match = true;
-				break;
+				return ISC_R_SUCCESS;
+			}
+
+			result = dns_dnssec_keyfromrdata(name, &keyrdata, mctx,
+							 &dstkey);
+			if (result != ISC_R_SUCCESS) {
+				continue;
 			}
 
 			/*
@@ -1394,20 +1351,6 @@ selfsigned_dnskey(dns_validator_t *val) {
 			if (DNS_TRUST_PENDING(rdataset->trust) &&
 			    dns_view_istrusted(val->view, name, &key))
 			{
-				result = dns_dnssec_keyfromrdata(
-					name, &keyrdata, mctx, &dstkey);
-				if (result == DST_R_UNSUPPORTEDALG) {
-					/* don't count towards max fails */
-					break; /* continue with next key */
-				} else if (result != ISC_R_SUCCESS) {
-					consume_validation(val);
-					if (over_max_fails(val)) {
-						return ISC_R_QUOTA;
-					}
-					consume_validation_fail(val);
-					break; /* continue with next key */
-				}
-
 				if (over_max_validations(val)) {
 					dst_key_free(&dstkey);
 					return ISC_R_QUOTA;
@@ -1441,8 +1384,6 @@ selfsigned_dnskey(dns_validator_t *val) {
 					}
 					consume_validation_fail(val);
 				}
-
-				dst_key_free(&dstkey);
 			} else if (rdataset->trust >= dns_trust_secure) {
 				/*
 				 * We trust this RRset so if the key is
@@ -1450,14 +1391,12 @@ selfsigned_dnskey(dns_validator_t *val) {
 				 */
 				dns_view_untrust(val->view, name, &key);
 			}
+
+			dst_key_free(&dstkey);
 		}
 	}
 
-	if (!match) {
-		return DNS_R_NOKEYMATCH;
-	}
-
-	return ISC_R_SUCCESS;
+	return DNS_R_NOKEYMATCH;
 }
 
 /*%
@@ -1534,11 +1473,6 @@ again:
 		 * Temporal errors don't count towards max validations nor max
 		 * fails.
 		 */
-		validator_addede(val,
-				 result == DNS_R_SIGEXPIRED
-					 ? DNS_EDE_SIGNATUREEXPIRED
-					 : DNS_EDE_SIGNATURENOTYETVALID,
-				 NULL);
 		break;
 	case ISC_R_SUCCESS:
 		consume_validation(val);
@@ -1603,8 +1537,6 @@ cleanup:
 		return;
 	}
 
-	val->unsupported_algorithm = 0;
-	val->unsupported_digest = 0;
 	result = validate_async_run(val, validate_answer_process);
 	INSIST(result == DNS_R_WAIT);
 }
@@ -1642,7 +1574,7 @@ validate_answer_signing_key_done(void *arg);
 static void
 validate_answer_signing_key(void *arg) {
 	dns_validator_t *val = arg;
-	isc_result_t result;
+	isc_result_t result = ISC_R_NOTFOUND;
 
 	if (CANCELED(val) || CANCELING(val)) {
 		val->result = ISC_R_CANCELED;
@@ -1665,21 +1597,15 @@ validate_answer_signing_key(void *arg) {
 	default:
 		/* Select next signing key */
 		result = select_signing_key(val, val->keyset);
-		if (result == ISC_R_SUCCESS) {
-			INSIST(val->key != NULL);
-		} else if (result == ISC_R_NOTFOUND) {
-			INSIST(val->key == NULL);
-		} else {
-			val->result = result;
-			if (over_max_fails(val)) {
-				INSIST(val->key == NULL);
-				val->result = ISC_R_QUOTA;
-			}
-			consume_validation_fail(val);
-		}
 		break;
 	}
 
+	if (result == ISC_R_SUCCESS) {
+		INSIST(val->key != NULL);
+	} else {
+		INSIST(val->key == NULL);
+	}
+
 	(void)validate_async_run(val, validate_answer_signing_key_done);
 }
 
@@ -1728,13 +1654,9 @@ validate_answer_process(void *arg) {
 	 * At this point we could check that the signature algorithm
 	 * was known and "sufficiently good".
 	 */
-	if (!dns_resolver_algorithm_supported(val->view->resolver,
-					      &val->siginfo->signer,
+	if (!dns_resolver_algorithm_supported(val->view->resolver, val->name,
 					      val->siginfo->algorithm))
 	{
-		if (val->unsupported_algorithm == 0) {
-			val->unsupported_algorithm = val->siginfo->algorithm;
-		}
 		goto next_key;
 	}
 
@@ -1858,10 +1780,6 @@ validate_answer_iter_done(dns_validator_t *val, isc_result_t result) {
 		return;
 	}
 
-	if (result != ISC_R_SUCCESS && result != DNS_R_WAIT) {
-		validate_extendederror(val);
-	}
-
 	validator_log(val, ISC_LOG_INFO, "no valid signature found");
 	validate_async_done(val, val->result);
 }
@@ -1925,15 +1843,14 @@ check_signer(dns_validator_t *val, dns_rdata_t *keyrdata, uint16_t keyid,
 	dns_rdata_rrsig_t sig;
 	dst_key_t *dstkey = NULL;
 	isc_result_t result;
-	dns_rdataset_t rdataset = DNS_RDATASET_INIT;
-	dns_rdataset_clone(val->sigrdataset, &rdataset);
 
-	for (result = dns_rdataset_first(&rdataset); result == ISC_R_SUCCESS;
-	     result = dns_rdataset_next(&rdataset))
+	for (result = dns_rdataset_first(val->sigrdataset);
+	     result == ISC_R_SUCCESS;
+	     result = dns_rdataset_next(val->sigrdataset))
 	{
 		dns_rdata_t rdata = DNS_RDATA_INIT;
 
-		dns_rdataset_current(&rdataset, &rdata);
+		dns_rdataset_current(val->sigrdataset, &rdata);
 		result = dns_rdata_tostruct(&rdata, &sig, NULL);
 		RUNTIME_CHECK(result == ISC_R_SUCCESS);
 		if (keyid != sig.keyid || algorithm != sig.algorithm) {
@@ -1943,7 +1860,10 @@ check_signer(dns_validator_t *val, dns_rdata_t *keyrdata, uint16_t keyid,
 			result = dns_dnssec_keyfromrdata(
 				val->name, keyrdata, val->view->mctx, &dstkey);
 			if (result != ISC_R_SUCCESS) {
-				return result;
+				/*
+				 * This really shouldn't happen, but...
+				 */
+				continue;
 			}
 		}
 		result = verify(val, dstkey, &rdata, sig.keyid);
@@ -1955,7 +1875,6 @@ check_signer(dns_validator_t *val, dns_rdata_t *keyrdata, uint16_t keyid,
 	if (dstkey != NULL) {
 		dst_key_free(&dstkey);
 	}
-	dns_rdataset_disassociate(&rdataset);
 
 	return result;
 }
@@ -1984,8 +1903,9 @@ get_dsset(dns_validator_t *val, dns_name_t *tname, isc_result_t *resp) {
 		 * We have a DS RRset.
 		 */
 		val->dsset = &val->frdataset;
-		if (DNS_TRUST_PENDING(val->frdataset.trust) ||
-		    DNS_TRUST_ANSWER(val->frdataset.trust))
+		if ((DNS_TRUST_PENDING(val->frdataset.trust) ||
+		     DNS_TRUST_ANSWER(val->frdataset.trust)) &&
+		    dns_rdataset_isassociated(&val->fsigrdataset))
 		{
 			/*
 			 * ... which is signed but not yet validated.
@@ -1993,12 +1913,21 @@ get_dsset(dns_validator_t *val, dns_name_t *tname, isc_result_t *resp) {
 			result = create_validator(
 				val, tname, dns_rdatatype_ds, &val->frdataset,
 				&val->fsigrdataset, validator_callback_ds,
-				"get_dsset");
+				"validate_dnskey");
 			*resp = DNS_R_WAIT;
 			if (result != ISC_R_SUCCESS) {
 				*resp = result;
 			}
 			return ISC_R_COMPLETE;
+		} else if (DNS_TRUST_PENDING(val->frdataset.trust)) {
+			/*
+			 * There should never be an unsigned DS.
+			 */
+			disassociate_rdatasets(val);
+			validator_log(val, ISC_LOG_DEBUG(2),
+				      "unsigned DS record");
+			*resp = DNS_R_NOVALIDSIG;
+			return ISC_R_COMPLETE;
 		}
 		break;
 
@@ -2051,15 +1980,12 @@ validate_dnskey_dsset_done(dns_validator_t *val, isc_result_t result) {
 		validator_log(val, ISC_LOG_DEBUG(3), "marking as secure (DS)");
 		break;
 	case ISC_R_NOMORE:
-		if (val->unsupported_algorithm != 0 ||
-		    val->unsupported_digest != 0)
-		{
+		if (!val->supported_algorithm) {
 			validator_log(val, ISC_LOG_DEBUG(3),
 				      "no supported algorithm/digest (DS)");
 			result = markanswer(
 				val, "validate_dnskey (3)",
 				"no supported algorithm/digest (DS)");
-			validate_extendederror(val);
 			break;
 		}
 		FALLTHROUGH;
@@ -2096,21 +2022,17 @@ validate_dnskey_dsset(dns_validator_t *val) {
 	if (!dns_resolver_ds_digest_supported(val->view->resolver, val->name,
 					      ds.digest_type))
 	{
-		if (val->unsupported_digest == 0) {
-			val->unsupported_digest = ds.digest_type;
-		}
 		return DNS_R_BADALG;
 	}
 
 	if (!dns_resolver_algorithm_supported(val->view->resolver, val->name,
 					      ds.algorithm))
 	{
-		if (val->unsupported_algorithm == 0) {
-			val->unsupported_algorithm = ds.algorithm;
-		}
 		return DNS_R_BADALG;
 	}
 
+	val->supported_algorithm = true;
+
 	/*
 	 * Find the DNSKEY matching the DS...
 	 */
@@ -2282,8 +2204,8 @@ validate_dnskey(void *arg) {
 	 * key set and the matching signature.  For each such key, attempt
 	 * verification.
 	 */
-	val->unsupported_algorithm = 0;
-	val->unsupported_digest = 0;
+
+	val->supported_algorithm = false;
 
 	/*
 	 * If DNS_DSDIGEST_SHA256 or DNS_DSDIGEST_SHA384 is present we
@@ -3021,13 +2943,6 @@ check_ds_algs(dns_validator_t *val, dns_name_t *name,
 		}
 		dns_rdata_reset(&dsrdata);
 	}
-
-	/*
-	 * No unsupported alg/digest EDE error is raised here because the prove
-	 * unsecure flow always runs after a validate/validatenx flow. So if an
-	 * unsupported alg/digest was found while building the chain of trust,
-	 * it would be raised already.
-	 */
 	return false;
 }
 
@@ -3076,7 +2991,7 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 					val, ISC_LOG_DEBUG(3),
 					"no supported algorithm/digest (%s/DS)",
 					namebuf);
-				*resp = markanswer(val, "seek_ds (1)",
+				*resp = markanswer(val, "proveunsecure (5)",
 						   "no supported "
 						   "algorithm/digest (DS)");
 				return ISC_R_COMPLETE;
@@ -3088,12 +3003,22 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 		/*
 		 * Otherwise, try to validate it now.
 		 */
-		result = create_validator(val, tname, dns_rdatatype_ds,
-					  &val->frdataset, &val->fsigrdataset,
-					  validator_callback_ds, "seek_ds");
-		*resp = DNS_R_WAIT;
-		if (result != ISC_R_SUCCESS) {
-			*resp = result;
+		if (dns_rdataset_isassociated(&val->fsigrdataset)) {
+			result = create_validator(
+				val, tname, dns_rdatatype_ds, &val->frdataset,
+				&val->fsigrdataset, validator_callback_ds,
+				"proveunsecure");
+			*resp = DNS_R_WAIT;
+			if (result != ISC_R_SUCCESS) {
+				*resp = result;
+			}
+		} else {
+			/*
+			 * There should never be an unsigned DS.
+			 */
+			validator_log(val, ISC_LOG_DEBUG(3),
+				      "unsigned DS record");
+			*resp = DNS_R_NOVALIDSIG;
 		}
 
 		return ISC_R_COMPLETE;
@@ -3104,7 +3029,7 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 		 */
 		*resp = DNS_R_WAIT;
 		result = create_fetch(val, tname, dns_rdatatype_ds,
-				      fetch_callback_ds, "seek_ds");
+				      fetch_callback_ds, "proveunsecure");
 		if (result != ISC_R_SUCCESS) {
 			*resp = result;
 		}
@@ -3125,7 +3050,7 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 			result = create_validator(
 				val, tname, dns_rdatatype_ds, &val->frdataset,
 				&val->fsigrdataset, validator_callback_ds,
-				"seek_ds");
+				"proveunsecure");
 			*resp = DNS_R_WAIT;
 			if (result != ISC_R_SUCCESS) {
 				*resp = result;
@@ -3145,7 +3070,7 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 					 NULL) == ISC_R_SUCCESS &&
 		    dns_name_equal(tname, found))
 		{
-			*resp = markanswer(val, "seek_ds (2)",
+			*resp = markanswer(val, "proveunsecure (3)",
 					   "no DS at zone cut");
 			return ISC_R_COMPLETE;
 		}
@@ -3165,7 +3090,7 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 		}
 
 		if (isdelegation(tname, &val->frdataset, result)) {
-			*resp = markanswer(val, "seek_ds (3)",
+			*resp = markanswer(val, "proveunsecure (4)",
 					   "this is a delegation");
 			return ISC_R_COMPLETE;
 		}
@@ -3197,7 +3122,7 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 			result = create_validator(
 				val, tname, dns_rdatatype_ds, &val->frdataset,
 				&val->fsigrdataset, validator_callback_ds,
-				"seek_ds");
+				"proveunsecure");
 			if (result != ISC_R_SUCCESS) {
 				*resp = result;
 			}
@@ -3226,7 +3151,9 @@ seek_ds(dns_validator_t *val, isc_result_t *resp) {
 			result = create_validator(
 				val, tname, dns_rdatatype_cname,
 				&val->frdataset, &val->fsigrdataset,
-				validator_callback_cname, "seek_ds (cname)");
+				validator_callback_cname,
+				"proveunsecure "
+				"(cname)");
 			*resp = DNS_R_WAIT;
 			if (result != ISC_R_SUCCESS) {
 				*resp = result;
@@ -3465,9 +3392,8 @@ dns_validator_create(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
 		     dns_rdataset_t *rdataset, dns_rdataset_t *sigrdataset,
 		     dns_message_t *message, unsigned int options,
 		     isc_loop_t *loop, isc_job_cb cb, void *arg,
-		     isc_counter_t *nvalidations, isc_counter_t *nfails,
-		     isc_counter_t *qc, isc_counter_t *gqc,
-		     dns_edectx_t *edectx, dns_validator_t **validatorp) {
+		     uint32_t *nvalidations, uint32_t *nfails,
+		     isc_counter_t *qc, dns_validator_t **validatorp) {
 	isc_result_t result = ISC_R_FAILURE;
 	dns_validator_t *val = NULL;
 	dns_keytable_t *kt = NULL;
@@ -3476,7 +3402,6 @@ dns_validator_create(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
 	REQUIRE(rdataset != NULL ||
 		(rdataset == NULL && sigrdataset == NULL && message != NULL));
 	REQUIRE(validatorp != NULL && *validatorp == NULL);
-	REQUIRE(edectx != NULL);
 
 	result = dns_view_getsecroots(view, &kt);
 	if (result != ISC_R_SUCCESS) {
@@ -3498,31 +3423,19 @@ dns_validator_create(dns_view_t *view, dns_name_t *name, dns_rdatatype_t type,
 		.cb = cb,
 		.arg = arg,
 		.rdata = DNS_RDATA_INIT,
-		.cb_edectx = edectx,
+		.nvalidations = nvalidations,
+		.nfails = nfails,
 	};
 
-	dns_ede_init(view->mctx, &val->edectx);
-
 	isc_refcount_init(&val->references, 1);
 	dns_view_attach(view, &val->view);
 	if (message != NULL) {
 		dns_message_attach(message, &val->message);
 	}
 
-	if (nfails != NULL) {
-		isc_counter_attach(nfails, &val->nfails);
-	}
-
-	if (nvalidations != NULL) {
-		isc_counter_attach(nvalidations, &val->nvalidations);
-	}
-
 	if (qc != NULL) {
 		isc_counter_attach(qc, &val->qc);
 	}
-	if (gqc != NULL) {
-		isc_counter_attach(gqc, &val->gqc);
-	}
 
 	val->mustbesecure = dns_resolver_getmustbesecure(view->resolver, name);
 	dns_rdataset_init(&val->fdsset);
@@ -3610,24 +3523,11 @@ destroy_validator(dns_validator_t *val) {
 	if (val->message != NULL) {
 		dns_message_detach(&val->message);
 	}
-	if (val->nfails != NULL) {
-		isc_counter_detach(&val->nfails);
-	}
-	if (val->nvalidations != NULL) {
-		isc_counter_detach(&val->nvalidations);
-	}
 	if (val->qc != NULL) {
 		isc_counter_detach(&val->qc);
 	}
-	if (val->gqc != NULL) {
-		isc_counter_detach(&val->gqc);
-	}
-
-	dns_ede_invalidate(&val->edectx);
-
 	dns_view_detach(&val->view);
 	isc_loop_detach(&val->loop);
-
 	isc_mem_put(mctx, val, sizeof(*val));
 }
 
@@ -3650,9 +3550,8 @@ dns_validator_shutdown(dns_validator_t *val) {
 }
 
 static void
-validator_logv(dns_validator_t *val, isc_logcategory_t *category,
-	       isc_logmodule_t *module, int level, const char *fmt,
-	       va_list ap) {
+validator_logv(dns_validator_t *val, isc_logcategory_t category,
+	       isc_logmodule_t module, int level, const char *fmt, va_list ap) {
 	char msgbuf[2048];
 	static const char spaces[] = "        *";
 	int depth = val->depth * 2;
@@ -3688,11 +3587,11 @@ validator_logv(dns_validator_t *val, isc_logcategory_t *category,
 
 		dns_name_format(val->name, namebuf, sizeof(namebuf));
 		dns_rdatatype_format(val->type, typebuf, sizeof(typebuf));
-		isc_log_write(dns_lctx, category, module, level,
+		isc_log_write(category, module, level,
 			      "%s%s%s%.*svalidating %s/%s: %s", sep1, viewname,
 			      sep2, depth, spaces, namebuf, typebuf, msgbuf);
 	} else {
-		isc_log_write(dns_lctx, category, module, level,
+		isc_log_write(category, module, level,
 			      "%s%s%s%.*svalidator @%p: %s", sep1, viewname,
 			      sep2, depth, spaces, val, msgbuf);
 	}
@@ -3702,7 +3601,7 @@ static void
 validator_log(void *val, int level, const char *fmt, ...) {
 	va_list ap;
 
-	if (!isc_log_wouldlog(dns_lctx, level)) {
+	if (!isc_log_wouldlog(level)) {
 		return;
 	}
 
@@ -3726,58 +3625,6 @@ validator_logcreate(dns_validator_t *val, dns_name_t *name,
 		      caller, operation, namestr, typestr);
 }
 
-static void
-validator_addede(dns_validator_t *val, uint16_t code, const char *extra) {
-	REQUIRE(VALID_VALIDATOR(val));
-
-	char bdata[DNS_NAME_FORMATSIZE + DNS_RDATATYPE_FORMATSIZE +
-		   DNS_EDE_EXTRATEXT_LEN];
-	isc_buffer_t b;
-
-	isc_buffer_init(&b, bdata, sizeof(bdata));
-
-	if (extra != NULL) {
-		isc_buffer_putstr(&b, extra);
-		isc_buffer_putuint8(&b, ' ');
-	}
-
-	dns_name_totext(val->name, DNS_NAME_OMITFINALDOT, &b);
-	isc_buffer_putuint8(&b, '/');
-	dns_rdatatype_totext(val->type, &b);
-	isc_buffer_putuint8(&b, '\0');
-
-	dns_ede_add(&val->edectx, code, bdata);
-}
-
-static void
-validate_extendederror(dns_validator_t *val) {
-	dns_validator_t *edeval = val;
-	char bdata[DNS_EDE_EXTRATEXT_LEN];
-	isc_buffer_t b;
-
-	REQUIRE(VALID_VALIDATOR(edeval));
-
-	isc_buffer_init(&b, bdata, sizeof(bdata));
-
-	while (edeval->parent != NULL) {
-		edeval = edeval->parent;
-	}
-
-	if (val->unsupported_algorithm != 0) {
-		isc_buffer_clear(&b);
-		dns_secalg_totext(val->unsupported_algorithm, &b);
-		isc_buffer_putuint8(&b, '\0');
-		validator_addede(val, DNS_EDE_DNSKEYALG, bdata);
-	}
-
-	if (val->unsupported_digest != 0) {
-		isc_buffer_clear(&b);
-		dns_dsdigest_totext(val->unsupported_digest, &b);
-		isc_buffer_putuint8(&b, '\0');
-		validator_addede(val, DNS_EDE_DSDIGESTTYPE, bdata);
-	}
-}
-
 #if DNS_VALIDATOR_TRACE
 ISC_REFCOUNT_TRACE_IMPL(dns_validator, destroy_validator);
 #else
