@@ -51,7 +51,6 @@
 #include <dns/nta.h>
 #include <dns/order.h>
 #include <dns/peer.h>
-#include <dns/rbt.h>
 #include <dns/rdataset.h>
 #include <dns/request.h>
 #include <dns/resolver.h>
@@ -545,27 +544,6 @@ dns_view_detach(dns_view_t **viewp) {
 	}
 }
 
-static isc_result_t
-dialup(dns_zone_t *zone, void *dummy) {
-	UNUSED(dummy);
-	dns_zone_dialup(zone);
-	return ISC_R_SUCCESS;
-}
-
-void
-dns_view_dialup(dns_view_t *view) {
-	dns_zt_t *zonetable = NULL;
-
-	REQUIRE(DNS_VIEW_VALID(view));
-
-	rcu_read_lock();
-	zonetable = rcu_dereference(view->zonetable);
-	if (zonetable != NULL) {
-		(void)dns_zt_apply(zonetable, false, NULL, dialup, NULL);
-	}
-	rcu_read_unlock();
-}
-
 void
 dns_view_weakattach(dns_view_t *source, dns_view_t **targetp) {
 	REQUIRE(DNS_VIEW_VALID(source));
@@ -785,8 +763,6 @@ dns_view_delzone(dns_view_t *view, dns_zone_t *zone) {
 
 	REQUIRE(DNS_VIEW_VALID(view));
 
-	dns_zone_prepare_shutdown(zone);
-
 	rcu_read_lock();
 	zonetable = rcu_dereference(view->zonetable);
 	if (zonetable != NULL) {
@@ -1626,7 +1602,9 @@ dns_view_issecuredomain(dns_view_t *view, const dns_name_t *name,
 		return result;
 	}
 
-	SET_IF_NOT_NULL(ntap, false);
+	if (ntap != NULL) {
+		*ntap = false;
+	}
 	if (checknta && secure && view->ntatable_priv != NULL &&
 	    dns_ntatable_covered(view->ntatable_priv, now, name, anchor))
 	{
@@ -1865,9 +1843,8 @@ dns_view_setnewzones(dns_view_t *view, bool allow, void *cfgctx,
 
 	status = mdb_env_create(&env);
 	if (status != MDB_SUCCESS) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      ISC_LOGMODULE_OTHER, ISC_LOG_ERROR,
-			      "mdb_env_create failed: %s",
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, ISC_LOGMODULE_OTHER,
+			      ISC_LOG_ERROR, "mdb_env_create failed: %s",
 			      mdb_strerror(status));
 		CHECK(ISC_R_FAILURE);
 	}
@@ -1875,7 +1852,7 @@ dns_view_setnewzones(dns_view_t *view, bool allow, void *cfgctx,
 	if (mapsize != 0ULL) {
 		status = mdb_env_set_mapsize(env, mapsize);
 		if (status != MDB_SUCCESS) {
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
+			isc_log_write(DNS_LOGCATEGORY_GENERAL,
 				      ISC_LOGMODULE_OTHER, ISC_LOG_ERROR,
 				      "mdb_env_set_mapsize failed: %s",
 				      mdb_strerror(status));
@@ -1886,9 +1863,8 @@ dns_view_setnewzones(dns_view_t *view, bool allow, void *cfgctx,
 
 	status = mdb_env_open(env, view->new_zone_db, DNS_LMDB_FLAGS, 0600);
 	if (status != MDB_SUCCESS) {
-		isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,
-			      ISC_LOGMODULE_OTHER, ISC_LOG_ERROR,
-			      "mdb_env_open of '%s' failed: %s",
+		isc_log_write(DNS_LOGCATEGORY_GENERAL, ISC_LOGMODULE_OTHER,
+			      ISC_LOG_ERROR, "mdb_env_open of '%s' failed: %s",
 			      view->new_zone_db, mdb_strerror(status));
 		CHECK(ISC_R_FAILURE);
 	}
@@ -2185,8 +2161,8 @@ dns_view_loadnta(dns_view_t *view) {
 		} else {
 			char nb[DNS_NAME_FORMATSIZE];
 			dns_name_format(ntaname, nb, sizeof(nb));
-			isc_log_write(dns_lctx, DNS_LOGCATEGORY_DNSSEC,
-				      DNS_LOGMODULE_NTA, ISC_LOG_INFO,
+			isc_log_write(DNS_LOGCATEGORY_DNSSEC, DNS_LOGMODULE_NTA,
+				      ISC_LOG_INFO,
 				      "ignoring expired NTA at %s", nb);
 		}
 	}
@@ -2471,11 +2447,3 @@ dns_view_setmaxrestarts(dns_view_t *view, uint8_t max_restarts) {
 
 	view->max_restarts = max_restarts;
 }
-
-void
-dns_view_setmaxqueries(dns_view_t *view, uint16_t max_queries) {
-	REQUIRE(DNS_VIEW_VALID(view));
-	REQUIRE(max_queries > 0);
-
-	view->max_queries = max_queries;
-}
