@@ -1,9 +1,11 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
@@ -33,7 +35,7 @@ dns_zonekey_iszonekey(dns_rdata_t *keyrdata) {
 
 	result = dns_rdata_tostruct(keyrdata, &key, NULL);
 	if (result != ISC_R_SUCCESS) {
-		return (false);
+		return false;
 	}
 
 	if ((key.flags & DNS_KEYTYPE_NOAUTH) != 0) {
@@ -43,9 +45,10 @@ dns_zonekey_iszonekey(dns_rdata_t *keyrdata) {
 		iszonekey = false;
 	}
 	if (key.protocol != DNS_KEYPROTO_DNSSEC &&
-	    key.protocol != DNS_KEYPROTO_ANY) {
+	    key.protocol != DNS_KEYPROTO_ANY)
+	{
 		iszonekey = false;
 	}
 
-	return (iszonekey);
+	return iszonekey;
 }
