@@ -33,8 +33,7 @@ typedef void (*isc_assertioncallback_t)(const char *, int, isc_assertiontype_t,
 					const char *);
 
 /* coverity[+kill] */
-ISC_NORETURN
-void
+ISC_NORETURN void
 isc_assertion_failed(const char *, int, isc_assertiontype_t, const char *);
 
 void isc_assertion_setcallback(isc_assertioncallback_t);
@@ -66,4 +65,9 @@ isc_assertion_typetotext(isc_assertiontype_t type);
 					isc_assertiontype_invariant, #cond), \
 		 0)))
 
+#define ISC_UNREACHABLE()                                                   \
+	(isc_assertion_failed(__FILE__, __LINE__, isc_assertiontype_insist, \
+			      "unreachable"),                               \
+	 __builtin_unreachable())
+
 ISC_LANG_ENDDECLS
