@@ -49,10 +49,6 @@ typedef void(isc_httpdondestroy_t)(void *); /*%< Callback on destroying httpd */
 typedef struct isc_interface	 isc_interface_t;     /*%< Interface */
 typedef struct isc_interfaceiter isc_interfaceiter_t; /*%< Interface Iterator */
 typedef struct isc_lex		 isc_lex_t;	      /*%< Lex */
-typedef struct isc_log		 isc_log_t;	      /*%< Log */
-typedef struct isc_logcategory	 isc_logcategory_t;   /*%< Log Category */
-typedef struct isc_logconfig	 isc_logconfig_t;     /*%< Log Configuration */
-typedef struct isc_logmodule	 isc_logmodule_t;     /*%< Log Module */
 typedef struct isc_loop		 isc_loop_t;	      /*%< Event loop */
 typedef struct isc_loopmgr	 isc_loopmgr_t;	      /*%< Event loop manager */
 typedef struct isc_mem		 isc_mem_t;	      /*%< Memory */
@@ -86,12 +82,6 @@ typedef struct isc_nm_http_endpoints isc_nm_http_endpoints_t;
 /*%< HTTP endpoints set */
 #endif /* HAVE_LIBNGHTTP2 */
 
-/*% Used by isc_base64 and isc_hex for expected lower bound */
-enum {
-	isc_zero_or_more = -1,
-	isc_one_or_more = -2,
-};
-
 /*% Statistics formats (text file or XML) */
 typedef enum {
 	isc_statsformat_file,
