@@ -13,6 +13,7 @@
 
 /*! \file */
 
+#include <isc/crypto.h>
 #include <isc/hash.h>
 #include <isc/iterated_hash.h>
 #include <isc/md.h>
@@ -24,7 +25,6 @@
 #include <isc/uv.h>
 #include <isc/xml.h>
 
-#include "config.h"
 #include "mem_p.h"
 #include "mutex_p.h"
 #include "os_p.h"
@@ -47,10 +47,10 @@ isc__initialize(void) {
 	isc__os_initialize();
 	isc__mutex_initialize();
 	isc__mem_initialize();
-	isc__tls_initialize();
+	isc__log_initialize();
+	isc__crypto_initialize();
 	isc__uv_initialize();
 	isc__xml_initialize();
-	isc__md_initialize();
 	isc__hash_initialize();
 	isc__iterated_hash_initialize();
 	(void)isc_os_ncpus();
@@ -60,10 +60,10 @@ isc__initialize(void) {
 void
 isc__shutdown(void) {
 	isc__iterated_hash_shutdown();
-	isc__md_shutdown();
 	isc__xml_shutdown();
 	isc__uv_shutdown();
-	isc__tls_shutdown();
+	isc__crypto_shutdown();
+	isc__log_shutdown();
 	isc__mem_shutdown();
 	isc__mutex_shutdown();
 	isc__os_shutdown();
