@@ -167,19 +167,3 @@ end:
 
 	return res;
 }
-
-#define md_register_algorithm(alg)                        \
-	const isc_md_type_t *isc__md_##alg(void) {        \
-		const isc_md_type_t *value = EVP_##alg(); \
-		if (value == NULL) {                      \
-			ERR_clear_error();                \
-		}                                         \
-		return (value);                           \
-	}
-
-md_register_algorithm(md5);
-md_register_algorithm(sha1);
-md_register_algorithm(sha224);
-md_register_algorithm(sha256);
-md_register_algorithm(sha384);
-md_register_algorithm(sha512);
