@@ -12,7 +12,6 @@
  */
 
 #include <libgen.h>
-#include <string.h>
 #include <unistd.h>
 
 #include <isc/async.h>
@@ -75,6 +74,39 @@ accept_connection(isc_nmsocket_t *ssock);
 static void
 quota_accept_cb(void *arg);
 
+static void
+tcp_dbg_log(const isc_nmsocket_t *sock, const isc_result_t result,
+	    const char *msg) {
+	const int level = ISC_LOG_DEBUG(99);
+
+	if (!isc_log_wouldlog(level)) {
+		return;
+	}
+
+	char err_msg[256];
+	char peer_sabuf[ISC_SOCKADDR_FORMATSIZE];
+	char local_sabuf[ISC_SOCKADDR_FORMATSIZE];
+	const bool has_peer_info = !sock->accepting && sock->recv_cb != NULL;
+
+	err_msg[0] = peer_sabuf[0] = local_sabuf[0] = '\0';
+
+	isc_sockaddr_format(&sock->iface, local_sabuf, sizeof(local_sabuf));
+
+	if (has_peer_info) {
+		isc_sockaddr_format(&sock->peer, peer_sabuf,
+				    sizeof(peer_sabuf));
+	}
+
+	if (result != ISC_R_SUCCESS) {
+		snprintf(err_msg, sizeof(err_msg), " (error: %s)",
+			 isc_result_totext(result));
+	}
+
+	isc_log_write(NS_LOGCATEGORY_GENERAL, ISC_LOGMODULE_NETMGR, level,
+		      " (%s%son %s): %s%s", peer_sabuf,
+		      has_peer_info ? " " : "", local_sabuf, msg, err_msg);
+}
+
 static isc_result_t
 tcp_connect_direct(isc_nmsocket_t *sock, isc__nm_uvreq_t *req) {
 	isc__networker_t *worker = NULL;
@@ -226,7 +258,6 @@ isc_nm_tcpconnect(isc_nm_t *mgr, isc_sockaddr_t *local, isc_sockaddr_t *peer,
 	sa_family_t sa_family;
 	isc__networker_t *worker = NULL;
 	uv_os_sock_t fd = -1;
-	in_port_t port_low, port_high;
 
 	REQUIRE(VALID_NM(mgr));
 	REQUIRE(local != NULL);
@@ -263,18 +294,6 @@ isc_nm_tcpconnect(isc_nm_t *mgr, isc_sockaddr_t *local, isc_sockaddr_t *peer,
 
 	(void)isc__nm_socket_min_mtu(sock->fd, sa_family);
 	(void)isc__nm_socket_tcp_maxseg(sock->fd, NM_MAXSEG);
-	port_low = (sa_family == AF_INET) ? mgr->port_low4 : mgr->port_low6;
-	port_high = (sa_family == AF_INET) ? mgr->port_high4 : mgr->port_high6;
-	result = isc__nm_socket_max_port_range(sock->fd, sa_family, port_low,
-					       port_high);
-	if (result != ISC_R_SUCCESS) {
-		isc__nmsocket_log(sock, ISC_LOG_DEBUG(99),
-				  "setting up IP_BIND_ADDRESS_NO_PORT or "
-				  "IP_LOCAL_PORT_RANGE failed: %s\n",
-				  result == ISC_R_RANGE
-					  ? isc_result_totext(result)
-					  : strerror(errno));
-	}
 
 	sock->active = true;
 
@@ -540,6 +559,8 @@ tcp_connection_cb(uv_stream_t *server, int status) {
 
 	if (status != 0) {
 		result = isc_uverr2result(status);
+		tcp_dbg_log(ssock, result,
+			    "TCP peer connection attempt early failure");
 		goto done;
 	}
 
@@ -557,6 +578,8 @@ tcp_connection_cb(uv_stream_t *server, int status) {
 			   &ssock->iface, NULL);
 	isc__nmsocket_attach(ssock, &csock->server);
 
+	tcp_dbg_log(csock, ISC_R_SUCCESS, "TCP peer connection attempt");
+
 	if (csock->server->pquota != NULL) {
 		result = isc_quota_acquire_cb(csock->server->pquota,
 					      &csock->quotacb, quota_accept_cb,
@@ -736,9 +759,7 @@ isc__nm_tcp_read_stop(isc_nmhandle_t *handle) {
 
 	isc_nmsocket_t *sock = handle->sock;
 
-	if (!sock->manual_read_timer) {
-		isc__nmsocket_timer_stop(sock);
-	}
+	isc__nmsocket_timer_stop(sock);
 	isc__nm_stop_reading(sock);
 	sock->reading = false;
 
@@ -974,6 +995,8 @@ accept_connection(isc_nmsocket_t *csock) {
 
 	csock->accepting = false;
 
+	tcp_dbg_log(csock, ISC_R_SUCCESS, "TCP connection has been accepted");
+
 	isc__nm_incstats(csock, STATID_ACCEPT);
 
 	/*
@@ -1000,6 +1023,8 @@ failure:
 				  isc_result_totext(result));
 	}
 
+	tcp_dbg_log(csock, result, "TCP connection has NOT been accepted");
+
 	isc__nmsocket_prep_destroy(csock);
 
 	isc__nmsocket_detach(&csock);
@@ -1183,16 +1208,12 @@ tcp_send_direct(isc_nmsocket_t *sock, isc__nm_uvreq_t *req) {
 		}
 	}
 
-	if (!sock->client && sock->reading) {
-		sock->reading_throttled = true;
-		isc__nm_stop_reading(sock);
-	}
-	isc__nmsocket_log(sock, ISC_LOG_DEBUG(3),
-			  "%sthe other side is not "
-			  "reading the data, switching to uv_write()",
-			  !sock->client && sock->reading
-				  ? "throttling TCP connection, "
-				  : "");
+	isc_log_write(ISC_LOGCATEGORY_GENERAL, ISC_LOGMODULE_NETMGR,
+		      ISC_LOG_DEBUG(3),
+		      "throttling TCP connection, the other side is not "
+		      "reading the data, switching to uv_write()");
+	sock->reading_throttled = true;
+	isc__nm_stop_reading(sock);
 
 	r = uv_write(&req->uv_req.write, &sock->uv_handle.stream, bufs, nbufs,
 		     tcp_send_cb);
@@ -1229,6 +1250,8 @@ tcp_close_sock(isc_nmsocket_t *sock) {
 		isc__nmsocket_detach(&sock->server);
 	}
 
+	tcp_dbg_log(sock, ISC_R_SUCCESS, "TCP connection closed");
+
 	isc__nmsocket_prep_destroy(sock);
 }
 
