@@ -244,7 +244,8 @@ static const char *description[ISC_R_NRESULTS] = {
 	[DST_R_COMPUTESECRETFAILURE] = "failure computing a shared secret",
 	[DST_R_NORANDOMNESS] = "no randomness available",
 	[DST_R_BADKEYTYPE] = "bad key type",
-	[DST_R_NOENGINE] = "no engine",
+	[DST_R_NOENGINE] = "OpenSSL Engine support has been removed; "
+			   "use OpenSSL Providers for PKCS#11",
 	[DST_R_EXTERNALKEY] = "illegal operation for an external key",
 
 	[DNS_R_NOERROR] = "NOERROR",
@@ -529,13 +530,13 @@ static const char *identifier[ISC_R_NRESULTS] = {
 	[ISCCC_R_MAXDEPTH] = "ISCCC_R_MAXDEPTH",
 };
 
-STATIC_ASSERT(DNS_R_SERVFAIL - DNS_R_NOERROR == 2,
+STATIC_ASSERT((DNS_R_SERVFAIL - DNS_R_NOERROR == 2),
 	      "DNS_R_NOERROR has wrong value");
 
-STATIC_ASSERT(DNS_R_BADVERS - DNS_R_NOERROR == 16,
+STATIC_ASSERT((DNS_R_BADVERS - DNS_R_NOERROR == 16),
 	      "DNS_R_BADVERS has wrong value");
 
-STATIC_ASSERT(ISC_R_NRESULTS < INT32_MAX, "result.h enum too big");
+STATIC_ASSERT((ISC_R_NRESULTS < INT32_MAX), "result.h enum too big");
 
 const char *
 isc_result_totext(isc_result_t result) {
