@@ -71,7 +71,7 @@ typedef struct cfg_listelt cfg_listelt_t;
  * that needs to be interpreted at parsing time, like
  * "directory".
  */
-typedef isc_result_t (*cfg_parsecallback_t)(const char      *clausename,
+typedef isc_result_t (*cfg_parsecallback_t)(const char	    *clausename,
 					    const cfg_obj_t *obj, void *arg);
 
 /***
@@ -87,10 +87,10 @@ cfg_parser_attach(cfg_parser_t *src, cfg_parser_t **dest);
  */
 
 isc_result_t
-cfg_parser_create(isc_mem_t *mctx, isc_log_t *lctx, cfg_parser_t **ret);
+cfg_parser_create(isc_mem_t *mctx, cfg_parser_t **ret);
 /*%<
  * Create a configuration file parser.  Any warning and error
- * messages will be logged to 'lctx'.
+ * messages will be logged.
  *
  * The parser object returned can be used for a single call
  * to cfg_parse_file() or cfg_parse_buffer().  It must not
@@ -392,6 +392,12 @@ cfg_obj_issockaddr(const cfg_obj_t *obj);
  * Return true iff 'obj' is a socket address.
  */
 
+bool
+cfg_obj_issockaddrtls(const cfg_obj_t *obj);
+/*%<
+ * Return true iff 'obj' is a socket address with an optional tls configuration.
+ */
+
 const isc_sockaddr_t *
 cfg_obj_assockaddr(const cfg_obj_t *obj);
 /*%<
@@ -399,17 +405,17 @@ cfg_obj_assockaddr(const cfg_obj_t *obj);
  *
  * Requires:
  * \li     'obj' points to a valid configuration object of a socket address
- * type.
+ * type, or of a socket address type with an optional tls configuration.
  *
  * Returns:
  * \li     A pointer to a sockaddr.  The sockaddr must be copied by the caller
  *      if necessary.
  */
 
-isc_dscp_t
-cfg_obj_getdscp(const cfg_obj_t *obj);
+const char *
+cfg_obj_getsockaddrtls(const cfg_obj_t *obj);
 /*%<
- * Returns the DSCP value of a configuration object representing a
+ * Returns the TLS value of a configuration object representing a
  * socket address.
  *
  * Requires:
@@ -417,7 +423,7 @@ cfg_obj_getdscp(const cfg_obj_t *obj);
  *         socket address type.
  *
  * Returns:
- * \li     DSCP value associated with a sockaddr, or -1.
+ * \li     TLS value associated with a sockaddr, or NULL.
  */
 
 bool
@@ -551,8 +557,8 @@ cfg_obj_destroy(cfg_parser_t *pctx, cfg_obj_t **obj);
  */
 
 void
-cfg_obj_log(const cfg_obj_t *obj, isc_log_t *lctx, int level, const char *fmt,
-	    ...) ISC_FORMAT_PRINTF(4, 5);
+cfg_obj_log(const cfg_obj_t *obj, int level, const char *fmt, ...)
+	ISC_FORMAT_PRINTF(3, 4);
 /*%<
  * Log a message concerning configuration object 'obj' to the logging
  * channel of 'pctx', at log level 'level'.  The message will be prefixed
@@ -580,9 +586,9 @@ cfg_map_nextclause(const cfg_type_t *map, const void **clauses,
 
 typedef isc_result_t(pluginlist_cb_t)(const cfg_obj_t *config,
 				      const cfg_obj_t *obj,
-				      const char	 *plugin_path,
-				      const char	 *parameters,
-				      void		   *callback_data);
+				      const char      *plugin_path,
+				      const char      *parameters,
+				      void	      *callback_data);
 /*%<
  * Function prototype for the callback used with cfg_pluginlist_foreach().
  * Called once for each element of the list passed to cfg_pluginlist_foreach().
@@ -598,15 +604,13 @@ typedef isc_result_t(pluginlist_cb_t)(const cfg_obj_t *config,
 
 isc_result_t
 cfg_pluginlist_foreach(const cfg_obj_t *config, const cfg_obj_t *list,
-		       isc_log_t *lctx, pluginlist_cb_t *callback,
-		       void *callback_data);
+		       pluginlist_cb_t *callback, void *callback_data);
 /*%<
  * For every "plugin" stanza present in 'list' (which in turn is a part of
  * 'config'), invoke the given 'callback', passing 'callback_data' to it along
  * with a fixed set of arguments (see the definition of the #pluginlist_cb_t
- * type).  Use logging context 'lctx' for logging error messages.  Interrupt
- * processing if 'callback' returns something else than #ISC_R_SUCCESS for any
- * element of 'list'.
+ * type).  Interrupt processing if 'callback' returns something else than
+ * #ISC_R_SUCCESS for any element of 'list'.
  *
  * Requires:
  *
