@@ -67,7 +67,6 @@
 
 #include <isccfg/cfg.h>
 #include <isccfg/grammar.h>
-#include <isccfg/log.h>
 
 /* Shorthand */
 #define CAT CFG_LOGCATEGORY_CONFIG
@@ -76,6 +75,7 @@
 #define MAP_SYM 1 /* Unique type for isc_symtab */
 
 #define TOKEN_STRING(pctx) (pctx->token.value.as_textregion.base)
+#define TOKEN_REGION(pctx) (pctx->token.value.as_textregion)
 
 /* Check a return value. */
 #define CHECK(op)                            \
@@ -118,9 +118,6 @@ create_string(cfg_parser_t *pctx, const char *contents, const cfg_type_t *type,
 static void
 free_string(cfg_parser_t *pctx, cfg_obj_t *obj);
 
-static void
-copy_string(cfg_parser_t *pctx, const cfg_obj_t *obj, isc_textregion_t *dst);
-
 static void
 free_sockaddrtls(cfg_parser_t *pctx, cfg_obj_t *obj);
 
@@ -498,7 +495,7 @@ static cfg_type_t cfg_type_filelist = { "filelist",    NULL,
 					&cfg_rep_list, &cfg_type_qstring };
 
 isc_result_t
-cfg_parser_create(isc_mem_t *mctx, isc_log_t *lctx, cfg_parser_t **ret) {
+cfg_parser_create(isc_mem_t *mctx, cfg_parser_t **ret) {
 	isc_result_t result;
 	cfg_parser_t *pctx;
 	isc_lexspecials_t specials;
@@ -513,7 +510,6 @@ cfg_parser_create(isc_mem_t *mctx, isc_log_t *lctx, cfg_parser_t **ret) {
 
 	isc_refcount_init(&pctx->references, 1);
 
-	pctx->lctx = lctx;
 	pctx->lexer = NULL;
 	pctx->seen_eof = false;
 	pctx->ungotten = false;
@@ -539,9 +535,9 @@ cfg_parser_create(isc_mem_t *mctx, isc_log_t *lctx, cfg_parser_t **ret) {
 	isc_lex_create(pctx->mctx, 1024, &pctx->lexer);
 
 	isc_lex_setspecials(pctx->lexer, specials);
-	isc_lex_setcomments(pctx->lexer, ISC_LEXCOMMENT_C |
-						 ISC_LEXCOMMENT_CPLUSPLUS |
-						 ISC_LEXCOMMENT_SHELL);
+	isc_lex_setcomments(pctx->lexer,
+			    (ISC_LEXCOMMENT_C | ISC_LEXCOMMENT_CPLUSPLUS |
+			     ISC_LEXCOMMENT_SHELL));
 
 	CHECK(cfg_create_list(pctx, &cfg_type_filelist, &pctx->open_files));
 	CHECK(cfg_create_list(pctx, &cfg_type_filelist, &pctx->closed_files));
@@ -1550,18 +1546,6 @@ free_string(cfg_parser_t *pctx, cfg_obj_t *obj) {
 		    obj->value.string.length + 1);
 }
 
-static void
-copy_string(cfg_parser_t *pctx, const cfg_obj_t *obj, isc_textregion_t *dst) {
-	if (dst->base != NULL) {
-		INSIST(dst->length != 0);
-		isc_mem_put(pctx->mctx, dst->base, dst->length + 1);
-	}
-	dst->length = obj->value.string.length;
-	dst->base = isc_mem_get(pctx->mctx, dst->length + 1);
-	memmove(dst->base, obj->value.string.base, dst->length);
-	dst->base[dst->length] = '\0';
-}
-
 static void
 free_sockaddrtls(cfg_parser_t *pctx, cfg_obj_t *obj) {
 	if (obj->value.sockaddrtls.tls.base != NULL) {
@@ -2339,7 +2323,9 @@ cfg_parse_mapbody(cfg_parser_t *pctx, const cfg_type_t *type, cfg_obj_t **ret) {
 					 clause->name);
 			CHECK(ISC_R_FAILURE);
 		}
-		if ((clause->flags & CFG_CLAUSEFLAG_NOTCONFIGURED) != 0) {
+		if ((pctx->flags & CFG_PCTX_ALLCONFIGS) == 0 &&
+		    (clause->flags & CFG_CLAUSEFLAG_NOTCONFIGURED) != 0)
+		{
 			cfg_parser_error(pctx, 0,
 					 "option '%s' was not "
 					 "enabled at compile time",
@@ -3259,6 +3245,16 @@ cfg_type_t cfg_type_netprefix = { "netprefix",	      cfg_parse_netprefix,
 				  print_netprefix,    cfg_doc_terminal,
 				  &cfg_rep_netprefix, NULL };
 
+static void
+copy_textregion(isc_mem_t *mctx, isc_textregion_t *dest, isc_textregion_t src) {
+	size_t dest_mem_length = (dest->base != NULL) ? dest->length + 1 : 0;
+	dest->base = isc_mem_creget(mctx, dest->base, dest_mem_length,
+				    src.length + 1, sizeof(char));
+	dest->length = src.length;
+	memmove(dest->base, src.base, src.length);
+	dest->base[dest->length] = '\0';
+}
+
 static isc_result_t
 parse_sockaddrsub(cfg_parser_t *pctx, const cfg_type_t *type, int flags,
 		  cfg_obj_t **ret) {
@@ -3266,37 +3262,45 @@ parse_sockaddrsub(cfg_parser_t *pctx, const cfg_type_t *type, int flags,
 	isc_netaddr_t netaddr;
 	in_port_t port = 0;
 	cfg_obj_t *obj = NULL;
+	int have_address = 0;
 	int have_port = 0;
 	int have_tls = 0;
+	int is_port_ok = (flags & CFG_ADDR_PORTOK) != 0;
+	int is_tls_ok = (flags & CFG_ADDR_TLSOK) != 0;
+	int is_address_ok = (flags & CFG_ADDR_TRAILINGOK) != 0;
 
-	CHECK(cfg_create_obj(pctx, type, &obj));
-	CHECK(cfg_parse_rawaddr(pctx, flags, &netaddr));
+	isc_textregion_t tls = { .base = NULL, .length = 0 };
+
+	CHECK(cfg_peektoken(pctx, 0));
+	if (cfg_lookingat_netaddr(pctx, flags)) {
+		CHECK(cfg_parse_rawaddr(pctx, flags, &netaddr));
+		++have_address;
+	}
 
 	for (;;) {
 		CHECK(cfg_peektoken(pctx, 0));
 		if (pctx->token.type == isc_tokentype_string) {
-			if (strcasecmp(TOKEN_STRING(pctx), "port") == 0) {
-				if ((pctx->flags & CFG_PCTX_NODEPRECATED) ==
-					    0 &&
-				    (flags & CFG_ADDR_PORTOK) == 0)
-				{
-					cfg_parser_warning(
-						pctx, 0,
-						"token 'port' is deprecated");
-				}
+			if (is_address_ok &&
+			    strcasecmp(TOKEN_STRING(pctx), "address") == 0)
+			{
+				/* read "address" */
+				CHECK(cfg_gettoken(pctx, 0));
+				CHECK(cfg_parse_rawaddr(pctx, flags, &netaddr));
+				++have_address;
+			} else if (strcasecmp(TOKEN_STRING(pctx), "port") == 0)
+			{
 				CHECK(cfg_gettoken(pctx, 0)); /* read "port" */
 				CHECK(cfg_parse_rawport(pctx, flags, &port));
 				++have_port;
-			} else if ((flags & CFG_ADDR_TLSOK) != 0 &&
+			} else if (is_tls_ok &&
 				   strcasecmp(TOKEN_STRING(pctx), "tls") == 0)
 			{
-				cfg_obj_t *tls = NULL;
-
 				CHECK(cfg_gettoken(pctx, 0)); /* read "tls" */
-				CHECK(cfg_parse_astring(pctx, NULL, &tls));
-				copy_string(pctx, tls,
-					    &obj->value.sockaddrtls.tls);
-				CLEANUP_OBJ(tls);
+				CHECK(cfg_getstringtoken(pctx));
+
+				isc_textregion_t tok = TOKEN_REGION(pctx);
+				copy_textregion(pctx->mctx, &tls, tok);
+
 				++have_tls;
 			} else {
 				break;
@@ -3306,26 +3310,60 @@ parse_sockaddrsub(cfg_parser_t *pctx, const cfg_type_t *type, int flags,
 		}
 	}
 
+	if (have_address != 1) {
+		cfg_parser_error(pctx, 0, "expected exactly one address");
+		result = ISC_R_UNEXPECTEDTOKEN;
+		goto cleanup;
+	}
+
+	if (!is_port_ok && have_port > 0) {
+		cfg_parser_error(pctx, 0, "subconfig 'port' no longer exists");
+		result = ISC_R_UNEXPECTEDTOKEN;
+		goto cleanup;
+	}
 	if (have_port > 1) {
 		cfg_parser_error(pctx, 0, "expected at most one port");
 		result = ISC_R_UNEXPECTEDTOKEN;
 		goto cleanup;
 	}
+
 	if (have_tls > 1) {
 		cfg_parser_error(pctx, 0, "expected at most one tls");
 		result = ISC_R_UNEXPECTEDTOKEN;
 		goto cleanup;
 	}
 
+	CHECK(cfg_create_obj(pctx, type, &obj));
+	if (have_tls == 1) {
+		obj->value.sockaddrtls.tls = tls;
+	}
 	isc_sockaddr_fromnetaddr(&obj->value.sockaddr, &netaddr, port);
 	*ret = obj;
 	return ISC_R_SUCCESS;
 
 cleanup:
+	if (tls.base != NULL) {
+		isc_mem_put(pctx->mctx, tls.base, tls.length + 1);
+	}
 	CLEANUP_OBJ(obj);
 	return result;
 }
 
+isc_result_t
+cfg_parse_sockaddr_generic(cfg_parser_t *pctx, cfg_type_t *klass,
+			   const cfg_type_t *type, cfg_obj_t **ret) {
+	const unsigned int *flagp;
+
+	REQUIRE(pctx != NULL);
+	REQUIRE(klass != NULL);
+	REQUIRE(type != NULL);
+	REQUIRE(ret != NULL && *ret == NULL);
+
+	flagp = type->of;
+
+	return parse_sockaddrsub(pctx, klass, *flagp, ret);
+}
+
 static unsigned int sockaddr_flags = CFG_ADDR_V4OK | CFG_ADDR_V6OK |
 				     CFG_ADDR_PORTOK;
 cfg_type_t cfg_type_sockaddr = { "sockaddr",	     cfg_parse_sockaddr,
@@ -3341,29 +3379,14 @@ cfg_type_t cfg_type_sockaddrtls = { "sockaddrtls",	  cfg_parse_sockaddrtls,
 isc_result_t
 cfg_parse_sockaddr(cfg_parser_t *pctx, const cfg_type_t *type,
 		   cfg_obj_t **ret) {
-	const unsigned int *flagp;
-
-	REQUIRE(pctx != NULL);
-	REQUIRE(type != NULL);
-	REQUIRE(ret != NULL && *ret == NULL);
-
-	flagp = type->of;
-
-	return parse_sockaddrsub(pctx, &cfg_type_sockaddr, *flagp, ret);
+	return cfg_parse_sockaddr_generic(pctx, &cfg_type_sockaddr, type, ret);
 }
 
 isc_result_t
 cfg_parse_sockaddrtls(cfg_parser_t *pctx, const cfg_type_t *type,
 		      cfg_obj_t **ret) {
-	const unsigned int *flagp;
-
-	REQUIRE(pctx != NULL);
-	REQUIRE(type != NULL);
-	REQUIRE(ret != NULL && *ret == NULL);
-
-	flagp = type->of;
-
-	return parse_sockaddrsub(pctx, &cfg_type_sockaddrtls, *flagp, ret);
+	return cfg_parse_sockaddr_generic(pctx, &cfg_type_sockaddrtls, type,
+					  ret);
 }
 
 void
@@ -3690,20 +3713,19 @@ parser_complain(cfg_parser_t *pctx, bool is_warning, unsigned int flags,
 	} else {
 		tokenbuf[0] = '\0';
 	}
-	isc_log_write(pctx->lctx, CAT, MOD, level, "%s%s%s%s", where, message,
-		      prep, tokenbuf);
+	isc_log_write(CAT, MOD, level, "%s%s%s%s", where, message, prep,
+		      tokenbuf);
 }
 
 void
-cfg_obj_log(const cfg_obj_t *obj, isc_log_t *lctx, int level, const char *fmt,
-	    ...) {
+cfg_obj_log(const cfg_obj_t *obj, int level, const char *fmt, ...) {
 	va_list ap;
 	char msgbuf[2048];
 
 	REQUIRE(obj != NULL);
 	REQUIRE(fmt != NULL);
 
-	if (!isc_log_wouldlog(lctx, level)) {
+	if (!isc_log_wouldlog(level)) {
 		return;
 	}
 
@@ -3712,10 +3734,10 @@ cfg_obj_log(const cfg_obj_t *obj, isc_log_t *lctx, int level, const char *fmt,
 	va_end(ap);
 
 	if (obj->file != NULL) {
-		isc_log_write(lctx, CAT, MOD, level, "%s:%u: %s", obj->file,
+		isc_log_write(CAT, MOD, level, "%s:%u: %s", obj->file,
 			      obj->line, msgbuf);
 	} else {
-		isc_log_write(lctx, CAT, MOD, level, "%s", msgbuf);
+		isc_log_write(CAT, MOD, level, "%s", msgbuf);
 	}
 }
 
@@ -3940,8 +3962,7 @@ cleanup:
 
 isc_result_t
 cfg_pluginlist_foreach(const cfg_obj_t *config, const cfg_obj_t *list,
-		       isc_log_t *lctx, pluginlist_cb_t *callback,
-		       void *callback_data) {
+		       pluginlist_cb_t *callback, void *callback_data) {
 	isc_result_t result = ISC_R_SUCCESS;
 	const cfg_listelt_t *element;
 
@@ -3962,7 +3983,7 @@ cfg_pluginlist_foreach(const cfg_obj_t *config, const cfg_obj_t *list,
 
 		/* Only query plugins are supported currently. */
 		if (strcasecmp(type, "query") != 0) {
-			cfg_obj_log(obj, lctx, ISC_LOG_ERROR,
+			cfg_obj_log(obj, ISC_LOG_ERROR,
 				    "unsupported plugin type");
 			return ISC_R_FAILURE;
 		}
