@@ -58,7 +58,6 @@
 
 #include <ns/client.h>
 #include <ns/interfacemgr.h>
-#include <ns/log.h>
 #include <ns/notify.h>
 #include <ns/server.h>
 #include <ns/stats.h>
@@ -98,13 +97,11 @@
 #define COOKIE_SIZE 24U /* 8 + 4 + 4 + 8 */
 #define ECS_SIZE    20U /* 2 + 1 + 1 + [0..16] */
 
-#define TCPBUFFERS_FILLCOUNT 1U
-#define TCPBUFFERS_FREEMAX   8U
-
-#define WANTNSID(x)	(((x)->attributes & NS_CLIENTATTR_WANTNSID) != 0)
-#define WANTEXPIRE(x)	(((x)->attributes & NS_CLIENTATTR_WANTEXPIRE) != 0)
-#define WANTPAD(x)	(((x)->attributes & NS_CLIENTATTR_WANTPAD) != 0)
 #define USEKEEPALIVE(x) (((x)->attributes & NS_CLIENTATTR_USEKEEPALIVE) != 0)
+#define WANTEXPIRE(x)	(((x)->attributes & NS_CLIENTATTR_WANTEXPIRE) != 0)
+#define WANTNSID(x)	(((x)->attributes & NS_CLIENTATTR_WANTNSID) != 0)
+#define WANTPAD(x)	(((x)->attributes & NS_CLIENTATTR_WANTPAD) != 0)
+#define WANTRC(x)	(((x)->attributes & NS_CLIENTATTR_WANTRC) != 0)
 
 #define MANAGER_MAGIC	 ISC_MAGIC('N', 'S', 'C', 'm')
 #define VALID_MANAGER(m) ISC_MAGIC_VALID(m, MANAGER_MAGIC)
@@ -142,7 +139,6 @@ static void
 compute_cookie(ns_client_t *client, uint32_t when, const unsigned char *secret,
 	       isc_buffer_t *buf);
 
-#ifdef HAVE_DNSTAP
 static dns_transport_type_t
 ns_client_transport_type(const ns_client_t *client) {
 	/*
@@ -187,7 +183,6 @@ ns_client_transport_type(const ns_client_t *client) {
 
 	return DNS_TRANSPORT_UDP;
 }
-#endif /* HAVE_DNSTAP */
 
 void
 ns_client_recursing(ns_client_t *client) {
@@ -223,6 +218,57 @@ ns_client_settimeout(ns_client_t *client, unsigned int seconds) {
 	/* XXXWPK TODO use netmgr to set timeout */
 }
 
+static void
+client_extendederror_reset(ns_client_t *client) {
+	if (client->ede == NULL) {
+		return;
+	}
+	isc_mem_put(client->manager->mctx, client->ede->value,
+		    client->ede->length);
+	isc_mem_put(client->manager->mctx, client->ede, sizeof(dns_ednsopt_t));
+	client->ede = NULL;
+}
+
+void
+ns_client_extendederror(ns_client_t *client, uint16_t code, const char *text) {
+	unsigned char ede[DNS_EDE_EXTRATEXT_LEN + 2];
+	isc_buffer_t buf;
+	uint16_t len = sizeof(uint16_t);
+
+	REQUIRE(NS_CLIENT_VALID(client));
+
+	if (client->ede != NULL) {
+		ns_client_log(client, NS_LOGCATEGORY_CLIENT,
+			      NS_LOGMODULE_CLIENT, ISC_LOG_DEBUG(1),
+			      "already have ede, ignoring %u %s", code,
+			      text == NULL ? "(null)" : text);
+		return;
+	}
+
+	ns_client_log(client, NS_LOGCATEGORY_CLIENT, NS_LOGMODULE_CLIENT,
+		      ISC_LOG_DEBUG(1), "set ede: info-code %u extra-text %s",
+		      code, text == NULL ? "(null)" : text);
+
+	isc_buffer_init(&buf, ede, sizeof(ede));
+	isc_buffer_putuint16(&buf, code);
+	if (text != NULL && strlen(text) > 0) {
+		if (strlen(text) < DNS_EDE_EXTRATEXT_LEN) {
+			isc_buffer_putstr(&buf, text);
+			len += (uint16_t)(strlen(text));
+		} else {
+			ns_client_log(client, NS_LOGCATEGORY_CLIENT,
+				      NS_LOGMODULE_CLIENT, ISC_LOG_WARNING,
+				      "ede extra-text too long, ignoring");
+		}
+	}
+
+	client->ede = isc_mem_get(client->manager->mctx, sizeof(dns_ednsopt_t));
+	client->ede->code = DNS_OPT_EDE;
+	client->ede->length = len;
+	client->ede->value = isc_mem_get(client->manager->mctx, len);
+	memmove(client->ede->value, ede, len);
+}
+
 static void
 ns_client_endrequest(ns_client_t *client) {
 	INSIST(client->state == NS_CLIENTSTATE_WORKING ||
@@ -263,11 +309,15 @@ ns_client_endrequest(ns_client_t *client) {
 		dns_message_puttemprdataset(client->message, &client->opt);
 	}
 
+	client_extendederror_reset(client);
 	client->signer = NULL;
 	client->udpsize = 512;
 	client->extflags = 0;
 	client->ednsversion = -1;
 	client->additionaldepth = 0;
+	if (dns_name_dynamic(&client->rad)) {
+		dns_name_free(&client->rad, client->manager->mctx);
+	}
 	dns_ecs_init(&client->ecs);
 	dns_message_reset(client->message, DNS_MESSAGE_INTENTPARSE);
 
@@ -908,7 +958,7 @@ ns_client_error(ns_client_t *client, isc_result_t result) {
 		} else {
 			loglevel = ISC_LOG_DEBUG(1);
 		}
-		wouldlog = isc_log_wouldlog(ns_lctx, loglevel);
+		wouldlog = isc_log_wouldlog(loglevel);
 		rrl_result = dns_rrl(client->view, NULL, &client->peeraddr,
 				     TCP_CLIENT(client), dns_rdataclass_in,
 				     dns_rdatatype_none, NULL, result,
@@ -1181,20 +1231,28 @@ no_nsid:
 		count++;
 	}
 
-	for (size_t i = 0; i < DNS_EDE_MAX_ERRORS; i++) {
-		dns_ednsopt_t *ede = client->edectx.ede[i];
-
-		if (ede == NULL) {
-			break;
-		}
-
+	if (client->ede != NULL) {
 		INSIST(count < DNS_EDNSOPTIONS);
 		ednsopts[count].code = DNS_OPT_EDE;
-		ednsopts[count].length = ede->length;
-		ednsopts[count].value = ede->value;
+		ednsopts[count].length = client->ede->length;
+		ednsopts[count].value = client->ede->value;
 		count++;
 	}
 
+	if (WANTRC(client)) {
+		dns_name_t *rad = NULL;
+		if (dns_name_dynamic(&client->rad)) {
+			rad = &client->rad;
+		}
+		if (rad != NULL && !dns_name_equal(rad, dns_rootname)) {
+			INSIST(count < DNS_EDNSOPTIONS);
+			ednsopts[count].code = DNS_OPT_REPORT_CHANNEL;
+			ednsopts[count].length = rad->length;
+			ednsopts[count].value = rad->ndata;
+			count++;
+		}
+	}
+
 	/* Padding must be added last */
 	if ((view != NULL) && (view->padding > 0) && WANTPAD(client) &&
 	    (TCP_CLIENT(client) ||
@@ -1278,7 +1336,6 @@ process_cookie(ns_client_t *client, isc_buffer_t *buf, size_t optlen) {
 	isc_stdtime_t now;
 	uint32_t when;
 	isc_buffer_t db;
-	bool alwaysvalid;
 
 	/*
 	 * If we have already seen a cookie option skip this cookie option.
@@ -1324,24 +1381,13 @@ process_cookie(ns_client_t *client, isc_buffer_t *buf, size_t optlen) {
 	when = isc_buffer_getuint32(buf);
 	isc_buffer_forward(buf, 8);
 
-	/*
-	 * For '-T cookiealwaysvalid' still process everything to not skew any
-	 * performance tests involving cookies, but make sure that the cookie
-	 * check passes in the end, given the cookie was structurally correct.
-	 */
-	alwaysvalid = ns_server_getoption(client->manager->sctx,
-					  NS_SERVER_COOKIEALWAYSVALID);
-
 	/*
 	 * Allow for a 5 minute clock skew between servers sharing a secret.
 	 * Only accept COOKIE if we have talked to the client in the last hour.
 	 */
 	now = isc_stdtime_now();
-	if (alwaysvalid) {
-		now = when;
-	}
-	if (isc_serial_gt(when, now + 300) /* In the future. */ ||
-	    isc_serial_lt(when, now - 3600) /* In the past. */)
+	if (isc_serial_gt(when, (now + 300)) /* In the future. */ ||
+	    isc_serial_lt(when, (now - 3600)) /* In the past. */)
 	{
 		client->attributes |= NS_CLIENTATTR_BADCOOKIE;
 		ns_stats_increment(client->manager->sctx->nsstats,
@@ -1352,7 +1398,7 @@ process_cookie(ns_client_t *client, isc_buffer_t *buf, size_t optlen) {
 	isc_buffer_init(&db, dbuf, sizeof(dbuf));
 	compute_cookie(client, when, client->manager->sctx->secret, &db);
 
-	if (isc_safe_memequal(old, dbuf, COOKIE_SIZE) || alwaysvalid) {
+	if (isc_safe_memequal(old, dbuf, COOKIE_SIZE)) {
 		ns_stats_increment(client->manager->sctx->nsstats,
 				   ns_statscounter_cookiematch);
 		client->attributes |= NS_CLIENTATTR_HAVECOOKIE;
@@ -1550,6 +1596,17 @@ process_opt(ns_client_t *client, dns_rdataset_t *opt) {
 	 * XXXRTH need library support for this!
 	 */
 	client->ednsversion = (opt->ttl & 0x00FF0000) >> 16;
+	if (client->ednsversion > DNS_EDNS_VERSION) {
+		ns_stats_increment(client->manager->sctx->nsstats,
+				   ns_statscounter_badednsver);
+		result = ns_client_addopt(client, client->message,
+					  &client->opt);
+		if (result == ISC_R_SUCCESS) {
+			result = DNS_R_BADVERS;
+		}
+		ns_client_error(client, result);
+		return result;
+	}
 
 	/* Check for NSID request */
 	result = dns_rdataset_first(opt);
@@ -1561,20 +1618,6 @@ process_opt(ns_client_t *client, dns_rdataset_t *opt) {
 		while (isc_buffer_remaininglength(&optbuf) >= 4) {
 			optcode = isc_buffer_getuint16(&optbuf);
 			optlen = isc_buffer_getuint16(&optbuf);
-
-			INSIST(isc_buffer_remaininglength(&optbuf) >= optlen);
-
-			/*
-			 * When returning BADVERSION, only process
-			 * DNS_OPT_NSID or DNS_OPT_COOKIE options.
-			 */
-			if (client->ednsversion > DNS_EDNS_VERSION &&
-			    optcode != DNS_OPT_NSID &&
-			    optcode != DNS_OPT_COOKIE)
-			{
-				isc_buffer_forward(&optbuf, optlen);
-				continue;
-			}
 			switch (optcode) {
 			case DNS_OPT_NSID:
 				if (!WANTNSID(client)) {
@@ -1646,18 +1689,6 @@ process_opt(ns_client_t *client, dns_rdataset_t *opt) {
 		}
 	}
 
-	if (client->ednsversion > DNS_EDNS_VERSION) {
-		ns_stats_increment(client->manager->sctx->nsstats,
-				   ns_statscounter_badednsver);
-		result = ns_client_addopt(client, client->message,
-					  &client->opt);
-		if (result == ISC_R_SUCCESS) {
-			result = DNS_R_BADVERS;
-		}
-		ns_client_error(client, result);
-		return result;
-	}
-
 	ns_stats_increment(client->manager->sctx->nsstats,
 			   ns_statscounter_edns0in);
 	client->attributes |= NS_CLIENTATTR_WANTOPT;
@@ -1726,7 +1757,7 @@ ns__client_put_cb(void *client0) {
 	 * Call this first because it requires a valid client.
 	 */
 	ns_query_free(client);
-	dns_ede_invalidate(&client->edectx);
+	client_extendederror_reset(client);
 
 	client->magic = 0;
 
@@ -2003,6 +2034,7 @@ ns_client_request(isc_nmhandle_t *handle, isc_result_t eresult,
 	}
 
 	client->message->rcode = dns_rcode_noerror;
+	client->ede = NULL;
 
 	/*
 	 * Deal with EDNS.
@@ -2191,7 +2223,7 @@ ns_client_request_continue(void *arg) {
 					      "no matching view in class");
 		}
 
-		dns_ede_add(&client->edectx, DNS_EDE_PROHIBITED, NULL);
+		ns_client_extendederror(client, DNS_EDE_PROHIBITED, NULL);
 		ns_client_error(client, DNS_R_REFUSED);
 
 		goto cleanup;
@@ -2213,7 +2245,7 @@ ns_client_request_continue(void *arg) {
 					     client->view->proxyacl,
 					     false) != ISC_R_SUCCESS)
 		{
-			if (isc_log_wouldlog(ns_lctx, log_level)) {
+			if (isc_log_wouldlog(log_level)) {
 				isc_sockaddr_format(&real_peer, fmtbuf,
 						    sizeof(fmtbuf));
 				ns_client_log(
@@ -2234,7 +2266,7 @@ ns_client_request_continue(void *arg) {
 					     client->view->proxyonacl,
 					     true) != ISC_R_SUCCESS)
 		{
-			if (isc_log_wouldlog(ns_lctx, log_level)) {
+			if (isc_log_wouldlog(log_level)) {
 				isc_sockaddr_format(&real_local, fmtbuf,
 						    sizeof(fmtbuf));
 				ns_client_log(
@@ -2522,8 +2554,6 @@ ns__client_setup(ns_client_t *client, ns_clientmgr_t *mgr, bool new) {
 		 */
 		client->magic = NS_CLIENT_MAGIC;
 		ns_query_init(client);
-
-		dns_ede_init(client->manager->mctx, &client->edectx);
 	} else {
 		REQUIRE(NS_CLIENT_VALID(client));
 		REQUIRE(client->manager->tid == isc_tid());
@@ -2536,11 +2566,8 @@ ns__client_setup(ns_client_t *client, ns_clientmgr_t *mgr, bool new) {
 			.magic = 0,
 			.manager = client->manager,
 			.message = client->message,
-			.edectx = client->edectx,
 			.query = client->query,
 		};
-
-		dns_ede_reset(&client->edectx);
 	}
 
 	client->query.attributes &= ~NS_QUERYATTR_ANSWERED;
@@ -2548,6 +2575,7 @@ ns__client_setup(ns_client_t *client, ns_clientmgr_t *mgr, bool new) {
 	client->udpsize = 512;
 	client->ednsversion = -1;
 	dns_name_init(&client->signername, NULL);
+	dns_name_init(&client->rad, NULL);
 	dns_ecs_init(&client->ecs);
 	isc_sockaddr_any(&client->formerrcache.addr);
 	client->formerrcache.time = 0;
@@ -2715,7 +2743,7 @@ ns_client_checkacl(ns_client_t *client, isc_sockaddr_t *sockaddr,
 			      NS_LOGMODULE_CLIENT, ISC_LOG_DEBUG(3),
 			      "%s approved", opname);
 	} else {
-		dns_ede_add(&client->edectx, DNS_EDE_PROHIBITED, NULL);
+		ns_client_extendederror(client, DNS_EDE_PROHIBITED, NULL);
 		ns_client_log(client, DNS_LOGCATEGORY_SECURITY,
 			      NS_LOGMODULE_CLIENT, log_level, "%s denied",
 			      opname);
@@ -2734,9 +2762,8 @@ ns_client_name(ns_client_t *client, char *peerbuf, size_t len) {
 }
 
 void
-ns_client_logv(ns_client_t *client, isc_logcategory_t *category,
-	       isc_logmodule_t *module, int level, const char *fmt,
-	       va_list ap) {
+ns_client_logv(ns_client_t *client, isc_logcategory_t category,
+	       isc_logmodule_t module, int level, const char *fmt, va_list ap) {
 	char msgbuf[4096];
 	char signerbuf[DNS_NAME_FORMATSIZE], qnamebuf[DNS_NAME_FORMATSIZE];
 	char peerbuf[ISC_SOCKADDR_FORMATSIZE];
@@ -2778,17 +2805,17 @@ ns_client_logv(ns_client_t *client, isc_logcategory_t *category,
 		snprintf(peerbuf, sizeof(peerbuf), "(no-peer)");
 	}
 
-	isc_log_write(ns_lctx, category, module, level,
+	isc_log_write(category, module, level,
 		      "client @%p %s%s%s%s%s%s%s%s: %s", client, peerbuf, sep1,
 		      signer, sep2, qname, sep3, sep4, viewname, msgbuf);
 }
 
 void
-ns_client_log(ns_client_t *client, isc_logcategory_t *category,
-	      isc_logmodule_t *module, int level, const char *fmt, ...) {
+ns_client_log(ns_client_t *client, isc_logcategory_t category,
+	      isc_logmodule_t module, int level, const char *fmt, ...) {
 	va_list ap;
 
-	if (!isc_log_wouldlog(ns_lctx, level)) {
+	if (!isc_log_wouldlog(level)) {
 		return;
 	}
 
@@ -2818,7 +2845,7 @@ ns_client_dumpmessage(ns_client_t *client, const char *reason) {
 	int len = 1024;
 	isc_result_t result;
 
-	if (!isc_log_wouldlog(ns_lctx, ISC_LOG_DEBUG(1))) {
+	if (!isc_log_wouldlog(ISC_LOG_DEBUG(1))) {
 		return;
 	}
 
@@ -2908,10 +2935,12 @@ ns_client_dumprecursing(FILE *f, ns_clientmgr_t *manager) {
 		}
 		UNLOCK(&client->query.fetchlock);
 		fprintf(f,
-			"; client %s%s%s: id %u '%s/%s/%s'%s%s "
+			"; client %s (%s)%s%s: id %u '%s/%s/%s'%s%s "
 			"requesttime %u\n",
-			peerbuf, sep, name, client->message->id, namebuf,
-			typebuf, classbuf, origfor, original,
+			peerbuf,
+			dns_transport_totext(ns_client_transport_type(client)),
+			sep, name, client->message->id, namebuf, typebuf,
+			classbuf, origfor, original,
 			isc_time_seconds(&client->requesttime));
 		client = ISC_LIST_NEXT(client, rlink);
 	}
