@@ -16,6 +16,7 @@
 #include <stdbool.h>
 
 #include <isc/interfaceiter.h>
+#include <isc/log.h>
 #include <isc/loop.h>
 #include <isc/netmgr.h>
 #include <isc/os.h>
@@ -29,7 +30,6 @@
 
 #include <ns/client.h>
 #include <ns/interfacemgr.h>
-#include <ns/log.h>
 #include <ns/server.h>
 #include <ns/stats.h>
 
@@ -58,9 +58,6 @@
 #define IFMGR_MAGIC		 ISC_MAGIC('I', 'F', 'M', 'G')
 #define NS_INTERFACEMGR_VALID(t) ISC_MAGIC_VALID(t, IFMGR_MAGIC)
 
-#define IFMGR_COMMON_LOGARGS \
-	ns_lctx, NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR
-
 /*% nameserver interface manager structure */
 struct ns_interfacemgr {
 	unsigned int magic; /*%< Magic number */
@@ -203,7 +200,8 @@ route_recv(isc_nmhandle_t *handle, isc_result_t eresult, isc_region_t *region,
 	struct MSGHDR *rtm = NULL;
 	size_t rtmlen;
 
-	isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_DEBUG(9), "route_recv: %s",
+	isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+		      ISC_LOG_DEBUG(9), "route_recv: %s",
 		      isc_result_totext(eresult));
 
 	if (handle == NULL) {
@@ -214,7 +212,8 @@ route_recv(isc_nmhandle_t *handle, isc_result_t eresult, isc_region_t *region,
 	case ISC_R_SUCCESS:
 		break;
 	default:
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR,
 			      "automatic interface scanning terminated: %s",
 			      isc_result_totext(eresult));
 		FALLTHROUGH;
@@ -230,7 +229,8 @@ route_recv(isc_nmhandle_t *handle, isc_result_t eresult, isc_region_t *region,
 
 #ifdef RTM_VERSION
 	if (rtm->rtm_version != RTM_VERSION) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR,
 			      "automatic interface rescanning disabled: "
 			      "rtm->rtm_version mismatch (%u != %u) "
 			      "recompile required",
@@ -255,8 +255,9 @@ static void
 route_connected(isc_nmhandle_t *handle, isc_result_t eresult, void *arg) {
 	ns_interfacemgr_t *mgr = (ns_interfacemgr_t *)arg;
 
-	isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_DEBUG(9),
-		      "route_connected: %s", isc_result_totext(eresult));
+	isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+		      ISC_LOG_DEBUG(9), "route_connected: %s",
+		      isc_result_totext(eresult));
 
 	if (eresult != ISC_R_SUCCESS) {
 		ns_interfacemgr_detach(&mgr);
@@ -352,8 +353,8 @@ ns_interfacemgr_routeconnect(ns_interfacemgr_t *mgr) {
 	isc_result_t result = isc_nm_routeconnect(mgr->nm, route_connected,
 						  mgr);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_INFO,
-			      "unable to open route socket: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_INFO, "unable to open route socket: %s",
 			      isc_result_totext(result));
 		ns_interfacemgr_unref(mgr);
 	}
@@ -505,8 +506,8 @@ ns_interface_listentcp(ns_interface_t *ifp, isc_nm_proxy_type_t proxy) {
 		ifp, ns__client_tcpconn, ifp, ifp->mgr->backlog,
 		&ifp->mgr->sctx->tcpquota, NULL, proxy, &ifp->tcplistensocket);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "creating TCP socket: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR, "creating TCP socket: %s",
 			      isc_result_totext(result));
 	}
 
@@ -517,8 +518,8 @@ ns_interface_listentcp(ns_interface_t *ifp, isc_nm_proxy_type_t proxy) {
 	 */
 	result = ns__client_tcpconn(NULL, ISC_R_SUCCESS, ifp);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "connecting TCP socket: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR, "connecting TCP socket: %s",
 			      isc_result_totext(result));
 	}
 
@@ -541,8 +542,8 @@ ns_interface_listentls(ns_interface_t *ifp, isc_nm_proxy_type_t proxy,
 		&ifp->tlslistensocket);
 
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "creating TLS socket: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR, "creating TLS socket: %s",
 			      isc_result_totext(result));
 		return result;
 	}
@@ -554,8 +555,8 @@ ns_interface_listentls(ns_interface_t *ifp, isc_nm_proxy_type_t proxy,
 	 */
 	result = ns__client_tcpconn(NULL, ISC_R_SUCCESS, ifp);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "updating TCP stats: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR, "updating TCP stats: %s",
 			      isc_result_totext(result));
 	}
 
@@ -616,8 +617,8 @@ ns_interface_listenhttp(ns_interface_t *ifp, isc_nm_proxy_type_t proxy,
 	}
 
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "creating %s socket: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR, "creating %s socket: %s",
 			      sslctx ? "HTTPS" : "HTTP",
 			      isc_result_totext(result));
 		return result;
@@ -636,8 +637,8 @@ ns_interface_listenhttp(ns_interface_t *ifp, isc_nm_proxy_type_t proxy,
 	 */
 	result = ns__client_tcpconn(NULL, ISC_R_SUCCESS, ifp);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "updating TCP stats: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR, "updating TCP stats: %s",
 			      isc_result_totext(result));
 	}
 
@@ -808,8 +809,8 @@ static void
 log_interface_shutdown(const ns_interface_t *ifp) {
 	char sabuf[ISC_SOCKADDR_FORMATSIZE];
 	isc_sockaddr_format(&ifp->addr, sabuf, sizeof(sabuf));
-	isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_INFO,
-		      "no longer listening on %s", sabuf);
+	isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+		      ISC_LOG_INFO, "no longer listening on %s", sabuf);
 }
 
 /*%
@@ -876,7 +877,8 @@ setup_locals(isc_interface_t *interface, dns_acl_t *localhost,
 	}
 
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_WARNING,
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_WARNING,
 			      "omitting IPv4 interface %s from "
 			      "localnets ACL: %s",
 			      interface->name, isc_result_totext(result));
@@ -884,7 +886,8 @@ setup_locals(isc_interface_t *interface, dns_acl_t *localhost,
 	}
 
 	if (prefixlen == 0U) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_WARNING,
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_WARNING,
 			      "omitting %s interface %s from localnets ACL: "
 			      "zero prefix length detected",
 			      (netaddr->family == AF_INET) ? "IPv4" : "IPv6",
@@ -951,8 +954,8 @@ replace_listener_tlsctx(ns_interface_t *ifp, isc_tlsctx_t *newctx) {
 	char sabuf[ISC_SOCKADDR_FORMATSIZE];
 
 	isc_sockaddr_format(&ifp->addr, sabuf, sizeof(sabuf));
-	isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_INFO,
-		      "updating TLS context on %s", sabuf);
+	isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+		      ISC_LOG_INFO, "updating TLS context on %s", sabuf);
 	if (ifp->tlslistensocket != NULL) {
 		isc_nmsocket_set_tlsctx(ifp->tlslistensocket, newctx);
 	} else if (ifp->http_secure_listensocket != NULL) {
@@ -1082,12 +1085,9 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 	isc_interfaceiter_t *iter = NULL;
 	bool scan_ipv4 = false;
 	bool scan_ipv6 = false;
-	bool ipv6only = true;
-	bool ipv6pktinfo = true;
 	isc_result_t result;
 	isc_netaddr_t zero_address, zero_address6;
 	ns_listenelt_t *le = NULL;
-	isc_sockaddr_t listen_addr;
 	ns_interface_t *ifp = NULL;
 	bool log_explicit = false;
 	bool dolistenon;
@@ -1100,7 +1100,7 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 	if (isc_net_probeipv6() == ISC_R_SUCCESS) {
 		scan_ipv6 = true;
 	} else if ((mgr->sctx->options & NS_SERVER_DISABLE6) == 0) {
-		isc_log_write(IFMGR_COMMON_LOGARGS,
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
 			      verbose ? ISC_LOG_INFO : ISC_LOG_DEBUG(1),
 			      "no IPv6 interfaces found");
 	}
@@ -1108,67 +1108,11 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 	if (isc_net_probeipv4() == ISC_R_SUCCESS) {
 		scan_ipv4 = true;
 	} else if ((mgr->sctx->options & NS_SERVER_DISABLE4) == 0) {
-		isc_log_write(IFMGR_COMMON_LOGARGS,
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
 			      verbose ? ISC_LOG_INFO : ISC_LOG_DEBUG(1),
 			      "no IPv4 interfaces found");
 	}
 
-	/*
-	 * A special, but typical case; listen-on-v6 { any; }.
-	 * When we can make the socket IPv6-only, open a single wildcard
-	 * socket for IPv6 communication.  Otherwise, make separate
-	 * socket for each IPv6 address in order to avoid accepting IPv4
-	 * packets as the form of mapped addresses unintentionally
-	 * unless explicitly allowed.
-	 */
-	if (scan_ipv6 && isc_net_probe_ipv6only() != ISC_R_SUCCESS) {
-		ipv6only = false;
-		log_explicit = true;
-	}
-	if (scan_ipv6 && isc_net_probe_ipv6pktinfo() != ISC_R_SUCCESS) {
-		ipv6pktinfo = false;
-		log_explicit = true;
-	}
-	if (scan_ipv6 && ipv6only && ipv6pktinfo) {
-		for (le = ISC_LIST_HEAD(mgr->listenon6->elts); le != NULL;
-		     le = ISC_LIST_NEXT(le, link))
-		{
-			struct in6_addr in6a;
-
-			if (!listenon_is_ip6_any(le)) {
-				continue;
-			}
-
-			in6a = in6addr_any;
-			isc_sockaddr_fromin6(&listen_addr, &in6a, le->port);
-
-			ifp = find_matching_interface(mgr, &listen_addr);
-			if (ifp != NULL) {
-				bool cont = interface_update_or_shutdown(
-					mgr, ifp, le, config);
-				if (cont) {
-					continue;
-				}
-			}
-
-			isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_INFO,
-				      "listening on IPv6 "
-				      "interfaces, port %u",
-				      le->port);
-			result = interface_setup(mgr, &listen_addr, "<any>",
-						 &ifp, le, NULL);
-			if (result == ISC_R_SUCCESS) {
-				ifp->flags |= NS_INTERFACEFLAG_ANYADDR;
-			} else {
-				isc_log_write(IFMGR_COMMON_LOGARGS,
-					      ISC_LOG_ERROR,
-					      "listening on all IPv6 "
-					      "interfaces failed");
-			}
-			/* Continue. */
-		}
-	}
-
 	isc_netaddr_any(&zero_address);
 	isc_netaddr_any6(&zero_address6);
 
@@ -1248,7 +1192,6 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 		{
 			int match;
 			bool addr_in_use = false;
-			bool ipv6_wildcard = false;
 			isc_sockaddr_t listen_sockaddr;
 
 			isc_sockaddr_fromnetaddr(&listen_sockaddr,
@@ -1274,16 +1217,6 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 				dolistenon = false;
 			}
 
-			/*
-			 * The case of "any" IPv6 address will require
-			 * special considerations later, so remember it.
-			 */
-			if (family == AF_INET6 && ipv6only && ipv6pktinfo &&
-			    listenon_is_ip6_any(le))
-			{
-				ipv6_wildcard = true;
-			}
-
 			ifp = find_matching_interface(mgr, &listen_sockaddr);
 			if (ifp != NULL) {
 				bool cont = interface_update_or_shutdown(
@@ -1293,14 +1226,11 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 				}
 			}
 
-			if (ipv6_wildcard) {
-				continue;
-			}
-
 			if (log_explicit && family == AF_INET6 &&
 			    listenon_is_ip6_any(le))
 			{
-				isc_log_write(IFMGR_COMMON_LOGARGS,
+				isc_log_write(NS_LOGCATEGORY_NETWORK,
+					      NS_LOGMODULE_INTERFACEMGR,
 					      verbose ? ISC_LOG_INFO
 						      : ISC_LOG_DEBUG(1),
 					      "IPv6 socket API is "
@@ -1311,7 +1241,8 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 			}
 			isc_sockaddr_format(&listen_sockaddr, sabuf,
 					    sizeof(sabuf));
-			isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_INFO,
+			isc_log_write(NS_LOGCATEGORY_NETWORK,
+				      NS_LOGMODULE_INTERFACEMGR, ISC_LOG_INFO,
 				      "listening on %s interface "
 				      "%s, %s",
 				      (family == AF_INET) ? "IPv4" : "IPv6",
@@ -1327,20 +1258,22 @@ do_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 			}
 
 			if (result != ISC_R_SUCCESS) {
-				isc_log_write(
-					IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-					"creating %s interface "
-					"%s failed; interface ignored",
-					(family == AF_INET) ? "IPv4" : "IPv6",
-					interface.name);
+				isc_log_write(NS_LOGCATEGORY_NETWORK,
+					      NS_LOGMODULE_INTERFACEMGR,
+					      ISC_LOG_ERROR,
+					      "creating %s interface "
+					      "%s failed; interface ignored",
+					      (family == AF_INET) ? "IPv4"
+								  : "IPv6",
+					      interface.name);
 			}
 			/* Continue. */
 		}
 		continue;
 
 	ignore_interface:
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_ERROR,
-			      "ignoring %s interface %s: %s",
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_ERROR, "ignoring %s interface %s: %s",
 			      (family == AF_INET) ? "IPv4" : "IPv6",
 			      interface.name, isc_result_totext(result));
 		continue;
@@ -1392,7 +1325,8 @@ ns_interfacemgr_scan(ns_interfacemgr_t *mgr, bool verbose, bool config) {
 	 * Warn if we are not listening on any interface.
 	 */
 	if (ISC_LIST_EMPTY(mgr->interfaces)) {
-		isc_log_write(IFMGR_COMMON_LOGARGS, ISC_LOG_WARNING,
+		isc_log_write(NS_LOGCATEGORY_NETWORK, NS_LOGMODULE_INTERFACEMGR,
+			      ISC_LOG_WARNING,
 			      "not listening on any interfaces");
 	}
 
