@@ -15,6 +15,7 @@
 #include <stdbool.h>
 
 #include <isc/formatcheck.h>
+#include <isc/log.h>
 #include <isc/mem.h>
 #include <isc/netmgr.h>
 #include <isc/result.h>
@@ -41,7 +42,6 @@
 #include <dns/zt.h>
 
 #include <ns/client.h>
-#include <ns/log.h>
 #include <ns/server.h>
 #include <ns/stats.h>
 #include <ns/xfrout.h>
@@ -56,14 +56,14 @@
  *  - IXFR over UDP
  */
 
-#define XFROUT_COMMON_LOGARGS \
-	ns_lctx, DNS_LOGCATEGORY_XFER_OUT, NS_LOGMODULE_XFER_OUT
+#define XFROUT_PROTOCOL_LOGARGS \
+	DNS_LOGCATEGORY_XFER_OUT, NS_LOGMODULE_XFER_OUT, ISC_LOG_INFO
 
-#define XFROUT_PROTOCOL_LOGARGS XFROUT_COMMON_LOGARGS, ISC_LOG_INFO
+#define XFROUT_DEBUG_LOGARGS(n) \
+	DNS_LOGCATEGORY_XFER_OUT, NS_LOGMODULE_XFER_OUT, ISC_LOG_DEBUG(n)
 
-#define XFROUT_DEBUG_LOGARGS(n) XFROUT_COMMON_LOGARGS, ISC_LOG_DEBUG(n)
-
-#define XFROUT_RR_LOGARGS XFROUT_COMMON_LOGARGS, XFROUT_RR_LOGLEVEL
+#define XFROUT_RR_LOGARGS \
+	DNS_LOGCATEGORY_XFER_OUT, NS_LOGMODULE_XFER_OUT, XFROUT_RR_LOGLEVEL
 
 #define XFROUT_RR_LOGLEVEL ISC_LOG_DEBUG(8)
 
@@ -80,7 +80,7 @@
 			      "bad zone transfer request: %s (%s)", msg, \
 			      isc_result_totext(code));                  \
 		if (result != ISC_R_SUCCESS)                             \
-			goto cleanup;                                    \
+			goto failure;                                    \
 	} while (0)
 
 #define FAILQ(code, msg, question, rdclass)                                  \
@@ -95,7 +95,14 @@
 			      "bad zone transfer request: '%s/%s': %s (%s)", \
 			      _buf1, _buf2, msg, isc_result_totext(code));   \
 		if (result != ISC_R_SUCCESS)                                 \
-			goto cleanup;                                        \
+			goto failure;                                        \
+	} while (0)
+
+#define CHECK(op)                            \
+	do {                                 \
+		result = (op);               \
+		if (result != ISC_R_SUCCESS) \
+			goto failure;        \
 	} while (0)
 
 /**************************************************************************/
@@ -242,7 +249,7 @@ ixfr_rrstream_create(isc_mem_t *mctx, const char *journal_filename,
 	*sp = (rrstream_t *)s;
 	return ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	ixfr_rrstream_destroy((rrstream_t **)(void *)&s);
 	return result;
 }
@@ -323,7 +330,7 @@ axfr_rrstream_create(isc_mem_t *mctx, dns_db_t *db, dns_dbversion_t *ver,
 	*sp = (rrstream_t *)s;
 	return ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	axfr_rrstream_destroy((rrstream_t **)(void *)&s);
 	return result;
 }
@@ -443,7 +450,7 @@ soa_rrstream_create(isc_mem_t *mctx, dns_db_t *db, dns_dbversion_t *ver,
 	*sp = (rrstream_t *)s;
 	return ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	soa_rrstream_destroy((rrstream_t **)(void *)&s);
 	return result;
 }
@@ -765,9 +772,9 @@ ns_xfr_start(ns_client_t *client, dns_rdatatype_t reqtype) {
 	 */
 	result = isc_quota_acquire(&client->manager->sctx->xfroutquota);
 	if (result != ISC_R_SUCCESS) {
-		isc_log_write(XFROUT_COMMON_LOGARGS, ISC_LOG_WARNING,
-			      "%s request denied: %s", mnemonic,
-			      isc_result_totext(result));
+		isc_log_write(DNS_LOGCATEGORY_XFER_OUT, NS_LOGMODULE_XFER_OUT,
+			      ISC_LOG_WARNING, "%s request denied: %s",
+			      mnemonic, isc_result_totext(result));
 		goto max_quota;
 	}
 
@@ -824,7 +831,7 @@ ns_xfr_start(ns_client_t *client, dns_rdatatype_t reqtype) {
 					      ISC_LOG_ERROR,
 					      "zone transfer '%s/%s' denied",
 					      _buf1, _buf2);
-				goto cleanup;
+				goto failure;
 			}
 			if (result != ISC_R_SUCCESS) {
 				FAILQ(DNS_R_NOTAUTH, "non-authoritative zone",
@@ -1027,8 +1034,7 @@ got_soa:
 			xfrout_log1(client, question_name, question_class,
 				    ISC_LOG_INFO,
 				    "IXFR version not in journal, "
-				    "falling back to AXFR (serial %u)",
-				    begin_serial);
+				    "falling back to AXFR");
 			mnemonic = "AXFR-style IXFR";
 			goto axfr_fallback;
 		}
@@ -1166,7 +1172,7 @@ have_stream:
 
 	result = ISC_R_SUCCESS;
 
-cleanup:
+failure:
 	if (result == DNS_R_REFUSED) {
 		inc_stats(client, zone, ns_statscounter_xfrrej);
 	}
@@ -1276,7 +1282,7 @@ xfrout_ctx_create(isc_mem_t *mctx, ns_client_t *client, unsigned int id,
 	xfr->txmemlen = len;
 
 	/*
-	 * These MUST be after the last "goto cleanup;" / CHECK to
+	 * These MUST be after the last "goto failure;" / CHECK to
 	 * prevent a double free by the caller.
 	 */
 	xfr->stream = stream;
@@ -1516,12 +1522,13 @@ sendstream(xfrout_ctx_t *xfr) {
 					   "(%d bytes)",
 					   size);
 				/* XXX DNS_R_RRTOOLARGE? */
-				CHECK(ISC_R_NOSPACE);
+				result = ISC_R_NOSPACE;
+				goto failure;
 			}
 			break;
 		}
 
-		if (isc_log_wouldlog(ns_lctx, XFROUT_RR_LOGLEVEL)) {
+		if (isc_log_wouldlog(XFROUT_RR_LOGLEVEL)) {
 			log_rr(name, rdata, ttl); /* XXX */
 		}
 
@@ -1614,7 +1621,7 @@ sendstream(xfrout_ctx_t *xfr) {
 	/* Advance lasttsig to be the last TSIG generated */
 	CHECK(dns_message_getquerytsig(msg, xfr->mctx, &xfr->lasttsig));
 
-cleanup:
+failure:
 	if (tcpmsg != NULL) {
 		dns_message_detach(&tcpmsg);
 	}
