@@ -236,7 +236,7 @@ create_bundle(FILE *fp, isc_stdtime_t btime, int bnum) {
 	ISC_LIST_APPEND(dnskeylist->rdata, &test_bundles[bnum].ksk, link);
 	dns_rdatalist_tordataset(dnskeylist, dnskeyset);
 	dns_rdata_init(&test_bundles[bnum].dnskey_rrsig);
-	sign_rrset(fp, btime, btime + LIFETIME, dnskeyset,
+	sign_rrset(fp, btime, (btime + LIFETIME), dnskeyset,
 		   test_bundles[bnum].rrsig1buf,
 		   &test_bundles[bnum].dnskey_rrsig);
 	for (dns_rdata_t *rd = ISC_LIST_HEAD(dnskeylist->rdata); rd != NULL;
@@ -263,7 +263,7 @@ create_bundle(FILE *fp, isc_stdtime_t btime, int bnum) {
 	ISC_LIST_APPEND(cdnskeylist->rdata, &test_bundles[bnum].cdnskey, link);
 	dns_rdatalist_tordataset(cdnskeylist, cdnskeyset);
 	dns_rdata_init(&test_bundles[bnum].cdnskey_rrsig);
-	sign_rrset(fp, btime, btime + LIFETIME, cdnskeyset,
+	sign_rrset(fp, btime, (btime + LIFETIME), cdnskeyset,
 		   test_bundles[bnum].rrsig2buf,
 		   &test_bundles[bnum].cdnskey_rrsig);
 	for (dns_rdata_t *rd = ISC_LIST_HEAD(cdnskeylist->rdata); rd != NULL;
@@ -289,7 +289,7 @@ create_bundle(FILE *fp, isc_stdtime_t btime, int bnum) {
 	ISC_LIST_APPEND(cdslist->rdata, &test_bundles[bnum].cds, link);
 	dns_rdatalist_tordataset(cdslist, cdsset);
 	dns_rdata_init(&test_bundles[bnum].cds_rrsig);
-	sign_rrset(fp, btime, btime + LIFETIME, cdsset,
+	sign_rrset(fp, btime, (btime + LIFETIME), cdsset,
 		   test_bundles[bnum].rrsig3buf, &test_bundles[bnum].cds_rrsig);
 	for (dns_rdata_t *rd = ISC_LIST_HEAD(cdslist->rdata); rd != NULL;
 	     rd = ISC_LIST_HEAD(cdslist->rdata))
@@ -442,8 +442,6 @@ ISC_RUN_TEST_IMPL(skr_read) {
 	isc_result_t result;
 	size_t count = 0;
 
-	dst_lib_init(mctx, NULL);
-
 	/* Owner name */
 	dname = dns_fixedname_initname(&dfname);
 	isc_buffer_init(&b, name, strlen(name));
@@ -494,8 +492,6 @@ ISC_RUN_TEST_IMPL(skr_read) {
 		dst_key_free(&key->key);
 		dns_dnsseckey_destroy(mctx, &key);
 	}
-
-	dst_lib_destroy();
 }
 
 ISC_TEST_LIST_START
