@@ -38,30 +38,14 @@
 
 #define TEST_ORIGIN "test"
 
-static int
-setup_test(void **state) {
-	isc_result_t result;
-
-	UNUSED(state);
-
-	result = dst_lib_init(mctx, NULL);
-
-	if (result != ISC_R_SUCCESS) {
-		return 1;
+#define CHECK(r)                               \
+	{                                      \
+		result = (r);                  \
+		if (result != ISC_R_SUCCESS) { \
+			goto cleanup;          \
+		}                              \
 	}
 
-	return 0;
-}
-
-static int
-teardown_test(void **state) {
-	UNUSED(state);
-
-	dst_lib_destroy();
-
-	return 0;
-}
-
 static isc_result_t
 add_mac(dst_context_t *tsigctx, isc_buffer_t *buf) {
 	dns_rdata_any_tsig_t tsig;
@@ -110,7 +94,7 @@ add_tsig(dst_context_t *tsigctx, dns_tsigkey_t *key, isc_buffer_t *target,
 	tsig.common.rdtype = dns_rdatatype_tsig;
 	ISC_LINK_INIT(&tsig.common, link);
 	dns_name_init(&tsig.algorithm, NULL);
-	dns_name_clone(dns_tsigkey_algorithm(key), &tsig.algorithm);
+	dns_name_clone(key->algorithm, &tsig.algorithm);
 
 	tsig.timesigned = now;
 	tsig.fudge = DNS_TSIG_FUDGE;
@@ -438,16 +422,7 @@ tsig_tcp(isc_stdtime_t now, isc_result_t expected_result, bool mangle_sig) {
 		assert_int_equal(msg->verified_sig, 1);
 		assert_int_equal(msg->tsigstatus, dns_tsigerror_badtime);
 		break;
-	case DNS_R_TSIGVERIFYFAILURE:
-		assert_int_equal(result, DNS_R_TSIGVERIFYFAILURE);
-		assert_int_equal(msg->verified_sig, 0);
-		assert_int_equal(msg->tsigstatus, dns_tsigerror_badsig);
-		break;
 	default:
-		if (debug) {
-			fprintf(stderr, "# result = %s\n",
-				isc_result_totext(result));
-		}
 		UNREACHABLE();
 	}
 
@@ -499,7 +474,7 @@ ISC_RUN_TEST_IMPL(tsig_badtime) {
 }
 
 ISC_RUN_TEST_IMPL(tsig_badsig) {
-	tsig_tcp(isc_stdtime_now(), DNS_R_TSIGVERIFYFAILURE, true);
+	tsig_tcp(isc_stdtime_now(), DNS_R_TSIGERRORSET, true);
 }
 
 /* Tests the dns__tsig_algvalid function */
@@ -518,10 +493,9 @@ ISC_RUN_TEST_IMPL(algvalid) {
 }
 
 ISC_TEST_LIST_START
+ISC_TEST_ENTRY(tsig_tcp)
+ISC_TEST_ENTRY(tsig_badtime)
 ISC_TEST_ENTRY(algvalid)
-ISC_TEST_ENTRY_CUSTOM(tsig_badsig, setup_test, teardown_test)
-ISC_TEST_ENTRY_CUSTOM(tsig_badtime, setup_test, teardown_test)
-ISC_TEST_ENTRY_CUSTOM(tsig_tcp, setup_test, teardown_test)
 ISC_TEST_LIST_END
 
 ISC_TEST_MAIN
